# -*- coding: utf-8 -*-

"""
vtoolboxplugin.py

A custom tool box widget plugin for Qt Designer.

Copyright (C) 2014-2016 Vincent Vande Vyvre. <vincent.vandevyvre@oqapy.eu>

GNU General Public License Usage
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""


from PyQt5.QtGui import QPixmap, QIcon
from PyQt5.QtDesigner import QPyDesignerCustomWidgetPlugin

from VWidgets_PyQt5.vtoolbox import VToolBox

class VToolBoxPlugin(QPyDesignerCustomWidgetPlugin):
    """Provides a Python custom plugin for Qt Designer by implementing the
    QDesignerCustomWidgetPlugin using a PyQt-specific custom plugin class.
    """

    # The __init__() method is only used to set up the plugin and define its
    # initialized variable.
    def __init__(self, parent=None):
    
        QPyDesignerCustomWidgetPlugin.__init__(self)
        self.initialized = False

    # The initialize() and isInitialized() methods allow the plugin to set up
    # any required resources, ensuring that this can only happen once for each
    # plugin.
    def initialize(self, formEditor):

        if self.initialized:
            return

        manager = formEditor.extensionManager()
        
        self.initialized = True

    def isInitialized(self):
        return self.initialized

    # This factory method creates new instances of our custom widget with the
    # appropriate parent.
    def createWidget(self, parent):
        return VToolBox(parent)

    # This method returns the name of the custom widget class that is provided
    # by this plugin.
    def name(self):
        return "VToolBox"

    # Returns the name of the group in Qt Designer's widget box that this
    # widget belongs to.
    def group(self):
        return "VWidgets PyQt5"

    # Returns the icon used to represent the custom widget in Qt Designer's
    # widget box.
    def icon(self):
        return QIcon()

    # Returns a short description of the custom widget for use in a tool tip.
    def toolTip(self):
        return "Tool box"

    # Returns a short description of the custom widget for use in a "What's
    # This?" help message for the widget.
    def whatsThis(self):
        return "Tool box collapsible with header bar."

    # Returns True if the custom widget acts as a container for other widgets;
    # otherwise returns False. Note that plugins for custom containers also
    # need to provide an implementation of the QDesignerContainerExtension
    # interface if they need to add custom editing support to Qt Designer.
    def isContainer(self):
        return True

    # Returns an XML description of a custom widget instance that describes
    # default values for its properties. Each custom widget created by this
    # plugin will be configured using this description.
    def domXml(self):
        return '<widget class="VToolBox" name="toolBox" />\n'

    # Returns the module containing the custom widget class. It may include
    # a module path.
    def includeFile(self):
        return "VWidgets_PyQt5.vtoolbox"
