# -*- coding: utf-8 -*-

import os
import sys
import unittest

from PyQt5.QtGui import QColor
from PyQt5.QtWidgets import  QWidget, QVBoxLayout, QApplication
from PyQt5.QtCore import pyqtProperty

pth = os.path.split(os.path.dirname(os.path.abspath(__file__)))[0]
sys.path.append(pth)

from vcolorcombobox import VColorComboBox
from colorsdata import BASIC_PLT

TESTPALETTE = {'Red': (255, 0, 0), 'Orangered': (255, 69, 0),
                'Crimson': (220, 20, 60), 'Dark red': (128, 0, 0),
                'Green': (0, 255, 0), 'Limegreen': (50, 205, 50),
                'Light green': (14, 255, 187), 'Dark green': (0, 128, 0),
                'Blue': (0, 0, 255), 'Royal blue': (65, 105, 225),
                'Indigo': (75, 0, 130), 'Dark blue': (0, 0, 128),
                'Cyan': (0, 255, 255), 'Magenta': (255, 0, 255),
                'Yellow': (255, 255, 0)}
app = QApplication([])

class TestVColorComboBox(unittest.TestCase):

    def setUp(self):
        # Create a VColorComboBox
        self.widget = QWidget()
        self.layout = QVBoxLayout(self.widget)
        self.ccbx = VColorComboBox(self.widget)
        self.layout.addWidget(self.ccbx)

    def test_attributes(self):
        self.assertEqual(self.ccbx.border_icon_color, QColor(0, 0, 0))
        self.assertEqual(self.ccbx.palette_name, BASIC_PLT)
        self.assertEqual(type(self.ccbx.palette_), dict)
        self.assertEqual(len(self.ccbx.colors), 20)
        self.assertEqual(self.ccbx.is_named_color, True)
        self.assertEqual(self.ccbx.sorting, 'provided')
        self.assertEqual(self.ccbx.color_space, 'RGB')
        self.assertEqual(self.ccbx.add_more_choice, True)
        self.assertEqual(self.ccbx.call_color_dialog, True)
        self.assertEqual(self.ccbx.use_svg_naming, True)
        self.assertEqual(self.ccbx.more_text, 'More...')

    def test_pyqtproperties(self):
        self.assertEqual(type(self.ccbx.borderIconColor_), QColor)
        self.assertEqual(self.ccbx.borderIconColor_, self.ccbx.border_icon_color)
        self.assertRaises(TypeError, self.ccbx.setBorderIconColor, 'blue')
        self.assertEqual(type(self.ccbx.namedColor), bool)
        self.assertRaises(TypeError, self.ccbx.setNamedColor, 'blue')
        self.assertEqual(type(self.ccbx.addMoreChoice_), bool)
        self.assertRaises(TypeError, self.ccbx.addMoreChoice, 'blue')
        self.assertRaises(TypeError, self.ccbx.addMoreChoice, True, call='blue')
        self.assertEqual(type(self.ccbx.moreText_), str)
        self.assertRaises(TypeError, self.ccbx.setMoreText, 1)
        self.assertEqual(type(self.ccbx.callColorDialog_), bool)
        self.assertRaises(TypeError, self.ccbx.setCallColorDialog, 'blue')
        self.assertEqual(type(self.ccbx.useSvgNaming_), bool)
        self.assertRaises(TypeError, self.ccbx.setUseSvgNaming, bool)

    def test_methods(self):
        # Change some properties
        self.ccbx.set_palette(TESTPALETTE)
        self.ccbx.set_sorting_mode('green')
        self.ccbx.setBorderIconColor(QColor(255, 0, 0))
        self.ccbx.setCallColorDialog(False)
        self.ccbx.setMoreText('Other...')
        self.ccbx.setNamedColor(False)
        self.ccbx.setUseSvgNaming(False)
        self.ccbx.set_current_color(QColor(255, 255, 0)) # Yellow
        self.assertEqual(self.ccbx.borderIconColor(), QColor(255, 0, 0))
        self.assertEqual(self.ccbx.sorting, 'green')
        self.assertEqual(self.ccbx.get_current_color(), ('Yellow',  QColor(255, 255, 0)))  
        self.assertEqual(self.ccbx.moreText(), 'Other...')
        self.assertEqual(self.ccbx.isNamedColor(), False)
        self.assertEqual(self.ccbx.useSvgNaming(), False)
        self.assertEqual(self.ccbx.find_color(QColor(65, 105, 225)), 6)
        self.assertRaises(TypeError, self.ccbx.find_color, 'Royal blue')
        self.assertEqual(type(self.ccbx.get_colors()), list)
        self.assertEqual(self.ccbx.get_color_space(), 'RGB')
        self.assertEqual(self.ccbx.get_current_name(), 'Yellow')
        self.assertEqual(type(self.ccbx.get_height()), int)
        self.assertEqual(type(self.ccbx.get_icon_height()), int)
        self.assertEqual(type(self.ccbx.get_icon_size()), tuple)
        self.assertEqual(type(self.ccbx.get_icon_width()), int)
        self.assertEqual(type(self.ccbx.get_size()), tuple)
        self.assertEqual(type(self.ccbx.get_width()), int)

