# -*- coding: utf-8 -*-

import os
import sys
import unittest

from PyQt5.QtWidgets import  QWidget, QVBoxLayout, QApplication
from PyQt5.QtCore import Qt

pth = os.path.split(os.path.dirname(os.path.abspath(__file__)))[0]
sys.path.append(pth)

from vlineedit import VLineEdit

class TestVLineEdit(unittest.TestCase):

    def setUp(self):
        # Create a VColorComboBox
        self.widget = QWidget()
        self.layout = QVBoxLayout(self.widget)
        self.vled = VLineEdit(self.widget)
        self.layout.addWidget(self.vled)

    def test_attributes(self):
        # Qt.LeftToRight == 0, Qt.RightToLeft == 1
        self.assertEqual(self.vled.getOrientation(), Qt.LeftToRight)
        self.assertEqual(self.vled.hasCommandButton(), False)
        self.assertEqual(self.vled.hasHistoricButton(), False)
        self.assertEqual(self.vled.hasResetButton(), False)
        self.assertEqual(self.vled.informativeText(), '')

    def test_methods(self):
        # Set some properties and values
        self.vled.setResetButton(True)
        self.vled.setHistoricButton(True)
        self.vled.setCommandButton(True)
        self.vled.setInformativeText('Déjà vu')
        self.vled.add_historic_entry('First line')
        self.vled.add_historic_entry('Second line')
        self.vled.setText('Usefull information')
        self.assertEqual(self.vled.informativeText(), 'Déjà vu')
        self.vled.command_button.click()
        self.assertEqual(self.vled.get_historic_entries(), ['Usefull information',
                                                            'Second line',
                                                            'First line'])
        self.vled.setText('Other information')
        self.vled.reset_button.click()
        self.assertEqual(self.vled.text(), 'Usefull information')
        self.vled.resetTriggered.connect(self.vled.clear)
        self.vled.reset_button.click()
        self.assertEqual(self.vled.text(), '')
        self.vled.clear_historic()
        self.vled.setResetButton(False)
        self.vled.setHistoricButton(False)
        self.vled.setCommandButton(False)
        self.assertEqual(self.vled.get_historic_entries(), [])
        self.assertRaises(TypeError, self.vled.add_historic_entry, 1)
        self.assertRaises(TypeError, self.vled.add_historic_entries, 1)
        self.assertRaises(TypeError, self.vled.add_historic_entries, ['text'], clear='clear')
        self.assertRaises(TypeError, self.vled.setHistoricButton, 'text')
        self.assertRaises(TypeError, self.vled.setCommandButton, 'text')
        self.assertRaises(TypeError, self.vled.setResetButton, 'text')
        self.assertRaises(TypeError, self.vled.setInformativeText, None)

