# -*- coding: utf-8 -*-

import os
import sys
import unittest

from PyQt5.QtWidgets import  QWidget, QVBoxLayout, QApplication, QFrame
from PyQt5.QtCore import Qt

pth = os.path.split(os.path.dirname(os.path.abspath(__file__)))[0]
sys.path.append(pth)

from vtoolheader import VToolHeader
app = QApplication([])

class TestVToolHeader(unittest.TestCase):

    def setUp(self):
        # Create a VToolHeader
        self.widget = QWidget()
        self.layout = QVBoxLayout(self.widget)
        self.thd = VToolHeader(self.widget)
        self.layout.addWidget(self.thd)

    def test_attributes(self):
        self.assertEqual(self.thd.getFrameShadow(), QFrame.Raised)
        self.assertEqual(self.thd.getFrameShape(), QFrame.Panel)
        self.assertEqual(self.thd.getTitle(), 'Tool Header')
        self.assertEqual(self.thd.getOrientation(), Qt.LeftToRight)
        self.assertEqual(self.thd.hasCollapseButton(), True)
        self.assertEqual(self.thd.isOrientedByLanguage(), False)

    def Test_methods(self):
        self.assertRaises(TypeError, self.thd.setTitle, True)
        self.assertRaises(TypeError, self.thd.setCollapseButton, 'text')
        self.assertRaises(TypeError, self.thd.setFrameShadow_, 'shadow')
        self.assertRaises(TypeError, self.thd.setFrameShape_, 'shape')
        self.assertRaises(TypeError, self.thd.setIcon, True)
        self.assertRaises(TypeError, self.thd.setOrientation, 'orientation')
        self.assertRaises(TypeError, self.thd.setOrientedByLanguage, 'left')
        # Set some properties
        self.thd.setTitle('Edit tools')
        self.thd.setOrientedByLanguage(True)
        self.thd.setFrameShadow_(QFrame.Sunken)
        self.thd.setFrameShape_(QFrame.Box)
        self.thd.add_button('medias/add.png', 'Action')
        self.thd.add_button('medias/add.png', 'View')
        self.assertEqual(self.thd.getTitle(), 'Edit tools')
        # Assuming this test is executed with a leftToRigth language
        self.assertEqual(self.thd.getOrientation(), Qt.LeftToRight)
        self.assertEqual(self.thd.getFrameShadow(), QFrame.Sunken)
        self.assertEqual(self.thd.getFrameShape(), QFrame.Box)
        self.assertEqual(self.thd.count_buttons(), 2)

