# -*- coding: utf-8 -*-

import sys

from PyQt5 import QtCore, QtGui, QtWidgets


class VProgressCircle(QtWidgets.QWidget):
    clockWise = -1
    counterClockWise = 1
    def __init__(self, parent=None):
        super(VProgressCircle, self).__init__(parent)
        dct = dict(minimum_ = 0,
                    maximum_ = 100,
                    ratio = 1,
                    value_ = 0,
                    origin_ = 1440,
                    direction_ = self.clockWise,
                    radius_ = 11,
                    tool_tip = True,
                    anim_step = 288,
                    border_color = QtGui.QColor(0, 0, 0, 255),
                    fill_color = QtGui.QColor(255, 255, 255, 255),
                    pie_border_color = QtGui.QColor(140, 140, 160, 255),
                    pie_fill_color = QtGui.QColor(211, 211, 211, 255))

        for key, val in dct.items():
            setattr(self, key, val)

        self.setBaseSize(50, 50)
        size_policy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred,
                                        QtWidgets.QSizePolicy.Preferred)
        size_policy.setHeightForWidth(True)
        self.setSizePolicy(size_policy)
        self.setMinimumSize(35, 35)
        self.__set_angle()
        self.__set_background()

    def minimum(self):
        return self.minimum_

    def setMinimum(self, value):
        if not isinstance(value, int):
            raise("VProgressCircle.setMinimum(value): arg 1 has"
                  "unexpected type: '{0}'".format(type(value)))

        self.minimum_ = abs(value)
        self.maximum_ = max(self.minimum_, self.maximum_)
        self.__set_ratio()

    minimum__ = QtCore.pyqtProperty(int, minimum, setMinimum)

    def maximum(self):
        return self.maximum_

    def setMaximum(self, value):
        if not isinstance(value, int):
            raise("VProgressCircle.setMaximum(value): arg 1 has"
                  "unexpected type: '{0}'".format(type(value)))

        self.maximum_ = max(self.minimum_, abs(value))
        self.__set_ratio()

    maximum__ = QtCore.pyqtProperty(int, maximum, setMaximum)

    def value(self):
        return self.value_

    def setValue(self, val):
        self.value_ = abs(val)
        self.__set_angle()
        self.repaint()
        if self.tool_tip:
            self.setToolTip(self.value_)

    value__ = QtCore.pyqtProperty(int, value, setValue)

    def setRange(self, minimum, maximum):
        self.setMinimum(minimum)
        self.setMaximum(maximum)

    def origin(self):
        return self.origin_

    def setOrigin(self, value):
        self.origin_ = int(value)

    origin__ = QtCore.pyqtProperty(int, origin, setOrigin)

    def hasToolTip(self):
        return self.tool_tip

    def showToolTip(self, show):
        if not isinstance(show, bool):
            raise("VProgressCircle.showToolTip(bool): arg 1 has"
                  "unexpected type: '{0}'".format(type(show)))

        self.tool_tip = show
        if show:
            self.setToolTip(self.value_)

        else:
            self.setToolTip('')

    toolTip = QtCore.pyqtProperty(bool, hasToolTip, showToolTip)

    def __set_ratio(self):
        if not any([self.maximum_, self.minimum_]):
            self.angle = 0.0
            self.set_perpetual_animation()
            return

        self.ratio = 1 / ((self.maximum_ - self.minimum_) / 100)

    def __set_angle(self):
        val = self.value_ * self.ratio
        self.angle = 57.60 * val * self.direction_

    def __set_background(self):
        self.bgd = QtGui.QPixmap(self.width()-2, self.height()-2)
        self.bgd.fill(QtGui.QColor(0, 0, 0, 0))

    def resizeEvent(self, event):
        self.radius_ = self.width() / 2 - 2
        self.__set_background()
        QtWidgets.QWidget.resizeEvent(self, event)

    def heightForWidth(self, width):
        return width

    def paintEvent(self, event):
        pix = self.draw()
        painter = QtGui.QPainter()
        painter.begin(self)
        painter.drawPixmap(2, 2, self.width()-2, self.height()-2, pix)
        painter.end()

    def draw(self):
        size = self.radius_ * 2
        crect = QtCore.QRectF(1.0, 1.0, size, size)
        prect = QtCore.QRectF(2.0, 2.0, size-2, size-2) 
        pen = QtGui.QPen(self.border_color, 1, QtCore.Qt.SolidLine, 
                            QtCore.Qt.RoundCap, QtCore.Qt.MiterJoin)
        brush = QtGui.QBrush(self.fill_color)
        painter = QtGui.QPainter()
        painter.begin(self.bgd)
        painter.setRenderHints(QtGui.QPainter.Antialiasing, True)
        painter.setPen(pen)
        painter.setBrush(brush)
        painter.drawEllipse(crect)
        painter.setPen(self.pie_border_color)
        painter.setBrush(self.pie_fill_color)
        painter.drawPie(prect, self.origin_, self.angle)
        painter.end()
        return self.bgd

    def set_perpetual_animation(self):
        self.setToolTip('')
        self.timer = QtCore.QTimeLine(2000)
        self.timer.setFrameRange(0, 21)
        self.timer.frameChanged.connect(self.change_value)
        # Don't use the finished signal, the last frame is not updated.
        self.timer.start()

    def change_value(self):
        self.angle -= self.anim_step
        self.repaint()
        if self.angle in [0.0, -5760]:
            self.restart()

    def restart(self):
        self.anim_step *= -1
        if not any([self.maximum_, self.minimum_]):
            self.set_perpetual_animation()

    def reset(self):
        self.value_ = self.minimum_
        self.__set_angle()
        self.repaint()

class MainWindow(object):
    def setupUi(self, mw):
        mw.resize(200, 50)
        self.centralwidget = QtWidgets.QWidget(mw)
        self.grid = QtWidgets.QGridLayout(self.centralwidget)
        self.vlayout = QtWidgets.QVBoxLayout()
        self.hlayout = QtWidgets.QHBoxLayout()
        self.vpc = VProgressCircle(mw)
        self.hlayout.addWidget(self.vpc)
        self.button = QtWidgets.QPushButton(mw)
        self.hlayout.addWidget(self.button)
        spacer = QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Expanding, 
                                    QtWidgets.QSizePolicy.Minimum)
        self.hlayout.addItem(spacer)
        self.vlayout.addLayout(self.hlayout)
        spacer1 = QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Minimum, 
                                    QtWidgets.QSizePolicy.Expanding)
        self.vlayout.addItem(spacer1)
        self.grid.addLayout(self.vlayout, 0, 0, 1, 1)
        mw.setCentralWidget(self.centralwidget)
        self.button.clicked.connect(self.set_values)

    def run_timeline(self):
        self.timer = QtCore.QTimeLine(3000)
        self.timer.setFrameRange(0, 21)
        self.timer.frameChanged.connect(self.change_value)
        self.timer.finished.connect(self.reset)
        self.timer.start()

    def change_value(self):
        v = self.vpc.value() + 5
        self.vpc.setValue(v)

    def reset(self):
       self.vpc.setValue(0)

    def set_values(self):
        self.vpc.setRange(0, 0)


if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    mw = QtWidgets.QMainWindow()
    ui = MainWindow()
    ui.setupUi(mw)
    mw.show()
    sys.exit(app.exec_())
        
