# -*- coding: utf-8 -*-

# camerapage.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2012-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Tool box camera

from PyQt5.QtCore import Qt, QSize, QCoreApplication
from PyQt5.QtGui import QIcon, QPixmap
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QComboBox, QSizePolicy, QSpacerItem, QToolButton,
                             QPlainTextEdit, QLabel, QPushButton, QProgressBar)

import stylesheets as styles

class CameraPage(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        sp = QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.Expanding)
        self.setSizePolicy(sp)
        self.setMinimumSize(QSize(0, 200))
        self.setMaximumSize(QSize(275, 250))
        self.gridLayout = QGridLayout(self)
        self.gridLayout.setContentsMargins(4, 4, 4, 4)
        self.gridLayout.setSpacing(4)
        self.vLayout = QVBoxLayout()
        self.horizontalLayout = QHBoxLayout()
        self.cam_cmb = QComboBox(self)
        self.cam_cmb.setMinimumSize(QSize(220, 25))
        self.cam_cmb.setMaximumSize(QSize(250, 25))
        self.horizontalLayout.addWidget(self.cam_cmb)
        self.info_btn = QToolButton(self)
        self.info_btn.setMaximumSize(QSize(25, 25))
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/infos.png"), QIcon.Normal, QIcon.Off)
        self.info_btn.setIcon(icon)
        self.info_btn.setAutoRaise(True)
        self.horizontalLayout.addWidget(self.info_btn)
        self.vLayout.addLayout(self.horizontalLayout)
        self.camera_edit = QPlainTextEdit(self)
        self.camera_edit.setReadOnly(True)
        self.vLayout.addWidget(self.camera_edit)
        self.horizontalLayout_2 = QHBoxLayout()
        self.refresh_btn = QToolButton(self)
        self.refresh_btn.setMaximumSize(QSize(25, 25))
        icon_1 = QIcon()
        icon_1.addPixmap(QPixmap("medias/gtk-refresh.png"), QIcon.Normal, 
                         QIcon.Off)
        self.refresh_btn.setIcon(icon_1)
        self.refresh_btn.setAutoRaise(True)
        self.horizontalLayout_2.addWidget(self.refresh_btn)
        self.cam_com_lbl = QLabel(self)
        self.horizontalLayout_2.addWidget(self.cam_com_lbl)
        sp1 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(sp1)
        self.show_content_btn = QPushButton(self)
        self.show_content_btn.setEnabled(False)
        self.horizontalLayout_2.addWidget(self.show_content_btn)
        self.vLayout.addLayout(self.horizontalLayout_2)
        self.cam_pgb = QProgressBar(self)
        self.cam_pgb.setMaximumSize(QSize(16777215, 12))
        self.cam_pgb.setMinimum(0)
        self.cam_pgb.setMaximum(0)
        self.cam_pgb.setTextVisible(False)
        self.vLayout.addWidget(self.cam_pgb)
        self.gridLayout.addLayout(self.vLayout, 0, 0, 1, 1)

        self.cam_com_lbl.setText(_("Search camera"))
        QCoreApplication.processEvents()
        self.show_content_btn.setText(_("Show content"))
        self.show_content_btn.setToolTip(_("Show the content of the camera"))
        self.refresh_btn.setToolTip(_("Refresh"))

        self.cam_cmb.setStyleSheet(styles.COMBO)
        self.info_btn.setStyleSheet(styles.TBUTTON)
        self.camera_edit.setStyleSheet(styles.TEXTEDIT)
        self.refresh_btn.setStyleSheet(styles.TBUTTON)
        self.show_content_btn.setStyleSheet(styles.PUSHBUTTON)
        self.cam_pgb.setStyleSheet(styles.PROGRESSBAR)
