# -*- coding: utf-8 -*-

# capturepage.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2012-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# http://gphoto.sourceforge.net/doc/remote/

from PyQt5 import QtCore, QtGui, QtWidgets


class CapturePage(QtWidgets.QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding,
                                        QtWidgets.QSizePolicy.Expanding)
        self.setSizePolicy(sizePolicy)
        self.setMinimumSize(QtCore.QSize(0, 400))
        self.setMaximumSize(QtCore.QSize(260, 9999))
        self.gridLayout_2 = QtWidgets.QGridLayout(self)
        self.gridLayout_2.setContentsMargins(2, 2, 2, 2)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.viewer = QtWidgets.QGraphicsView(self)
        self.viewer.setObjectName("graphicsView")
        self.viewer.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.viewer.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.viewer.setRenderHints(QtGui.QPainter.Antialiasing|
                                    QtGui.QPainter.HighQualityAntialiasing|
                                    QtGui.QPainter.TextAntialiasing)
        self.verticalLayout.addWidget(self.viewer)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.left_btn = QtWidgets.QToolButton(self)
        self.left_btn.setText("L")
        self.left_btn.setObjectName("left_btn")
        self.horizontalLayout.addWidget(self.left_btn)
        self.right_btn = QtWidgets.QToolButton(self)
        self.right_btn.setText("R")
        self.right_btn.setObjectName("right_btn")
        self.horizontalLayout.addWidget(self.right_btn)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.preview_btn = QtWidgets.QPushButton(self)
        self.preview_btn.setObjectName("preview_btn")
        self.horizontalLayout.addWidget(self.preview_btn)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        spacerItem1 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Fixed, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem1)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtWidgets.QLabel(self)
        self.label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                        QtCore.Qt.AlignVCenter)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.delay_spb = QtWidgets.QSpinBox(self)
        self.delay_spb.setSuffix(" sec.")
        self.delay_spb.setMaximum(999)
        self.delay_spb.setObjectName("delay_spb")
        self.gridLayout.addWidget(self.delay_spb, 0, 1, 1, 1)
        self.label_1 = QtWidgets.QLabel(self)
        self.label_1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                        QtCore.Qt.AlignVCenter)
        self.label_1.setObjectName("label_1")
        self.gridLayout.addWidget(self.label_1, 1, 0, 1, 1)
        self.frames_spb = QtWidgets.QSpinBox(self)
        self.frames_spb.setMinimum(1)
        self.frames_spb.setMaximum(999)
        self.frames_spb.setObjectName("frames_spb")
        self.gridLayout.addWidget(self.frames_spb, 1, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(self)
        self.label_2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                        QtCore.Qt.AlignVCenter)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 2, 0, 1, 1)
        self.interval_spb = QtWidgets.QSpinBox(self)
        self.interval_spb.setSuffix(" sec.")
        self.interval_spb.setObjectName("interval_spb")
        self.gridLayout.addWidget(self.interval_spb, 2, 1, 1, 1)
        self.horizontalLayout_5.addLayout(self.gridLayout)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem2)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.start_btn = QtWidgets.QPushButton(self)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("medias/dialog-apply.svg"), 
                                        QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.start_btn.setIcon(icon)
        self.start_btn.setObjectName("start_btn")
        self.horizontalLayout_2.addWidget(self.start_btn)
        self.report_lbl = QtWidgets.QLabel(self)
        self.report_lbl.setObjectName("report_lbl")
        self.horizontalLayout_2.addWidget(self.report_lbl)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.stop_btn = QtWidgets.QToolButton(self)
        self.stop_btn.setText("None")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap("medias/process-stop.png"), 
                                        QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.stop_btn.setIcon(icon1)
        self.stop_btn.setAutoRaise(True)
        self.stop_btn.setObjectName("stop_btn")
        self.horizontalLayout_2.addWidget(self.stop_btn)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.line = QtWidgets.QFrame(self)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.tethered_chb = QtWidgets.QCheckBox(self)
        self.tethered_chb.setObjectName("tethered_chb")
        self.horizontalLayout_3.addWidget(self.tethered_chb)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem4)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.line_2 = QtWidgets.QFrame(self)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout.addWidget(self.line_2)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_3 = QtWidgets.QLabel(self)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_4.addWidget(self.label_3)
        self.mem_lbl = QtWidgets.QLabel(self)
        self.mem_lbl.setObjectName("mem_lbl")
        self.horizontalLayout_4.addWidget(self.mem_lbl)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem5)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.gridLayout_2.addLayout(self.verticalLayout, 0, 0, 1, 1)

        self.scene = QtWidgets.QGraphicsScene()
        self.viewer.setScene(self.scene)

        self.retranslateUi()
        QtCore.QMetaObject.connectSlotsByName(self)

    def display_preview(self, path):
        self.img = QtGui.QImage(path)
        self.set_pixmap()

    def set_pixmap(self):
        w, h = self.img.width(), self.img.height()
        width, height = 256, 220

        if w > h:
            height = int(256.0 * (h / float(w)))
            
        else:
            width = int(220.0 * (w / float(h)))

        self.pixmap = QtGui.QPixmap(self.img.scaled(width, height,
                                    QtCore.Qt.KeepAspectRatio, 
                                    QtCore.Qt.SmoothTransformation))

        self.viewer.setSize(QtCore.QSize(256, height))
        self.scene.clear()
        self.scene.setSceneRect(0, 0, width, height)
        self.scene.addPixmap(self.pix)


    def retranslateUi(self):
        self.preview_btn.setText(_("Preview"))
        self.label.setText(_("Delay: "))
        self.label_1.setText(_("Frames: "))
        self.label_2.setText(_("Interval: "))
        self.start_btn.setText(_("Start"))
        self.report_lbl.setText(_("TextLabel"))
        self.tethered_chb.setText(_("Tethered mode"))
        self.label_3.setText(_("Memory free: "))
        self.mem_lbl.setText(_("0 Ko"))
