# -*- coding: utf-8 -*-

# importpage.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2012-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Import tool box

from PyQt5.QtCore import QSize
from PyQt5.QtGui import QIcon, QPixmap
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QComboBox, QSizePolicy, QSpacerItem, QPushButton,
                             QLabel, QRadioButton, QProgressBar)

import stylesheets as styles

class ImportPage(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        sp = QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.Expanding)
        self.setSizePolicy(sp)
        self.setMaximumSize(QSize(260, 500))
        self.gridLayout_2 = QGridLayout(self)
        self.gridLayout_2.setContentsMargins(4, 4, 4, 4)
        self.verticalLayout = QVBoxLayout()
        self.gridLayout = QGridLayout()
        self.gridLayout.setVerticalSpacing(2)
        self.all_rdo = QRadioButton(self)
        self.all_rdo.setChecked(True)
        self.gridLayout.addWidget(self.all_rdo, 0, 0, 1, 1)
        spacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout.addItem(spacer, 0, 1, 1, 1)
        self.select_rdo = QRadioButton(self)
        self.gridLayout.addWidget(self.select_rdo, 1, 0, 1, 1)
        space1 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout.addItem(space1, 1, 1, 1, 1)
        self.unselect_rdo = QRadioButton(self)
        self.gridLayout.addWidget(self.unselect_rdo, 2, 0, 1, 1)
        space2 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout.addItem(space2, 2, 1, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.horizontalLayout_5 = QHBoxLayout()
        self.only_rdo = QRadioButton(self)
        self.horizontalLayout_5.addWidget(self.only_rdo)
        self.ext_cmb = QComboBox(self)
        self.ext_cmb.setMaximumSize(QSize(9999, 22))
        self.ext_cmb.setEnabled(False)
        self.horizontalLayout_5.addWidget(self.ext_cmb)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_3 = QHBoxLayout()
        self.select_all_btn = QPushButton(self)
        self.select_all_btn.setMaximumSize(QSize(9999, 22))
        self.horizontalLayout_3.addWidget(self.select_all_btn)
        self.clear_btn = QPushButton(self)
        self.clear_btn.setMaximumSize(QSize(9999, 22))
        self.horizontalLayout_3.addWidget(self.clear_btn)
        space3 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(space3)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_2 = QHBoxLayout()
        self.import_btn = QPushButton(self)
        self.import_btn.setMaximumSize(QSize(9999, 22))
        self.horizontalLayout_2.addWidget(self.import_btn)
        self.report_lbl = QLabel(self)
        self.horizontalLayout_2.addWidget(self.report_lbl)
        space4 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(space4)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setSpacing(2)
        self.import_pgb = QProgressBar(self)
        self.import_pgb.setMaximumSize(QSize(16777215, 12))
        self.import_pgb.setProperty("value", 0)
        self.import_pgb.setTextVisible(False)
        self.horizontalLayout.addWidget(self.import_pgb)
        self.cancel_btn = QPushButton(self)
        self.cancel_btn.setMaximumSize(QSize(40, 16777215))
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/process-stop.png"),
                                        QIcon.Normal, QIcon.Off)
        self.cancel_btn.setIcon(icon)
        self.cancel_btn.setFlat(True)
        self.horizontalLayout.addWidget(self.cancel_btn)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.gridLayout_2.addLayout(self.verticalLayout, 0, 0, 1, 1)

        self.all_rdo.setText(_("Import all files"))
        self.select_rdo.setText(_("Import selected files"))
        self.unselect_rdo.setText(_("Import unselected files"))
        self.only_rdo.setText(_("Import only:"))
        self.select_all_btn.setToolTip(_("Select all files"))
        self.select_all_btn.setText(_("Select all"))
        self.clear_btn.setToolTip(_("Unselect all"))
        self.clear_btn.setText(_("Clear"))
        self.import_btn.setToolTip(_("Start the import"))
        self.import_btn.setText(_("Import"))
        self.cancel_btn.setToolTip(_("Stop the importation"))

        self.all_rdo.setStyleSheet(styles.RADIO)
        self.select_rdo.setStyleSheet(styles.RADIO)
        self.unselect_rdo.setStyleSheet(styles.RADIO)
        self.only_rdo.setStyleSheet(styles.RADIO)
        self.ext_cmb.setStyleSheet(styles.COMBO)
        self.select_all_btn.setStyleSheet(styles.PUSHBUTTON)
        self.clear_btn.setStyleSheet(styles.PUSHBUTTON)
        self.import_btn.setStyleSheet(styles.PUSHBUTTON)
        self.import_pgb.setStyleSheet(styles.PROGRESSBAR)
        self.cancel_btn.setStyleSheet(styles.PUSHBUTTON)

        self.set_widgets_enabled(False)

    def set_widgets_enabled(self, b):
        #self.ext_cmb.setEnabled(b)
        self.select_all_btn.setEnabled(b)
        self.clear_btn.setEnabled(b)
        self.import_btn.setEnabled(b)
        self.cancel_btn.setEnabled(b)
