# -*- coding: utf-8 -*-

# picturepage.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2012-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Image preview tool box

import os
import logging
logger = logging.getLogger(__name__)

from PyQt5.QtCore import Qt, QSize
from PyQt5.QtGui import QIcon, QPixmap, QImage, QFont, QPainter
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QSizePolicy, QSpacerItem, QToolButton, QLabel,
                             QGraphicsView, QGraphicsScene, QSlider)

import stylesheets as styles

class PicturePage(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        sizePolicy = QSizePolicy(QSizePolicy.MinimumExpanding,
                                        QSizePolicy.MinimumExpanding)
        self.setSizePolicy(sizePolicy)
        self.setMaximumSize(QSize(299, 500))
        self.gridLayout_2 = QGridLayout(self)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setSpacing(2)
        self.viewer = QGraphicsView(self)
        self.viewer.setMinimumSize(250, 200)
        self.viewer.setMaximumSize(299, 300)
        self.viewer.setDragMode(QGraphicsView.ScrollHandDrag)      
        self.viewer.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.viewer.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.viewer.setRenderHints(QPainter.Antialiasing|
                                    QPainter.HighQualityAntialiasing|
                                    QPainter.TextAntialiasing)
        self.verticalLayout.addWidget(self.viewer)
        self.horizontalLayout_2 = QHBoxLayout()
        self.left_btn = QToolButton(self)
        icon3 = QIcon()
        icon3.addPixmap(QPixmap("medias/left_arrow_20x20.png"))
        self.left_btn.setIcon(icon3)
        self.left_btn.setIconSize(QSize(15, 15))
        self.horizontalLayout_2.addWidget(self.left_btn)
        self.right_btn = QToolButton(self)
        icon4 = QIcon()
        icon4.addPixmap(QPixmap("medias/right_arrow_20x20.png"))
        self.right_btn.setIcon(icon4)
        self.right_btn.setIconSize(QSize(15, 15))
        self.horizontalLayout_2.addWidget(self.right_btn)
        self.zoom_min_btn = QToolButton(self)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/minus_white_20x5.png"))
        self.zoom_min_btn.setIcon(icon)
        self.zoom_min_btn.setIconSize(QSize(15, 15))
        self.zoom_min_btn.setAutoRaise(True)
        self.horizontalLayout_2.addWidget(self.zoom_min_btn)
        self.zoom_sld = QSlider(self)
        self.zoom_sld.setOrientation(Qt.Horizontal)
        self.zoom_sld.setTickPosition(QSlider.TicksBelow)
        self.zoom_sld.setMinimum(100)
        self.zoom_sld.setMaximum(150)
        self.zoom_sld.setSingleStep(5)
        self.zoom_sld.setPageStep(5)
        self.zoom_sld.setValue(100)
        self.horizontalLayout_2.addWidget(self.zoom_sld)
        self.zoom_max_btn = QToolButton(self)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/plus_white_20x20.png"))
        self.zoom_max_btn.setIcon(icon1)
        self.zoom_max_btn.setIconSize(QSize(15, 15))
        self.zoom_max_btn.setAutoRaise(True)
        self.horizontalLayout_2.addWidget(self.zoom_max_btn)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout = QHBoxLayout()
        sp = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(sp)
        self.name_lbl = QLabel(self)
        font = QFont()
        font.setFamily("DejaVu Sans")
        font.setWeight(75)
        font.setBold(True)
        self.name_lbl.setFont(font)
        self.horizontalLayout.addWidget(self.name_lbl)
        sp1 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(sp1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.gridLayout = QGridLayout()
        self.gridLayout.setVerticalSpacing(2)
        self.date_lbl = QLabel(self)
        self.date_lbl.setAlignment(Qt.AlignRight | Qt.AlignTrailing | 
                                   Qt.AlignVCenter)
        self.gridLayout.addWidget(self.date_lbl, 0, 0, 1, 1)
        self.date_val_lbl = QLabel(self)
        self.gridLayout.addWidget(self.date_val_lbl, 0, 1, 1, 2)
        self.iso_lbl = QLabel(self)
        self.iso_lbl.setText("ISO: ")
        self.iso_lbl.setAlignment(Qt.AlignRight | Qt.AlignTrailing |
                                    Qt.AlignVCenter)
        self.gridLayout.addWidget(self.iso_lbl, 1, 0, 1, 1)
        self.iso_val_lbl = QLabel(self)
        self.gridLayout.addWidget(self.iso_val_lbl, 1, 1, 1, 2)
        self.expo_lbl = QLabel(self)
        self.expo_lbl.setAlignment(Qt.AlignRight | Qt.AlignTrailing |
                                    Qt.AlignVCenter)
        self.gridLayout.addWidget(self.expo_lbl, 2, 0, 1, 1)
        self.expo_val_lbl = QLabel(self)
        self.gridLayout.addWidget(self.expo_val_lbl, 2, 1, 1, 2)
        self.fnum_lbl = QLabel(self)
        self.fnum_lbl.setAlignment(Qt.AlignRight | Qt.AlignTrailing |
                                    Qt.AlignVCenter)
        self.gridLayout.addWidget(self.fnum_lbl, 3, 0, 1, 1)
        self.fnum_val_lbl = QLabel(self)
        self.gridLayout.addWidget(self.fnum_val_lbl, 3, 1, 1, 2)

        self.reso_lbl = QLabel(self)
        self.reso_lbl.setAlignment(Qt.AlignRight | Qt.AlignTrailing |
                                    Qt.AlignVCenter)
        self.gridLayout.addWidget(self.reso_lbl, 4, 0, 1, 1)
        self.reso_val_lbl = QLabel(self)
        self.gridLayout.addWidget(self.reso_val_lbl, 4, 1, 1, 2)
        self.size_lbl = QLabel(self)
        self.size_lbl.setAlignment(Qt.AlignRight | Qt.AlignTrailing |
                                    Qt.AlignVCenter)
        self.gridLayout.addWidget(self.size_lbl, 5, 0, 1, 1)
        self.size_val_lbl = QLabel(self)
        self.gridLayout.addWidget(self.size_val_lbl, 5, 1, 1, 2)
        self.verticalLayout.addLayout(self.gridLayout)
        self.gridLayout_2.addLayout(self.verticalLayout, 0, 0, 1, 1)
        self.fsize_lbl = QLabel(self)
        self.fsize_lbl.setAlignment(Qt.AlignRight | Qt.AlignTrailing |
                                    Qt.AlignVCenter)
        self.gridLayout.addWidget(self.fsize_lbl, 6, 0, 1, 1)
        self.fsize_val_lbl = QLabel(self)
        self.gridLayout.addWidget(self.fsize_val_lbl, 6, 1, 1, 1)
        self.type_lbl = QLabel(self)
        self.type_lbl.setAlignment(Qt.AlignRight | Qt.AlignTrailing |
                                    Qt.AlignVCenter)
        self.gridLayout.addWidget(self.type_lbl, 7, 0, 1, 1)
        self.type_val_lbl = QLabel(self)
        self.gridLayout.addWidget(self.type_val_lbl, 7, 1, 1, 1)

        self.scene = QGraphicsScene()
        self.viewer.setScene(self.scene)
        self.zoom = 1
        self.image = None

        self.clear_all()

        self.viewer.setStyleSheet(styles.VIEWER)
        self.left_btn.setStyleSheet(styles.TBUTTON)
        self.right_btn.setStyleSheet(styles.TBUTTON)
        self.zoom_min_btn.setStyleSheet(styles.TBUTTON)
        self.zoom_sld.setStyleSheet(styles.SLIDER)
        self.zoom_max_btn.setStyleSheet(styles.TBUTTON)

        self.left_btn.clicked.connect(self.rotate_to_left)
        self.right_btn.clicked.connect(self.rotate_to_right)
        self.zoom_min_btn.clicked.connect(self.on_zoom_min_clicked)
        self.zoom_sld.valueChanged.connect(self.on_zoom_changed)
        self.zoom_max_btn.clicked.connect(self.on_zoom_max_clicked)

        self.retranslateUi()

    def retranslateUi(self):
        self.date_lbl.setText(_("Date: "))
        self.expo_lbl.setText(_("Exposure: "))
        self.fnum_lbl.setText(_("FNumber: "))
        self.size_lbl.setText(_("Size: "))
        self.reso_lbl.setText(_("Resolution: "))
        self.fsize_lbl.setText(_("File Size: "))
        self.type_lbl.setText(_("Type: "))

    def set_picture(self, pict):
        self.picture = pict
        self.clear_all()
        self.image = QImage(os.path.join(pict.orig, pict.fname))
        if self.image.isNull():
            return

        self.set_dimension(self.image.size())

        self.show_image()
        self.show_data()

    def show_image(self):
        logger.info("Show image at zoom: {0}".format(self.zoom))
        self.pix = QPixmap(self.image.scaled(self.img_width * self.zoom,
                                    self.img_height * self.zoom,
                                    Qt.KeepAspectRatio, 
                                    Qt.SmoothTransformation))

        self.scene.clear()
        self.scene.setSceneRect(0, 0, self.pix.width(), self.pix.height())
        self.scene.addPixmap(self.pix)

    def set_dimension(self, size):
        self.img_width, self.img_height = size.width(), size.height()
        if self.img_width > self.img_height:
            self.min_width = 100
            self.min_heigth = 66

        else:
            self.min_width = 66
            self.min_heigth = 100

        self.max_width = self.img_width + (self.img_width / 5.0)
        self.max_height = self.img_height + (self.img_height / 5.0)

        if self.img_height > 300:
            # Max width for the viewer
            zoom = 300.0 / self.img_width

        else:
            zoom = 1

        self.reset_zoom(zoom, True)

    def reset_zoom(self, val, block=False):
        logger.info("Reset_zoom: {0}".format(val))
        self.zoom = val
        self.zoom_sld.blockSignals(block)
        self.zoom_sld.setValue(val*100)
        self.zoom_sld.blockSignals(False)
        self.zoom_min_btn.setEnabled(val > 1.0)
        self.zoom_max_btn.setEnabled(val < 1.5)

    def on_zoom_changed(self, val):
        self.zoom = val / 100.0
        if self.image is not None:
            self.show_image()
            self.zoom_min_btn.setEnabled(val > 100)
            self.zoom_max_btn.setEnabled(val < 150)

    def on_zoom_min_clicked(self):
        self.reset_zoom(self.zoom - 0.05)

    def on_zoom_max_clicked(self):
        self.reset_zoom(self.zoom + 0.05)

    def rotate_to_left(self):
        if self.image is None:
            return

        self.rotate_view(-90)

    def rotate_to_right(self):
        if self.image is None:
            return

        self.rotate_view(90)

    def rotate_view(self, val):
        logger.info('Rotate preview: {0}'.format(val))
        matrix = QTransform()
        self.image = self.image.transformed(matrix.rotate(val), 
                                 Qt.SmoothTransformation)
        self.show_image()

    def show_data(self):
        size = ' x '.join([str(x) for x in self.picture.size])
        reso = ' x '.join([str(x) for x in self.picture.resolution])
        self.name_lbl.setText(self.picture.fname.replace('thumb_', ''))
        self.date_val_lbl.setText(self.picture.date)
        self.iso_val_lbl.setText(self.picture.iso)
        self.expo_val_lbl.setText(self.picture.speed)
        self.fnum_val_lbl.setText(self.picture.fnumber)
        self.reso_val_lbl.setText(reso)
        self.size_val_lbl.setText(size)
        self.fsize_val_lbl.setText(str(self.picture.fsize) + " Kb")
        self.type_val_lbl.setText(self.picture.type_)

    def clear_all(self):
        self.scene.clear()
        self.reset_zoom(1, False)
        self.name_lbl.setText("")
        self.date_val_lbl.setText(" --")
        self.iso_val_lbl.setText(" --")
        self.expo_val_lbl.setText(" --")
        self.fnum_val_lbl.setText(" --")
        self.reso_val_lbl.setText(" --")
        self.size_val_lbl.setText(" --")
        self.fsize_val_lbl.setText(" --")
        self.type_val_lbl.setText(" --")
        self.image = None
