# -*- coding: utf-8 -*-

# settingpage.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2012-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Import setting tool box

from PyQt5.QtCore import QSize
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QLineEdit, QSizePolicy, QSpacerItem, QPushButton,
                             QLabel, QCheckBox)

import stylesheets  as styles

class SettingPage(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        sp = QSizePolicy(QSizePolicy.MinimumExpanding,
                                        QSizePolicy.MinimumExpanding)
        self.setSizePolicy(sp)
        self.setMaximumSize(QSize(260, 300))
        self.gridLayout_2 = QGridLayout(self)
        self.gridLayout_2.setContentsMargins(4, 4, 4, 4)
        self.verticalLayout = QVBoxLayout()
        self.horizontalLayout = QHBoxLayout()
        self.label = QLabel(self)
        self.horizontalLayout.addWidget(self.label)
        sp0 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(sp0)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.gridLayout = QGridLayout()
        self.imp_fld_led = QLineEdit(self)
        self.gridLayout.addWidget(self.imp_fld_led, 0, 0, 1, 1)
        self.browse_btn = QPushButton(self)
        self.browse_btn.setMaximumSize(QSize(40, 16777215))
        self.browse_btn.setText("...")
        self.gridLayout.addWidget(self.browse_btn, 0, 1, 1, 1)
        self.rename_chb = QCheckBox(self)
        self.gridLayout.addWidget(self.rename_chb, 1, 0, 1, 1)
        self.rename_setting_btn = QPushButton(self)
        self.rename_setting_btn.setMaximumSize(QSize(40, 16777215))
        self.gridLayout.addWidget(self.rename_setting_btn, 1, 1, 1, 1)
        self.remove_after_chb = QCheckBox(self)
        self.gridLayout.addWidget(self.remove_after_chb, 2, 0, 1, 1)
        sp1 = QSpacerItem(40, 20, QSizePolicy.Preferred, QSizePolicy.Minimum)
        self.gridLayout.addItem(sp1, 2, 1, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.gridLayout_2.addLayout(self.verticalLayout, 0, 0, 1, 1)

        # TODO Renaming feature not yet implemented
        self.rename_chb.hide()
        self.rename_setting_btn.hide()

        self.setWindowTitle(_("Settings"))
        self.label.setText(_("Import folder"))
        self.browse_btn.setToolTip(_("Browse"))
        self.rename_chb.setText(_("Rename image\'s files"))
        self.remove_after_chb.setToolTip(_(
                                "Remove imported files from the camera"))
        self.remove_after_chb.setText(_("Remove after import"))

        self.imp_fld_led.setStyleSheet(styles.LINEEDIT)
        self.browse_btn.setStyleSheet(styles.PUSHBUTTON)
        self.rename_chb.setStyleSheet(styles.CHECKBOX)
        self.rename_setting_btn.setStyleSheet(styles.PUSHBUTTON)
        self.remove_after_chb.setStyleSheet(styles.CHECKBOX)
