# -*- coding: utf-8 -*-

# ui_camera.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2012-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# 'Import from camera' dialog.

import sys

from PyQt5.QtCore import Qt, QSize, QMetaObject
from PyQt5.QtGui import QIcon, QPixmap
from PyQt5.QtWidgets import (QDialog, QWidget, QGridLayout, QVBoxLayout,
                             QHBoxLayout, QSizePolicy, QSpacerItem, QToolButton,
                             QPushButton, QSlider, QPlainTextEdit, QListView,
                             QSplitter, QListWidget, QAbstractItemView, QLabel)

import stylesheets as styles
from camerahandler.camerapage import CameraPage
from camerahandler.settingpage import SettingPage
from camerahandler.importpage import ImportPage
from camerahandler.picturepage import PicturePage
from VWidgets_PyQt5.vtoolpanel import VToolPanel


class Ui_Camera(QDialog):
    def __init__(self, ifc, parent=None):
        super().__init__(parent)
        self.ifc = ifc
        self.icon_size = 150

        try:
            size = ifc.cfg.get(("camera", "import", "size"))
            self.resize(size[0], size[1])
        except:
            self.resize(850, 600)
            ifc.cfg.update(("camera", "import", "size"), [850, 600])

        try:
            icosize = ifc.cfg.get(("camera", "import", "thumb"))
        except:
            icosize = 150
            ifc.cfg.update(("camera", "import", "thumb"), 150)

        self.setWindowTitle("Import from Camera - Oqapy")
        self.gridLayout = QGridLayout(self)
        self.gridLayout.setContentsMargins(2, 2, 2, 2)
        self.h_splitter = QSplitter(self)
        self.h_splitter.setOrientation(Qt.Horizontal)
        self.v_splitter = QSplitter(self.h_splitter)
        self.v_splitter.setOrientation(Qt.Vertical)
        self.layoutWidget = QWidget(self.v_splitter)
        self.verticalLayout = QVBoxLayout(self.layoutWidget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.content_view = QListWidget(self.layoutWidget)
        self.content_view.setTabKeyNavigation(True)
        self.content_view.setProperty("showDropIndicator", False)
        self.content_view.setSelectionMode(QAbstractItemView.MultiSelection)
        self.content_view.setSelectionBehavior(QAbstractItemView.SelectItems)
        self.content_view.setMovement(QListView.Static)
        self.content_view.setResizeMode(QListView.Adjust)
        self.content_view.setViewMode(QListView.IconMode)
        self.content_view.setIconSize(QSize(icosize, icosize))
        self.content_view.setUniformItemSizes(False)
        self.verticalLayout.addWidget(self.content_view)
        self.h_layout = QHBoxLayout()
        self.selection_lbl = QLabel(self.layoutWidget)
        self.selection_lbl.setText(_("No selection"))
        self.h_layout.addWidget(self.selection_lbl)
        sp2 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.h_layout.addItem(sp2)
        self.sizer_min_btn = QToolButton(self)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/minus_white_20x5.png"))
        self.sizer_min_btn.setIcon(icon)
        self.sizer_min_btn.setIconSize(QSize(15, 15))
        self.sizer_min_btn.setAutoRaise(True)
        self.h_layout.addWidget(self.sizer_min_btn)
        self.sizer_sld = QSlider(self)
        self.sizer_sld.setOrientation(Qt.Horizontal)
        self.sizer_sld.setTickPosition(QSlider.TicksBelow)
        self.sizer_sld.setMinimum(100)
        self.sizer_sld.setMaximum(200)
        self.sizer_sld.setSingleStep(10)
        self.sizer_sld.setPageStep(10)
        self.sizer_sld.setValue(150)
        self.h_layout.addWidget(self.sizer_sld)
        self.sizer_max_btn = QToolButton(self)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/plus_white_20x20.png"))
        self.sizer_max_btn.setIcon(icon1)
        self.sizer_max_btn.setIconSize(QSize(15, 15))
        self.sizer_max_btn.setAutoRaise(True)
        self.h_layout.addWidget(self.sizer_max_btn)
        self.verticalLayout.addLayout(self.h_layout)
        self.editor = QPlainTextEdit(self.v_splitter)
        self.editor.setReadOnly(True)
        self.editor.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.panel = QWidget(self.h_splitter)
        self.panel.setMaximumSize(QSize(305, 16777215))
        self.verticalLayout_3 = QVBoxLayout(self.panel)
        self.verticalLayout_3.setContentsMargins(2, 0, 2, 0)
        self.verticalLayout_3.setSpacing(0)
        self.tool_wdg = VToolPanel(self.panel)
        self.tool_wdg.setMinimumSize(QSize(250, 0))
        self.tool_wdg.setMaximumSize(QSize(305, 16777215))
        self.verticalLayout_3.addWidget(self.tool_wdg)
        self.horizontalLayout = QHBoxLayout()
        self.tool_btn = QPushButton(self.panel)
        self.tool_btn.hide()
        self.horizontalLayout.addWidget(self.tool_btn) 
        sp1 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(sp1)
        self.close_btn = QPushButton(self.panel)
        self.close_btn.setIconSize(QSize(18, 18))
        self.close_btn.setText(_("Close"))
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/close.png"), QIcon.Normal, QIcon.Off)
        self.close_btn.setIcon(icon1)
        self.horizontalLayout.addWidget(self.close_btn)
        sp3 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(sp3)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.gridLayout.addWidget(self.h_splitter, 0, 0, 1, 1)
        self.setSizeGripEnabled(True)

        self.set_theme()
        self.show()
 
        self.set_splitters_size()
        self.set_tools()

        self.content_view.itemSelectionChanged.connect(self.set_selection)
        self.sizer_min_btn.clicked.connect(self.reduce_thumbnails)
        self.sizer_sld.valueChanged.connect(self.on_slider_changed)
        self.sizer_max_btn.clicked.connect(self.enlarge_thumbnails)  
        self.close_btn.clicked.connect(self.accept)
        QMetaObject.connectSlotsByName(self)

    def set_splitters_size(self):
        # Vertical splitter
        ch = self.content_view.height()
        eh = self.editor.height()
        f = (ch + eh) / 10.0
        self.v_splitter.setSizes([f*8, f*2])
        # Horizontal splitter
        ss = sum(self.h_splitter.sizes())
        self.h_splitter.setSizes([ss-305, 305])

    def set_tools(self):
        """Sets the tool pages into the tool panel.

        How to add a tool page:
            - Create, if needed, a new tool group with:
                    group = VToolPanel.addToolGroup(name, icon)
            - Create a new page to this group with:
                    myNewPage = VToolPanel.addToolPage(group)
            - Set the properties of the page (title, icon, collapsible, ...)
            - Instanciate the tool widget with the page as parent:
                    myWidget = MyWidget(myNewPage)
            - Place the tool widget into the page layout with:
                    myNewPage.layout.addWidget(myWidget)
        """
        self.tool_wdg.setTitle('Actions')
        # Tool panel header
        # Import
        self.import_grp = self.tool_wdg.addToolPage('import', None)
        # Tool pages
        # Camera
        self.camera_pg = self.tool_wdg.addToolBox(0)
        self.camera_pg.setTitle('Camera - Settings')
        self.camera_pg.header.setMaximumSize(99999, 22)
        self.camera_pg.header.label.setMinimumSize(50, 20)
        self.camera_pg.header.setStyleSheet(styles.HEADER)
        self.camera_pg.header.label.setStyleSheet(styles.TITLE)
        self.camera_pg.header.collapse_btn.setStyleSheet(styles.EXPAND)
        self.camera_pg.header.set_expanding_buttons(
                                        'medias/collapse_white_down.png',
                                        'medias/collapse_white_left.png',
                                        'medias/collapse_white_right.png')
        self.camera_box = CameraPage(self.camera_pg)
        self.camera_pg.layout.addWidget(self.camera_box)
        # Import settings
        self.setting_pg = self.tool_wdg.addToolBox(self.import_grp.index)
        self.setting_pg.setTitle('Import - Settings')
        self.setting_pg.header.setMaximumSize(99999, 22)
        self.setting_pg.header.label.setMinimumSize(50, 20)
        self.setting_pg.header.setStyleSheet(styles.HEADER)
        self.setting_pg.header.label.setStyleSheet(styles.TITLE)
        self.setting_pg.header.collapse_btn.setStyleSheet(styles.EXPAND)
        self.setting_pg.header.set_expanding_buttons(
                                        'medias/collapse_white_down.png',
                                        'medias/collapse_white_left.png',
                                        'medias/collapse_white_right.png')
        self.importsett_box = SettingPage(self.setting_pg)
        self.setting_pg.layout.addWidget(self.importsett_box)
        # Import
        self.import_pg = self.tool_wdg.addToolBox(self.import_grp.index)
        self.import_pg.setTitle('Import Images')
        self.import_pg.header.setMaximumSize(99999, 22)
        self.import_pg.header.label.setMinimumSize(50, 20)
        self.import_pg.header.setStyleSheet(styles.HEADER)
        self.import_pg.header.label.setStyleSheet(styles.TITLE)
        self.import_pg.header.collapse_btn.setStyleSheet(styles.EXPAND)
        self.import_pg.header.set_expanding_buttons(
                                        'medias/collapse_white_down.png',
                                        'medias/collapse_white_left.png',
                                        'medias/collapse_white_right.png')
        self.import_box = ImportPage(self.import_pg)
        self.import_pg.layout.addWidget(self.import_box)
        # Preview
        self.picture_pg = self.tool_wdg.addToolBox(self.import_grp.index)
        self.picture_pg.setTitle('Selected Picture')
        self.picture_pg.header.setMaximumSize(99999, 22)
        self.picture_pg.header.label.setMinimumSize(50, 20)
        self.picture_pg.header.setStyleSheet(styles.HEADER)
        self.picture_pg.header.label.setStyleSheet(styles.TITLE)
        self.picture_pg.header.collapse_btn.setStyleSheet(styles.EXPAND)
        self.picture_pg.header.set_expanding_buttons(
                                        'medias/collapse_white_down.png',
                                        'medias/collapse_white_left.png',
                                        'medias/collapse_white_right.png')
        self.picture_box = PicturePage(self.picture_pg)
        self.picture_pg.layout.addWidget(self.picture_box)
        self.tool_wdg.setCurrentPage(0)

    def reduce_thumbnails(self):
        val = self.sizer_sld.value() - 10
        self.sizer_sld.setValue(val)
        if val == 100:
            self.sizer_min_btn.setEnabled(False)
        self.sizer_max_btn.setEnabled(True)
        
    def on_slider_changed(self, val):
        self.content_view.setIconSize(QSize(val, val))
        self.sizer_sld.setToolTip('%s pixels' % val)

    def enlarge_thumbnails(self):
        val = self.sizer_sld.value() + 10
        self.sizer_sld.setValue(val)
        if val == 200:
            self.sizer_max_btn.setEnabled(False)
        self.sizer_min_btn.setEnabled(True)

    def set_selection(self):
        selected = self.content_view.selectedItems()
        self.on_selection_changed(len(selected))

    def on_selection_changed(self, num):
        if not num:
            txt = _(" No selection")

        elif num == 1:
            txt = _(" One item selected")

        else:
            txt = _(" {0} items selected".format(num))

        self.selection_lbl.setText(txt)

    def set_theme(self):
        self.setStyleSheet("""QWidget {background: #181E26;
                        color: #FFFFFF;}""")
        self.content_view.setStyleSheet(styles.VIEWER)
        self.content_view.horizontalScrollBar().setStyleSheet(
                                                    styles.HSCROLLBAR)
        self.content_view.verticalScrollBar().setStyleSheet(
                                                    styles.VSCROLLBAR)
        self.sizer_min_btn.setStyleSheet(styles.TBUTTON)
        self.sizer_sld.setStyleSheet(styles.SLIDER)
        self.sizer_max_btn.setStyleSheet(styles.TBUTTON)
        self.editor.setStyleSheet(styles.TEXTEDIT)
        self.editor.verticalScrollBar().setStyleSheet(styles.VSCROLLBAR)
        self.tool_wdg.scrollArea.setStyleSheet(styles.SCROLLAREA)
        self.tool_wdg.header.setStyleSheet(styles.HEADER)
        self.tool_wdg.header.label.setStyleSheet(styles.TITLE)
        self.tool_wdg.scrollArea.horizontalScrollBar().setStyleSheet(
                                                    styles.HSCROLLBAR)
        self.tool_wdg.scrollArea.verticalScrollBar().setStyleSheet(
                                                    styles.VSCROLLBAR)
        self.close_btn.setStyleSheet(styles.PUSHBUTTON)
        self.tool_btn.setStyleSheet(styles.PUSHBUTTON)

    def resizeEvent(self, event):
        s = event.size()
        self.ifc.cfg.update(("camera", "import", "size"),  
                            [s.width(), s.height()])
