# contextmenuscene.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2012-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Context menu on QGraphicsView

from PyQt5.QtGui import QFont
from PyQt5.QtWidgets import QMenu, QAction

#import stylesheets

class ContextMenuScene(QMenu):
    def __init__(self, core, pos, parent=None):
        """Instanciate the contextual menu for the viewing area.

        Args:
        core -- OqapyCore instance
        pos -- cursor pos
        """
        super().__init__(parent)
        self.core = core
        self.font_1 = QFont()
        self.font_1.setFamily("Arial")
        self.font_1.setWeight(75)
        self.font_1.setBold(True)
        self.font_2 = QFont()
        self.font_2.setFamily("Trebuchet")
        self.font_2.setWeight(50)
        # Image: rotate left, rotate right, 
        # Zoom: plus, minus, center here, fit in view, full size
        # Tools: magnifier, aerial view, crop, grid
        
        mag = self.core.magnifier
        try:
            avw = self.core.aerial_view.isVisible()
        except:
            avw = False

        grd = self.core.grid_is_visible()
        scr = self.core.cfg.get(('style', 'scrlbars'))
        self.title_1 = QAction(None)
        self.title_1.setFont(self.font_1)
        self.title_1.setText(_("Image"))
        self.addAction(self.title_1)
        self.lrotate = QAction(None)
        self.lrotate.setText(_("Rotate to Left"))
        self.lrotate.setFont(self.font_2)
        self.addAction(self.lrotate)
        self.rrotate = QAction(None)
        self.rrotate.setText(_("Rotate to Right"))
        self.rrotate.setFont(self.font_2)
        self.addAction(self.rrotate)
        self.addSeparator()
        self.title_2 = QAction(None)
        self.title_2.setText(_("Zoom"))
        self.title_2.setFont(self.font_1)
        self.addAction(self.title_2)
        self.upZoom = QAction(None)
        self.upZoom.setText(_("Zoom Up"))
        self.upZoom.setFont(self.font_2)
        self.addAction(self.upZoom)
        self.downZoom = QAction(None)
        self.downZoom.setText(_("Zoom Down"))
        self.downZoom.setFont(self.font_2)
        self.addAction(self.downZoom)
        self.activZoom = QAction(None)
        self.activZoom.setText(_("Zoom on Area"))
        self.activZoom.setFont(self.font_2)
        self.addAction(self.activZoom)
        self.center = QAction(None)
        self.center.setText(_("Center Here"))
        self.center.setFont(self.font_2)
        self.addAction(self.center)
        self.fitInView = QAction(None)
        self.fitInView.setText(_("Fit in view"))
        self.fitInView.setFont(self.font_2)
        self.addAction(self.fitInView)
        self.fullSize = QAction(None)
        self.fullSize.setText(_("Full size"))
        self.fullSize.setFont(self.font_2)
        self.addAction(self.fullSize)
        self.addSeparator()
        self.title_3 = QAction(None)
        self.title_3.setText(_("Tools"))
        self.title_3.setFont(self.font_1)
        self.addAction(self.title_3)
        self.magnf = QAction(None) 
        self.magnf.setText(_("Magnifier"))
        self.magnf.setFont(self.font_2)
        self.magnf.setCheckable(True)
        self.magnf.setChecked(mag and 1)
        self.addAction(self.magnf)
        self.aerial = QAction(None) 
        self.aerial.setText(_("Aerial View"))
        self.aerial.setFont(self.font_2)
        self.aerial.setCheckable(True)
        self.aerial.setChecked(avw)
        self.addAction(self.aerial)
        self.grid = QAction(None) 
        self.grid.setText(_("Grid"))
        self.grid.setFont(self.font_2)
        self.grid.setCheckable(True)
        self.grid.setChecked(grd)
        self.addAction(self.grid)
        if grd:
            self.grd_conf = QAction(None)
            self.grd_conf.setText(_("Grid Settings"))
            self.grd_conf.setFont(self.font_2)
            self.addAction(self.grd_conf)
            self.grd_conf.triggered.connect(self.core.grid.call_settings_menu)
        self.scrollbars = QAction(None)
        self.scrollbars.setText(_("Scroll bars"))
        self.scrollbars.setFont(self.font_2)
        self.addAction(self.scrollbars)
        self.scrollbars.setCheckable(True)
        self.scrollbars.setChecked(scr)
        self.lrotate.triggered.connect(self.core.rotate_to_left)
        self.rrotate.triggered.connect(self.core.rotate_to_right)
        self.upZoom.triggered.connect(self.core.on_zoom_up)
        self.downZoom.triggered.connect(self.core.on_zoom_down)
        self.activZoom.triggered.connect(self.core.get_zoom_area)
        self.center.triggered.connect(self.core.zoom_on_cursor_pos)
        self.fitInView.triggered.connect(self.core.on_fit_in_view)
        self.fullSize.triggered.connect(self.core.on_zoom_full_size)
        self.magnf.toggled.connect(self.core.toggle_magnifier)
        #self.aerial.toggled.connect(self.core.show_aerial_view)
        self.grid.toggled.connect(self.core.toggle_grid)
        self.scrollbars.toggled.connect(self.core.ui.toggle_scroll_bars)

        #p = stylesheets.get_palette()
        #self.setPalette(p)

        self.popup(pos)
