# data.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2012-2018 Vincent Vande Vyvre
# Licence: GPL3 


VERSION = (3, 0, 3)

VERSION_STR = "3.0.3"

XMP_VERSION_STR = "1.0"

CONFIG_DEFAULT = {
    "app": {"name": "Oqapy",
            "vers": (3, 0, 3),
            "svers": "3.0.3"},
    "startup": {"size": (1000, 700),
                "mode": 0,
                "trailer": 1,
                "panel": 2,
                "open": "",
                "always_open": "",
                "empty_trash": False,
                "2screens": True,
                "map_server": 0,
                "numb_recents": 20},
    "style": {"bgrd": "#080C14",
              "scrlbars": True,
              "stretch": False,
              "thumb_size": 150,
              "sel_color": 2,
              "tab_icon_size": 150},
    "zoom": {"step": 0.06},
	"trailer": {"size": 80,
                "style": 3},
    "folders": {"current": "",
                "trash": "",
                "thumbnails": "",
                "previews": "",
                "camera": "",
                "recents": [],
                "last_dir": ""},
    "sorting": {"number": 6,
                "paths": []},
    "aerialview": {"size": 250,
                   "pos": False},
    "magnifier": {"factor": 1.0,
                  "size": 170,
                  "shape": "circle"},
    "dataeditor": {"expandeds": [True, False, False, False, False, False, 
                                True, True, True, True, False]},
    "g9n": {"server": 1,
            "expandsize": (700, 600),
            "default": (50.0, 4.0, 6),
            "userpoints": {}},
    "crop": {"constrain": False,
             "current_ratio": None,
             "reverse": False,
             "force_width": True,
             "grid": False,
             "mode": ("same_as",),
             "ratios": ["1:1", "4:3", "3:2", "16:9", "1.85:1", "2.39:1"]},
    "rawfiles": {"ignore": True},
    "grid": {"line_style": "solid", 
             "color": "#c0c0c0", 
             "thickness": 1,
             "type_ratio": "sameAs", 
             "ratio": 0, 
             "mode": "lines",
             "numH": 3, 
             "numV": 5, 
             "offsetX": 0, 
             "offsetY": 0},
    "filtering": {"preview":
                    {"size": 250}},
    "camera": {"import":
                {"size": (800, 600),
                 "thumb": 150}},
    "manual": {"size": (1000, 600),
               "opened_tabs": []},
    "toolpanel": {"size": (310, 99999)},
    "multiviews": {"rows": 2, "columns": 4}
    }

SHORTCUTS = {"viewing": {"next_image": "Space",
                         "prev_image": "Ctrl+Space",
                         "rotate_r": "R",
                         "rotate_l": "L",
                         "del_image": "Del"},
             "sorting": {"next_right_tab": "Ctrl+Right",
                         "next_left_tab": "Ctrl+Left",
                         "next_right_button": "Shift+Right",
                         "next_left_button": "Shift+Left",
                         "unselect": "Shift+Down",
                         "copy": "C",
                         "move": "V",
                         "close_folder": "W",
                         "open_folder": "N"},
             "trailer": {"show": "T",
                         "forward": "J",
                         "reverse": "H",
                         "fast_forward": "G",
                         "fast_reverse": "F"},
             "aerial_view": {"show_hide": "A",
                             "permanent": "D"},
             "crop_tool": {"activate": "Ctrl+C",
                           "apply": "Ctrl+A"},
             "magnifier": {"show_hide": "M",
                           "set_focus": "Ctrl+M"},
             "grid": {"show_hide": "Ctrl+G"},
             "zoom": {"plus": "+",
                      "minus": "-",
                      "fit_in_view": "Home",
                      "full_size": "End"},
             "global_menu": {"show_hide": "F1"}
             }


COLORS = ["black", "blue", "cyan", "darkBlue", "darkCyan", "darkGrey",
                "darkGreen", "darkMagenta", "darkRed", "grey",
                "green", "lightGrey", "magenta", "red", "white", "yellow"]

HEX_COLORS = {"black": "#000000", "blue": "#0000ff", "cyan": "#00ffff", 
                "darkBlue": "#000080", "darkCyan": "#008080", 
                "darkGrey": "#808080", "darkGreen": "#008000", 
                "darkMagenta": "#800080", "darkRed": "#800000", 
                "grey": "#a0a0a4", "green": "#00ff00", "lightGrey": "#c0c0c0", 
                "magenta": "#ff00ff", "red": "#ff0000", "white": "#ffffff", 
                "yellow": "#ffff00"}

BASIC_PLT = {'Red': (255, 0, 0), 'Orangered': (255, 69, 0),
                'Crimson': (220, 20, 60), 'Dark red': (128, 0, 0),
                'Green': (0, 255, 0), 'Limegreen': (50, 205, 50),
                'Light green': (14, 255, 187), 'Dark green': (0, 128, 0),
                'Blue': (0, 0, 255), 'Royal blue': (65, 105, 225),
                'Indigo': (75, 0, 130), 'Dark blue': (0, 0, 128),
                'Cyan': (0, 255, 255), 'Magenta': (255, 0, 255),
                'Yellow': (255, 255, 0), 'Gray': (160, 160, 160),
                'Light gray': (192, 192, 192), 'Dark gray': (128, 128, 128),
                'White': (255, 255, 255), 'Black': (0, 0, 0),
                'ORDER': ['Red', 'Orangered', 'Crimson', 'Dark red',
                'Green', 'Light green', 'Limegreen', 'Dark green',
                'Blue', 'Royal blue', 'Indigo', 'Dark blue', 
                'Cyan', 'Magenta', 'Yellow', 'Gray', 
                'Light gray', 'Dark gray', 'White', 'Black']}

VIEWER_PLT = {'Ubangi light': (244, 244, 244), 'Ubangi mid': (232, 232, 232),
              'Ubangi dark': (154, 154, 154), 
              'Casamance light': (138, 142, 150), 
              'Casamance mid': (120, 124, 132), 'Casamance dark': (80, 84, 96),
              'Bafing light': (50, 58, 70), 'Bafing mid': (24, 30, 38),
              'Bafing dark': (8, 12, 20), 'Dark gray': (128, 128, 128),
              'White': (255, 255, 255), 'Black': (0, 0, 0),
              'ORDER': ['Bafing light', 'Bafing mid', 'Bafing dark', 
              'Casamance light', 'Casamance mid', 'Casamance dark', 
              'Ubangi light', 'Ubangi mid', 'Ubangi dark', 'White', 'Black',
              'Dark gray']}

FORMATS = ['.bmp', '.gif', '.ico', '.jpg', '.JPG', '.jpeg', '.JPEG', 
            '.png', 'PNG', '.ppm', '.tif', '.tiff', '.TIFF', '.xbm', '.xpm']


