# -*- coding: utf-8 -*-

# contrastbrightnessfilter.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2014-2016 Vincent Vande Vyvre
# Licence: GPL3
#
# Image contrast-brightness filter

import os

from oqapy_iproc import OqapyIProc

class ContrastBrightnessFilter:
    def __init__(self, fname=None, outfname=None):
        self.name = 'Contrast-Brightness'
        self.fname = fname
        self.outfname = outfname
        self.image = None
        self.cfg = None
        self.last_error = None

    def set_file_name(self, fname):
        self.fname = fname

    def set_out_file_name(self, fname):
        self.outfname = fname

    def configure(self, **kwargs):
        self.cfg = kwargs

    def load_image(self):
        if self.fname is None:
            self.last_error = 'ContrastBrightnessFilter.load_image Error: '\
                                'No file name defined'
            return False

        if not os.path.isfile(self.fname):
            self.last_error =  'ContrastBrightnessFilter.load_image Error: '\
                                'No such file: %s' % self.fname
            return False

        self.oip = OqapyIProc()
        try:
            self.oip.loadImage(self.fname)
        except Exception as why:
            self.last_error = 'ContrastBrightnessFilter.load_image Error: %s'\
                                % why
            return False
        
        if not self.oip.isLoaded():
            self.last_error = 'ContrastBrightnessFilter unable to load: %s'\
                                % self.fname
            return False

        return True

    def apply_filter(self):
        if not self.check_config():
            return False

        gain, bias = self.get_gain_bias(self.cfg['cnt'], self.cfg['brg'])
        try:
            reply = self.oip.applyContrastBrightness(gain, bias,
                                                     self.cfg['low'], 
                                                     self.cfg['up'],
                                                     self.outfname)
            return True

        except Exception as why:
            self.last_error = 'ContrastBrightnessFilter Error: %s' % why
            return False


    def get_gain_bias(self, cnt, brg):
        if cnt > 0:
            delta = 127.0 * cnt / 100
            gain = 255.0 / (255.0 - delta * 2)
            bias = gain * (brg - delta)

        else:
            delta = -128. * cnt / 100
            gain = (256. - delta * 2) / 255
            bias = gain * brg + delta

        return gain, bias

    def check_config(self):
        if self.outfname is None:
            self.last_error =  'ContrastBrightnessFilter.check_config Error: '\
                                'No out file name defined'
            return False        

        if self.cfg is None:
            self.last_error =  'ContrastBrightnessFilter.check_config Error: '\
                                'No configuration'
            return False

        return True

