# -*- coding: utf-8 -*-

# laplace.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2014-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Laplacian filter
#
# docs/2.4.13.2/modules/imgproc/doc/filtering.html#laplacian


import sys
import os

from oqapy_iproc import OqapyIProc

class Laplace:
    def __init__(self, fname=None, outfname=None):
        self.name = "Laplace"
        self.fname = fname
        self.outfname = outfname
        self.image = None
        self.cfg = None
        self.last_error = None

    def set_file_name(self, fname):
        self.fname = fname

    def set_out_file_name(self, fname):
        self.outfname = fname

    def configure(self, **kwargs):
        self.cfg = kwargs

    def load_image(self):
        if self.fname is None:
            self.last_error = 'Laplace.load_image Error: '\
                                'No file name defined'
            return False

        if not os.path.isfile(self.fname):
            self.last_error =  'Laplace.load_image Error: '\
                                'No such file: %s' % self.fname
            return False

        self.oip = OqapyIProc()
        try:
            self.oip.loadImage(self.fname)
        except Exception as why:
            self.last_error = 'Laplace.load_image Error: %s'\
                                % why
            return False
        
        if not self.oip.isLoaded():
            self.last_error = 'Laplace unable to load: %s'\
                                % self.fname
            return False

        return True

    def apply_filter(self):
        if not self.check_config():
            return False

        try:
            res = self.oip.applyLaplacianFilter(self.cfg['aperture'],
                                                self.outfname)
        except Exception as why:
            self.last_error = 'Laplace.apply_filter Error: %s' % why
            return False

        return True

    def check_config(self):
        if self.outfname is None:
            self.last_error =  'Laplace.check_config Error: '\
                                'No out file name defined'
            return False        

        if self.cfg is None:
            self.last_error =  'Laplace.check_config Error: '\
                                'No configuration'
            return False

        if not self.cfg['aperture'] % 2:
            self.cfg['aperture'] += 1

        return True
