# -*- coding: utf-8 -*-

# toning.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Image toning
#
# docs/2.4.13.2/modules/core/doc/basic_structures.html?highlight=matx33f#matx

import sys
import os

from oqapy_iproc import OqapyIProc


class Toning:
    def __init__(self, fname=None, outfname=None):
        self.name = 'Toning'
        self.fname = fname
        self.outfname = outfname
        self.image = None
        self.cfg = None
        self.last_error = None

    def set_file_name(self, fname):
        self.fname = fname

    def set_out_file_name(self, fname):
        self.outfname = fname

    def configure(self, **kwargs):
        self.cfg = kwargs

    def load_image(self):
        if self.fname is None:
            self.last_error = 'Toning.load_image Error: No file name defined'
            return False

        if not os.path.isfile(self.fname):
            self.last_error =  'Toning.load_image Error: '\
                                'No such file: %s' % self.fname
            return False

        self.oip = OqapyIProc()
        try:
            self.oip.loadImage(self.fname)
        except Exception as why:
            self.last_error = 'Toning.load_image Error: %s' % why
            return False
        
        if not self.oip.isLoaded():
            self.last_error = 'Toning unable to load: %s' % self.fname
            return False

        return True

    def get_preview(self):
        if not self.check_config():
            return False

        toning, level = self.cfg['color'], self.cfg['level']
        if toning == 'greyscale':
            try:
                res = self.oip.convertToBrightGreyMap(level, self.outfname)
            except Exception as why:
                self.last_error = 'Toning.apply_filter Error: %s' % why
                return False

        elif toning == 'greymap':
            try:
                res = self.oip.convertToGreyMap(self.outfname)
            except Exception as why:
                self.last_error = 'Toning.apply_filter Error: %s' % why
                return False

        elif toning == 'sepia':
            try:
                res = self.oip.applyToningSepia(level, self.outfname)
            except Exception as why:
                self.last_error = 'Toning.apply_filter Error: %s' % why
                return False

        elif toning == 'blue':
            try:
                res = self.oip.applyToningBlue(level, self.outfname)
            except Exception as why:
                self.last_error = 'Toning.apply_filter Error: %s' % why
                return False

        return self.outfname

    def check_config(self):
        if self.outfname is None:
            self.last_error = 'Toning.check_config Error: '\
                              'No out file name defined'
            return False        

        if self.cfg is None:
            self.last_error = 'Toning.check_config Error: No configuration'
            return False

        return True




