# -*- coding: utf-8 -*-

# expandedmap.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2013-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# expanded map viewer
#
# St Python '50.19345734865621', '3.479976593078728'


from PyQt5.QtCore import Qt, QSize, QUrl, pyqtSignal
from PyQt5.QtGui import QIcon, QPixmap 
from PyQt5.QtWidgets import (QGridLayout, QVBoxLayout, QHBoxLayout,
                             QSpacerItem, QLabel, QSizePolicy, QPushButton,
                             QComboBox, QDialog, QLineEdit, QTableWidget,
                             QTableWidgetItem, QToolButton, QAbstractItemView)

from g9n.mapviewer import MapViewer
from VWidgets_PyQt5.vlineedit import VLineEdit


class ExpandedMap(QDialog):
    def __init__(self, view, cfg, url, urls, parent=None):
        super(ExpandedMap, self).__init__(parent)
        self.view = view
        self.cfg = cfg.get(('g9n', ))
        s = self.cfg['expandsize']
        self.resize(s[0], s[1])
        self.setSizeGripEnabled(True)
        self.latitude = 0
        self.longitude = 0
        self.gridLayout = QGridLayout(self)
        self.gridLayout.setContentsMargins(2, 4, 2, 4)
        self.verticalLayout = QVBoxLayout()
        self.horizontalLayout = QHBoxLayout()
        self.server_cmb = QComboBox(self)
        self.server_cmb.addItems(['Open Street Map', 'Open Street Map'])
        self.horizontalLayout.addWidget(self.server_cmb)
        self.points_cmb = QComboBox(self)
        self.points_cmb.setMinimumSize(QSize(150, 0))
        self.horizontalLayout.addWidget(self.points_cmb)
        spacerItem = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                            QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.webView = MapViewer(self)
        self.webView.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.webView.urls = urls
        self.verticalLayout.addWidget(self.webView)
        self.horizontalLayout_2 = QHBoxLayout()
        self.lat_lbl = QLabel(self)
        self.horizontalLayout_2.addWidget(self.lat_lbl)
        self.lat_val_lbl = QLabel(self)
        self.horizontalLayout_2.addWidget(self.lat_val_lbl)
        self.lon_lbl = QLabel(self)
        self.horizontalLayout_2.addWidget(self.lon_lbl)
        self.lon_val_lbl = QLabel(self)
        self.horizontalLayout_2.addWidget(self.lon_val_lbl)
        spacerItem1 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                            QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QHBoxLayout()
        self.save_lbl = QLabel(self)
        self.horizontalLayout_3.addWidget(self.save_lbl)
        self.point_led = VLineEdit(self)
        self.point_led.setHistoricButton(True)
        self.horizontalLayout_3.addWidget(self.point_led)
        self.comment_led = QLineEdit(self)
        self.comment_led.setPlaceholderText(_('Add a comment'))
        self.horizontalLayout_3.addWidget(self.comment_led)
        self.create_btn = QPushButton(_('Create'), self)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/apply.svg"), QIcon.Normal, QIcon.Off)
        self.create_btn.setIcon(icon)
        self.horizontalLayout_3.addWidget(self.create_btn)
        spacerItem2 = QSpacerItem(40, 20, QSizePolicy.Expanding, 
                                            QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem2)
        self.close_btn = QPushButton(_('Close'), self)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/close.png"), QIcon.Normal, QIcon.Off)
        self.close_btn.setIcon(icon1)
        self.horizontalLayout_3.addWidget(self.close_btn)
        spacerItem3 = QSpacerItem(30, 20, QSizePolicy.Fixed, 
                                            QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem3)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.gridLayout.addLayout(self.verticalLayout, 0, 0, 1, 1)

        self.setWindowTitle(_("Map Viewer - Oqapy"))
        self.lat_lbl.setText(_("Latitude:"))
        self.lon_lbl.setText(_("Longitude:"))
        self.save_lbl.setText(_("Save as:"))
        self.point_led.setInformativeText(_("Enter a name"))
        self.close_btn.setText(_("Close"))
        if self.cfg['userpoints']:
            self.set_user_points()
        self.server_cmb.setCurrentIndex(url)

        self.last_point = None
        self.lock_update = True
        self.is_selected = False
        self.webView.coordinateChanged.connect(self.on_mapViewer_updated)
        #self.server_cmb.currentIndexChanged.connect(self.on_server_selected)
        self.points_cmb.currentIndexChanged[str].connect(self.on_point_selected)
        self.create_btn.clicked.connect(self.on_new_point_created)
        self.close_btn.clicked.connect(self.accept)
        self.webView.server = url
        self.webView.setUrl(urls[url])

    def configure(self, lat, lon, zoom=16):
        self.webView.go_to(lat, lon, zoom)

    def set_coordinate(self, lat, lon, zoom=16):
        """Update the map with the new coordinates.

        Args:
        lat -- latitude in decimal
        lon -- longitude in decimal
        zoom -- zoom factor
        """
        self.webView.fly_to(lat, lon, zoom)
        self.view.webView.go_to(lat, lon, zoom)

    def set_user_points(self):
        """Set the user's geolocation into the comboBox.

        The user's points is a dict with the point's name as key and a list
        [latitude, longitude, comment] as value
        """
        self.points_cmb.addItem(_('Go to ...'))
        points = self.cfg['userpoints']
        for idx, key in enumerate(sorted(points.keys())):
            self.points_cmb.addItem(key)
            self.points_cmb.setItemData(idx+1, points[key][2], 
                                        Qt.ToolTipRole)

        self.points_cmb.addItem(_('Edit'))
        c = self.points_cmb.count() - 1
        self.points_cmb.setItemData(c, _('Edit the list'), Qt.ToolTipRole)

    def add_user_point(self, point):
        """Update the combo box when a new point has been created.

        Args:
        point -- the name of the point
        """
        self.block_points_combo(True)
        self.points_cmb.clear()
        self.set_user_points()
        self.points_cmb.setCurrentIndex(self.points_cmb.findText(point))
        self.block_points_combo(False)

    def check_name(self, value):
        """Verify the name of the new point.

        If the name is already existing, an nummer is appended.
        e.g. Central Park --> Central Park (1)

        Args:
        value -- the name

        Return:
        The name, modified or not
        """
        name = value
        count = 1
        while name in self.cfg['userpoints']:
            name = "%s (%s)" %(value, count)
            count += 1

        return name

    def block_points_combo(self, b):
        self.points_cmb.blockSignals(b)

    #--- Slots -------------------------------------------------------------

    def on_mapViewer_updated(self, coord):
        """The map in the webView has been changed.

        Args:
        coord -- list(lat, long, zoom)
        """
        coord = [str(c) for c in coord]
        self.lat_val_lbl.setText(coord[0][:8])
        self.lon_val_lbl.setText(coord[1][:8])
        if not self.lock_update:
            self.view.webView.go_to(coord[0], coord[1], coord[2])

        else:
            self.lock_update = False

        if not self.is_selected:
            self.points_cmb.setCurrentIndex(0)
            
        self.is_selected = False

    def on_server_selected(self, idx):
        self.webView.change_server()
        self.view.change_map_server(idx)

    def on_point_selected(self, name):
        """Called when the user select a point in the combo box.

        Args:
        name -- the name of the point
        """
        if name == self.points_cmb.itemText(0):
            return

        if name == self.points_cmb.itemText(self.points_cmb.count()-1):
            self.edit_points()

        else:
            self.last_point = name
            coord = self.cfg['userpoints'][name]
            self.is_selected = True
            self.set_coordinate(coord[0], coord[1])

    def on_new_point_created(self):
        """Called when the user create a new coordinate's point.

        Args:
        name -- the content of the name line edit
        """ 
        name = self.check_name(self.point_led.text())
        lat = float(self.lat_val_lbl.text())
        lon = float(self.lon_val_lbl.text())
        cmt = self.comment_led.text()
        self.cfg['userpoints'][name] = [lat, lon, cmt]
        self.add_user_point(name)
        self.point_led.clear()
        self.comment_led.clear()

    def resizeEvent(self, event):
        self.cfg['expandsize'] = self.width(), self.height()

    def edit_points(self):
        """Call the user's points editor.

        """
        self.editor = UserPointsEditor(self.cfg['userpoints'], self)
        self.editor.exec_()
        self.block_points_combo(True)
        self.points_cmb.clear()
        if self.cfg['userpoints']:
            self.set_user_points()

        if self.last_point is not None:
            idx = self.points_cmb.findText(self.last_point)
            # The last point may be removed
            if idx != -1:
                self.points_cmb.setCurrentIndex(idx)

        self.block_points_combo(False)


class UserPointsEditor(QDialog):
    """Define the user's points editor.

    """
    def __init__(self, points, parent=None):
        super(UserPointsEditor, self).__init__(parent)
        self.points = points
        self.resize(500, 400)
        self.setSizeGripEnabled(True)
        self.gl = QGridLayout(self)
        self.gl.setContentsMargins(2, 4, 2, 4)
        self.vl = QVBoxLayout()
        self.table = PointsTable(self)
        self.vl.addWidget(self.table)
        self.hl = QHBoxLayout()
        self.spacer = QSpacerItem(50, 20, QSizePolicy.Expanding, 
                                    QSizePolicy.Minimum)
        self.hl.addItem(self.spacer)
        self.apply_btn = QPushButton(_('Apply'), self)
        self.hl.addWidget(self.apply_btn)
        self.close_btn = QPushButton(_('Close'), self)
        self.hl.addWidget(self.close_btn)
        self.vl.addLayout(self.hl)
        self.gl.addLayout(self.vl, 0, 0, 1, 1)

        self.apply_btn.clicked.connect(self.apply_new_values)
        self.close_btn.clicked.connect(self.accept)
        self.table.set_points(points)
        self.apply_btn.setEnabled(False)

    def apply_new_values(self):
        self.apply_btn.setEnabled(False)
        self.parent().cfg['userpoints'] = self.table.data


class PointsTable(QTableWidget):
    def __init__(self, parent=None):
        super(PointsTable, self).__init__(parent)
        self.parent = parent
        self.setSelectionMode(QAbstractItemView.SingleSelection)
        self.setRowCount(0)
        self.setColumnCount(3)
        self.setColumnWidth(0, 150)
        self.setColumnWidth(1, 300)
        self.setColumnWidth(2, 40)
        self.verticalHeader().hide()
        labels = [_(u'Name'), _(u'Comment'), '']
        self.setHorizontalHeaderLabels(labels)

    def set_points(self, dct):
        self.data = {}
        for key, value in dct.items():
            self.data[key] = value

        self.items = sorted(self.data.keys())
        for key in self.items:
            r = self.rowCount()
            self.insertRow(r)
            obj = QTableWidgetItem()
            obj.setText(key)
            self.setItem(r, 0, obj)
            obj = QTableWidgetItem()
            obj.setText(self.data[key][2])
            self.setItem(r, 1, obj) 
            obj = TableButton(r)
            self.setCellWidget(r, 2, obj)
            obj.removeClicked.connect(self.remove_requested)

        self.itemChanged.connect(self.on_item_edited)

    def on_item_edited(self, item):
        idx = item.row()
        orig = self.items[item.row()]
        if not item.column():
            # Change the name
            name = self.check_name(item.text())
            self.data[name] = self.data[orig]
            del self.data[orig]
            self.items.pop(idx)
            self.items.insert(idx, name)

        else:
            # Change the comment
            self.data[orig][2] = item.text()
            
        self.parent.apply_btn.setEnabled(True)
        self.clearSelection()

    def remove_requested(self, idx):
        name = self.item(idx, 0).text()
        self.items.remove(name)
        del self.data[name]
        self.parent.apply_btn.setEnabled(True)
        self.removeRow(idx)
        self.clearSelection()
        self.reset_buttons_index()

    def check_name(self, value):
        name = value
        count = 1
        while name in self.data:
            name = "%s(%s)" %(value, count)
            count += 1

        return name

    def reset_buttons_index(self):
        for row in range(self.rowCount()):
            self.cellWidget(row, 2).idx = row


class TableButton(QToolButton):
    """Define the delete button for the edit dialog box.

    """
    removeClicked = pyqtSignal(int)
    def __init__(self, idx, parent=None):
        super(TableButton, self).__init__(parent)
        self.idx = idx
        self.setIcon(QIcon('medias/clean_24x27.png'))
        self.setToolTip(_('Delete entry'))
        self.clicked.connect(self.on_click)

    def on_click(self):
        """Emit the signal removeClicked(button index).

        """
        self.removeClicked.emit(self.idx)


#----------------------------------------------------------------------------
# --- Devel usage ---

class Main(object):
    def __init__(self):
        import sys
        urls = [QUrl('file:///home/vincent/oqapy-2/tempoqapy/g9n/GM.html'),
                QUrl('file:///home/vincent/oqapy-2/tempoqapy/g9n/OSM.html')]
        url = urls[0]
        lat = 47.847160647818754
        lon = 5.185136077602821
        cfg = {'g9n': {'expandsize': [700, 500],
               'userpoints': {'St Andrew': 
                    ['50.08079', '3.415389', 'Old church'],
                    'Etcétéra':
                    ['48.080798', '3.4153891', 'Café de quartier'],
                    'Les suicidés':
                    ['50.080798', '5.4153891', 'Viaduc sur le Rhin']}}}
        self.em = ExpandedMap(self, cfg, url, urls)
        self.em.set_coordinate(lat, lon, 4)
        reply = self.em.exec_()
        sys.exit()

if __name__ == '__main__':
    from PyQt5.QtWidgets import  QApplication
    app = QApplication([])
    m = Main()
    sys.exit(app.exec_())
