# -*- coding: utf-8 -*-

# ui_geolocation.py
# 
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2009-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Gui for the geolocation module

import os

from datetime import date, datetime, timedelta

from PyQt5 import QtCore, QtGui, QtWidgets


import stylesheets as styles

from g9n.mapviewer import MapViewer
from VWidgets_PyQt5.vlineedit import VLineEdit

class Ui_Geolocation(QtWidgets.QDialog):
    def __init__(self, gl, place, utc, parent=None):
        super().__init__(parent)
        self.gl = gl
        self.temp = gl.temp
        self.place = (place[0], place[1], 16)
        self.resize(915, 583)
        self.setSizeGripEnabled(True)
        self.gridLayout = QtWidgets.QGridLayout(self)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.fname_lbl = QtWidgets.QLabel(self)
        self.horizontalLayout.addWidget(self.fname_lbl)
        self.gpx_fname_led = VLineEdit(self)
        self.gpx_fname_led.setResetButton(True)
        self.gpx_fname_led.setHistoricButton(True)
        self.gpx_fname_led.setCommandButton(True)
        self.gpx_fname_led.setInformativeText(_("Enter the file name"))
        self.horizontalLayout.addWidget(self.gpx_fname_led)
        self.browse_btn = QtWidgets.QPushButton(self)
        self.browse_btn.setText("...")
        self.horizontalLayout.addWidget(self.browse_btn)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.splitter = QtWidgets.QSplitter(self)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.wdg = QtWidgets.QWidget(self.splitter)
        self.v_layout = QtWidgets.QVBoxLayout()
        self.v_layout.setContentsMargins(0, 0, 0, 0)
        self.tracks_tbw = TrackSelector(self, self.wdg)
        self.v_layout.addWidget(self.tracks_tbw)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        spacerItem1 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Fixed, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_10.addItem(spacerItem1)
        self.info_track_lbl = QtWidgets.QLabel(self)
        self.horizontalLayout_10.addWidget(self.info_track_lbl)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_10.addItem(spacerItem2)
        self.v_layout.addLayout(self.horizontalLayout_10)
        self.wdg.setLayout(self.v_layout)
        self.map = MapViewer(self.splitter)
        self.map.setUrl(QtCore.QUrl('about:blank'))
        self.map.is_enable = True
        self.verticalLayout_3.addWidget(self.splitter)
        self.line = QtWidgets.QFrame(self)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.verticalLayout_3.addWidget(self.line)
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.hours_lbl = QtWidgets.QLabel(self)
        self.horizontalLayout_2.addWidget(self.hours_lbl)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Fixed,
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem4)
        self.gmt_lbl = QtWidgets.QLabel(self)
        self.gmt_lbl.setText("GMT:")
        self.horizontalLayout_5.addWidget(self.gmt_lbl)
        self.gmt_val_lbl = QtWidgets.QLabel(self)
        self.horizontalLayout_5.addWidget(self.gmt_val_lbl)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        spacerItem5 = QtWidgets.QSpacerItem(60, 20, QtWidgets.QSizePolicy.Fixed, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem5)
        self.gmt_rdo = QtWidgets.QRadioButton(self)
        self.gmt_rdo.setObjectName('gmt_rdo')
        self.buttonGroup = QtWidgets.QButtonGroup(self)
        self.buttonGroup.addButton(self.gmt_rdo)
        self.horizontalLayout_6.addWidget(self.gmt_rdo)
        self.gmt_rdo.setEnabled(False)
        self.verticalLayout_2.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        spacerItem6 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Fixed, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem6)
        self.local_lbl = QtWidgets.QLabel(self)
        self.horizontalLayout_7.addWidget(self.local_lbl)
        self.local_val_lbl = QtWidgets.QLabel(self)
        self.horizontalLayout_7.addWidget(self.local_val_lbl)
        self.verticalLayout_2.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        spacerItem7 = QtWidgets.QSpacerItem(60, 20, QtWidgets.QSizePolicy.Fixed, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem7)
        self.local_rdo = QtWidgets.QRadioButton(self)
        self.local_rdo.setObjectName('local_rdo')
        self.buttonGroup.addButton(self.local_rdo)
        self.horizontalLayout_8.addWidget(self.local_rdo)
        self.local_rdo.setEnabled(False)
        self.verticalLayout_2.addLayout(self.horizontalLayout_8)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        spacerItem8 = QtWidgets.QSpacerItem(60, 20, QtWidgets.QSizePolicy.Fixed, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem8)
        self.other_rdo = QtWidgets.QRadioButton(self)
        self.other_rdo.setObjectName('other_rdo')
        self.buttonGroup.addButton(self.other_rdo)
        self.horizontalLayout_9.addWidget(self.other_rdo)
        self.other_rdo.setEnabled(False)
        self.other_date_lbl = QtWidgets.QLabel(self)
        self.horizontalLayout_9.addWidget(self.other_date_lbl)
        self.hour_spb = QtWidgets.QSpinBox(self)
        self.horizontalLayout_9.addWidget(self.hour_spb)
        self.hour_spb.setEnabled(False)
        self.min_spb = QtWidgets.QSpinBox(self)
        self.horizontalLayout_9.addWidget(self.min_spb)
        self.min_spb.setEnabled(False)

        self.verticalLayout_2.addLayout(self.horizontalLayout_9)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        spacerItem9 = QtWidgets.QSpacerItem(60, 20, QtWidgets.QSizePolicy.Fixed, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(spacerItem9)
        self.choose_rdo = QtWidgets.QRadioButton(self)
        self.choose_rdo.setObjectName('choose_rdo')
        self.buttonGroup.addButton(self.choose_rdo)
        self.horizontalLayout_12.addWidget(self.choose_rdo)
        self.choose_rdo.setEnabled(False)
        self.verticalLayout_2.addLayout(self.horizontalLayout_12)
        spacerItem10 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Minimum, 
                                        QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem10)
        self.horizontalLayout_13.addLayout(self.verticalLayout_2)
        self.preview = QtWidgets.QWidget(self)
        self.preview.setMinimumSize(QtCore.QSize(250, 300))
        self.preview.setMaximumSize(QtCore.QSize(400, 16777215))
        self.verticalLayout = QtWidgets.QVBoxLayout(self.preview)
        self.picture_name_lbl = QtWidgets.QLabel(self.preview)
        self.picture_name_lbl.setAlignment(QtCore.Qt.AlignCenter)
        self.picture_name_lbl.setText('Name - Date')
        self.verticalLayout.addWidget(self.picture_name_lbl)
        self.viewer = Viewer(self.preview)
        self.verticalLayout.addWidget(self.viewer)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.sort_btn = QtWidgets.QPushButton(self)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap("medias/magnifier_36x30.png"), 
                                        QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.sort_btn.setIcon(icon4)
        self.sort_btn.setIconSize(QtCore.QSize(22, 22))
        self.horizontalLayout_3.addWidget(self.sort_btn)
        spacerItem11 = QtWidgets.QSpacerItem(40, 20, 
                                        QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem11)
        self.previous_btn = QtWidgets.QToolButton(self.preview)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("medias/arrow_grey_left.png"), 
                                        QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.previous_btn.setIcon(icon)
        self.previous_btn.setIconSize(QtCore.QSize(22, 22))
        self.horizontalLayout_3.addWidget(self.previous_btn)
        self.next_btn = QtWidgets.QToolButton(self.preview)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap("medias/arrow_grey_right.png"), 
                                        QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.next_btn.setIcon(icon1)
        self.next_btn.setIconSize(QtCore.QSize(22, 22))
        self.horizontalLayout_3.addWidget(self.next_btn)

        spacerItem12 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem12)
        self.select_btn = QtWidgets.QPushButton(self.preview)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap("medias/dialog-apply.svg"), 
                                        QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.select_btn.setIcon(icon2)
        self.select_btn.setIconSize(QtCore.QSize(22, 22))
        self.horizontalLayout_3.addWidget(self.select_btn)
        self.verticalLayout.addLayout(self.horizontalLayout_3)

        self.horizontalLayout_13.addWidget(self.preview)
        self.preview.setEnabled(False)
        self.verticalLayout_3.addLayout(self.horizontalLayout_13)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.message_lbl = QtWidgets.QLabel(self)
        self.horizontalLayout_4.addWidget(self.message_lbl)
        spacerItem13 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem13)
        self.apply_btn = QtWidgets.QPushButton(self)
        self.apply_btn.setIcon(icon2)
        self.apply_btn.setIconSize(QtCore.QSize(22, 22))
        self.horizontalLayout_4.addWidget(self.apply_btn)
        self.apply_btn.setEnabled(False)
        self.progress_pgb = QtWidgets.QProgressBar(self)
        self.horizontalLayout_4.addWidget(self.progress_pgb)
        self.close_btn = QtWidgets.QPushButton(self)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap("medias/close.png"), 
                                        QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.close_btn.setIcon(icon3)
        self.close_btn.setIconSize(QtCore.QSize(22, 22))
        self.horizontalLayout_4.addWidget(self.close_btn)
        spacerItem14 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Fixed, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem14)
        self.verticalLayout_3.addLayout(self.horizontalLayout_4)
        self.gridLayout.addLayout(self.verticalLayout_3, 0, 0, 2, 2)

        self.retranslateUi()
        self.set_theme()

        # sets the last user's choice
        if utc == 'gmt':
            self.gmt_rdo.setChecked(True)
        elif utc == 'other':
            self.other_rdo.setChecked(True)
        else:
            self.local_rdo.setChecked(True)

        self.closeEvent = self.close_event
        self.gpx_fname_led.editingFinished.connect(self.on_editing_finished)
        self.gpx_fname_led.returnPressed.connect(self.on_editing_finished)
        self.gpx_fname_led.commandTriggered.connect(self.on_editing_finished)
        self.browse_btn.clicked.connect(self.on_browse_clicked)
        self.buttonGroup.buttonClicked.connect(self.on_time_changed)
        self.hour_spb.valueChanged.connect(self.set_other_time)
        self.min_spb.valueChanged.connect(self.set_other_time)
        self.viewer.viewChanged.connect(self.gl.view_changed)
        self.viewer.fileRejected.connect(self.gl.file_rejected)
        self.sort_btn.clicked.connect(self.on_sort_clicked)
        self.previous_btn.clicked.connect(self.viewer.previous_image)
        self.next_btn.clicked.connect(self.viewer.next_image)
        self.select_btn.clicked.connect(self.gl.set_first_image)
        self.apply_btn.clicked.connect(self.gl.tag_images)
        self.close_btn.clicked.connect(self.close_event)

        self.time_delta = 0

    def setup_widgets(self):
        d = sum(self.splitter.sizes()) / 10
        self.splitter.setSizes([d*6, d*4])
        self.map._setup(self.place, self.temp)
        dt = date.today().strftime('%A %d %b %Y')
        self.other_date_lbl.setText(dt)
        tm = QtCore.QTime().currentTime()
        self.hour_spb.setValue(tm.hour())
        self.min_spb.setValue(tm.minute())
        self.set_time_delta()

    def enable_time_widgets(self, state):
        self.gmt_rdo.setEnabled(state)
        self.local_rdo.setEnabled(state)
        self.other_rdo.setEnabled(state)
        self.choose_rdo.setEnabled(state)

    def enable_other_time_widgets(self, state):
        self.hour_spb.setEnabled(state)
        self.min_spb.setEnabled(state)

    def enable_viewer_widgets(self, state):
        self.next_btn.setEnabled(state)
        self.previous_btn.setEnabled(state)
        self.select_btn.setEnabled(state)
        self.picture_name_lbl.setText('')

    def reset_preview(self, enable=True):
        self.picture_name_lbl.setText('')
        self.viewer.clear()
        self.preview.setEnabled(enable)
        self.next_btn.setEnabled(False)
        self.previous_btn.setEnabled(False)
        self.select_btn.setEnabled(False)
        self.apply_btn.setEnabled(False)

    def on_editing_finished(self, path=None):
        if path is None:
            path = self.gpx_fname_led.text()

        if path:
            if os.path.isfile(path):
                self.gl.read_gpx()

            else:
                self.gpx_fname_led.clear()

    def on_browse_clicked(self):
        self.gpx_fname_led.setText('')
        self.gl.load_gpx()

    def on_time_changed(self, sender):
        name = sender.objectName()
        self.enable_other_time_widgets(False)
        self.enable_viewer_widgets(False)
        self.apply_btn.setEnabled(False)
        self.viewer.clear()
        if name == 'other_rdo':
            self.enable_other_time_widgets(True)

        elif name == 'choose_rdo':
            self.apply_btn.setEnabled(False)

        self.set_time_delta()
        self.gl.set_message(6)
        self.gl.update_config(name[:-4])

    def set_other_time(self, val):
        utc = datetime.utcnow()
        h = self.hour_spb.value()
        m = self.min_spb.value()
        new = datetime(year=utc.year, month=utc.month, day=utc.day, 
                        hour=h, minute=m, second=utc.second)
        self.time_delta = new - utc

    def on_sort_clicked(self):
        self.viewer.clear()
        self.gl.sort_images()

    def set_time_delta(self):
        """time_delta = time - utcnow

        picture time - time_delta = gmt
        """
        utc = datetime.utcnow()
        if self.gmt_rdo.isChecked():
            self.time_delta = timedelta(0)

        elif self.local_rdo.isChecked():
            lcl = datetime.today()
            self.time_delta = lcl - utc

        elif self.other_rdo.isChecked():
            h = self.hour_spb.value()
            m = self.min_spb.value()
            new = datetime(year=utc.year, month=utc.month, day=utc.day, 
                            hour=h, minute=m, second=utc.second)
            self.time_delta = new - utc

        else:
            self.time_delta = None

    def set_rolling_progress(self, state):
        """Sets the buzy indicator into the progressBar.

        Args:
        state -- bool
        """
        mx = 100
        if state:
            mx = 0

        self.progress_pgb.setRange(0, mx)

    def close_event(self, ev=None):
        self.accept()

    def retranslateUi(self):
        self.setWindowTitle(_("Geolocation - Oqapy"))
        self.fname_lbl.setText(_("File .gpx:"))
        self.browse_btn.setToolTip(_("Browse"))
        self.hours_lbl.setText(_("Hour settings:"))
        self.gmt_val_lbl.setText(_("TextLabel"))
        self.gmt_rdo.setText(_("The hour of the camera is the GMT hour"))
        self.local_lbl.setText(_("Local:"))
        self.local_val_lbl.setText(_("TextLabel"))
        self.local_rdo.setText(_("The hour of the camera is the local hour"))
        self.other_rdo.setText(_("Other:"))
        self.hour_spb.setSuffix(_(" Hours"))
        self.min_spb.setSuffix(_(" Min."))
        self.choose_rdo.setText(_("Choose the first picture"))
        self.previous_btn.setToolTip(_("Previous picture"))
        self.next_btn.setToolTip(_("Next picture"))
        self.select_btn.setToolTip(_("Select the current picture \nas the first one"))
        self.select_btn.setText(_("Select"))
        self.sort_btn.setToolTip(_("Sort the pictures"))
        self.sort_btn.setText(_("Sort"))
        self.apply_btn.setToolTip(_("Apply the geolocation"))
        self.apply_btn.setText(_("Apply"))
        self.close_btn.setText(_("Close"))

    def set_theme(self):
        self.gpx_fname_led.setStyleSheet(styles.LINEEDIT)
        self.browse_btn.setStyleSheet(styles.PUSHBUTTONDIAL)
        self.tracks_tbw.setStyleSheet(styles.TABLEVIEW)
        self.tracks_tbw.horizontalHeader().setStyleSheet(styles.DATAHEADER)
        self.tracks_tbw.verticalHeader().setStyleSheet(styles.DATAHEADER)
        self.line.setStyleSheet(styles.UNDERLINE)
        self.gmt_rdo.setStyleSheet(styles.RADIO)
        self.local_rdo.setStyleSheet(styles.RADIO)
        self.other_rdo.setStyleSheet(styles.RADIO)
        self.hour_spb.setStyleSheet(styles.SPINBOX)
        self.min_spb.setStyleSheet(styles.SPINBOX)
        self.viewer.setStyleSheet(styles.VIEWER)
        self.choose_rdo.setStyleSheet(styles.RADIO)
        self.sort_btn.setStyleSheet(styles.PUSHBUTTONDIAL)
        self.previous_btn.setStyleSheet(styles.TOOLBUTTONDIAL)
        self.next_btn.setStyleSheet(styles.TOOLBUTTONDIAL)
        self.select_btn.setStyleSheet(styles.PUSHBUTTONDIAL)
        self.apply_btn.setStyleSheet(styles.PUSHBUTTONDIAL)
        self.progress_pgb.setStyleSheet(styles.PROGRESSBAR)
        self.close_btn.setStyleSheet(styles.PUSHBUTTONDIAL)


class TrackSelector(QtWidgets.QTableWidget):
    def __init__(self, ui, parent=None):
        super().__init__(parent)
        self.ui = ui
        self.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.setColumnCount(5)
        self.horizontalHeader().setStretchLastSection(True)
        self.setAlternatingRowColors(True)
        self.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        for idx, width in enumerate([80, 90, 180, 80, 80]):
            self.setColumnWidth(idx, width)
        h = [_("Name"), _("Description"), _("Start time"), _("Lat."), _("Long.")]
        self.setHorizontalHeaderLabels(h)
        self.itemSelectionChanged.connect(self.on_selection_changed)

    def display_tracks(self, tracks):
        """Sets the track's infos into the table widget.

        args:
        tracks -- List of tracks 
                  [[name, description, date, longitude, latitude], [...]]
        """
        self.clean()
        self.setRowCount(len(tracks))
        for idx, track in enumerate(tracks):
            for idy, data in enumerate(track):
                item = QtWidgets.QTableWidgetItem()
                item.setText(data)
                self.setItem(idx, idy, item)

    def on_selection_changed(self):
        sel = False
        if self.selectedItems():
            sel = self.selectedItems()[0].text()

        self.ui.gl.on_track_selected(sel)

    def clean(self):
        self.clearContents()

class Viewer(QtWidgets.QGraphicsView):
    viewChanged = QtCore.pyqtSignal(int)
    fileRejected = QtCore.pyqtSignal(int)
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.ui = parent
        self.images = None
        self.current_view = 0
        sc = QtWidgets.QGraphicsScene()
        self.setScene(sc)
        self.pix = False

    def load_image(self, idx):
        self.current_view = idx
        self.image = QtGui.QImage(self.images[idx])
        if self.image.isNull():
            self.images.pop(idx)
            self.fileRejected.emit(idx)
            self.load_image(idx)

        self.show_image()
        self.viewChanged.emit(idx)

    def show_image(self):
        self.pix = QtGui.QPixmap.fromImage(self.image.scaled
                                            (self.width()-2, self.height()-2,
                                            QtCore.Qt.KeepAspectRatio, 
                                            QtCore.Qt.SmoothTransformation))
        self.scene().clear()
        self.scene().setSceneRect(0, 0, self.pix.width(), self.pix.height())
        self.scene().addPixmap(self.pix)

    def next_image(self):
        """Display the next image in the viewer.

        """
        idx = self.current_view + 1
        if idx == len(self.images):
            idx = 0

        self.load_image(idx)

    def previous_image(self):
        """Display the previous image in the viewer.

        """
        idx = self.current_view - 1
        if idx == -1:
            idx = len(self.images) - 1

        self.load_image(idx)

    def clear(self):
        self.scene().clear()

    def resizeEvent(self, event):
        if self.pix:
            self.show_image()

    def display_no_images(self):
        self.pix = QtGui.QPixmap('medias/no_images.png')
        self.scene().clear()
        self.scene().setSceneRect(0, 0, self.pix.width(), self.pix.height())
        self.scene().addPixmap(self.pix)


