# -*- coding: utf-8 -*-

# contrastbrightnessbox.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2014-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Contrast-Brightness tool box.

from PyQt5.QtCore import Qt, QSize, QCoreApplication
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QSpacerItem, QSizePolicy, QLabel, QSpinBox,
                             QPushButton, QProgressBar)

import stylesheets as styles

from commonwidgets.horizontalslider import HorizontalSlider

class ContrastBrightnessBox(QWidget):
    def __init__(self, core, parent=None):
        super().__init__ (parent)
        self.name = 'ContrastBrightnessBox'
        self.core = core
        self.i18n_name = 'Contrast-Brightness'
        self.glayout_2 = QGridLayout(self)
        self.glayout_2.setContentsMargins(0, 2, 8, 2)
        self.vlayout = QVBoxLayout()
        self.vlayout.setSpacing(0)
        self.hlayout_6 = QHBoxLayout()
        spacer = QSpacerItem(20, 20, QSizePolicy.Preferred, QSizePolicy.Minimum)
        self.hlayout_6.addItem(spacer)
        self.gridLayout = QGridLayout()
        self.gridLayout.setHorizontalSpacing(8)
        self.label = QLabel(self)
        self.label.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.hlayout = QHBoxLayout()
        self.hlayout.setSpacing(8)
        self.label_2 = QLabel(self)
        self.label_2.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.hlayout.addWidget(self.label_2)
        self.upper_spb = QSpinBox(self)
        self.upper_spb.setMinimum(155)
        self.upper_spb.setMaximum(255)
        self.upper_spb.setProperty("value", 255)
        self.hlayout.addWidget(self.upper_spb)
        self.gridLayout.addLayout(self.hlayout, 0, 1, 1, 1)
        spacer2 = QSpacerItem(40, 20, QSizePolicy.Preferred, QSizePolicy.Minimum)
        self.gridLayout.addItem(spacer2, 1, 0, 1, 1)
        self.hlayout_2 = QHBoxLayout()
        self.hlayout_2.setSpacing(8)
        self.label_3 = QLabel(self)
        self.label_3.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.hlayout_2.addWidget(self.label_3)
        self.lower_spb = QSpinBox(self)
        self.lower_spb.setMaximum(100)
        self.hlayout_2.addWidget(self.lower_spb)
        self.gridLayout.addLayout(self.hlayout_2, 1, 1, 1, 1)
        self.cont_lbl = QLabel(self)
        self.cont_lbl.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.gridLayout.addWidget(self.cont_lbl, 2, 0, 1, 1)
        self.hlayout_3 = QHBoxLayout()
        self.hlayout_3.setSpacing(8)
        self.cont_val_lbl = QLabel(self)
        self.cont_val_lbl.setMinimumSize(QSize(10, 0))
        self.cont_val_lbl.setText("0")
        self.cont_val_lbl.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.hlayout_3.addWidget(self.cont_val_lbl)
        self.contrast_wdg = HorizontalSlider(self)
        self.contrast_wdg.set_layout_style('right')
        self.contrast_wdg.set_range((-100, 100))
        self.hlayout_3.addWidget(self.contrast_wdg)
        self.gridLayout.addLayout(self.hlayout_3, 2, 1, 1, 1)
        self.brg_lbl = QLabel(self)
        self.brg_lbl.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.gridLayout.addWidget(self.brg_lbl, 3, 0, 1, 1)
        self.hlayout_4 = QHBoxLayout()
        self.hlayout_4.setSpacing(8)
        self.brg_val_lbl = QLabel(self)
        self.brg_val_lbl.setMinimumSize(QSize(10, 0))
        self.brg_val_lbl.setText("0")
        self.brg_val_lbl.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.hlayout_4.addWidget(self.brg_val_lbl)
        self.brightness_wdg = HorizontalSlider(self)
        self.brightness_wdg.set_layout_style('right')
        self.brightness_wdg.set_range((-100, 100))
        self.hlayout_4.addWidget(self.brightness_wdg)
        self.gridLayout.addLayout(self.hlayout_4, 3, 1, 1, 1)
        self.hlayout_6.addLayout(self.gridLayout)
        self.vlayout.addLayout(self.hlayout_6)

        self.hlayout_7 = QHBoxLayout()
        self.hlayout_7.setContentsMargins(20, 0, 5, 0)
        self.progress = QProgressBar(self)
        self.progress.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        self.progress.setMaximumSize(QSize(1000, 12))
        self.hlayout_7.addWidget(self.progress)
        self.vlayout.addLayout(self.hlayout_7)

        self.hlayout_8 = QHBoxLayout()
        spacer = QSpacerItem(10, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout_8.addItem(spacer)
        self.preview_btn = QPushButton(self)
        self.preview_btn.setEnabled(False)
        icon = QIcon('medias/eye_red.png')
        self.preview_btn.setIcon(icon)
        self.preview_btn.setIconSize(QSize(30, 14))
        self.hlayout_8.addWidget(self.preview_btn)
        self.cancel_btn = QPushButton(self)
        self.cancel_btn.setObjectName("reverse_btn")
        self.cancel_btn.setEnabled(False)
        icon = QIcon('medias/edit-undo.png')
        self.cancel_btn.setIcon(icon)
        self.cancel_btn.setIconSize(QSize(18, 18))
        self.hlayout_8.addWidget(self.cancel_btn)
        self.apply_btn = QPushButton(self)
        self.apply_btn.setObjectName("apply_btn")
        self.apply_btn.setEnabled(False)
        icon1 = QIcon('medias/apply.png')
        self.apply_btn.setIcon(icon1)
        self.apply_btn.setIconSize(QSize(18, 18))
        self.hlayout_8.addWidget(self.apply_btn)
        self.vlayout.addLayout(self.hlayout_8)
        self.glayout_2.addLayout(self.vlayout, 0, 0, 1, 1)

        self.contrast = 0
        self.brightness = 0
        self.translate()
        self.set_style()
        self.progress.hide()

        self.contrast_wdg.valueChanged.connect(self.on_contrast_changed)
        self.brightness_wdg.valueChanged.connect(self.on_brightness_changed)
        self.preview_btn.clicked.connect(self.on_preview_clicked)
        self.apply_btn.clicked.connect(self.apply_contrast)
        self.cancel_btn.clicked.connect(self.undo)

    def on_contrast_changed(self, value):
        self.contrast = value
        self.cont_val_lbl.setText(str(value))
        self.check_values()

    def on_brightness_changed(self, value):
        self.brightness = value
        self.brg_val_lbl.setText(str(value))
        self.check_values()

    def on_preview_clicked(self):
        """Call the preview window.

        """
        self.preview_btn.setEnabled(False)
        self.core.set_preview_window(self)

    def apply_contrast(self):
        self.core.state.disable_undo_buttons(self)
        self.core.close_preview_window()
        self.set_progress_bar(True)
        self.apply_btn.setEnabled(False)
        self.preview_btn.setEnabled(False)
        QCoreApplication.processEvents()
        self.core.set_filtered_mode(True)
        self.core.filtering.apply_contrast_brightness(self.get_values(), self)

    def check_values(self):
        b = self.contrast or self.brightness
        self.apply_btn.setEnabled(b)
        self.preview_btn.setEnabled(b)

    def get_values(self):
        return (self.lower_spb.value(), self.upper_spb.value(),
                self.contrast, self.brightness)

    def set_apply_button_enabled(self, state):
        pass

    def undo(self):
        self.core.filtering.undo_last_change()
        self.cancel_btn.setEnabled(False)

    def reset(self):
        self.contrast = 0
        self.brightness = 0
        self.lower_spb.setValue(0)
        self.upper_spb.setValue(255)
        self.contrast_wdg.reset_value()
        self.brightness_wdg.reset_value()
        self.set_progress_bar(False)

    def set_progress_bar(self, b):
        if b:
            self.progress.setRange(0, 0)
            self.progress.show()

        else:
            self.progress.setRange(0, 10)
            self.progress.hide()

        QCoreApplication.processEvents()

    def translate(self):
        self.label.setText(_("Tresholds:"))
        self.label_2.setText(_("Upper:"))
        self.label_3.setText(_("Lower:"))
        self.cont_lbl.setText(_("Contrast:"))
        self.brg_lbl.setText(_("Brightness:"))
        self.contrast_wdg.minus.setStatusTip(_('Contrast -1'))
        self.contrast_wdg.plus.setStatusTip(_('Contrast +1'))
        self.brightness_wdg.minus.setStatusTip(_('Brightness -1'))
        self.brightness_wdg.plus.setStatusTip(_('Brightness +1'))
        self.contrast_wdg.reset.setStatusTip(_("Reset to 0"))
        self.brightness_wdg.reset.setStatusTip(_("Reset to 0"))
        self.preview_btn.setStatusTip(_("Show the preview"))
        self.apply_btn.setText(_("Apply"))
        self.cancel_btn.setText(_("Cancel"))
        self.apply_btn.setStatusTip(_("Apply the contrast"))
        self.cancel_btn.setStatusTip(_("Cancel the contrast"))

    def set_style(self):
        self.label.setStyleSheet(styles.TITLE)
        self.label_2.setStyleSheet(styles.TITLE)
        self.label_3.setStyleSheet(styles.TITLE)
        self.upper_spb.setStyleSheet(styles.SPINBOX)
        self.lower_spb.setStyleSheet(styles.SPINBOX)
        self.cont_lbl.setStyleSheet(styles.TITLE)
        self.brg_lbl.setStyleSheet(styles.TITLE)
        self.contrast_wdg.minus.setStyleSheet(styles.TBUTTON)
        self.contrast_wdg.slider.setStyleSheet(styles.SLIDER)
        self.contrast_wdg.plus.setStyleSheet(styles.TBUTTON)
        self.contrast_wdg.reset.setStyleSheet(styles.TBUTTON)
        self.brightness_wdg.minus.setStyleSheet(styles.TBUTTON)
        self.brightness_wdg.slider.setStyleSheet(styles.SLIDER)
        self.brightness_wdg.plus.setStyleSheet(styles.TBUTTON)
        self.brightness_wdg.reset.setStyleSheet(styles.TBUTTON)
        self.progress.setStyleSheet(styles.PROGRESSBAR)
        self.preview_btn.setStyleSheet(styles.PUSHBUTTON)
        self.apply_btn.setStyleSheet(styles.PUSHBUTTON)
        self.cancel_btn.setStyleSheet(styles.PUSHBUTTON)

