# -*- coding: utf-8 -*-

# cropbox.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2014-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Crop tool box.

import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import QSize, pyqtSignal
from PyQt5.QtGui import QPixmap, QIcon
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QCheckBox, QComboBox, QLabel, QToolButton,
                             QPushButton, QSpacerItem, QSizePolicy)

import stylesheets as styles


class CropBox(QWidget):
    getFocus = pyqtSignal(QWidget)
    def __init__(self, ui, parent=None):
        super().__init__(parent)
        self.name = 'CropBox'
        self.ui = ui
        self.core = ui.core
        self.grid = QGridLayout(self)
        self.grid.setContentsMargins(0, 0, 8, 0)
        self.grid.setVerticalSpacing(2)
        self.vlayout = QVBoxLayout()
        self.vlayout.setSpacing(2)
        self.hlayout_3 = QHBoxLayout()
        spacer = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.hlayout_3.addItem(spacer)
        self.grid1 = QGridLayout()
        self.constrain_chb = QCheckBox(self)
        self.grid1.addWidget(self.constrain_chb, 0, 0, 1, 1)
        self.hlayout_5 = QHBoxLayout()
        spacer5 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout_5.addItem(spacer5)
        self.constrain_cmb = QComboBox(self)
        self.hlayout_5.addWidget(self.constrain_cmb)
        self.grid1.addLayout(self.hlayout_5, 1, 0, 1, 1)

        #self.hlayout = QHBoxLayout()
        #self.grid_chb = QCheckBox(self)
        #self.hlayout.addWidget(self.grid_chb)
        #spacer1 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        #self.hlayout.addItem(spacer1)
        #self.grid1.addLayout(self.hlayout, 2, 0, 1, 1)

        self.hlayout_2 = QHBoxLayout()
        self.label = QLabel(self)
        self.hlayout_2.addWidget(self.label)
        self.size_lbl = QLabel(self)
        self.hlayout_2.addWidget(self.size_lbl)
        self.grid1.addLayout(self.hlayout_2, 3, 0, 1, 1)
        spacer2 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.grid1.addItem(spacer2, 2, 1, 1, 1)
        self.hlayout_3.addLayout(self.grid1)
        self.vlayout.addLayout(self.hlayout_3)
        self.hlayout_4 = QHBoxLayout()
        spacer3 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.hlayout_4.addItem(spacer3)
        self.pick_btn = QToolButton(self)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/crop_tool_white_24.png"), QIcon.Normal, 
                            QIcon.Off)
        self.pick_btn.setIcon(icon)
        self.pick_btn.setIconSize(QSize(18, 18))
        self.hlayout_4.addWidget(self.pick_btn)
        self.setting_btn = QToolButton(self)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/settings.png"), QIcon.Normal, 
                            QIcon.Off)
        self.setting_btn.setIcon(icon1)
        self.setting_btn.setIconSize(QSize(18, 18))
        self.hlayout_4.addWidget(self.setting_btn)
        spacer4 = QSpacerItem(18, 20, QSizePolicy.Preferred, QSizePolicy.Minimum)
        self.hlayout_4.addItem(spacer4)
        self.cancel_btn = QPushButton(self)
        self.cancel_btn.setEnabled(False)
        icon2 = QIcon('medias/edit-undo.png')
        self.cancel_btn.setIcon(icon2)
        self.cancel_btn.setIconSize(QSize(18, 18))
        self.hlayout_4.addWidget(self.cancel_btn)
        self.apply_btn = QPushButton(self)
        icon3 = QIcon('medias/apply.png')
        self.apply_btn.setIcon(icon3)
        self.apply_btn.setIconSize(QSize(18, 18))
        self.hlayout_4.addWidget(self.apply_btn)
        self.vlayout.addLayout(self.hlayout_4)
        self.grid.addLayout(self.vlayout, 0, 0, 1, 1)

        self.set_size_label(False)
        self.retranslateUi()
        self.set_style()
        self.set_widgets_enabled(False)
        self.constrain_chb.toggled.connect(self.on_constrain_checked)
        self.constrain_cmb.currentIndexChanged[int].connect(self.on_ratio_changed)
        self.pick_btn.clicked.connect(self.activate_crop_tool)
        self.setting_btn.clicked.connect(self.call_settings_dialog)
        self.apply_btn.clicked.connect(self.apply_crop)
        self.cancel_btn.clicked.connect(self.undo_crop)
        self.is_active = False
        self.is_drawn = False

    def set_ratios(self):
        self.ratios = self.cfg['ratios'][:]
        self.ratios.insert(0, _('Same as image'))
        self.ratios.append(_('Settings...'))
        self.constrain_cmb.blockSignals(True)
        self.constrain_cmb.clear()
        self.constrain_cmb.addItems(self.ratios)
        self.constrain_cmb.blockSignals(False)

    def set_widgets_enabled(self, b):
        self.constrain_chb.setEnabled(b)
        self.constrain_cmb.setEnabled(b)
        self.cancel_btn.setEnabled(b)
        self.setting_btn.setEnabled(b)

    def set_size_label(self, size):
        if not size:
            self.label.hide()
            self.size_lbl.hide()
            self.apply_btn.setEnabled(False)

        else:
            self.label.show()
            self.size_lbl.setText(" %s x %s pxl" % size)
            self.size_lbl.show()
            self.apply_btn.setEnabled(True)

    def apply_crop(self):
        self.apply_btn.setEnabled(False)
        self.pick_btn.setEnabled(True)
        self.setting_btn.setEnabled(False)
        self.core.state.disable_undo_buttons(self)
        self.core.crop.prepare_crop()

    def configure(self, cfg):
        if -1 in cfg['ratios']:
            cfg['ratios'].remove(-1)
        self.cfg = cfg
        self.set_ratios()
        self.constrain_cmb.blockSignals(True)
        self.constrain_chb.blockSignals(True)
        self.constrain_chb.setChecked(cfg['constrain'])
        val = cfg['mode']
        lgg.info('Set config values')
        if val[0] == 'same_as':
            idx = 0

        else:
            if val[0] == 'defined':
                ratio = val[1]

            else:
                ratio = '%s:%s' %(val[1], val[2])

            if ratio not in self.ratios:
                self.ratios.insert(-2, ratio)
                self.constrain_cmb.clear()
                self.constrain_cmb.addItems(self.ratios)

            idx = self.ratios.index(ratio)

        self.constrain_cmb.setCurrentIndex(idx)
        self.constrain_chb.blockSignals(False)
        self.constrain_cmb.blockSignals(False)

    def update_settings(self, sett):
        """Called when the settings are changed by the crop tool dialog.

        Args:
        sett -- dict
        """
        self.constrain_chb.blockSignals(True)
        self.configure(sett)
        self.constrain_chb.blockSignals(False)

    def unset_focus(self):
        self.set_widgets_enabled(False)
        self.is_active = False

    def reset(self):
        self.set_size_label(0)

    #----------------------------------------------------------
    # Slots
    #----------------------------------------------------------

    def on_constrain_checked(self, b):
        lgg.info('Crop constrain ratio changed: %s' % b)
        try:
            self.core.crop.update_constrain(b)
        except Exception as why:
            # Crop tool not active
            lgg.warning('on_constrain_checked: %s' % why)

    def on_ratio_changed(self, ratio):
        lgg.info('Crop ratio changed: %s' % ratio)
        if self.is_active:
            sett = self.core.cfg.get(('crop',))
            if ratio == len(self.ratios) - 1:
                self.core.crop.edit_settings()
                self.configure(self.core.cfg)

            elif not ratio:
                sett['mode'] = ('same_as',)

            else:
                if ratio == -1:
                    # If the combo is cleared ...
                    return

                w, h = self.ratios[ratio].split(':')
                value = float(w) / float(h)
                sett['current_ratio'] = value
                sett['mode'] = ('defined', self.ratios[ratio])

            self.core.crop.update_constrain_settings()

    def set_grid(self, b):
        lgg.info('Set grid: %s' %b)
        if b:
            self.core.show_grid()

        else:
            self.core.hide_grid()

    def activate_crop_tool(self):
        self.getFocus.emit(self)
        self.pick_btn.setEnabled(False)
        self.set_widgets_enabled(True)
        self.is_active = True
        self.is_drawn = False
        self.core.activate_crop_tool()

    def call_settings_dialog(self):
        from croptool import Settings
        dial = Settings(self)
        dial.set_values(self.core.cfg.get(('crop', )))
        dial.exec_()
        if self.core.crop:
            self.core.crop.update_settings()
            
        self.update_settings(self.core.cfg.get(('crop', )))

    def undo_crop(self):
        self.cancel_btn.setEnabled(False)
        self.set_size_label(False)
        if self.is_drawn:
            self.core.filtering.undo_last_change()

        else:
            self.core.abort_crop()

        self.pick_btn.setEnabled(True)
        self.set_widgets_enabled(False)
        self.is_active = False
        self.is_drawn = False

    def set_style(self):
        self.constrain_chb.setStyleSheet(styles.CHECKBOX)
        self.constrain_cmb.setStyleSheet(styles.COMBO)
        self.pick_btn.setStyleSheet(styles.TBUTTON)
        self.cancel_btn.setStyleSheet(styles.PUSHBUTTON)
        self.apply_btn.setStyleSheet(styles.PUSHBUTTON)

    def retranslateUi(self):
        self.constrain_chb.setText(_("Constrain ratio"))
        self.label.setText(_("Size:"))
        self.pick_btn.setStatusTip(_("Tool crop"))
        self.setting_btn.setStatusTip(_("Settings"))
        self.cancel_btn.setText(_("Cancel"))
        self.apply_btn.setText(_("Crop"))

