# -*- coding: utf-8 -*-

# denoisingbox.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2014-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Image denoising tool box.

from PyQt5.QtCore import Qt, QSize, QCoreApplication
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QSpacerItem, QSizePolicy, QLabel, QPushButton,
                             QProgressBar, QRadioButton)


from commonwidgets.horizontalslider import HorizontalSlider

import stylesheets as styles


class DenoisingBox(QWidget):
    def __init__(self, core, parent=None):
        super().__init__ (parent)
        self.name = 'DenoisingBox'
        self.i18n_name = 'Denoising'
        self.core = core
        self.gridLayout = QGridLayout(self)
        self.gridLayout.setContentsMargins(20, 2, 8, 2)
        self.vlayout = QVBoxLayout()
        self.vlayout.setSpacing(0)
        self.hlayout = QHBoxLayout()
        spacer = QSpacerItem(20, 20, QSizePolicy.Preferred, QSizePolicy.Minimum)
        self.hlayout.addItem(spacer)
        self.color_rdo = QRadioButton(self)
        self.hlayout.addWidget(self.color_rdo)
        self.bw_rdo = QRadioButton(self)
        self.hlayout.addWidget(self.bw_rdo)
        self.vlayout.addLayout(self.hlayout)
        self.glayout = QGridLayout()
        self.glayout.setContentsMargins(0, 0, 0, 0)
        self.glayout.setHorizontalSpacing(8)
        self.lm_lbl = QLabel(self)
        self.lm_lbl.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.glayout.addWidget(self.lm_lbl, 0, 0, 1, 1)
        self.hlayout1 = QHBoxLayout()
        self.hlayout1.setSpacing(8)
        self.lm_val_lbl = QLabel(self)
        self.lm_val_lbl.setMinimumSize(QSize(10, 0))
        self.lm_val_lbl.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.lm_val_lbl.setText("8")
        self.hlayout1.addWidget(self.lm_val_lbl)
        self.lm_wdg = HorizontalSlider(self)
        self.lm_wdg.reset.hide()
        self.lm_wdg.set_layout_style('right')
        self.lm_wdg.set_range((2, 30))
        self.lm_wdg.set_value(8)
        self.lm_wdg.set_default_value(8)
        self.hlayout1.addWidget(self.lm_wdg)
        self.glayout.addLayout(self.hlayout1, 0, 1, 1, 1)

        self.cl_lbl = QLabel(self)
        self.cl_lbl.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.glayout.addWidget(self.cl_lbl, 1, 0, 1, 1)
        self.hlayout2 = QHBoxLayout()
        self.hlayout2.setSpacing(8)
        self.cl_val_lbl = QLabel(self)
        self.cl_val_lbl.setMinimumSize(QSize(10, 0))
        self.cl_val_lbl.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.cl_val_lbl.setText("8")
        self.hlayout2.addWidget(self.cl_val_lbl)
        self.cl_wdg = HorizontalSlider(self)
        self.cl_wdg.reset.hide()
        self.cl_wdg.set_layout_style('right')
        self.cl_wdg.set_range((2, 30))
        self.cl_wdg.set_value(8)
        self.cl_wdg.set_default_value(8)
        self.hlayout2.addWidget(self.cl_wdg)
        self.glayout.addLayout(self.hlayout2, 1, 1, 1, 1)

        self.template_lbl = QLabel(self)
        self.template_lbl.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.glayout.addWidget(self.template_lbl, 2, 0, 1, 1)
        self.hlayout3 = QHBoxLayout()
        self.hlayout3.setSpacing(8)
        self.template_val_lbl = QLabel(self)
        self.template_val_lbl.setMinimumSize(QSize(10, 0))
        self.template_val_lbl.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.template_val_lbl.setText("5")
        self.hlayout3.addWidget(self.template_val_lbl)
        self.template_wdg = HorizontalSlider(self)
        self.template_wdg.reset.hide()
        self.template_wdg.set_layout_style('right')
        self.template_wdg.set_range((1, 15))
        self.template_wdg.set_single_step(2)
        self.template_wdg.set_value(5)
        self.template_wdg.set_default_value(5)
        self.hlayout3.addWidget(self.template_wdg)
        self.glayout.addLayout(self.hlayout3, 2, 1, 1, 1)

        self.sample_lbl = QLabel(self)
        self.sample_lbl.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.glayout.addWidget(self.sample_lbl, 3, 0, 1, 1)
        self.hlayout4 = QHBoxLayout()
        self.hlayout4.setSpacing(8)
        self.sample_val_lbl = QLabel(self)
        self.sample_val_lbl.setMinimumSize(QSize(10, 0))
        self.sample_val_lbl.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.sample_val_lbl.setText("10")
        self.hlayout4.addWidget(self.sample_val_lbl)
        self.sample_wdg = HorizontalSlider(self)
        self.sample_wdg.reset.hide()
        self.sample_wdg.set_layout_style('right')
        self.sample_wdg.set_range((2, 30))
        self.sample_wdg.set_value(10)
        self.sample_wdg.set_default_value(10)
        self.hlayout4.addWidget(self.sample_wdg)
        self.glayout.addLayout(self.hlayout4, 3, 1, 1, 1)

        self.vlayout.addLayout(self.glayout)

        self.hlayout5 = QHBoxLayout()
        self.hlayout5.setContentsMargins(20, 0, 5, 0)
        self.progress = QProgressBar(self)
        self.progress.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        self.progress.setMaximumSize(QSize(1000, 12))
        self.hlayout5.addWidget(self.progress)
        self.vlayout.addLayout(self.hlayout5)

        self.hlayout6 = QHBoxLayout()
        spacer = QSpacerItem(10, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout6.addItem(spacer)
        self.preview_btn = QPushButton(self)
        self.preview_btn.setEnabled(False)
        icon = QIcon('medias/eye_red.png')
        self.preview_btn.setIcon(icon)
        self.preview_btn.setIconSize(QSize(30, 14))
        self.hlayout6.addWidget(self.preview_btn)
        self.cancel_btn = QPushButton(self)
        self.cancel_btn.setEnabled(False)
        icon1 = QIcon('medias/edit-undo.png')
        self.cancel_btn.setIcon(icon1)
        self.cancel_btn.setIconSize(QSize(18, 18))
        self.hlayout6.addWidget(self.cancel_btn)
        self.apply_btn = QPushButton(self)
        self.apply_btn.setObjectName("apply_btn")
        self.apply_btn.setEnabled(False)
        icon2 = QIcon('medias/apply.png')
        self.apply_btn.setIcon(icon2)
        self.apply_btn.setIconSize(QSize(18, 18))
        self.hlayout6.addWidget(self.apply_btn)
        self.vlayout.addLayout(self.hlayout6)

        self.gridLayout.addLayout(self.vlayout, 0, 0, 1, 1)

        self.color_rdo.setChecked(True)
        self.progress.hide()
        self.translate()
        self.set_style()

        self.color_rdo.toggled.connect(self.on_space_changed)
        self.lm_wdg.valueChanged.connect(self.on_luminance_changed)
        self.cl_wdg.valueChanged.connect(self.on_color_changed)
        self.template_wdg.valueChanged.connect(self.on_template_changed)
        self.sample_wdg.valueChanged.connect(self.on_sample_changed)
        self.preview_btn.clicked.connect(self.on_preview_clicked)
        self.cancel_btn.clicked.connect(self.undo)
        self.apply_btn.clicked.connect(self.apply_denoising)

    def get_values(self):
        """Returns the filter parametres.

        """
        values = [self.lm_wdg.get_value(),
                  self.cl_wdg.get_value(),
                  self.template_wdg.get_value(),
                  self.sample_wdg.get_value()]

        if self.bw_rdo.isChecked():
            values[1] = False

        return values

    def set_progress_bar(self, b):
        """Run or stop the progress bar.

        Args:
        b -- Boolean
        """
        if b:
            self.progress.setRange(0, 0)
            self.progress.show()

        else:
            self.progress.setRange(0, 10)
            self.progress.hide()

        QCoreApplication.processEvents()

    def on_space_changed(self, state):
        """Called when the color radio button has been toggled.

        Args:
        state -- Boolean
        """
        self.cl_wdg.setEnabled(state)
        self.enable_buttons(True, False, True)

    def on_luminance_changed(self, value):
        """Called when the luminance value has been changed.

        Args:
        value -- int(slider value)
        """
        self.lm_val_lbl.setText(str(value))
        self.enable_buttons(True, False, True)

    def on_color_changed(self, value):
        """Called when the color luminance value has been changed.

        Args:
        value -- int(slider value)
        """
        self.cl_val_lbl.setText(str(value))
        self.enable_buttons(True, False, True)

    def on_template_changed(self, value):
        """Called when the template value has been changed.

        Args:
        value -- int(slider value)
        """
        self.template_val_lbl.setText(str(value))
        self.enable_buttons(True, False, True)

    def on_sample_changed(self, value):
        """Called when the sample value has been changed.

        Args:
        value -- int(slider value)
        """
        self.sample_val_lbl.setText(str(value))
        self.enable_buttons(True, False, True)

    def enable_buttons(self, prev, cancel, app):
        """Enable the buttons.

        Args:
        prev -- Boolean for the Preview button
        cancel -- Boolean for the Cancel button
        app -- Boolean for the Apply button 
        """
        self.preview_btn.setEnabled(prev)
        self.cancel_btn.setEnabled(cancel)
        self.apply_btn.setEnabled(app)

    def on_preview_clicked(self):
        """Call the preview window.

        """
        self.preview_btn.setEnabled(False)
        self.core.set_preview_window(self)

    def undo(self):
        """Cancel the filter.

        """
        self.core.filtering.undo_last_change()
        self.cancel_btn.setEnabled(False)

    def apply_denoising(self):
        """Apply the filter.

        """
        self.cancel_btn.setEnabled(True)
        self.apply_btn.setEnabled(False)
        self.set_progress_bar(True)
        self.core.state.disable_undo_buttons(self)
        self.core.close_preview_window()
        self.core.set_filtered_mode(True)
        QCoreApplication.processEvents()
        self.core.filtering.apply_denoising_filter(self.get_values(), self)

    def reset(self):
        self.lm_wdg.set_value(8, True)
        self.cl_wdg.set_value(8, True)
        self.template_wdg.set_value(5, True)
        self.sample_wdg.set_value(10, True)
        self.set_progress_bar(False)

    def translate(self):
        self.color_rdo.setText(_(u'Colour'))
        self.bw_rdo.setText(_(u'Black && White'))
        self.lm_lbl.setText(_('Luminance:'))
        self.lm_wdg.minus.setStatusTip(_('Luminance -1'))
        self.lm_wdg.plus.setStatusTip(_('Luminance +1'))
        self.cl_lbl.setText(_('Colour:'))
        self.cl_wdg.minus.setStatusTip(_('Luminance colour -1'))
        self.cl_wdg.plus.setStatusTip(_('Luminance colour +1'))
        self.template_lbl.setText(_('Template:'))
        self.template_wdg.minus.setStatusTip(_('Template -2'))
        self.template_wdg.plus.setStatusTip(_('Template +2'))
        self.sample_lbl.setText(_('Sample:'))
        self.sample_wdg.minus.setStatusTip(_('Sample -1'))
        self.sample_wdg.plus.setStatusTip(_('Sample +1'))
        self.preview_btn.setStatusTip(_('Show the preview'))
        self.cancel_btn.setText(_('Cancel'))
        self.apply_btn.setText(_('Apply'))
        self.apply_btn.setStatusTip(_('Apply the denoising'))
        self.cancel_btn.setStatusTip(_('Cancel the denoising'))

    def set_style(self):
        self.color_rdo.setStyleSheet(styles.RADIO)
        self.bw_rdo.setStyleSheet(styles.RADIO)
        self.lm_lbl.setStyleSheet(styles.TITLE)
        self.lm_val_lbl.setStyleSheet(styles.TITLE)
        self.lm_wdg.minus.setStyleSheet(styles.TBUTTON)
        self.lm_wdg.slider.setStyleSheet(styles.SLIDER)
        self.lm_wdg.plus.setStyleSheet(styles.TBUTTON)
        self.lm_wdg.reset.setStyleSheet(styles.TBUTTON)
        self.cl_lbl.setStyleSheet(styles.TITLE)
        self.cl_val_lbl.setStyleSheet(styles.TITLE)
        self.cl_wdg.minus.setStyleSheet(styles.TBUTTON)
        self.cl_wdg.slider.setStyleSheet(styles.SLIDER)
        self.cl_wdg.plus.setStyleSheet(styles.TBUTTON)
        self.cl_wdg.reset.setStyleSheet(styles.TBUTTON)
        self.template_lbl.setStyleSheet(styles.TITLE)
        self.template_val_lbl.setStyleSheet(styles.TITLE)
        self.template_wdg.minus.setStyleSheet(styles.TBUTTON)
        self.template_wdg.slider.setStyleSheet(styles.SLIDER)
        self.template_wdg.plus.setStyleSheet(styles.TBUTTON)
        self.template_wdg.reset.setStyleSheet(styles.TBUTTON)
        self.sample_lbl.setStyleSheet(styles.TITLE)
        self.sample_val_lbl.setStyleSheet(styles.TITLE)
        self.sample_wdg.minus.setStyleSheet(styles.TBUTTON)
        self.sample_wdg.slider.setStyleSheet(styles.SLIDER)
        self.sample_wdg.plus.setStyleSheet(styles.TBUTTON)
        self.sample_wdg.reset.setStyleSheet(styles.TBUTTON)
        self.progress.setStyleSheet(styles.PROGRESSBAR)
        self.preview_btn.setStyleSheet(styles.PUSHBUTTON)
        self.cancel_btn.setStyleSheet(styles.PUSHBUTTON)
        self.apply_btn.setStyleSheet(styles.PUSHBUTTON)

