# -*- coding: utf-8 -*-

# exportbox.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2013-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Export tool box

import os
import re
import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import Qt, QSize, QCoreApplication
from PyQt5.QtGui import QIcon, QPixmap
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QLabel, QSpacerItem, QLineEdit, QSizePolicy,
                             QToolButton, QCheckBox, QPushButton, QSlider,
                             QFileDialog, QProgressBar)

import stylesheets as styles

from VWidgets_PyQt5.vlineedit import VLineEdit
from commonwidgets.imageformatcombobox import ImageFormatComboBox
from tools.path import Path


class ExportBox(QWidget):
    def __init__(self, panel, parent=None):
        super().__init__(parent)
        self.name = 'ExportBox'
        self.panel = panel
        self.ui = panel.ui
        self.core = panel.core
        self.gridLayout_2 = QGridLayout(self)
        self.gridLayout_2.setContentsMargins(8, 2, 8, 2)
        self.gridLayout_2.setVerticalSpacing(6)
        self.verticalLayout = QVBoxLayout()
        self.orig_lbl = QLabel(self)
        self.orig_lbl.setAlignment(Qt.AlignHCenter|Qt.AlignVCenter)
        self.verticalLayout.addWidget(self.orig_lbl)
        self.gridLayout = QGridLayout()
        self.gridLayout.setHorizontalSpacing(8)
        self.name_lbl = QLabel(self)
        self.name_lbl.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.gridLayout.addWidget(self.name_lbl, 0, 0, 1, 1)
        self.name_led = QLineEdit(self)
        self.gridLayout.addWidget(self.name_led, 0, 1, 1, 1)
        self.format_lbl = QLabel(self)
        self.format_lbl.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.gridLayout.addWidget(self.format_lbl, 1, 0, 1, 1)
        self.horizontalLayout = QHBoxLayout()
        self.format_cmb = ImageFormatComboBox(self)
        self.format_cmb.setMinimumSize(QSize(100, 25))
        self.horizontalLayout.addWidget(self.format_cmb)
        space = QSpacerItem(18, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(space)
        self.gridLayout.addLayout(self.horizontalLayout, 1, 1, 1, 1)
        self.folder_lbl = QLabel(self)
        self.folder_lbl.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.gridLayout.addWidget(self.folder_lbl, 2, 0, 1, 1)
        self.horizontalLayout_2 = QHBoxLayout()
        self.folder_led = VLineEdit(self)
        self.folder_led.setHistoricButton(True)
        self.horizontalLayout_2.addWidget(self.folder_led)
        self.browse_btn = QToolButton(self)
        self.browse_btn.setText("...")
        self.browse_btn.setEnabled(False)
        self.horizontalLayout_2.addWidget(self.browse_btn)
        self.gridLayout.addLayout(self.horizontalLayout_2, 2, 1, 1, 1)
        self.warning_lbl = QLabel(self)
        self.gridLayout.addWidget(self.warning_lbl, 3, 1, 1, 1)
        self.comp_lbl = QLabel(self)
        self.comp_lbl.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.gridLayout.addWidget(self.comp_lbl, 4, 0, 1, 1)
        self.horizontalLayout_3 = QHBoxLayout()
        self.comp_sld = QSlider(self)
        self.comp_sld.setOrientation(Qt.Horizontal)
        self.horizontalLayout_3.addWidget(self.comp_sld)
        self.comp_val_lbl = QLabel(self)
        self.comp_val_lbl.setText("00")
        self.horizontalLayout_3.addWidget(self.comp_val_lbl)
        self.gridLayout.addLayout(self.horizontalLayout_3, 4, 1, 1, 1)
        space1 = QSpacerItem(40, 2, QSizePolicy.Preferred, QSizePolicy.Minimum)
        self.gridLayout.addItem(space1, 5, 0, 1, 1)
        self.comp_lzw_chb = QCheckBox(self)
        self.comp_lzw_chb.setChecked(True)
        self.gridLayout.addWidget(self.comp_lzw_chb, 4, 1, 1, 1)
        space2 = QSpacerItem(40, 20, QSizePolicy.Preferred, QSizePolicy.Minimum)
        self.gridLayout.addItem(space2, 6, 0, 1, 1)
        self.data_chb = QCheckBox(self)
        self.gridLayout.addWidget(self.data_chb, 6, 1, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.horizontalLayout_5 = QHBoxLayout()
        space3 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(space3)
        self.progress = QProgressBar(self)
        self.progress.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        self.progress.setMaximumSize(QSize(1000, 12))
        self.horizontalLayout_5.addWidget(self.progress)
        self.save_btn = QPushButton(self)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/apply.svg"), QIcon.Normal, QIcon.Off)
        self.save_btn.setIcon(icon)
        self.save_btn.setIconSize(QSize(18, 18))
        self.horizontalLayout_5.addWidget(self.save_btn)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.gridLayout_2.addLayout(self.verticalLayout, 0, 0, 1, 1)
        self.progress.hide()
        self.set_compression_widgets(None)
        self.retranslateUi()
        self.set_style()

        self.name_led.textChanged.connect(self.on_filename_edited)
        self.format_cmb.currentIndexChanged.connect(self.on_extension_changed)
        self.folder_led.textChanged.connect(self.on_folder_edited)
        self.browse_btn.clicked.connect(self.get_folder_name)
        self.comp_sld.valueChanged.connect(self.on_slider_moved)
        self.save_btn.clicked.connect(self.export_image)

    def set_recents(self):
        self.folder_led.add_historic_entries(self.core.recents.paths, True)

    def set_data(self, orig, qimage=None, bckp=None):
        """Sets the data into the widgets.

        Args:
        orig -- Picture instance of the original image
        qimage -- QImage instance
        bckp -- Backup instance
        """
        self.orig = orig
        self.backup = bckp
        if bckp is not None:
            name = bckp.name

        else:
            name = self.orig.name

        self.orig_lbl.setText(_('Export from %s' % name))
        self.qimage = qimage
        self.name_led.blockSignals(True)
        self.set_filename(self.orig.name)
        self.set_format()
        self.set_folder_name()
        self.browse_btn.setEnabled(True)
        name = self.check_file_name()
        if name is not None:
            self.set_filename(name)
        self.name_led.blockSignals(False)

    def clear(self):
        self.orig_lbl.setText('Export from --')
        self.name_led.clear()

    def set_filename(self, name):
        self.name_led.setText(name)

    def set_format(self):
        self.update_format(self.orig.ext)

    def update_filename(self, name):
        """Set the target filename, called by the quick save option.

        Args:
        name -- file name without extension
        """
        self.name_led.blockSignals(True)
        self.name_led.setText(name)
        self.name_led.blockSignals(False)

    def update_format(self, format):
        """Set the image format, called by the quick save option.

        Args:
        format -- the image format
        """
        self.format_cmb.blockSignals(True)
        self.format_cmb.set_current_item(format)
        self.format_cmb.blockSignals(False)
        
    def set_folder_name(self):
        if isinstance(self.orig.uri, str):
            self.folder = os.path.dirname(self.orig.uri)

        else:
            self.folder = self.orig.uri.parents
        self.folder_led.setText(self.folder)

    def set_compression_widgets(self, ext):
        """Sets the compression widgets depending to the file format.

        Args:
        ext -- the file extension in lower case
        """
        if ext in ['.tif', '.tiff']:
            j, t = False, True

        elif ext in ['.jpg', '.jpeg']:
            j, t = True, False

        else:
            j, t = False, False

        self.comp_lbl.setVisible(j)
        self.comp_sld.setVisible(j)
        self.comp_val_lbl.setVisible(j)
        self.comp_lzw_chb.setVisible(t)
        self.comp_sld.setEnabled(j)

    def set_backup_name(self, bckp):
        self.backup = bckp
        self.orig_lbl.setText(_('Export from %s' % bck.name))

    def set_qimage(self, qimage):
        self.qimage = qimage

    def check_file_name(self):
        fld = self.folder_led.text()
        fname = self.name_led.text()
        ext = self.format_cmb.currentText()
        if not os.path.isfile(os.path.join(fld, fname + ext)):
            return

        count = self.find_suffix(fname+ext, ext)
        if count:
            fname = '_('.join(fname.split('_(')[:-1])

        else:
            count = 1

        while 1:
            name = "%s_(%s)" % (fname, count)
            if not os.path.isfile(os.path.join(fld, name + ext)):
                return name

            count += 1

    def find_suffix(self, name, ext):
        suff = re.compile('(?<=_\()(\d+)(?=\)%s)' % ext)
        match = suff.search(name)
        if match is not None:
            return int(match.group(0))

        return 0

    #------ Slots ------------------------------------------

    def on_extension_changed(self, ext, update=True):
        """Invoked when the extension is changed.

        Args:
        ext -- the extension
        update -- True if the viewer header must be updated, False if called
                  from the viewer header himself
        """
        ext = self.format_cmb.currentText().lower()
        if ext in ['.jpg', '.jpeg', '.tif', '.tiff']:
            b = True

        else:
            b = False

        self.data_chb.setChecked(b)
        self.data_chb.setVisible(b)
        self.set_compression_widgets(ext)
        if update:
            self.ui.stackedviews.update_from_export_tool(self)

    def on_filename_edited(self, name):
        if not name:
            self.save_btn.setEnabled(False)

        else:
            self.save_btn.setEnabled(True)

        self.ui.stackedviews.update_from_export_tool(self)

    def on_folder_edited(self, path):
        """Invoked when the folder line edit is edited.

        Args:
        path -- the new path edited
        """
        if not os.path.isdir(path):
            color = '#FF0000'
            self.save_btn.setEnabled(False)
            self.warning_lbl.hide()

        else:
            if not os.access(path, os.W_OK):
                color = '#FF0000'
                self.save_btn.setEnabled(False)
                self.warning_lbl.setText(_('The folder is read only'))
                self.warning_lbl.show()

            else:
                color = '#000000'
                self.save_btn.setEnabled(True)
                self.warning_lbl.hide()

        self.folder_led.setStyleSheet("""QLineEdit {border: 2px solid #A0A5B2;
                                    border-radius: 6px;
                                    background: white;
                                    color: %s;
                                    selection-background-color: darkgray;}"""
                                    %(color))
        self.ui.stackedviews.update_from_export_tool(self)

    def on_slider_moved(self, value):
        self.comp_val_lbl.setText(str(value))

    def get_folder_name(self):
        """Call the file dialog box when the button 'Browse' is clicked.

        """
        try:
            folder = os.path.dirname(self.orig.uri)
        except:
            folder = os.path.expanduser("~")

        fld = QFileDialog.getExistingDirectory(None, "Open folder", folder,
                                            QFileDialog.DontResolveSymlinks
                                            | QFileDialog.ShowDirsOnly)
        if fld:
            self.folder_led.setText(fld)
            self.core.recents.add_path(Path(fld))
            lgg.info('Set new folder: {0}'.format(fld))

    def export_image(self):
        self.set_progress_bar(True)
        cfg = self.get_export_config()
        self.core.save_from_export(cfg)

    def get_export_config(self):
        quality, data = False, False
        format = self.format_cmb.currentText()
        if format.lower() in ('.jpg', '.jpeg'):
            # For the Qt image writer the quality is into the range 
            # 0 to 100, this is the opposite of the compression, so we 
            # reverse the compression value.
            quality = 100 - self.comp_sld.value()
            data = self.data_chb.isChecked()

        elif format.lower() in ('.tif', '.tiff'):
            quality = self.comp_lzw_chb.isChecked()
            data = self.data_chb.isChecked()

        return {'orig': self.orig,
                 'backup': self.backup,
                 'dest': self.folder_led.text(),
                 'name': self.name_led.text(),
                 'format': format,
                 'quality': quality,
                 'data': data,
                 'qimage': self.qimage}

    def set_progress_bar(self, b):
        if b:
            self.progress.setRange(0, 0)
            self.progress.show()

        else:
            self.progress.setRange(0, 10)
            self.progress.hide()

        QCoreApplication.processEvents()

    def reset(self):
        self.clear()

    def set_style(self):
        self.name_led.setStyleSheet(styles.LINEEDIT)
        self.format_cmb.setStyleSheet(styles.COMBO)
        self.folder_led.setStyleSheet(styles.LINEEDIT)
        self.folder_led.historic_button.setStyleSheet(styles.LINEEDITBUTTON)
        self.browse_btn.setStyleSheet(styles.TBUTTON)
        self.comp_lzw_chb.setStyleSheet(styles.CHECKBOX)
        self.comp_sld.setStyleSheet(styles.SLIDER)
        self.data_chb.setStyleSheet(styles.CHECKBOX)
        self.progress.setStyleSheet(styles.PROGRESSBAR)
        self.save_btn.setStyleSheet(styles.PUSHBUTTON)

    def retranslateUi(self):
        self.name_lbl.setText(_("Name:"))
        self.format_lbl.setText(_("Format:"))
        self.folder_lbl.setText(_("Folder:"))
        self.browse_btn.setStatusTip(_("Browse"))
        self.comp_lbl.setText(_("Compression:"))
        self.comp_lzw_chb.setText(_("Compression LZW"))
        self.data_chb.setText(_("Copy the metadata"))
        self.save_btn.setText(_("Save"))

