# imageinfos.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2013-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Image infos tool box

from PyQt5.QtCore import Qt, QLocale
from PyQt5.QtGui import QFont
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QSpacerItem, QLabel, QSizePolicy)

LOC = QLocale.system().name()
LANG = QLocale(LOC)

class ImageInfos(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.name = 'ImageInfos'
        self.gridLayout = QGridLayout(self)
        self.gridLayout.setContentsMargins(2, 2, 2, 2)
        self.gridLayout.setVerticalSpacing(2)
        self.verticalLayout_3 = QVBoxLayout()
        self.verticalLayout_3.setSpacing(2)
        font = QFont()
        font.setFamily("Ubuntu Mono")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.horizontalLayout_2 = QHBoxLayout()
        spacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacer)
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setSpacing(2)
        self.date_lbl = QLabel(self)
        self.date_lbl.setAlignment(Qt.AlignRight|Qt.AlignTrailing|
                                    Qt.AlignVCenter)
        self.date_lbl.setIndent(5)
        self.verticalLayout.addWidget(self.date_lbl)
        self.width_lbl = QLabel(self)
        self.width_lbl.setAlignment(Qt.AlignRight|Qt.AlignTrailing|
                                    Qt.AlignVCenter)
        self.width_lbl.setIndent(5)
        self.verticalLayout.addWidget(self.width_lbl)
        self.height_lbl = QLabel(self)
        self.height_lbl.setAlignment(Qt.AlignRight|Qt.AlignTrailing|
                                    Qt.AlignVCenter)
        self.height_lbl.setIndent(5)
        self.verticalLayout.addWidget(self.height_lbl)
        self.fsize_lbl = QLabel(self)
        self.fsize_lbl.setAlignment(Qt.AlignRight|Qt.AlignTrailing|
                                    Qt.AlignVCenter)
        self.fsize_lbl.setIndent(5)
        self.verticalLayout.addWidget(self.fsize_lbl)
        self.horizontalLayout_2.addLayout(self.verticalLayout)
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setSpacing(2)
        self.date_val_lbl = QLabel(self)
        font = QFont()
        font.setBold(True)
        font.setWeight(75)
        self.date_val_lbl.setFont(font)
        self.date_val_lbl.setAlignment(Qt.AlignRight|Qt.AlignTrailing|
                                        Qt.AlignVCenter)
        self.verticalLayout_2.addWidget(self.date_val_lbl)
        self.width_val_lbl = QLabel(self)
        self.width_val_lbl.setFont(font)
        self.width_val_lbl.setAlignment(Qt.AlignRight|Qt.AlignTrailing|
                                        Qt.AlignVCenter)
        self.verticalLayout_2.addWidget(self.width_val_lbl)
        self.height_val_lbl = QLabel(self)
        self.height_val_lbl.setFont(font)
        self.height_val_lbl.setAlignment(Qt.AlignRight|Qt.AlignTrailing|
                                        Qt.AlignVCenter)
        self.verticalLayout_2.addWidget(self.height_val_lbl)
        self.fsize_val_lbl = QLabel(self)
        self.fsize_val_lbl.setFont(font)
        self.fsize_val_lbl.setAlignment(Qt.AlignRight|Qt.AlignTrailing|
                                        Qt.AlignVCenter)
        self.verticalLayout_2.addWidget(self.fsize_val_lbl)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)
        spac3 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spac3)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.gridLayout.addLayout(self.verticalLayout_3, 0, 0, 1, 1)
        self.date_lbl.hide()
        self.date_val_lbl.hide()

        self.retranslateUi()
        self.set_style()

    def set_image_data(self, img):
        """Update the image infos.

        args:
        img -- the Picture instance
        """
        width = "  %s pxl" % LANG.toString(img.width)
        self.width_val_lbl.setText(width)
        height = "  %s pxl" % LANG.toString(img.height)
        self.height_val_lbl.setText(height)
        weight = "  %s bytes" % LANG.toString(img.weight)
        self.fsize_val_lbl.setText(weight)
        date = img.date
        if date is not None:
            #date = date.strftime("%Y-%m-%d %H:%M:%S")
            self.date_lbl.show()
            self.date_val_lbl.show()
            self.date_val_lbl.setText(date)

        else:
            self.date_lbl.hide()
            self.date_val_lbl.hide()

        self.parent().setTitle(img.base)

    def clear(self):
        self.date_val_lbl.setText("    --")
        self.width_val_lbl.setText("    --")
        self.height_val_lbl.setText("    --")
        self.fsize_val_lbl.setText("    --")       

    def retranslateUi(self):
        self.date_lbl.setText(_("Date:"))
        self.width_lbl.setText(_("Width:"))
        self.height_lbl.setText(_("Height:"))
        self.fsize_lbl.setText(_("File size:"))
        self.width_val_lbl.setText("    --")
        self.height_val_lbl.setText("    --")
        self.fsize_val_lbl.setText("    --")

    def set_raw_image_data(self, width, height, weight, date):
        width = "  %s pxl" % LANG.toString(width)
        self.width_val_lbl.setText(width)
        height = "  %s pxl" % LANG.toString(height)
        self.height_val_lbl.setText(height)
        weight = "  %s bytes" % LANG.toString(weight)
        self.fsize_val_lbl.setText(weight)
        if date is not None:
            self.date_lbl.show()
            self.date_val_lbl.show()
            self.date_val_lbl.setText(date)
        else:
            self.date_lbl.hide()
            self.date_val_lbl.hide()

    def reset(self):
        self.clear()

    def set_style(self):
        pass

