# -*- coding: utf-8 -*-

# metadatabox.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2013-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Metadata tool box

from PyQt5.QtCore import QSize
from PyQt5.QtGui import QIcon, QPixmap, QFont
from PyQt5.QtWidgets import (QWidget, QVBoxLayout, QHBoxLayout, QPushButton,
                             QTreeWidget, QFrame, QAbstractItemView,
                             QSpacerItem, QSizePolicy)

import stylesheets as styles

class MetadataBox(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.name = 'MetadataBox'
        self.v_layout = QVBoxLayout(self)
        self.v_layout.setContentsMargins(2, 0, 2, 0)
        self.v_layout.setSpacing(0)
        self.data_tree = DataTree(self)
        self.v_layout.addWidget(self.data_tree)
        self.h_layout = QHBoxLayout()
        self.h_layout.setContentsMargins(0, 0, 0, 0)
        self.h_layout.setSpacing(6)
        space = QSpacerItem(20, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.h_layout.addItem(space)
        self.apply_btn = QPushButton(self)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/apply.png"), 
                                QIcon.Normal, QIcon.Off)
        self.apply_btn.setIcon(icon)
        self.apply_btn.setIconSize(QSize(16, 16))

        self.h_layout.addWidget(self.apply_btn)
        self.expand_btn = QPushButton(self)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/gtk-edit.svg"), 
                                QIcon.Normal, QIcon.Off)
        self.expand_btn.setIcon(icon1)
        self.expand_btn.setIconSize(QSize(16, 16))
        self.h_layout.addWidget(self.expand_btn)
        space1 = QSpacerItem(6, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.h_layout.addItem(space1)
        self.v_layout.addLayout(self.h_layout)
        self.apply_btn.setText(_("Apply"))
        self.apply_btn.setStatusTip(_("Apply the new metadata."))
        self.expand_btn.setText(_("Expand") )
        self.expand_btn.setStatusTip(_("Expand the editor."))
        self.set_style()

    def set_style(self):
        self.data_tree.setStyleSheet(styles.DATATREE)
        self.data_tree.header().setStyleSheet(styles.DATATREEHEADER)
        self.data_tree.horizontalScrollBar().setStyleSheet(styles.HSCROLLBAR)
        self.data_tree.verticalScrollBar().setStyleSheet(styles.VSCROLLBAR)
        self.apply_btn.setStyleSheet(styles.PUSHBUTTON)
        self.expand_btn.setStyleSheet(styles.PUSHBUTTON)

    def clear(self):
        self.data_tree.clear()

    def reset(self):
        self.clear()

class DataTree(QTreeWidget):
    def __init__(self, parent=None):
        super(DataTree, self).__init__(parent)
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(9)
        self.setFont(font)
        self.setFrameShape(QFrame.NoFrame)
        self.setFrameShadow(QFrame.Plain)
        self.setIndentation(8)
        self.setUniformRowHeights(True)
        self.setColumnCount(2)
        self.headerItem().setText(0, _("Name"))
        self.headerItem().setText(1, _("Value"))
        self.setColumnWidth(0, 100)
        self.header().setStretchLastSection(True)
        self.setSelectionMode(QAbstractItemView.NoSelection)
    
