# -*- coding: utf-8 -*-

# rawinquiriesbox.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2015-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# RAW files spec's viewer

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QFont
from PyQt5.QtWidgets import (QGridLayout, QHBoxLayout, QSizePolicy, QWidget,
                             QSpacerItem, QLabel)

import stylesheets as styles


class RawInquiriesBox(QWidget):
    def __init__(self, ui, parent=None):
        super().__init__(parent)
        self.name = 'RawInquiriesBox'
        self.ui = ui
        alignment = Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter
        self.gridLayout_2 = QGridLayout(self)
        self.gridLayout_2.setContentsMargins(2, 2, 8, 2)
        self.horizontalLayout = QHBoxLayout()
        spacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacer)
        self.gridLayout = QGridLayout()
        self.gridLayout.setVerticalSpacing(2)
        self.name_lbl = QLabel(self)
        self.name_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.name_lbl, 0, 0, 1, 1)
        font = QFont()
        font.setFamily("Ubuntu Mono")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.name_val_lbl = QLabel(self)
        self.name_val_lbl.setFont(font)
        self.name_val_lbl.setText("    --")
        self.gridLayout.addWidget(self.name_val_lbl, 0, 1, 1, 1)
        self.rwidth_lbl = QLabel(self)
        self.rwidth_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.rwidth_lbl, 1, 0, 1, 1)
        self.rwidth_val_lbl = QLabel(self)
        self.rwidth_val_lbl.setFont(font)
        self.rwidth_val_lbl.setText("    --")
        self.gridLayout.addWidget(self.rwidth_val_lbl, 1, 1, 1, 1)
        self.rheight_lbl = QLabel(self)
        self.rheight_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.rheight_lbl, 2, 0, 1, 1)
        self.rheight_val_lbl = QLabel(self)
        self.rheight_val_lbl.setFont(font)
        self.rheight_val_lbl.setText("    --")
        self.gridLayout.addWidget(self.rheight_val_lbl, 2, 1, 1, 1)
        self.width_lbl = QLabel(self)
        self.width_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.width_lbl, 3, 0, 1, 1)
        self.width_val_lbl = QLabel(self)
        self.width_val_lbl.setFont(font)
        self.width_val_lbl.setText("    --")
        self.gridLayout.addWidget(self.width_val_lbl, 3, 1, 1, 1)
        self.height_lbl = QLabel(self)
        self.height_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.height_lbl, 4, 0, 1, 1)
        self.height_val_lbl = QLabel(self)
        self.height_val_lbl.setFont(font)
        self.height_val_lbl.setText("    --")
        self.gridLayout.addWidget(self.height_val_lbl, 4, 1, 1, 1)
        self.iso_lbl = QLabel(self)
        self.iso_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.iso_lbl, 5, 0, 1, 1)
        self.iso_val_lbl = QLabel(self)
        self.iso_val_lbl.setFont(font)
        self.iso_val_lbl.setText("    --")
        self.gridLayout.addWidget(self.iso_val_lbl, 5, 1, 1, 1)
        self.fnumber_lbl = QLabel(self)
        self.fnumber_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.fnumber_lbl, 6, 0, 1, 1)
        self.fnumber_val_lbl = QLabel(self)
        self.fnumber_val_lbl.setFont(font)
        self.fnumber_val_lbl.setText("    --")
        self.gridLayout.addWidget(self.fnumber_val_lbl, 6, 1, 1, 1)
        self.shutter_lbl = QLabel(self)
        self.shutter_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.shutter_lbl, 7, 0, 1, 1)
        self.shutter_val_lbl = QLabel(self)
        self.shutter_val_lbl.setFont(font)
        self.shutter_val_lbl.setText("    --")
        self.gridLayout.addWidget(self.shutter_val_lbl, 7, 1, 1, 1)
        self.count_lbl = QLabel(self)
        self.count_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.count_lbl, 8, 0, 1, 1)
        self.count_val_lbl = QLabel(self)
        self.count_val_lbl.setFont(font)
        self.count_val_lbl.setText("    --")
        self.gridLayout.addWidget(self.count_val_lbl, 8, 1, 1, 1)
        self.horizontalLayout.addLayout(self.gridLayout)
        self.gridLayout_2.addLayout(self.horizontalLayout, 0, 0, 1, 1)

        self.retranslate()
        self.set_style()

    def clear(self):
        self.name_val_lbl.setText("    --")
        self.rwidth_val_lbl.setText("    --")
        self.rheight_val_lbl.setText("    --")
        self.width_val_lbl.setText("    --")
        self.height_val_lbl.setText("    --")
        self.iso_val_lbl.setText("    --")
        self.fnumber_val_lbl.setText("    --")
        self.shutter_val_lbl.setText("    --")
        self.count_val_lbl.setText("    --")

    def set_values(self, fname, values):
        if len(fname) > 22:
            self.name_val_lbl.setToolTip(fname)
            fname = "...%s" % fname[-18:]

        self.name_val_lbl.setText(fname)
        self.rwidth_val_lbl.setText(str(values['raw_size'][0]))
        self.rheight_val_lbl.setText(str(values['raw_size'][1]))
        self.width_val_lbl.setText(str(values['out_size'][0]))
        self.height_val_lbl.setText(str(values['out_size'][1]))
        self.iso_val_lbl.setText(str(values['ISO']))
        self.fnumber_val_lbl.setText(str(values['aperture']))
        self.shutter_val_lbl.setText(str(values['shutter']))
        self.count_val_lbl.setText(str(values['n_images']))

    def reset(self):
        self.clear()

    def retranslate(self):
        self.name_lbl.setText(_("Name:"))
        self.rwidth_lbl.setText(_("Raw width:"))
        self.rheight_lbl.setText(_("Raw height:"))
        self.width_lbl.setText(_("Width:"))
        self.height_lbl.setText(_("Height:"))
        self.iso_lbl.setText(_("ISO:"))
        self.fnumber_lbl.setText(_("Aperture:"))
        self.shutter_lbl.setText(_("Shutter speed:"))
        self.count_lbl.setText(_("Images:"))

    def set_style(self):
        pass


