# -*- coding: utf-8 -*-

# recentsbox.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2014-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Recent folders tool box

from PyQt5.QtCore import  QSize
from PyQt5.QtGui import QIcon, QPixmap
from PyQt5.QtWidgets import (QWidget, QFormLayout, QHBoxLayout, QSizePolicy,
                             QLabel, QSpacerItem, QSpinBox, QPushButton)

import stylesheets as styles

from recentscleaner import RecentsCleaner

class RecentsBox(QWidget):
    def __init__(self,  ui, parent=None):
        super().__init__(parent)
        self.name = 'RecentsBox'
        self.ui = ui
        self.core = ui.core
        self.formLayout = QFormLayout(self)
        self.formLayout.setContentsMargins(0, 0, 8, 0)
        self.horizontalLayout = QHBoxLayout()
        spacerItem = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.label = QLabel(self)
        self.horizontalLayout.addWidget(self.label)
        self.number_spb = QSpinBox(self)
        self.number_spb.setProperty("value", 20)
        self.horizontalLayout.addWidget(self.number_spb)
        self.clean_btn = QPushButton(self)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/clean_24x27.png"), QIcon.Normal, QIcon.Off)
        self.clean_btn.setIcon(icon)
        self.clean_btn.setIconSize(QSize(18, 18))
        self.horizontalLayout.addWidget(self.clean_btn)
        self.formLayout.setLayout(0, QFormLayout.LabelRole, self.horizontalLayout)

        self.number_spb.valueChanged.connect(self.on_number_changed)
        self.clean_btn.clicked.connect(self.clean_recents_list)

        self.retranslateUi()
        self.set_style()

    def set_up(self):
        self.number_spb.blockSignals(True)
        self.number_spb.setValue(self.core.cfg[2000])
        self.number_spb.blockSignals(False)

    def on_number_changed(self, value):
        self.core.cfg[2000] = value

    def clean_recents_list(self):
        cleaner = RecentsCleaner(self.core)
        cleaner.exec_()

    def reset(self):
        pass

    def retranslateUi(self):
        self.label.setText(_("Number:"))
        self.clean_btn.setStatusTip(_("Edit the recent folder list"))
        self.clean_btn.setText(_("Clean"))

    def set_style(self):
        self.number_spb.setStyleSheet(styles.SPINBOX)
        self.clean_btn.setStyleSheet(styles.PUSHBUTTON)

