# -*- coding: utf-8 -*-

# resizingbox.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2013-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Resizing tool box.

import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import Qt, pyqtSignal, QSize
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QSpacerItem, QSizePolicy, QRadioButton, QCheckBox,
                             QLabel, QSpinBox, QDoubleSpinBox, QPushButton)

import stylesheets as styles


class ResizingBox(QWidget):
    getFocus = pyqtSignal(QWidget)
    def __init__(self, ui, parent=None):
        super().__init__(parent)
        self.name = 'ResizingBox'
        self.ui = ui
        self.core = ui.core
        self.v_layout = QVBoxLayout(self)
        self.v_layout.setContentsMargins(0, 0, 8, 2)
        self.h_layout = QHBoxLayout()
        sp = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.h_layout.addItem(sp)
        self.unity_lbl = QLabel(self)
        self.h_layout.addWidget(self.unity_lbl)
        self.pixel_rdo = QRadioButton(self)
        self.pixel_rdo.setChecked(True)
        self.h_layout.addWidget(self.pixel_rdo)
        self.percent_rdo = QRadioButton(self)
        self.percent_rdo.setText("%")
        self.h_layout.addWidget(self.percent_rdo)
        sp1 = QSpacerItem(20, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.h_layout.addItem(sp1)
        self.v_layout.addLayout(self.h_layout)
        self.g_layout = QGridLayout()
        self.g_layout.setHorizontalSpacing(8)
        sp2 = QSpacerItem(40, 20, QSizePolicy.Preferred, QSizePolicy.Minimum)
        self.g_layout.addItem(sp2, 0, 0, 1, 1)
        self.width_lbl = QLabel(self)
        self.width_lbl.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.g_layout.addWidget(self.width_lbl, 0, 1, 1, 1)
        self.width_spb = QSpinBox(self)
        self.width_spb.setSuffix(" px")
        self.width_spb.setMinimum(10)
        self.width_spb.setMaximum(99999)
        self.width_spb.setSingleStep(10)
        self.g_layout.addWidget(self.width_spb, 0, 2, 1, 1)
        self.width_dsp = QDoubleSpinBox(self)
        self.width_dsp.setSuffix(' %')
        self.width_dsp.setMaximum(1000.0)
        self.g_layout.addWidget(self.width_dsp, 0, 3, 1, 1)
        sp3 = QSpacerItem(40, 20, QSizePolicy.Preferred, QSizePolicy.Minimum)
        self.g_layout.addItem(sp3, 1, 0, 1, 1)
        self.height_lbl = QLabel(self)
        self.height_lbl.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.g_layout.addWidget(self.height_lbl, 1, 1, 1, 1)
        self.height_spb = QSpinBox(self)
        self.height_spb.setSuffix(" px")
        self.height_spb.setMinimum(10)
        self.height_spb.setMaximum(99999)
        self.height_spb.setSingleStep(10)
        self.g_layout.addWidget(self.height_spb, 1, 2, 1, 1)
        self.height_dsp = QDoubleSpinBox(self)
        self.height_dsp.setSuffix(" %")
        self.height_dsp.setMaximum(1000.0)
        self.g_layout.addWidget(self.height_dsp, 1, 3, 1, 1)
        self.v_layout.addLayout(self.g_layout)
        self.h_layout1 = QHBoxLayout()
        sp3 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.h_layout1.addItem(sp3)
        self.proportionnal_chb = QCheckBox(self)
        self.proportionnal_chb.setChecked(True)
        self.h_layout1.addWidget(self.proportionnal_chb)
        sp4 = QSpacerItem(20, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.h_layout1.addItem(sp4)
        self.v_layout.addLayout(self.h_layout1)
        self.h_layout2 = QHBoxLayout()
        sp5 = QSpacerItem(40, 20, QSizePolicy.Preferred, QSizePolicy.Minimum)
        self.h_layout2.addItem(sp5)
        self.cancel_btn = QPushButton(self)
        self.cancel_btn.setEnabled(False)
        icon = QIcon('medias/edit-undo.png')
        self.cancel_btn.setIcon(icon)
        self.cancel_btn.setIconSize(QSize(18, 18))
        self.h_layout2.addWidget(self.cancel_btn)
        self.apply_btn = QPushButton(self)
        self.apply_btn.setEnabled(False)
        icon1 = QIcon('medias/apply.png')
        self.apply_btn.setIcon(icon1)
        self.apply_btn.setIconSize(QSize(18, 18))
        self.h_layout2.addWidget(self.apply_btn)
        self.v_layout.addLayout(self.h_layout2)
        self.width_dsp.hide()
        self.height_dsp.hide()
        self.img_width = 0
        self.img_height = 0
        # Proportionnal resizing
        self.is_linked = True
        # 0 > pixel, 1 > percent
        self.unity = 0
        # 0 > width, 1 > height
        self.last_edited = None

        self.retranslateUi()
        self.set_style()

        self.pixel_rdo.toggled.connect(self.on_unity_changed)
        self.proportionnal_chb.toggled.connect(self.on_proportionnal_changed)
        self.width_spb.valueChanged.connect(self.on_width_pixel_changed)
        self.height_spb.valueChanged.connect(self.on_height_pixel_changed)
        self.width_dsp.valueChanged.connect(self.on_width_pcent_changed)
        self.height_dsp.valueChanged.connect(self.on_height_pcent_changed)
        self.cancel_btn.clicked.connect(self.cancel_change)
        self.apply_btn.clicked.connect(self.check_sizes)

    def set_config(self, unit, prop):
        if unit:
            self.percent_rdo.setChecked(True)

        if not prop:
            self.proportionnal_chb.setChecked(False)

    def get_config(self):
        return self.unity, self.is_linked

    def setup(self, picture):
        self.img_width = picture.width
        self.img_height = picture.height
        self.reset()

    def change_size(self, width, height):
        self.img_width = width
        self.img_height = height
        self.reset()

    def reset(self):
        self.block_signals(True)
        self.width_spb.setValue(self.img_width)
        self.height_spb.setValue(self.img_height)
        self.width_dsp.setValue(100.0)
        self.height_dsp.setValue(100.0)
        self.last_edited = None
        self.block_signals(False)
        self.cancel_btn.setEnabled(False)
        self.apply_btn.setEnabled(False)
        self.old_size = (self.img_width, self.img_height)
        self.new_size = self.old_size

    def on_unity_changed(self, state):
        self.getFocus.emit(self)
        self.width_spb.setVisible(state)
        self.height_spb.setVisible(state)
        self.width_dsp.setVisible(not state)
        self.height_dsp.setVisible(not state)
        self.unity = not state

    def on_proportionnal_changed(self, state):
        self.is_linked = state

    def on_width_pixel_changed(self, value):
        self.getFocus.emit(self)
        self.last_edited = 0
        self.apply_btn.setEnabled(True)
        if self.is_linked:
            self._update_values(value)

        else:
            self.update_width_percent(value)

    def on_height_pixel_changed(self, value):
        self.getFocus.emit(self)
        self.last_edited = 1
        self.apply_btn.setEnabled(True)
        if self.is_linked:
            self._update_values(value)

        else:
            self.update_height_percent(value)

    def on_width_pcent_changed(self, value):
        self.getFocus.emit(self)
        self.last_edited = 0
        self.apply_btn.setEnabled(True)
        if self.is_linked:
            self._update_values(value)

        else:
            self.update_width_pixel(value)

    def on_height_pcent_changed(self, value):
        self.getFocus.emit(self)
        self.last_edited = 1
        self.apply_btn.setEnabled(True)
        if self.is_linked:
            self._update_values(value)

        else:
            self.update_height_pixel(value)

    def _update_values(self, val):
        self.block_signals(True)
        if isinstance(val, int):
            # val in pixel
            if not self.last_edited:
                # val is width 
                f = self.img_width / float(val)
                self.height_spb.setValue(int(self.img_height / f))
                p = 100 * (1 / f)

            else:
                # val is height
                f = self.img_height / float(val)
                self.width_spb.setValue(int(self.img_width / f))
                p = 100 * (1 / f)

            self.width_dsp.setValue(p)
            self.height_dsp.setValue(p)

        else:
            # val in percent
            if not self.last_edited:
                # val is width
                self.height_dsp.setValue(val)

            else:
                # val is height
                self.width_dsp.setValue(val)

            self.width_spb.setValue(int(self.img_width * (val / 100)))
            self.height_spb.setValue(int(self.img_height * (val / 100)))

        self.block_signals(False)

    def update_width_percent(self, val):
        self.block_signals(True)
        p = 100 * (1 / (self.img_width / float(val)))
        self.width_dsp.setValue(p)
        self.block_signals(False)

    def update_height_percent(self, val):
        self.block_signals(True)
        p = 100 * (1 / (self.img_height / float(val)))
        self.height_dsp.setValue(p)
        self.block_signals(False)

    def update_width_pixel(self, val):
        self.block_signals(True)
        p = self.img_width * (val / 100)
        self.width_spb.setValue(p)
        self.block_signals(False)

    def update_height_pixel(self, val):
        self.block_signals(True)
        p = self.img_height * (val / 100)
        self.height_spb.setValue(p)
        self.block_signals(False)

    def check_sizes(self):
        w, h = self.width_spb.value(), self.height_spb.value()
        self.old_size = self.new_size
        if self.img_width != w or self.img_height != h:
            self.resize_image(w, h)
            self.new_size = (w, h)

    def resize_image(self, width, height):
        lgg.info('Resizing, new size: {0} x {1}'.format(width, height))
        self.core.state.disable_undo_buttons(self)
        self.core.set_filtered_mode(True)
        self.core.filtering.resize_image(self, width, height, 
                                         self.proportionnal_chb.isChecked())
        self.apply_btn.setEnabled(False)

    def cancel_change(self):
        self.reset_old_values()
        self.cancel_btn.setEnabled(False)
        self.core.filtering.undo_last_change()

    def reset_old_values(self):
        for i, size in enumerate(self.old_size):
            self.last_edited = i
            self._update_values(size)

    def set_buttons_enabled(self, b):
        self.cancel_btn.setEnabled(b)
        self.apply_btn.setEnabled(b)

    def block_signals(self, b):
        self.proportionnal_chb.blockSignals(b)
        self.pixel_rdo.blockSignals(b)
        self.width_spb.blockSignals(b)
        self.height_spb.blockSignals(b)
        self.width_dsp.blockSignals(b)
        self.height_dsp.blockSignals(b)

    def unset_focus(self):
        self.reset()

    def clear(self):
        self.img_width = 0
        self.img_height = 0
        self.reset()

    def set_style(self):
        self.pixel_rdo.setStyleSheet(styles.RADIO)
        self.percent_rdo.setStyleSheet(styles.RADIO)
        self.proportionnal_chb.setStyleSheet(styles.CHECKBOX)
        self.width_spb.setStyleSheet(styles.SPINBOX)
        self.height_spb.setStyleSheet(styles.SPINBOX)
        self.width_dsp.setStyleSheet(styles.DSPINBOX)
        self.height_dsp.setStyleSheet(styles.DSPINBOX)
        self.cancel_btn.setStyleSheet(styles.PUSHBUTTON)
        self.apply_btn.setStyleSheet(styles.PUSHBUTTON)

    def retranslateUi(self):
        self.unity_lbl.setText(_("Unity:"))
        self.pixel_rdo.setText(_("Pixels"))
        self.proportionnal_chb.setText(_("Proportionnal"))
        self.width_lbl.setText(_("width:"))
        self.height_lbl.setText(_("Height:"))
        self.cancel_btn.setText(_("Cancel"))
        self.apply_btn.setText(_("Apply"))

