# -*- coding: utf-8 -*-

# straightenbox.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2014-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Straighten tool box

from PyQt5.QtCore import Qt, pyqtSignal, QSize
from PyQt5.QtGui import QIcon, QPixmap
from PyQt5.QtWidgets import (QWidget, QGridLayout, QHBoxLayout, QVBoxLayout,
                             QSpacerItem, QLabel, QToolButton, QPushButton,
                             QComboBox, QSizePolicy, QButtonGroup,
                             QDoubleSpinBox)

import stylesheets as styles


class StraightenBox(QWidget):
    getFocus = pyqtSignal(QWidget)
    def __init__(self, ui, parent=None):
        super().__init__(parent)
        self.name = 'StraightenBox'
        self.ui = ui
        self.core = ui.core
        self.gridLayout_2 = QGridLayout(self)
        self.gridLayout_2.setContentsMargins(0, 2, 8, 2)
        self.vlayout = QVBoxLayout()
        self.hlayout_4 = QHBoxLayout()
        spacer = QSpacerItem(20, 12, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.hlayout_4.addItem(spacer)
        self.gridLayout = QGridLayout()
        self.gridLayout.setVerticalSpacing(1)
        alignment = Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter
        self.angle_lbl = QLabel(self)
        self.angle_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.angle_lbl, 0, 0, 1, 1)
        self.angles_spb = QDoubleSpinBox(self)
        self.angles_spb.setRange(-45.0, 45.0)
        self.angles_spb.setSingleStep((0.5))
        self.angles_spb.setValue(0)
        self.angles_spb.setSuffix(u' °')
        self.gridLayout.addWidget(self.angles_spb, 0, 1, 1, 1)
        self.apply_angle_btn = QToolButton(self)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/apply.png"), QIcon.Normal, QIcon.Off)
        self.apply_angle_btn.setIcon(icon)
        self.apply_angle_btn.setIconSize(QSize(18, 18))
        self.apply_angle_btn.setEnabled(False)
        self.gridLayout.addWidget(self.apply_angle_btn, 0, 2, 1, 1)
        self.aligne_lbl = QLabel(self)
        self.aligne_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.aligne_lbl, 1, 0, 1, 1)
        self.align_cmb = QComboBox(self)
        self.align_cmb.addItems([_('Vertical'), _('Horizontal')])
        self.gridLayout.addWidget(self.align_cmb, 1, 1, 1, 1)
        self.pick_btn = QToolButton(self)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/cursor.png"), QIcon.Normal, QIcon.Off)
        self.pick_btn.setIcon(icon1)
        self.pick_btn.setIconSize(QSize(16, 16))
        self.gridLayout.addWidget(self.pick_btn, 1, 2, 1, 1)
        self.flip_lbl = QLabel(self)
        self.flip_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.flip_lbl, 2, 0, 1, 1)
        self.flip_cmb = QComboBox(self)
        self.flip_cmb.addItems([_('Vertical'), _('Horizontal')])
        self.gridLayout.addWidget(self.flip_cmb, 2, 1, 1, 1)
        self.apply_flip_btn = QToolButton(self)
        self.apply_flip_btn.setIcon(icon)
        self.apply_flip_btn.setIconSize(QSize(18, 18))
        self.gridLayout.addWidget(self.apply_flip_btn, 2, 2, 1, 1)
        self.hlayout_4.addLayout(self.gridLayout)
        self.vlayout.addLayout(self.hlayout_4)
        self.hlayout_5 = QHBoxLayout()
        spacer_1 = QSpacerItem(40, 12, QSizePolicy.Preferred, 
                                QSizePolicy.Minimum)
        self.hlayout_5.addItem(spacer_1)
        self.cancel_btn = QPushButton(self)
        icon2 = QIcon('medias/edit-undo.png')
        self.cancel_btn.setIcon(icon2)
        self.cancel_btn.setIconSize(QSize(18, 18))
        self.hlayout_5.addWidget(self.cancel_btn)
        self.crop_btn = QPushButton(self)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/crop_tool_white_24.png"), QIcon.Normal, 
                                QIcon.Off)
        self.crop_btn.setIcon(icon1)
        self.crop_btn.setIconSize(QSize(18, 18))
        self.hlayout_5.addWidget(self.crop_btn)
        self.apply_btn = QPushButton(self)
        self.apply_btn.setIcon(icon)
        self.apply_btn.setIconSize(QSize(18, 18))
        self.hlayout_5.addWidget(self.apply_btn)
        self.vlayout.addLayout(self.hlayout_5)
        self.gridLayout_2.addLayout(self.vlayout, 0, 0, 1, 1)

        self.retranslateUi()
        self.set_style()
        self.set_buttons_enabled(False)
        self.angles_spb.valueChanged.connect(self.on_angle_value_changed)
        #self.angles_spb.editingFinished.connect(self.apply_angle)
        self.apply_angle_btn.clicked.connect(self.apply_angle)
        self.pick_btn.clicked.connect(self.set_freehand_mode)
        self.apply_flip_btn.clicked.connect(self.apply_flip)
        self.cancel_btn.clicked.connect(self.cancel_rotation)
        self.crop_btn.clicked.connect(self.set_crop_tool)
        self.apply_btn.clicked.connect(self.apply_rotation)
        self.applied = False

    def set_buttons_enabled(self, b):
        self.cancel_btn.setEnabled(b)
        self.crop_btn.setEnabled(b)
        self.apply_btn.setEnabled(b)

    def on_angle_value_changed(self, value):
        self.apply_angle_btn.setEnabled(True)

    def apply_angle(self):
        self.getFocus.emit(self)
        self.core.set_filtered_mode(True)
        self.core.filtering.rotate_image(self.angles_spb.value())
        self.set_buttons_enabled(True)

    def set_freehand_mode(self):
        self.getFocus.emit(self)
        self.enable_tools(False)
        self.angles_spb.setEnabled(True)
        self.apply_angle_btn.setEnabled(True)
        self.core.state.disable_undo_buttons(self)
        self.core.set_straighten_mode(self, self.align_cmb.currentIndex())

    def apply_flip(self):
        self.getFocus.emit(self)
        self.core.set_filtered_mode(True)
        self.core.filtering.flip_image(self, self.flip_cmb.currentIndex())
        self.cancel_btn.setEnabled(True)
        self.applied = True

    def cancel_rotation(self):
        self.set_buttons_enabled(False)
        self.reset()
        self.enable_tools(True)
        if self.applied:
            self.core.filtering.undo_last_change()
            self.applied = False

        else:
            self.core.filtering.cancel_rotation()

    def set_crop_tool(self):
        self.core.imgchanges.crop_on_rotated_image()
        self.set_buttons_enabled(False)

    def apply_rotation(self):
        self.set_buttons_enabled(False)
        self.enable_tools(True)
        self.change_angle_value(0)
        self.applied = True
        self.core.state.disable_undo_buttons(self)
        self.core.filtering.apply_rotation()

    def unset_focus(self):
        self.core.imgchanges.unset_straighten_mode()
        self.angles_spb.setValue(0.0)
        self.apply_angle_btn.setEnabled(False)

    def change_angle_value(self, value):
        """Change the angle spinBox value, called after a straighten action.

        Args:
        value -- the value in degrees
        """
        self.angles_spb.blockSignals(True)
        self.angles_spb.setValue(float(value))
        self.angles_spb.blockSignals(False)

    def reset(self):
        self.angles_spb.setValue(0.0)

    def enable_tools(self, state):
        self.angles_spb.setEnabled(state)
        self.apply_angle_btn.setEnabled(state)
        self.align_cmb.setEnabled(state)
        self.pick_btn.setEnabled(state)
        self.flip_cmb.setEnabled(state)
        self.apply_flip_btn.setEnabled(state)

    def set_style(self):
        self.angles_spb.setStyleSheet(styles.DSPINBOX)
        self.apply_angle_btn.setStyleSheet(styles.TBUTTON)
        self.align_cmb.setStyleSheet(styles.COMBO)
        self.pick_btn.setStyleSheet(styles.TBUTTON)
        self.flip_cmb.setStyleSheet(styles.COMBO)
        self.apply_flip_btn.setStyleSheet(styles.TBUTTON)
        self.cancel_btn.setStyleSheet(styles.PUSHBUTTON)
        self.crop_btn.setStyleSheet(styles.PUSHBUTTON)
        self.apply_btn.setStyleSheet(styles.PUSHBUTTON)

    def retranslateUi(self):
        self.angle_lbl.setText(_("Angle:"))
        self.apply_angle_btn.setStatusTip(_("Rotate the image"))
        self.aligne_lbl.setText(_("Straighten:"))
        self.pick_btn.setStatusTip(_("Click to draw a line on the image"))
        self.flip_lbl.setText(_("Flip:"))
        self.apply_flip_btn.setStatusTip(_("Flip the image"))
        self.cancel_btn.setText(_("Cancel"))
        self.crop_btn.setText(_("Crop"))
        self.apply_btn.setText(_("Apply"))

