# -*- coding: utf-8 -*-

# targetfoldersbox.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2014-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Target folders tool box

import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import Qt, QSize
from PyQt5.QtGui import QIcon, QPixmap
from PyQt5.QtWidgets import (QWidget, QGridLayout, QHBoxLayout, QLabel,
                             QSpinBox, QSizePolicy, QPushButton, QToolButton,
                             QSpacerItem, QCheckBox)

import stylesheets as styles

from VWidgets_PyQt5.vlineedit import VLineEdit


class TargetFoldersBox(QWidget):
    def __init__(self, widget, parent=None):
        super().__init__(parent)
        self.name = 'TargetFoldersBox'
        self.ui = widget.ui
        self.core = widget.core
        self.gridLayout_2 = QGridLayout(self)
        self.gridLayout_2.setContentsMargins(0, 0, 8, 0)
        self.gridLayout_2.setVerticalSpacing(2)
        self.gridLayout_2.setHorizontalSpacing(6)
        self.gridLayout = QGridLayout()
        self.gridLayout.setHorizontalSpacing(8)
        sp = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout.addItem(sp, 0, 0, 1, 1)
        self.label = QLabel(self)
        self.label.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label, 0, 1, 1, 1)
        self.number_spb = QSpinBox(self)
        self.number_spb.setMinimum(6)
        self.number_spb.setMaximum(24)
        self.gridLayout.addWidget(self.number_spb, 0, 2, 1, 1)
        sp1 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout.addItem(sp1, 1, 0, 1, 1)
        self.label_2 = QLabel(self)
        self.label_2.setAlignment(Qt.AlignRight|Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label_2, 1, 1, 1, 1)
        self.target_spb = QSpinBox(self)
        self.target_spb.setMinimum(1)
        self.target_spb.setMaximum(24)
        self.gridLayout.addWidget(self.target_spb, 1, 2, 1, 1)
        self.gridLayout_2.addLayout(self.gridLayout, 0, 0, 1, 1)
        self.horizontalLayout = QHBoxLayout()
        sp2 = QSpacerItem(20, 20, QSizePolicy.Minimum, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(sp2)
        self.path_led = VLineEdit(self)
        self.path_led.setHistoricButton(True)
        self.horizontalLayout.addWidget(self.path_led)
        self.gridLayout_2.addLayout(self.horizontalLayout, 1, 0, 1, 1)
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setSpacing(4)
        sp3 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(sp3)
        self.browse_btn = QToolButton(self)
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.browse_btn.sizePolicy()
                                    .hasHeightForWidth())
        self.browse_btn.setSizePolicy(sizePolicy)
        self.browse_btn.setText("...")
        self.horizontalLayout_2.addWidget(self.browse_btn)
        self.open_btn = QPushButton(self)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/apply.png"), QIcon.Normal, QIcon.Off)
        self.open_btn.setIcon(icon)
        self.open_btn.setIconSize(QSize(18, 18))
        self.horizontalLayout_2.addWidget(self.open_btn)
        self.close_btn = QPushButton(self)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/close.png"), QIcon.Normal, QIcon.Off)
        self.close_btn.setIcon(icon1)
        self.close_btn.setIconSize(QSize(18, 18))
        self.horizontalLayout_2.addWidget(self.close_btn)
        self.gridLayout_2.addLayout(self.horizontalLayout_2, 2, 0, 1, 1)
        self.horizontalLayout_4 = QHBoxLayout()
        sp4 = QSpacerItem(17, 17, QSizePolicy.Minimum, QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(sp4)
        self.check_size_chk = QCheckBox(self)
        self.horizontalLayout_4.addWidget(self.check_size_chk)
        self.gridLayout_2.addLayout(self.horizontalLayout_4, 3, 0, 1, 1)
        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setSpacing(4)
        sp5 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(sp5)
        self.label_3 = QLabel(self)
        self.horizontalLayout_3.addWidget(self.label_3)
        self.limit_spb = QSpinBox(self)
        self.limit_spb.setMaximum(99999)
        self.limit_spb.setProperty("value", 750)
        self.horizontalLayout_3.addWidget(self.limit_spb)
        self.gridLayout_2.addLayout(self.horizontalLayout_3, 4, 0, 1, 1)

        self.retranslateUi()
        self.set_style()

    def set_config(self):
        self.number_spb.setValue(self.core.cfg.get(('sorting', 'number')))
        self.on_target_changed(1)
        self.number_spb.valueChanged.connect(self.change_number_targets)
        self.target_spb.valueChanged.connect(self.on_target_changed)
        self.check_size_chk.stateChanged.connect(self.on_check_size_changed)
        self.limit_spb.valueChanged.connect(self.on_check_size_changed)
        self.open_btn.clicked.connect(self.on_link_clicked)
        self.close_btn.clicked.connect(self.on_unlink_clicked)

    def change_number_targets(self, num):
        self.ui.targetarea.change_number_targets(num)

    def on_target_changed(self, target):
        fld = self.core.targets[target-1]
        self.path_led.setText('')
        if fld is not None:
            self.path_led.setText(fld.path.path)

    def set_limit(self, b, limit):
        self.check_size_chk.blockSignals(True)
        self.check_size_chk.setChecked(b)
        self.check_size_chk.blockSignals(False)
        self.limit_spb.setValue(limit)

    def on_check_size_changed(self, args):
        checked = self.check_size_chk.isChecked()
        size = self.limit_spb.value() * 1048576
        fld = self.core.targets[self.target_spb.value()-1]
        if fld.name is None:
            return

        fld.is_gauged = checked
        fld.button.is_gauged = checked
        if checked:
            fld.limit = size
            fld.button.max = size
            fld.button.reset_gauge()

        else:
            fld.limit = None

    def on_link_clicked(self):
        fld = self.target_spb.value() - 1
        fname = self.path_led.text()
        lgg.info('Link folder %s, %s' %(fld, fname))
        self.core.link_new_target(fld, fname)

    def on_unlink_clicked(self):
        idx = self.target_spb.value() - 1
        lgg.info('Unlink folder %s' % idx)
        self.core.unlink_target(idx)
        self.path_led.setText('')

    def reset(self):
        pass

    def retranslateUi(self):
        self.label.setText(_("Number:"))
        self.number_spb.setStatusTip(_("Number of targets"))
        self.label_2.setText(_("Target:"))
        self.path_led.setStatusTip(_("Enter the target path"))
        self.path_led.setProperty("informativeText_", _("Empty"))
        self.browse_btn.setStatusTip(_("Browse ..."))
        self.open_btn.setStatusTip(_("Link the target"))
        self.open_btn.setText(_("Link"))
        self.close_btn.setStatusTip(_("Unlink the target"))
        self.close_btn.setText(_("Unlink"))
        self.check_size_chk.setStatusTip(_("Control the maximum size of the folder"))
        self.check_size_chk.setText(_("Check the size of the folder"))
        self.label_3.setText(_("Size:"))
        self.limit_spb.setStatusTip(_("Maximum"))
        self.limit_spb.setSuffix(" Mo")

    def set_style(self):
        self.number_spb.setStyleSheet(styles.SPINBOX)
        self.target_spb.setStyleSheet(styles.SPINBOX)
        self.path_led.setStyleSheet(styles.LINEEDIT)
        self.path_led.historic_button.setStyleSheet(styles.LINEEDITBUTTON)
        self.path_led.informative_text.setStyleSheet(styles.LINEEDITPLACEHOLDER)
        self.browse_btn.setStyleSheet(styles.TBUTTON)
        self.open_btn.setStyleSheet(styles.PUSHBUTTON)
        self.close_btn.setStyleSheet(styles.PUSHBUTTON)
        self.check_size_chk.setStyleSheet(styles.CHECKBOX)
        self.limit_spb.setStyleSheet(styles.SPINBOX)


