# -*- coding: utf-8 -*-

# toningbox.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2014-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Toning colored image tool box

from PyQt5.QtCore import Qt, pyqtSignal, QSize, QCoreApplication
from PyQt5.QtGui import QIcon, QPixmap
from PyQt5.QtWidgets import (QWidget, QVBoxLayout, QHBoxLayout, QGridLayout,
                             QLabel, QSpacerItem, QSizePolicy, QCheckBox,
                             QPushButton, QSlider, QProgressBar)

import stylesheets as styles

class ToningBox(QWidget):
    getFocus = pyqtSignal(QWidget)
    def __init__(self, ui, parent=None):
        super().__init__ (parent)
        self.name = 'ToningBox'
        self.ui = ui
        self.core = ui.core
        self.v_l = QVBoxLayout(self)
        self.v_l.setContentsMargins(2, 0, 8, 2)
        self.v_l.setSpacing(2)
        self.g_l = QGridLayout()
        sp = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.g_l.addItem(sp, 0, 0, 1, 1)
        self.sepia_chb = QCheckBox(self)
        self.g_l.addWidget(self.sepia_chb, 0, 1, 1, 1)
        self.h_l = QHBoxLayout()
        self.ico_lbl = QLabel(self)
        self.ico_lbl.setPixmap(QPixmap("medias/contrast.png"))
        self.h_l.addWidget(self.ico_lbl)
        self.sepia_sld = QSlider(self)
        self.sepia_sld.setMaximum(4)
        self.sepia_sld.setPageStep(1)
        self.sepia_sld.setOrientation(Qt.Horizontal)
        self.h_l.addWidget(self.sepia_sld)
        self.ico_2_lbl = QLabel(self)
        self.ico_2_lbl.setPixmap(QPixmap("medias/contrast_2.png"))
        self.h_l.addWidget(self.ico_2_lbl)
        self.g_l.addLayout(self.h_l, 0, 2, 1, 1)
        sp2 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.g_l.addItem(sp2, 1, 0, 1, 1)
        self.blue_chb = QCheckBox(self)
        self.g_l.addWidget(self.blue_chb, 1, 1, 1, 1)
        self.h_l_1 = QHBoxLayout()
        self.ico_lbl = QLabel(self)
        self.ico_lbl.setPixmap(QPixmap("medias/contrast.png"))
        self.h_l_1.addWidget(self.ico_lbl)
        self.blue_sld = QSlider(self)
        self.blue_sld.setMaximum(4)
        self.blue_sld.setPageStep(1)
        self.blue_sld.setOrientation(Qt.Horizontal)
        self.h_l_1.addWidget(self.blue_sld)
        self.ico_2_lbl = QLabel(self)
        self.ico_2_lbl.setPixmap(QPixmap("medias/contrast_2.png"))
        self.h_l_1.addWidget(self.ico_2_lbl)
        self.g_l.addLayout(self.h_l_1, 1, 2, 1, 1)
        self.h_l_3 = QHBoxLayout()
        sp_4 = QSpacerItem(20, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.h_l_3.addItem(sp_4)
        self.progress = QProgressBar(self)
        self.progress.setMaximumSize(QSize(100, 12))
        self.h_l_3.addWidget(self.progress)
        self.cancel_btn = QPushButton(self)
        icon = QIcon('medias/edit-undo.png')
        self.cancel_btn.setIcon(icon)
        self.cancel_btn.setIconSize(QSize(18, 18))
        self.h_l_3.addWidget(self.cancel_btn)
        self.apply_btn = QPushButton(self)
        icon1 = QIcon('medias/apply.png')
        self.apply_btn.setIcon(icon1)
        self.apply_btn.setIconSize(QSize(18, 18))
        self.h_l_3.addWidget(self.apply_btn)
        self.v_l.addLayout(self.g_l)
        self.v_l.addLayout(self.h_l_3)

        self.cancel_btn.setEnabled(False)
        self.apply_btn.setEnabled(False)
        self.sepia_sld.setEnabled(False)
        self.blue_sld.setEnabled(False)

        self.sepia_chb.setText(_('Sepia'))
        self.sepia_chb.setStatusTip(_('Toning sepia'))
        self.blue_chb.setText(_('Blue'))
        self.blue_chb.setStatusTip(_('Toning blue'))
        self.cancel_btn.setText(_('Cancel'))
        self.cancel_btn.setStatusTip(_('Cancel the toning'))
        self.apply_btn.setText(_('Apply'))
        self.apply_btn.setStatusTip(_('Apply the toning'))
        self.set_style()
        self.progress.hide()

        self.sepia_chb.stateChanged.connect(self.on_sepia_toggled)
        self.blue_chb.stateChanged.connect(self.on_blue_toggled)
        self.sepia_sld.valueChanged.connect(self.on_sepia_value_changed)
        self.blue_sld.valueChanged.connect(self.on_blue_value_changed)
        self.cancel_btn.clicked.connect(self.on_cancel_clicked)
        self.apply_btn.clicked.connect(self.on_apply_clicked)

    def on_sepia_toggled(self, state):
        if state:
            self.getFocus.emit(self)
            self.uncheck_blue()
            self.sepia_sld.setEnabled(True)
            self.apply_btn.setEnabled(True)
            self.core.set_filter_previewing_mode(True)
            self.core.filtering.show_toning_preview('sepia', 0)

        else:
            self.core.filtering.cancel_toning_preview()
            self.core.set_filter_previewing_mode(False)
            self.sepia_sld.setEnabled(False)
            self.reset_slider(self.sepia_sld)
            self.apply_btn.setEnabled(False)

    def on_blue_toggled(self, state):
        if state:
            self.getFocus.emit(self)
            self.uncheck_sepia()
            self.blue_sld.setEnabled(True)
            self.apply_btn.setEnabled(True)
            self.core.set_filter_previewing_mode(True)
            self.core.filtering.show_toning_preview('blue', 0)

        else:
            self.core.filtering.cancel_toning_preview()
            self.blue_sld.setEnabled(False)
            self.reset_slider(self.blue_sld)
            self.apply_btn.setEnabled(False)

    def on_sepia_value_changed(self, val):
        self.core.filtering.show_toning_preview('sepia', val)

    def on_blue_value_changed(self, val):
        self.core.filtering.show_toning_preview('blue', val)

    def on_cancel_clicked(self):
        self.cancel_btn.setEnabled(False)
        self.core.filtering.undo_last_change()

    def on_apply_clicked(self):
        tone = 'blue'
        val = self.blue_sld.value()
        if self.sepia_chb.isChecked():
            tone = 'sepia'
            val = self.sepia_sld.value()

        self.set_progress_bar(True)
        self.core.state.disable_undo_buttons(self)
        self.cancel_btn.setEnabled(True)
        self.uncheck_all()
        self.core.set_filtered_mode(True)
        self.core.filtering.apply_toning(tone, val, self)

    def uncheck_sepia(self):
        self.sepia_chb.blockSignals(True)
        self.sepia_sld.blockSignals(True)
        self.sepia_chb.setChecked(False)
        self.sepia_sld.setValue(0)
        self.sepia_sld.setEnabled(False)
        self.sepia_chb.blockSignals(False)
        self.sepia_sld.blockSignals(False)

    def uncheck_blue(self):
        self.blue_chb.blockSignals(True)
        self.blue_sld.blockSignals(True)
        self.blue_chb.setChecked(False)
        self.blue_sld.setValue(0)
        self.blue_sld.setEnabled(False)
        self.blue_chb.blockSignals(False)
        self.blue_sld.blockSignals(False)

    def reset_slider(self, slider):
        slider.blockSignals(True)
        slider.setValue(0)
        slider.blockSignals(False)

    def uncheck_all(self):
        self.uncheck_sepia()
        self.uncheck_blue()
        self.apply_btn.setEnabled(False)

    def unset_focus(self):
        self.uncheck_all()

    def set_progress_bar(self, b):
        if b:
            self.progress.setRange(0, 0)
            self.progress.show()
            QCoreApplication.processEvents()

        else:
            self.progress.setRange(0, 10)
            self.progress.hide()

    def reset(self):
        self.uncheck_all()
        self.set_progress_bar(False)

    def set_style(self):
        self.sepia_chb.setStyleSheet(styles.CHECKBOX)
        self.sepia_sld.setStyleSheet(styles.SLIDER)
        self.blue_chb.setStyleSheet(styles.CHECKBOX)
        self.blue_sld.setStyleSheet(styles.SLIDER)
        self.progress.setStyleSheet(styles.PROGRESSBAR)
        self.cancel_btn.setStyleSheet(styles.PUSHBUTTON)
        self.apply_btn.setStyleSheet(styles.PUSHBUTTON)
