# -*- coding: utf-8 -*-

# trashbox.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2013-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Trash can tool box


from PyQt5.QtWidgets import (QWidget, QGridLayout, QHBoxLayout, QLabel,
                             QPushButton, QCheckBox, QSizePolicy,
                             QSpacerItem)

import stylesheets as styles


class TrashBox(QWidget):
    def __init__(self, ui, parent=None):
        super().__init__(parent)
        self.name = 'TrashBox'
        self.ui = ui
        self.core = ui.core
        self.glayout = QGridLayout(self)
        self.glayout.setContentsMargins(0, 0, 8, 0)
        self.glayout.setVerticalSpacing(2)
        self.hlayout = QHBoxLayout()
        spacer = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.hlayout.addItem(spacer)
        self.label = QLabel(self)
        self.hlayout.addWidget(self.label)
        self.path_lbl = QLabel(self)
        self.hlayout.addWidget(self.path_lbl)
        spacer1 = QSpacerItem(20, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout.addItem(spacer1)
        self.glayout.addLayout(self.hlayout, 0, 0, 1, 1)
        self.hlayout1 = QHBoxLayout()
        spacer2 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.hlayout1.addItem(spacer)
        self.show_btn = QPushButton(self)
        self.hlayout1.addWidget(self.show_btn)
        spacer3 = QSpacerItem(20, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout1.addItem(spacer3)
        self.glayout.addLayout(self.hlayout1, 1, 0, 1, 1)
        self.hlayout2 = QHBoxLayout()
        spacer4 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.hlayout2.addItem(spacer4)
        self.empty_chb = QCheckBox(self)
        self.hlayout2.addWidget(self.empty_chb)
        self.glayout.addLayout(self.hlayout2, 2, 0, 1, 1)

        self.retranslateUi()
        self.set_style()

    def configure(self):
        self.path_lbl.setText('<b>/.oqapy/trash</b>')
        self.empty_chb.setChecked(self.core.cfg.get(('startup', 'empty_trash')))
        self.show_btn.clicked.connect(self.display_trash_can)
        self.empty_chb.toggled.connect(self.change_config)

    def display_trash_can(self):
        self.core.display_trash_tab()

    def change_config(self, b):
        self.core.cfg.update(('startup', 'empty_trash'), b)

    def reset(self):
        # Don't remove this function
        pass

    def retranslateUi(self):
        self.label.setText(_("Folder: "))
        self.show_btn.setText(_('Show'))
        self.show_btn.setStatusTip(_("Show the content of the trash can"))
        self.empty_chb.setText(_("To empty at startup"))

    def set_style(self):
        self.path_lbl.setStyleSheet(styles.LINEEDIT)
        self.empty_chb.setStyleSheet(styles.CHECKBOX)
        self.show_btn.setStyleSheet(styles.PUSHBUTTON)


