# -*- coding: utf-8 -*-

# unrawbox.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2015-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# RAW files parameters tool box

import os

from PyQt5.QtCore import Qt, QSize, QCoreApplication
from PyQt5.QtGui import QIcon, QPixmap
from PyQt5.QtWidgets import (QGridLayout, QHBoxLayout, QVBoxLayout, QSizePolicy,
                             QWidget, QSpacerItem, QLabel, QComboBox,
                             QProgressBar, QPushButton, QDoubleSpinBox,
                             QCheckBox, QSpinBox, QToolButton, QFileDialog)

from VWidgets_PyQt5.vlineedit import VLineEdit
import stylesheets as styles


class UnrawBox(QWidget):
    def __init__(self, ui, parent=None):
        super().__init__(parent)
        self.name = 'UnrawBox'
        self.ui = ui
        self.raw_viewer = None
        self.darkframes = []
        self.glayout_3 = QGridLayout(self)
        self.glayout_3.setContentsMargins(20, 2, 8, 2)
        self.vlayout = QVBoxLayout()
        self.vlayout.setSpacing(2)
        alignment = Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter
        self.hlayout_3 = QHBoxLayout()
        spc3 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout_3.addItem(spc3)
        self.n_image_lbl = QLabel(self)
        self.n_image_lbl.setAlignment(alignment)
        self.hlayout_3.addWidget(self.n_image_lbl)
        self.n_image_cmb = QComboBox(self)
        self.n_image_cmb.setMinimumSize(55, 20)
        self.hlayout_3.addWidget(self.n_image_cmb)
        self.n_image_lbl.hide()
        self.n_image_cmb.hide()
        self.glayout_2 = QGridLayout()
        self.glayout_2.setVerticalSpacing(2)
        self.balance_lbl = QLabel(self)
        self.balance_lbl.setAlignment(alignment)
        self.glayout_2.addWidget(self.balance_lbl, 0, 0, 1, 1)
        self.white_balance_cmb = QComboBox(self)
        self.white_balance_cmb.setMinimumSize(QSize(145, 0))
        self.glayout_2.addWidget(self.white_balance_cmb, 0, 1, 1, 1)
        spc = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.glayout_2.addItem(spc, 1, 0, 1, 1)
        self.glayout = QGridLayout()
        self.red_lbl = QLabel(self)
        self.red_lbl.setEnabled(False)
        self.glayout.addWidget(self.red_lbl, 0, 0, 1, 1)
        self.red_dsp = QDoubleSpinBox(self)
        self.red_dsp.setEnabled(False)
        self.red_dsp.setMinimum(0.1)
        self.red_dsp.setMaximum(20.0)
        self.red_dsp.setProperty("value", 1.0)
        self.glayout.addWidget(self.red_dsp, 0, 1, 1, 1)
        self.blue_lbl = QLabel(self)
        self.blue_lbl.setEnabled(False)
        self.glayout.addWidget(self.blue_lbl, 1, 0, 1, 1)
        self.blue_dsp = QDoubleSpinBox(self)
        self.blue_dsp.setEnabled(False)
        self.blue_dsp.setMinimum(0.1)
        self.blue_dsp.setMaximum(20.0)
        self.blue_dsp.setProperty("value", 1.0)
        self.glayout.addWidget(self.blue_dsp, 1, 1, 1, 1)
        self.glayout_2.addLayout(self.glayout, 1, 1, 1, 1)
        self.interpolation_lbl = QLabel(self)
        self.interpolation_lbl.setAlignment(alignment)
        self.glayout_2.addWidget(self.interpolation_lbl, 2, 0, 1, 1)
        self.interpolation_cmb = QComboBox(self)
        self.interpolation_cmb.setMinimumSize(QSize(145, 0))
        self.glayout_2.addWidget(self.interpolation_cmb, 2, 1, 1, 1)
        self.color_space_lbl = QLabel(self)
        self.color_space_lbl.setAlignment(alignment)
        self.glayout_2.addWidget(self.color_space_lbl, 3, 0, 1, 1)
        self.color_space_cmb = QComboBox(self)
        self.color_space_cmb.setMinimumSize(QSize(145, 0))
        self.glayout_2.addWidget(self.color_space_cmb, 3, 1, 1, 1)
        self.gamma_lbl = QLabel(self)
        self.gamma_lbl.setAlignment(alignment)
        self.glayout_2.addWidget(self.gamma_lbl, 4, 0, 1, 1)
        self.gamma_chb = QCheckBox(self)
        self.glayout_2.addWidget(self.gamma_chb, 4, 1, 1, 1)
        spc1 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.glayout_2.addItem(spc1, 5, 0, 1, 1)
        self.gamma_cmb = QComboBox(self)
        self.gamma_cmb.setMinimumSize(QSize(145, 0))
        self.glayout_2.addWidget(self.gamma_cmb, 5, 1, 1, 1)
        self.vlayout.addLayout(self.glayout_2)
        self.cleanup_chb = QCheckBox(self)
        self.vlayout.addWidget(self.cleanup_chb)
        self.hlayout_4 = QHBoxLayout()
        spc2 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout_4.addItem(spc2)
        self.cleanup_lbl = QLabel(self)
        self.cleanup_lbl.setEnabled(False)
        self.cleanup_lbl.setAlignment(alignment)
        self.hlayout_4.addWidget(self.cleanup_lbl)
        self.cleanup_spb = QSpinBox(self)
        self.cleanup_spb.setEnabled(False)
        self.cleanup_spb.setMinimum(1)
        self.cleanup_spb.setMaximum(5)
        self.hlayout_4.addWidget(self.cleanup_spb)
        self.vlayout.addLayout(self.hlayout_4)
        self.dark_chb = QCheckBox(self)
        self.vlayout.addWidget(self.dark_chb)
        self.hlayout_2 = QHBoxLayout()
        self.import_dark_led = VLineEdit(self)
        self.import_dark_led.setHistoricButton(True)
        self.hlayout_2.addWidget(self.import_dark_led)
        self.import_browse_btn = QToolButton(self)
        self.import_browse_btn.setText("...")
        self.hlayout_2.addWidget(self.import_browse_btn)
        self.import_dark_led.hide()
        self.import_browse_btn.hide()
        self.vlayout.addLayout(self.hlayout_2)
        self.progress = QProgressBar(self)
        self.progress.setProperty("value", 0)
        self.progress.setTextVisible(False)
        self.progress.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        self.progress.setMaximumSize(QSize(1000, 12))
        self.progress.hide()
        self.vlayout.addWidget(self.progress)
        self.hlayout = QHBoxLayout()
        spc4 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.hlayout.addItem(spc4)
        self.cancel_btn = QPushButton(self)
        self.cancel_btn.setEnabled(False)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/edit-undo.png"), QIcon.Normal, QIcon.Off)
        self.cancel_btn.setIcon(icon)
        self.hlayout.addWidget(self.cancel_btn)
        self.apply_btn = QPushButton(self)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/apply.png"), QIcon.Normal, QIcon.Off)
        self.apply_btn.setIcon(icon1)
        self.hlayout.addWidget(self.apply_btn)
        self.vlayout.addLayout(self.hlayout)
        self.glayout_3.addLayout(self.vlayout, 0, 0, 1, 1)

        self.white_balance_cmb.addItems([u"Camera's settings", 
                                         u"Full image average", 
                                         u"Custom"])
        self.interpolation_cmb.addItems(["Bilinear", "VNG", "PPG", 
                                         "AHD (default)",
                                         "RGB as four colors"])
        tips = ("High-speed, low-quality bilinear interpolation.",
                "Use Variable Number of Gradients interpolation.",
                "Use Patterned Pixel Grouping interpolation.",
                "Use Adaptive Homogeneity-Directed interpolation.",
                "Use this if the output shows false 2x2 meshes\n"\
                "with VNG or mazes with AHD.")
        for idx, tip in enumerate(tips):
            self.interpolation_cmb.setItemData(idx, tip, Qt.ToolTipRole)
        self.interpolation_cmb.setCurrentIndex(3)
        self.space_colors = ["Raw color", "sRGB D65 (default)",
                            "Adobe RGB (1998) D65",
                            "Wide Gamut RGB D65",
                            "Kodak ProPhoto RGB D65", "XYZ"]
        self.color_space_cmb.addItems(self.space_colors)
        self.color_space_cmb.setCurrentIndex(1)
        self.gamma_cmb.addItems(["BT.709 gamma (default)",
                                "2.2 gamma (Adobe RGB)",
                                "1.8 gamma (ProPhoto RGB)",
                                "sRGB gamma"])

        self.white_balance_cmb.currentIndexChanged.connect(
                                    self.on_white_balance_changed)
        self.gamma_chb.stateChanged.connect(self.on_gamma_checked)
        self.cleanup_chb.stateChanged.connect(self.on_cleanup_checked)
        self.dark_chb.stateChanged.connect(self.on_dark_checked)
        self.import_dark_led.textChanged.connect(self.on_dark_edited)
        self.import_browse_btn.clicked.connect(self.browse_to_pgm_file)
        self.cancel_btn.clicked.connect(self.on_cancel_clicked)
        self.apply_btn.clicked.connect(self.on_apply_clicked)

        self.retranslate()
        self.set_style()
        self.darkframe_ext = None

    def add_darkframe(self, darks, clear=False):
        if clear:
            self.import_dark_led.clear_historic()

        if isinstance(darks, (tuple, list)):
            self.import_dark_led.add_historic_entries(darks, clear)

        else:
            self.import_dark_led.add_historic_entry(darks)

    def remove_darkframe(self, dark):
        darks = self.import_dark_led.get_historic_entries()
        if dark in darks:
            darks.remove(dark)
            self.add_darkframe(darks, True)

    def on_white_balance_changed(self, idx):
        self.red_lbl.setEnabled(idx == 2)
        self.blue_lbl.setEnabled(idx == 2)
        self.red_dsp.setEnabled(idx == 2)
        self.blue_dsp.setEnabled(idx == 2)

    def on_gamma_checked(self, state):
        self.gamma_cmb.setEnabled(not state)

    def on_cleanup_checked(self, state):
        self.cleanup_lbl.setEnabled(state)
        self.cleanup_spb.setValue(1)
        self.cleanup_spb.setEnabled(state)

    def on_dark_checked(self, state):
        self.import_dark_led.setVisible(state)
        self.import_browse_btn.setVisible(state)
        self.apply_btn.setEnabled(not state)

    def on_dark_edited(self, path):
        self.apply_btn.setEnabled(os.path.isfile(path))

    def on_apply_clicked(self):
        self.set_progress_bar(True)
        self.raw_viewer.apply_parameters()

    def on_cancel_clicked(self):
        self.raw_viewer.cancel_process()

    def browse_to_pgm_file(self):
        self.import_dark_led.setText(self.browse_to('Import PGM File', 
                                  'RAW Files *.*'))

    def browse_to(self, title, files):
        fld = self.ui.main.folders[0].where
        fname = str(QFileDialog.getOpenFileName(None, title, fld, files)[0])
        return fname

    def reset_to_default(self):
        self.white_balance_cmb.setCurrentIndex(0)
        self.red_dsp.setValue(1.0)
        self.blue_dsp.setValue(1.0)
        self.interpolation_cmb.setCurrentIndex(3)
        self.color_space_cmb.setCurrentIndex(1)
        self.gamma_chb.setChecked(False)
        self.gamma_cmb.setCurrentIndex(0)
        self.cleanup_chb.setChecked(False)
        self.cleanup_spb.setValue(1)
        self.dark_chb.setChecked(False)
        self.import_dark_led.clear()
        self.n_image_cmb.setCurrentIndex(0)

    def set_values(self, values):
        self.reset_to_default()
        try:
            idx = ('default', 'average').index(values['whiteBalance'])
        except ValueError:
            red, blue = [float(x) for x in values['whiteBalance'].split(' ')]
            idx = 2
            self.red_dsp.setValue(red)
            self.blue_dsp.setValue(blue)
        self.white_balance_cmb.setCurrentIndex(idx)

        mt = values['interpolation']    
        if mt != 'AHD':
            idx = self.interpolation_cmb.findText(mt)
            self.interpolation_cmb.setCurrentIndex(idx)

        cs = values['colorSpace']
        if cs != 'sRGB D65':
            idx = self.color_space_cmb.findText(cs)
            self.color_space_cmb.setCurrentIndex(idx)

        if values['gamma'] == 'linear 16-bits':
            gamma = True
            self.gamma_chb.setChecked(True)

        else:
            gamma = self.gamma_cmb.findText(values['gamma'])
            if gamma == -1:
                gamma = 0
            self.gamma_cmb.setCurrentIndex(gamma)

        nmb = int(values['cleanupPasses'])
        self.cleanup_chb.setChecked(nmb)
        self.cleanup_spb.setValue(nmb)
        if 'substractDarkFrame' in values:
            self.dark_chb.setChecked(True)
            self.import_dark_led.setText(values['substractDarkFrame'])

        self.n_image_cmb.setCurrentIndex(values['imageIndex'])

    def set_progress_bar(self, b):
        max_ = 100
        if b:
            max_ = 0

        self.progress.setRange(0, max_)
        self.progress.setVisible(b)
        self.apply_btn.setEnabled(not b)
        self.cancel_btn.setEnabled(b)
        QCoreApplication.processEvents()

    def set_image_number(self, number):
        if number == 1:
            self.n_image_lbl.hide()
            self.n_image_cmb.hide()
            self.vlayout.removeItem(self.hlayout_3)

        else:
            self.n_image_cmb.clear()
            self.vlayout.insertLayout(0, self.hlayout_3)
            self.n_image_lbl.show()
            self.n_image_cmb.show()
            numbs = [str(i+1) for i in range(number)]
            self.n_image_cmb.addItems(numbs)
            self.n_image_cmb.setCurrentIndex(0)

    def add_dark_path(self, path, clear=False):
        if isinstance(path, list):
            self.import_dark_led.add_historic_entries(path, clear)

        else:
            self.import_dark_led.add_historic_entry(path)

    def reset(self):
        self.reset_to_default()

    def retranslate(self):
        self.balance_lbl.setText(_("White balance:"))
        self.white_balance_cmb.setStatusTip(_("Settings of white balance"))
        self.red_lbl.setText(_("Red:"))
        self.blue_lbl.setText(_("Blue:"))
        self.interpolation_lbl.setText(_("Interpolation:"))
        self.interpolation_cmb.setStatusTip(_("Interpolation method"))
        self.color_space_lbl.setText(_("Color:"))
        self.color_space_cmb.setStatusTip(_("Color space"))
        self.gamma_lbl.setText(_("Gamma curve:"))
        self.gamma_chb.setText(_("Linear 16-bit"))
        self.gamma_cmb.setStatusTip(_("Gamma curve"))
        self.cleanup_chb.setText(_("Cleanup color artefacts"))
        self.cleanup_lbl.setText(_("Number of passes:"))
        self.n_image_lbl.setText(_("Image n°:"))
        self.dark_chb.setText(_("Substract a dark frame"))
        self.import_dark_led.setInformativeText(_("Raw darkframe file name"))
        self.cancel_btn.setText(_("Cancel"))
        self.apply_btn.setText(_("Apply"))

    def set_style(self):
        self.white_balance_cmb.setStyleSheet(styles.COMBO)
        self.red_dsp.setStyleSheet(styles.DSPINBOX)
        self.blue_dsp.setStyleSheet(styles.DSPINBOX)
        self.interpolation_cmb.setStyleSheet(styles.COMBO)
        self.color_space_cmb.setStyleSheet(styles.COMBO)
        self.gamma_chb.setStyleSheet(styles.CHECKBOX)
        self.gamma_cmb.setStyleSheet(styles.COMBO)
        self.cleanup_chb.setStyleSheet(styles.CHECKBOX)
        self.cleanup_spb.setStyleSheet(styles.SPINBOX)
        self.n_image_cmb.setStyleSheet(styles.COMBO)
        self.dark_chb.setStyleSheet(styles.CHECKBOX)
        self.import_dark_led.setStyleSheet(styles.LINEEDIT)
        self.import_dark_led.historic_button.setStyleSheet(
                                            styles.LINEEDITBUTTON)
        self.import_dark_led.informative_text.setStyleSheet(
                                            styles.LINEEDITPLACEHOLDER)
        self.import_browse_btn.setStyleSheet(styles.TBUTTON)
        self.progress.setStyleSheet(styles.PROGRESSBAR)
        self.cancel_btn.setStyleSheet(styles.PUSHBUTTON)
        self.apply_btn.setStyleSheet(styles.PUSHBUTTON)



