# -*- coding: utf-8 -*-

# docinstaller.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2014-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Documentation installer dialog box

import os
import urllib
import hashlib
import tarfile
import shutil
import utils
import logging
lgg = logging.getLogger(__name__)

from PyQt5.QtCore import QCoreApplication
from PyQt5.QtGui import QFont
from PyQt5.QtWidgets import (QDialog, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QLabel, QComboBox, QCheckBox, QToolButton,
                             QPushButton, QSpacerItem, QSizePolicy,
                             QProgressBar, QPlainTextEdit, QFileDialog)

from VWidgets_PyQt5.vlineedit import VLineEdit



class DocInstaller(QDialog):
    def __init__(self, help, new):
        super(DocInstaller, self).__init__()
        self.help = help
        self.resize(384, 186)
        self.setSizeGripEnabled(True)
        self.setModal(True)
        self.gridLayout = QGridLayout(self)
        self.verticalLayout = QVBoxLayout()
        self.new_install_lbl = QLabel(self)
        font = QFont()
        font.setPointSize(11)
        self.new_install_lbl.setFont(font)
        self.verticalLayout.addWidget(self.new_install_lbl)
        self.update_instal_lbl = QLabel(self)
        self.update_instal_lbl.setFont(font)
        self.verticalLayout.addWidget(self.update_instal_lbl)
        self.horizontalLayout = QHBoxLayout()
        self.lang_lbl = QLabel(self)
        self.horizontalLayout.addWidget(self.lang_lbl)
        self.langs_cbx = QComboBox(self)
        self.horizontalLayout.addWidget(self.langs_cbx)
        sp = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(sp)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.local_file_chb = QCheckBox(self)
        self.verticalLayout.addWidget(self.local_file_chb)
        self.horizontalLayout_2 = QHBoxLayout()
        sp1 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(sp1)
        self.local_file_led = VLineEdit(self)
        self.horizontalLayout_2.addWidget(self.local_file_led)
        self.browse_btn = QToolButton(self)
        self.horizontalLayout_2.addWidget(self.browse_btn)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QHBoxLayout()
        self.progress = QProgressBar(self)
        self.progress.setMinimumWidth(100)
        self.horizontalLayout_3.addWidget(self.progress)
        self.message = QLabel(self)
        self.horizontalLayout_3.addWidget(self.message)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.warning_lbl = QLabel(self)
        self.verticalLayout.addWidget(self.warning_lbl)
        self.error_ted = QPlainTextEdit(self)
        self.error_ted.setReadOnly(True)
        self.verticalLayout.addWidget(self.error_ted)
        self.horizontalLayout_4 = QHBoxLayout()
        sp2 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(sp2)
        self.cancel_btn = QPushButton(self)
        self.horizontalLayout_4.addWidget(self.cancel_btn)
        self.install_btn = QPushButton(self)
        self.horizontalLayout_4.addWidget(self.install_btn)
        sp3 = QSpacerItem(18, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(sp3)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.gridLayout.addLayout(self.verticalLayout, 0, 0, 1, 1)
        self.new_install_lbl.setVisible(new)
        self.update_instal_lbl.setVisible(not new)
        self.langs_cbx.setEnabled(False)
        self.local_file_led.setEnabled(False)
        self.browse_btn.setEnabled(False)
        self.install_btn.setEnabled(False)
        self.local_file_chb.stateChanged.connect(self.on_local_changed)
        self.local_file_led.textChanged.connect(self.on_local_edited)
        self.browse_btn.clicked.connect(self.browse)
        self.install_btn.clicked.connect(self.install)
        self.cancel_btn.clicked.connect(self.reject)
        self.progress.hide()
        self.message.hide()
        self.error_ted.hide()
        self.retranslateUi()

    def setup(self, languages, folder, versions):
        self.languages = languages
        self.usr_fld = os.path.dirname(folder)
        self.temp = os.path.join(folder, 'temp')
        self.docs = os.path.join(folder, 'docs')
        self.versions = versions
        if languages:
            for l in languages:
                self.langs_cbx.addItem(self.versions[l]['lang'])
            self.langs_cbx.setEnabled(True)
            self.install_btn.setEnabled(True)

    def on_local_changed(self, state):
        self.langs_cbx.setEnabled(False)
        self.local_file_led.setEnabled(state)
        self.browse_btn.setEnabled(state)
        self.on_local_edited(self.local_file_led.text())
        if not state:
            if self.languages:
                self.langs_cbx.setEnabled(True)
                self.install_btn.setEnabled(True)

    def on_local_edited(self, text):
        text = text
        name = os.path.basename(text)
        if name.startswith(u'oqapyDoc-'):
            self.install_btn.setEnabled(os.path.isfile(text))

    def browse(self):
        d = QFileDialog.getOpenFileName(None,
                                        u"Open doc folder",
                                        self.usr_fld,
                                        "Archives (*.gz)")[0]

        if d:
            self.local_file_led.setText(d)

    def install(self):
        self.show_progress(True)
        installed = False
        if self.local_file_chb.isChecked():
            self.update_message('Check file')
            fname = self.local_file_led.text()
            if self.is_valid(fname):
                self.unpack_doc(fname)
                fld, name = os.path.split(fname)
                lang = os.path.splitext(name)[0].split('-')[1].split('.')[0]
                orig = os.path.join(self.temp, lang)
                dest = os.path.join(self.docs, lang)
                shutil.move(orig, dest)
                installed = True
                self.help.new_update = lang
            else:
                self.show_progress(False)
                self.display_error_message(0, _('Unvalid archive:\n%s' % fname))
                return

        else:
            installed = self.fetch_doc()

        self.show_progress(False)
        if installed:
            self.accept()

    def is_valid(self, arch):
        name = os.path.basename(arch)
        if not name.startswith('oqapyDoc-') or not name.endswith('.tar.gz'):
            return False

        _, tail = name.split('-')
        lang = tail.split('.')[0]
        return "_" in lang

    def fetch_doc(self):
        lang = self.languages[self.langs_cbx.currentIndex()]
        #lang = self.langs_cbx.currentText()
        pack = self.versions[lang]['pack']
        self.update_message('Get %s' % pack)
        url = 'http://www.oqapy.eu/releases/' + pack
        dest = os.path.join(self.temp, pack)
        user_agent = 'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:49.0)'\
                      ' Gecko/20100101 Oqapy/3.0'
        req = urllib.request.Request(url, data=None,
                                     headers={"User-Agent": user_agent})
        try:
            tar = urllib.request.urlopen(req, timeout=5)
        except Exception as why:
            self.display_error_message(1, "Request for:\n%s\n%s" 
                                        % (url, str(why)))
            return False

        try:
            content = tar.read()
        except Exception as why:
            self.display_error_message(2, why)
            return False

        with open(dest, 'wb') as outf:
            outf.write(content)

        if self.check_integrity(dest, lang):
            if self.unpack_doc(dest):
                orig = os.path.join(self.temp, lang)
                dest = os.path.join(self.docs, lang)
                self.clean_destination(dest)
                shutil.move(orig, self.docs)
                self.help.new_update = lang
                orig = os.path.join(self.temp, 'DOCVERSIONS.json')
                if os.path.isfile(orig):
                    dest = os.path.join(self.docs, 'DOCVERSIONS.json')
                    shutil.move(orig, dest)
                return True

        return False

    def clean_destination(self, dest):
        lgg.info('Clean:', dest)
        if os.path.isdir(dest):
            for i in (os.listdir(dest)):
                fpath = os.path.join(dest, i)
                if os.path.isfile(fpath):
                    try:
                        os.remove(fpath)
                    except Exception as why:
                        lgg.warning("Can't remove %s %s" %(i, why))
                else:
                    self.clean_destination(fpath)
            os.rmdir(dest)

    def unpack_doc(self, fname):
        def doc_files(members):
            for tarinfo in members:
                if not '/' in tarinfo.name[:3]:
                    yield tarinfo

        self.update_message('Unpack')
        old_dir = os.getcwd()
        os.chdir(self.temp)
        try:
            with tarfile.open(fname) as tar:
                tar.extractall(members=doc_files(tar))
        except Exception as why:
            self.display_error_message(3, why)
            os.chdir(old_dir)
            return False

        os.chdir(old_dir)
        return True

    def check_integrity(self, obj, key):
        self.update_message('Check integrity')
        ref = self.versions[key]['md5']
        with open(obj, 'rb') as inf:
            m = hashlib.sha224(inf.read()).hexdigest()

        if ref != m:
            self.display_error_message(3, _("The archive downloaded seems corrupted."))
            return False

        return True

    def display_error_message(self, idx, message=False):
        labels = [_("Can't install the archive"), _("Unable to find the archive"),
                  _("Unable to read the archive"), _("Unable to extract the tarball")]
        self.warning_lbl.setText(labels[idx])
        self.error_ted.setPlainText(message)
        self.show_error(True)

    def show_progress(self, b):
        if b:
            self.progress.show()
            self.progress.setRange(0, 0)
            self.message.setText("Install ...")
            self.message.show()

        else:
            self.progress.setRange(0, 100)
            self.progress.hide()
            self.message.hide()

        QCoreApplication.processEvents()

    def show_error(self, b):
        self.warning_lbl.setVisible(b)
        self.error_ted.setVisible(b)

    def update_message(self, message):
        self.message.setText(message)
        QCoreApplication.processEvents()

    def retranslateUi(self):
        self.setWindowTitle(_("Oqapy - Docs Installer"))
        self.new_install_lbl.setText(_("The documentation is not installed."))
        self.update_instal_lbl.setText(_("The documentation has been updated."))
        self.lang_lbl.setText(_("language:"))
        self.langs_cbx.setToolTip(_("Choose a language"))
        self.local_file_chb.setText(_("Use local file"))
        self.local_file_led.setProperty("informativeText_", _("Enter the file path"))
        self.browse_btn.setToolTip(_("Browse ..."))
        self.browse_btn.setText("...")
        self.cancel_btn.setText(_("Cancel"))
        self.install_btn.setText(_("Install"))


