# -*- coding: utf-8 -*-

# ui_helpviewer.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2014-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# User guide dialog

from PyQt5.QtCore import Qt, QCoreApplication, pyqtSignal
from PyQt5.QtGui import QPen, QColor
from PyQt5.QtWidgets import (QGridLayout, QVBoxLayout, QSplitter, QLabel,
                             QWidget, QTreeWidget, QTabWidget, QComboBox,
                             QPushButton, QSizePolicy, QSpacerItem,
                             QItemDelegate, QMenu, QAction,
                             QTreeWidgetItemIterator)
from PyQt5.QtWebKit import QWebSettings
from PyQt5.QtWebKitWidgets import QWebView, QWebPage, QWebFrame

#from VWidgets_PyQt5.vlineedit import VLineEdit

class Ui_HelpViewer(QWidget):
    def __init__(self, help, size):
        super().__init__()
        self.help = help
        self.resize(size[0], size[1])
        self.gridLayout_2 = QGridLayout(self)
        self.gridLayout_2.setContentsMargins(2, 2, 2, 2)
        self.splitter = QSplitter(self)
        self.splitter.setOrientation(Qt.Horizontal)
        self.splitter.setHandleWidth(10)
        self.widget = QWidget(self.splitter)
        self.gridLayout = QGridLayout(self.widget)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        # TODO implementation of indexation of doc
        #self.search_lbl = QLabel(self.widget)
        #self.gridLayout.addWidget(self.search_lbl, 0, 0, 1, 1)
        #self.search_led = VLineEdit(self.widget)
        #self.search_led.setResetButton(True)
        #self.search_led.setHistoricButton(True)
        #self.search_led.setCommandButton(True)
        #self.gridLayout.addWidget(self.search_led, 0, 1, 1, 3)
        self.lang_lbl = QLabel(self.widget)
        self.gridLayout.addWidget(self.lang_lbl, 1, 0, 1, 1)
        self.lang_cmb = QComboBox(self.widget)
        self.gridLayout.addWidget(self.lang_cmb, 1, 1, 1, 1)
        self.content_lbl = QLabel(self.widget)
        self.gridLayout.addWidget(self.content_lbl, 2, 0, 1, 2)
        spacer = QSpacerItem(88, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout.addItem(spacer, 1, 2, 1, 2)
        self.treeWidget = IndexTree(self)
        self.treeWidget.headerItem().setText(0, "1")
        self.gridLayout.addWidget(self.treeWidget, 3, 0, 1, 4)
        self.update_lbl = QLabel(self.widget)
        self.gridLayout.addWidget(self.update_lbl, 4, 0, 1, 3)
        self.update_btn = QPushButton("Update", self.widget)
        self.gridLayout.addWidget(self.update_btn, 4, 3, 1, 1)
        self.tabWidget = QTabWidget(self.splitter)
        tab = Tab(0, "", self)
        self.tabWidget.addTab(tab, "")
        self.gridLayout_2.addWidget(self.splitter, 0, 0, 1, 1)
        self.update_lbl.hide()
        self.update_btn.hide()
        self.retranslateUi()
        self.tabWidget.tabCloseRequested.connect(self.remove_tab)
        self.update_btn.clicked.connect(self.help.update_doc)

        self.show()
        QCoreApplication.processEvents()
        size = self.splitter.sizes()
        left = 220
        right = sum(size) - left
        self.splitter.setSizes([left, right])

    def add_tab(self, title, pos=None):
        if pos is None or pos >= self.tabWidget.count():
            tab = Tab(self.tabWidget.count(), None, self)
            self.tabWidget.addTab(tab, title)

        else:
            tab = Tab(pos)
            self.tabWidget.insertTab(pos, tab, title)
            for t in range(pos+1, self.tabWidget.count()):
                self.tabWidget.widget(t).index += 1

        if self.tabWidget.count() > 1:
            self.tabWidget.setTabsClosable(True)

        return tab

    def remove_tab(self, tab):
        self.help.current_pages.pop(tab)
        self.help.tabs.pop(tab)
        self.help.update_tree_index(tab)
        self.tabWidget.removeTab(tab)
        if self.tabWidget.count() == 1:
            # Don't close the last tab
            self.tabWidget.setTabsClosable(False)

    def set_page_title(self, idx, title):
        self.tabWidget.widget(idx).title = title
        self.tabWidget.setTabText(idx, title)

    def retranslateUi(self):
        self.setWindowTitle("Oqapy - Helpviewer")
        self.lang_lbl.setText("Language:")
        self.content_lbl.setText("Content")


class IndexTree(QTreeWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = parent
        self.setHeaderHidden(True)
        delegate = ItemDelegate(self)
        self.setItemDelegate(delegate)
        self.item = ""
        self.displayed = []
        self.expands = []

    def display_page(self):
        if self.item:
            self.ui.help.on_item_tree_selected(self.item, None)

    def display_new_tab(self):
        if self.item:
            self.ui.help.on_item_tree_selected(self.item, None, "new")

    def contextMenuEvent(self, event):
        self.item = self.itemAt(event.pos())
        self.menu = ContextMenu(self, event.globalPos())

    def reset_displayed(self):
        self.displayed = [tab.title for tab in self.ui.help.tabs]
        self.save_expanded()

    def save_expanded(self):
        iterator = QTreeWidgetItemIterator(self)
        self.expands = []
        model = self.model()
        while iterator.value():
            if iterator.value().isExpanded():
                self.expands.append(iterator.value())
            iterator += 1

    def reset_expanded(self):
        for item in self.expands:
            self.expandItem(item)

    def mousePressEvent(self, event):
        self.button = event.button()
        QTreeWidget.mousePressEvent(self, event)

class ItemDelegate(QItemDelegate):
    def __init__(self, tree, parent=None):
        super().__init__(parent)
        self.tree = tree

    def createEditor(self, parent, option, index):
        return QItemDelegate.createEditor(self, parent, option, index)

    def paint(self, painter, option, index):
        painter.save()
        x, y = option.rect.x(), option.rect.y()
        text = self.tree.itemFromIndex(index).text(0)
        if text in self.tree.displayed:
            painter.setPen(QPen(QColor(125, 125, 160, 255)))

        else:
            painter.setPen(QPen(QColor(0, 0, 0, 255)))

        painter.drawText(x+25, y+14, text)
        painter.restore() 


class ContextMenu(QMenu):
    def __init__(self, caller, pos, parent=None):
        super().__init__(parent)
        self.show = QAction(None)
        self.show.setText('Show')
        self.addAction(self.show)
        self.add = QAction(None)
        self.add.setText('Show into a new tab')
        self.addAction(self.add)
        self.show.triggered.connect(caller.display_page)
        self.add.triggered.connect(caller.display_new_tab)
        self.popup(pos)


class Tab(QWidget):
    def __init__(self, index, title=None, parent=None):
        super().__init__(parent.tabWidget)
        self.ui = parent
        self.index = index
        self.title = title
        self.url = None
        self.layout = QVBoxLayout(self)
        self.layout.setContentsMargins(2, 0, 2, 0)
        self.webview = WebView(self)
        self.layout.addWidget(self.webview)

    def update_page(self, title, url):
        self.webview.setUrl(url)
        self.index = self.ui.tabWidget.indexOf(self)
        self.ui.tabWidget.setTabText(self.index, title)
        self.title = title
        self.url = url

class WebView(QWebView):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.tab = parent
        self.ui = parent.ui
        self.button = 1
        self.urlChanged.connect(self.on_url_changed)
        self.linkClicked.connect(self.on_link_clicked)

    def contextMenuEvent(self, event):
        frame = self.page().frameAt(event.pos())
        res = frame.hitTestContent(event.pos())
        self.item = res.linkUrl()
        if res.linkUrl().isValid():
            self.menu = ContextMenu(self, event.globalPos())

    def display_page(self):
        if self.item:
            self.ui.help.show_page(None, self.item)

    def display_new_tab(self):
        if self.item:
            self.ui.help.show_page('new', self.item)

    def on_url_changed(self, qurl):
        self.page().setLinkDelegationPolicy(QWebPage.DelegateAllLinks)

    def on_link_clicked(self, link):
        tab = [None, None, None, None, "new"][self.button]
        anchor = None
        if not link.isLocalFile():
            self.ui.help.call_external_link(link.toString())

        if link.isValid():
            self.ui.help.show_page(tab, link)

        self.button = 1

    def mousePressEvent(self, event):
        self.button = event.button()
        QWebView.mousePressEvent(self, event)
