
#include "/usr/include/opencv2/highgui/highgui.hpp"
#include "/usr/include/opencv2/imgproc/imgproc.hpp"
#include <iostream>
#include <stdio.h>
#include <fstream>
#include <math.h>
#include "colortemp.h"

using namespace std;
using namespace cv;

ColorTemp::ColorTemp()
{
}


int ColorTemp::loadImage(const char *filename, int mode)
{
    src = imread(filename, mode);

    if( !src.data )
    { 
        cout << "Image " << filename << " not loaded\n";
        bool isLoaded_ = false;
        return 0; 
    }

    else
    {
        bool isLoaded_ = true;
        return 1;
    }
}

bool ColorTemp::isLoaded()
{
    if (!isLoaded_)
    {
        return false;
    }
    return true;
}

int ColorTemp::applyColorTemperature(int red, int green, int blue, float strength,
                                     const char* outf)
{
    float b, g, r, min, max;
    float strg = 1 - strength;
    for( int y = 0 ; y < src.rows ; y++ )
    { 
        for( int x = 0 ; x < src.cols ; x++ )
        {
            b = src.at<Vec3b>(y, x)[0];
            g = src.at<Vec3b>(y, x)[1];
            r = src.at<Vec3b>(y, x)[2];

            /* Keep the original luminance */
            min = fmin(b, fmin(g, r));
            max = fmax(b, fmax(g, r));
            float ol = (min + max) / 2;

            float bb = (b * strg) + (blue * strength);
            float gg = (g * strg) + (green * strength);
            float rr = (r * strg) + (red * strength);

            Mat hls(1, 1, CV_8UC3);
            hls.at<Vec3b>(0, 0)[0] = bb;
            hls.at<Vec3b>(0, 0)[1] = gg;
            hls.at<Vec3b>(0, 0)[2] = rr;

            /* Convert to HLS */
            cvtColor(hls, hls, CV_BGR2HLS, 0);

            /* Reset the original luminance */
            hls.at<Vec3b>(0, 0)[1] = ol;

            /* Return to BGR */
            cvtColor(hls, hls, CV_HLS2BGR, 0);

            src.at<Vec3b>(y, x)[0] = hls.at<Vec3b>(0, 0)[0];
            src.at<Vec3b>(y, x)[1] = hls.at<Vec3b>(0, 0)[1];
            src.at<Vec3b>(y, x)[2] = hls.at<Vec3b>(0, 0)[2];

        }
    }
    imwrite( outf, src );
    return 0;
}

ColorTemp::~ColorTemp()
{
}
