/*
 * oqapy-iproc
 * Image processor using openCV
 *
 * This lib is part of Oqapy
 *
 * Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
 * Home: http://www.oqapy.eu
 * Copyright: 2009-2015 Vincent Vande Vyvre
 * Licence: GPL3
 *
 *
 * This lib is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This lib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <http://www.gnu.org/licenses/>.
 * 
 */

#include "/usr/include/opencv2/highgui/highgui.hpp"
#include "/usr/include/opencv2/imgproc/imgproc.hpp"
#include "/usr/include/opencv2/photo/photo.hpp"

using namespace cv;

/**
 * class OqapyIProc
 */
class OqapyIProc
{

public :
    OqapyIProc();
    int loadImage(const char *filename, int mode=1);
    bool isLoaded_;
    Mat src;
    const char* version();
    const int version_hex();
    bool isLoaded();
    int applyContrastStretching(float lower, float upper, const char* outf);
    int applyContrastBrightness(float gain, float bias, float lower, float upper, 
                                const char* outf);
    int convertToGreyMap(const char* outf);
    int convertToBrightGreyMap(int level, const char* outf);
    int applyToningSepia(int level, const char* outf);
    int applyToningBlue(int level, const char* outf);
    int applyLaplacianFilter(int ksize, const char* outf);
    int denoiseGreyMap(float strength, int patch, int window,
                                const char* outf);
    int denoiseColorMap(float strength, int color, int patch,
                                int window, const char* outf);
    virtual ~OqapyIProc();

private :
    Mat bgr_planes;
    Mat b_hist;
    Mat g_hist;
    Mat r_hist;
    Mat dst;
    float c_hist;
    void getColorHistogram();
    void setContrastStretching(int min, int max, int index);

};
