# -*- coding: utf-8 -*-

# camerapage.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2015-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Camera page widget

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QFont
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QLabel, QSizePolicy, QSpacerItem, QFrame)

import stylesheets as style

class CameraPage(QWidget):
    def __init__(self, editor, parent=None):
        super().__init__(parent)
        self.gridLayout_3 = QGridLayout(self)
        self.verticalLayout_3 = QVBoxLayout()
        self.horizontalLayout = QHBoxLayout()
        self.label = QLabel(self)
        font = QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setItalic(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.horizontalLayout.addWidget(self.label)
        self.verticalLayout = QVBoxLayout()
        spacer = QSpacerItem(20, 12, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout.addItem(spacer)
        self.line = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line.sizePolicy().hasHeightForWidth())
        self.line.setSizePolicy(sizePolicy)
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)
        self.verticalLayout.addWidget(self.line)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.horizontalLayout_3 = QHBoxLayout()
        spacer1 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacer1)
        alignment = Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter
        self.gridLayout = QGridLayout()
        self.maker_lbl = QLabel(self)
        self.maker_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.maker_lbl, 0, 0, 1, 1)
        self.maker_value_lbl = QLabel(self)
        self.maker_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.maker_value_lbl, 0, 1, 1, 1)
        self.model_lbl = QLabel(self)
        self.model_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.model_lbl, 1, 0, 1, 1)
        self.model_value_lbl = QLabel(self)
        self.model_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.model_value_lbl, 1, 1, 1, 1)
        self.firmware_lbl = QLabel(self)
        self.firmware_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.firmware_lbl, 2, 0, 1, 1)
        self.firmware_value_lbl = QLabel(self)
        self.firmware_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.firmware_value_lbl, 2, 1, 1, 1)
        self.serial_lbl = QLabel(self)
        self.serial_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.serial_lbl, 3, 0, 1, 1)
        self.serial_value_lbl = QLabel(self)
        self.serial_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.serial_value_lbl, 3, 1, 1, 1)
        self.owner_lbl = QLabel(self)
        self.owner_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.owner_lbl, 4, 0, 1, 1)
        self.owner_value_lbl = QLabel(self)
        self.owner_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.owner_value_lbl, 4, 1, 1, 1)
        self.horizontalLayout_3.addLayout(self.gridLayout)
        spacer5 = QSpacerItem(40, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacer5)
        self.verticalLayout_3.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_2 = QHBoxLayout()
        self.label_2 = QLabel(self)
        self.label_2.setFont(font)
        self.horizontalLayout_2.addWidget(self.label_2)
        self.verticalLayout_2 = QVBoxLayout()
        spacer2 = QSpacerItem(20, 12, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_2.addItem(spacer2)
        self.line_2 = QFrame(self)
        self.line_2.setSizePolicy(sizePolicy)
        self.line_2.setFrameShape(QFrame.HLine)
        self.line_2.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_2.addWidget(self.line_2)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_4 = QHBoxLayout()
        spacer3 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacer3)
        self.gridLayout_2 = QGridLayout()
        # Lens specifications placeholder
        self.horizontalLayout_4.addLayout(self.gridLayout_2)
        spacer6 = QSpacerItem(40, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacer6)
        self.verticalLayout_3.addLayout(self.horizontalLayout_4)
        spacer4 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacer4)
        self.gridLayout_3.addLayout(self.verticalLayout_3, 0, 0, 1, 1)

        self.retranslateUi()

    def retranslateUi(self):
        self.label.setText(_("Camera"))
        self.maker_lbl.setText(_("Constructor:"))
        self.model_lbl.setText(_("Model:"))
        self.serial_lbl.setText(_("Serial number:"))
        self.firmware_lbl.setText(_("Firmware:"))
        self.owner_lbl.setText(_("Owner:"))
        self.label_2.setText(_("Lens"))

    def set_style(self):
        self.label.setStyleSheet(style.SUBTITLE)
        self.line.setStyleSheet(style.UNDERLINE)
        self.label_2.setStyleSheet(style.SUBTITLE)
        self.line_2.setStyleSheet(style.UNDERLINE)

