# -*- coding: utf-8 -*-

# imagepage.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2015-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Image page widget

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QFont
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QLabel, QLineEdit, QSizePolicy,
                             QSpacerItem, QFrame, QScrollArea)

import stylesheets as style

class ImagePage(QWidget):
    def __init__(self, editor, parent=None):
        super().__init__(parent)
        self.gridLayout_3 = QGridLayout(self)
        self.scrollArea = QScrollArea(self)
        self.scrollArea.setWidgetResizable(True)
        self.scrollwidget = QWidget()
        self.verticalLayout_3 = QVBoxLayout(self.scrollwidget)
        self.horizontalLayout = QHBoxLayout()
        self.label = QLabel(self)
        font = QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setItalic(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.horizontalLayout.addWidget(self.label)
        self.verticalLayout = QVBoxLayout()
        spacer = QSpacerItem(20, 12, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout.addItem(spacer)
        self.line = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line.sizePolicy().hasHeightForWidth())
        self.line.setSizePolicy(sizePolicy)
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)
        self.verticalLayout.addWidget(self.line)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.horizontalLayout_3 = QHBoxLayout()
        spacer1 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacer1)
        alignment = Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter
        self.gridLayout = QGridLayout()
        self.title_lbl = QLabel(self)
        self.title_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.title_lbl, 0, 0, 1, 1)
        self.title_value_lbl = QLabel(self)
        self.title_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.title_value_lbl, 0, 1, 1, 1)
        self.uid_lbl = QLabel(self)
        self.uid_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.uid_lbl, 1, 0, 1, 1)
        self.uid_value_lbl = QLabel(self)
        self.uid_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.uid_value_lbl, 1, 1, 1, 1)
        self.size_lbl = QLabel(self)
        self.size_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.size_lbl, 2, 0, 1, 1)
        self.size_value_lbl = QLabel(self)
        self.size_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.size_value_lbl, 2, 1, 1, 1)
        self.resolution_lbl = QLabel(self)
        self.resolution_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.resolution_lbl, 3, 0, 1, 1)
        self.resolution_value_lbl = QLabel(self)
        self.resolution_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.resolution_value_lbl, 3, 1, 1, 1)
        self.colorspace_lbl = QLabel(self)
        self.colorspace_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.colorspace_lbl, 4, 0, 1, 1)
        self.colorspace_value_lbl = QLabel(self)
        self.colorspace_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.colorspace_value_lbl, 4, 1, 1, 1)
        self.orientation_lbl = QLabel(self)
        self.orientation_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.orientation_lbl, 5, 0, 1, 1)
        self.orientation_value_lbl = QLabel(self)
        self.orientation_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.orientation_value_lbl, 5, 1, 1, 1)
        self.horizontalLayout_3.addLayout(self.gridLayout)
        spacer5 = QSpacerItem(40, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacer5)
        self.verticalLayout_3.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_2 = QHBoxLayout()
        self.label_2 = QLabel(self)
        self.label_2.setFont(font)
        self.horizontalLayout_2.addWidget(self.label_2)
        self.verticalLayout_2 = QVBoxLayout()
        spacer2 = QSpacerItem(20, 12, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_2.addItem(spacer2)
        self.line_2 = QFrame(self)
        self.line_2.setSizePolicy(sizePolicy)
        self.line_2.setFrameShape(QFrame.HLine)
        self.line_2.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_2.addWidget(self.line_2)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_4 = QHBoxLayout()
        spacer3 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacer3)
        self.gridLayout_2 = QGridLayout()
        self.type_lbl = QLabel(self)
        self.type_lbl.setAlignment(alignment)
        self.gridLayout_2.addWidget(self.type_lbl, 0, 0, 1, 1)
        self.horizontalLayout_4.addLayout(self.gridLayout_2)
        spacer6 = QSpacerItem(40, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacer6)
        self.verticalLayout_3.addLayout(self.horizontalLayout_4)
        spacer4 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacer4)
        self.scrollArea.setWidget(self.scrollwidget)
        self.gridLayout_3.addWidget(self.scrollArea, 0, 0, 1, 1)

        self.retranslateUi()
        self.set_style()

    def retranslateUi(self):
        self.label.setText(_("Image"))
        self.title_lbl.setText(_("Title:"))
        self.uid_lbl.setText("Unique identifier:")
        self.size_lbl.setText(_("Size:"))
        self.resolution_lbl.setText(_("Resolution:"))
        self.colorspace_lbl.setText(_("Color space:"))
        self.orientation_lbl.setText(_("Orientation:"))
        self.label_2.setText(_("Previews"))

    def set_style(self):
        self.label.setStyleSheet(style.SUBTITLE)
        self.line.setStyleSheet(style.UNDERLINE)
        self.label_2.setStyleSheet(style.SUBTITLE)
        self.line_2.setStyleSheet(style.UNDERLINE)
