# -*- coding: utf-8 -*-

# misc.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2012-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Some data used by the metadata handler

FLASH_VALUES = {0: 'No Flash',
                1: 'Fired',
                5: 'Fired, Return not detected',
                7: 'Fired, Return detected',
                9: 'On',
                13: 'On, Return not detected',
                15: 'On, Return detected',
                16: 'Off',
                24: 'Auto, Did not fire',
                25: 'Auto, Fired',
                29: 'Auto, Fired, Return not detected',
                31: 'Auto, Fired, Return detected',
                32: 'No flash function',
                65: 'Fired, Red-eye reduction',
                69: 'Fired, Red-eye reduction, Return not detected',
                71: 'Fired, Red-eye reduction, Return detected',
                73: 'On, Red-eye reduction',
                77: 'On, Red-eye reduction, Return not detected',
                79: 'On, Red-eye reduction, Return detected',
                89: 'Auto, Fired, Red-eye reduction',
                93: 'Auto, Fired, Red-eye reduction, Return not detected',
                95: 'Auto, Fired, Red-eye reduction, Return detected'}

LIGHTSOURCE_VALUES = {0: 'Auto',
                        1: 'Daylight',
                        2: 'Fluorescent',
                        3: 'Tungsten (incandescent light)',
                        4: 'Flash',
                        9: 'Fine weather',
                        10: 'Cloudy weather',
                        11: 'Shade',
                        12: 'Daylight fluorescent (D 5700 - 7100K)',
                        13: 'Day white fluorescent (N 4600 - 5400K)',
                        14: 'Cool white fluorescent (W 3900 - 4500K)',
                        15: 'White fluorescent (WW 3200 - 3700K)',
                        17: 'Standard light A',
                        18: 'Standard light B',
                        19: 'Standard light C',
                        20: 'D55',
                        21: 'D65',
                        22: 'D75',
                        23: 'D50',
                        24: 'ISO studio tungsten',
                        255: 'Unknow'}

GENERAL_KEYS = ['Title', 'Date', 'Author', 'Copyright', 'Comment', 
                'Unique Identifier', 'Exif.Image.ImageDescription',
                'Exif.Photo.DateTimeOriginal', 'Exif.Image.Artist', 
                'Exif.Image.Copyright', 'Exif.Photo.UserComment', 
                'Exif.Photo.ImageUniqueID']

IMAGE_KEYS = ['Size', 'Resolution', 'Scene type', 'Software',
                'Exif.Photo.PixelXDimension', 'Exif.Photo.PixelYDimension',
                'Exif.Image.XResolution', 'Exif.Image.YResolution',
                'Exif.Image.ResolutionUnit', 'Exif.Photo.SceneType',
                'Exif.Image.ProcessingSoftware']

EDITABLES = {0: ('Exif.Image.Artist', 'Iptc.Application2.Byline',
                'Xmp.dc.creator'), 
            1: ('Iptc.Application2.BylineTitle', 'AuthorsPosition'),
            2: ('Exif.Image.Copyright', 'Iptc.Application2.Copyright',
                'Xmp.dc.rights'),
            3: ('Iptc.Application2.0x00d0',),
            4: ('Iptc.Application2.0x00d1',),
            5: ('Iptc.Application2.0x00d2',),
            6: ('Iptc.Application2.0x00d3',),
            7: ('Iptc.Application2.0x00d4',),
            8: ('Iptc.Application2.0x00d5',),
            9: ('Iptc.Application2.0x00d6',),
            10: ('Iptc.Application2.0x00d7',),
            20: ('Iptc.Application2.Headline', 'Xmp.photoshop.Headline'),
            21: ('Exif.Image.ImageDescription', 'Iptc.Application2.Caption',
                'Xmp.dc.description'),
            22: ('Iptc.Application2.Category', 'Xmp.photoshop.Category'),
            23: ('Iptc.Application2.Writer', 'Xmp.photoshop.CaptionWriter'),
            30: ('Iptc.Application2.SubLocation', 'Xmp.iptc.Location'),
            31: ('Iptc.Application2.City', 'Xmp.photoshop.City'),
            32: ('Iptc.Application2.ProvinceState', 'Xmp.photoshop.State'),
            33: ('Iptc.Application2.CountryName', 'Xmp.photoshop.Country'),
            34: ('Iptc.Application2.CountryCode', 'Xmp.iptc.CountryCode'),
            50: ('Exif.Photo.UserComment', 'Iptc.Application2.Keywords',
                'Xmp.dc.subject')}

EDITABLES_DESC = {3: ('Address', 'Address of contact.'),
                    4: ('City', 'City of contact.'),
                    5: ('State', 'State or province.'),
                    6: ('Zip code', 'Postal code.'),
                    7: ('Country', 'Country name.'),
                    8: ('Telephone', 'Telephone number.'),
                    9: ('Mail', 'Mail address.'),
                    10: ('Site', 'Home page of personnal site.')}

XMP = ['Xmp.dc.creator', 'Xmp.photoshop.Headline', 
        'Xmp.photoshop.AuthorsPosition', 'Xmp.dc.rights',
        'Xmp.dc.subject', 'Xmp.dc.description', 'Xmp.photoshop.CaptionWriter',
        'Xmp.photoshop.Category', 'Xmp.iptc.Location', 'Xmp.photoshop.City',
        'Xmp.photoshop.State', 'Xmp.photoshop.Country']

COUNTRIES_ABBR_3 = ['AFG', 'ALA', 'ALB', 'DZA', 'ASM', 'AND', 'AGO', 'AIA',
        'ATA', 'ATG', 'ARG', 'ARM', 'ABW', 'AUS', 'AUT', 'AZE', 'BHS', 'BHR',
        'BGD', 'BRB', 'BLR', 'BEL', 'BLZ', 'BEN', 'BMU', 'BTN', 'BOL', 'BIH',
        'BWA', 'BVT', 'BRA', 'IOT', 'VGB', 'BRN', 'BGR', 'BFA', 'BDI', 'KHM',
        'CMR', 'CAN', 'CPV', 'CYM', 'CAF', 'TCD', 'CHL', 'CHN', 'CXR', 'CCK',
        'COL', 'COM', 'COG', 'COK', 'CRI', 'HRV', 'CUB', 'CYP', 'CZE', 'COD',
        'DNK', 'DJI', 'DMA', 'DOM', 'TLS', 'ECU', 'EGY', 'SLV', 'GNQ', 'ERI',
        'EST', 'ETH', 'FLK', 'FRO', 'FJI', 'FIN', 'FRA', 'GUF', 'PYF', 'ATF',
        'GAB', 'GMB', 'GEO', 'DEU', 'GHA', 'GIB', 'GRC', 'GRL', 'GRD', 'GLP',
        'GUM', 'GTM', 'GGY', 'GIN', 'GNB', 'GUY', 'HTI', 'HMD', 'HND', 'HKG',
        'HUN', 'ISL', 'IND', 'IDN', 'IRN', 'IRQ', 'IRL', 'IMN', 'ISR', 'ITA',
        'CIV', 'JAM', 'JPN', 'JEY', 'JOR', 'KAZ', 'KEN', 'KIR', 'XKX', 'KWT',
        'KGZ', 'LAO', 'LVA', 'LBN', 'LSO', 'LBR', 'LBY', 'LIE', 'LTU', 'LUX',
        'MAC', 'MKD', 'MDG', 'MWI', 'MYS', 'MDV', 'MLI', 'MLT', 'MHL', 'MTQ',
        'MRT', 'MUS', 'MYT', 'MEX', 'FSM', 'MDA', 'MCO', 'MNG', 'MNE', 'MSR',
        'MAR', 'MOZ', 'MMR', 'NAM', 'NRU', 'NPL', 'NLD', 'ANT', 'NCL', 'NZL',
        'NIC', 'NER', 'NGA', 'NIU', 'NFK', 'PRK', 'MNP', 'NOR', 'OMN', 'PAK',
        'PLW', 'PSE', 'PAN', 'PNG', 'PRY', 'PER', 'PHL', 'PCN', 'POL', 'PRT',
        'PRI', 'QAT', 'REU', 'ROU', 'RUS', 'RWA', 'BLM', 'SHN', 'KNA', 'LCA',
        'MAF', 'SPM', 'VCT', 'WSM', 'SMR', 'STP', 'SAU', 'SEN', 'SRB', 'SCG',
        'SYC', 'SLE', 'SGP', 'SVK', 'SVN', 'SLB', 'SOM', 'ZAF', 'SGS', 'KOR',
        'ESP', 'LKA', 'SDN', 'SUR', 'SJM', 'SWZ', 'SWE', 'CHE', 'SYR', 'TWN',
        'TJK', 'TZA', 'THA', 'TGO', 'TKL', 'TON', 'TTO', 'TUN', 'TUR', 'TKM',
        'TCA', 'TUV', 'VIR', 'UGA', 'UKR', 'ARE', 'GBR', 'USA', 'UMI', 'URY']

COUNTRIES_ABBR_2 = ['AF', 'AX', 'AL', 'DZ', 'AS', 'AD', 'AO', 'AI', 'AQ',
             'AG', 'AR', 'AM', 'AW', 'XSE', 'AU', 'AT', 'AZ', 'BS', 'BH', 'BD',
             'BB', 'BY', 'BE', 'BZ', 'BJ', 'BM', 'BT', 'BO', 'BQ', 'BA', 'BW',
             'BV', 'BR', 'IO', 'BN', 'BG', 'BF', 'BI', 'KH', 'CM', 'CA', 'CV',
             'KY', 'CF', 'TD', 'CL', 'CN', 'CX', 'CC', 'CO', 'KM', 'CG', 'CD',
             'CK', 'CR', 'CI', 'HR', 'CU', 'CW', 'CY', 'CZ', 'DK', 'DJ', 'DM',
             'DO', 'EC', 'EG', 'SV', 'XEN', 'GQ', 'ER', 'EE', 'ET', 'CE', 'FK',
             'FO', 'FJ', 'FI', 'FR', 'GF', 'PF', 'TF', 'GA', 'GM', 'GZA', 'GE',
             'DE', 'GH', 'GI', 'GR', 'GL', 'GD', 'GP', 'GU', 'GT', 'GG', 'GN',
             'GW', 'GY', 'HT', 'HM', 'VA', 'HN', 'HK', 'HU', 'IS', 'IN', 'ID',
             'XIF', 'IR', 'IQ', 'IE', 'IM', 'IL', 'IT', 'JM', 'JP', 'JE', 'JO',
             'KZ', 'KE', 'KI', 'KP', 'KR', 'KW', 'KG', 'LA', 'LV', 'LB', 'LS',
             'LR', 'LY', 'LI', 'LT', 'LU', 'MO', 'MK', 'MG', 'MW', 'MY', 'MV',
             'ML', 'MT', 'MH', 'MQ', 'MR', 'MU', 'YT', 'MX', 'FM', 'MD', 'MC',
             'MN', 'ME', 'MS', 'MA', 'MZ', 'MM', 'NA', 'NR', 'NP', 'NL', 'NC',
             'NZ', 'NI', 'NE', 'NG', 'NU', 'NF', 'MP', 'NO', 'OM', 'PK', 'PW',
             'PS', 'PA', 'PG', 'PY', 'PE', 'PH', 'PN', 'PL', 'PT', 'PR', 'QA',
             'RE', 'RO', 'RU', 'RW', 'BL', 'SH', 'KN', 'LC', 'MF', 'PM', 'VC',
             'WS', 'SM', 'ST', 'SA', 'SN', 'RS', 'SC', 'SL', 'SG', 'SX', 'SK',
             'SI', 'SB', 'SO', 'ZA', 'GS', 'SS', 'XSP', 'ES', 'LK', 'SD', 'SR',
             'SJ', 'SZ', 'SE', 'CH', 'SY', 'TW', 'TJ', 'TZ', 'TH', 'TL', 'TG',
             'TK', 'TO', 'TT', 'TN', 'TR', 'TM', 'TC', 'TV', 'UG', 'UA', 'AE',
             'GB', 'US', 'UM', 'UY', 'UZ', 'VU', 'VE', 'VN', 'VG', 'VI', 'WF',
             'EH', 'YE', 'ZM', 'ZW']

COUNTRIES = ['Afghanistan', 'Åland Islands', 'Albania', 'Algeria',
            'American Samoa', 'Andorra', 'Angola', 'Anguilla', 'Antarctica',
            'Antigua And Barbuda', 'Argentina', 'Armenia', 'Aruba', 'At Sea',
            'Australia', 'Austria', 'Azerbaijan', 'Bahamas', 'Bahrain',
            'Bangladesh', 'Barbados', 'Belarus', 'Belgium', 'Belize', 'Benin',
            'Bermuda', 'Bhutan', 'Bolivia, Plurinational State Of',
            'Bonaire, Sint Eustatius And Saba', 'Bosnia And Herzegovina',
            'Botswana', 'Bouvet Island', 'Brazil',
            'British Indian Ocean Territory', 'Brunei Darussalam', 'Bulgaria',
            'Burkina Faso', 'Burundi', 'Cambodia', 'Cameroon', 'Canada',
            'Cape Verde', 'Cayman Islands', 'Central African Republic', 'Chad',
            'Chile', 'China', 'Christmas Island', 'Cocos (keeling) Islands',
            'Colombia', 'Comoros', 'Congo',
            'Congo, The Democratic Republic Of The', 'Cook Islands',
            'Costa Rica', 'Côte D\'ivoire', 'Croatia', 'Cuba', 'Curacao',
            'Cyprus', 'Czech Republic', 'Denmark', 'Djibouti', 'Dominica',
            'Dominican Republic', 'Ecuador', 'Egypt', 'El Salvador', 'England',
            'Equatorial Guinea', 'Eritrea', 'Estonia', 'Ethiopia',
            'European Union', 'Falkland Islands (malvinas)', 'Faroe Islands',
            'Fiji', 'Finland', 'France', 'French Guiana', 'French Polynesia',
            'French Southern Territories', 'Gabon', 'Gambia', 'Gaza',
            'Georgia', 'Germany', 'Ghana', 'Gibraltar', 'Greece', 'Greenland',
            'Grenada', 'Guadeloupe', 'Guam', 'Guatemala', 'Guernsey', 'Guinea',
            'Guinea-bissau', 'Guyana', 'Haiti',
            'Heard Island And Mcdonald Islands',
            'Holy See (vatican City State)', 'Honduras', 'Hong Kong',
            'Hungary', 'Iceland', 'India', 'Indonesia', 'In Flight',
            'Iran, Islamic Republic Of', 'Iraq', 'Ireland', 'Isle Of Man',
            'Israel', 'Italy', 'Jamaica', 'Japan', 'Jersey', 'Jordan',
            'Kazakhstan', 'Kenya', 'Kiribati',
            'Korea, Democratic People\'s Republic Of', 'Korea, Republic Of',
            'Kuwait', 'Kyrgyzstan', 'Lao People\'s Democratic Republic',
            'Latvia', 'Lebanon', 'Lesotho', 'Liberia', 'Libya',
            'Liechtenstein', 'Lithuania', 'Luxembourg', 'Macao',
            'Macedonia, The Former Yugoslav Republic Of', 'Madagascar',
            'Malawi', 'Malaysia', 'Maldives', 'Mali', 'Malta',
            'Marshall Islands', 'Martinique', 'Mauritania', 'Mauritius',
            'Mayotte', 'Mexico', 'Micronesia, Federated States Of',
            'Moldova, Republic Of', 'Monaco', 'Mongolia', 'Montenegro',
            'Montserrat', 'Morocco', 'Mozambique', 'Myanmar', 'Namibia',
            'Nauru', 'Nepal', 'Netherlands', 'New Caledonia', 'New Zealand',
            'Nicaragua', 'Niger', 'Nigeria', 'Niue', 'Norfolk Island',
            'Northern Mariana Islands', 'Norway', 'Oman', 'Pakistan', 'Palau',
            'Palestinian Territory, Occupied', 'Panama', 'Papua New Guinea',
            'Paraguay', 'Peru', 'Philippines', 'Pitcairn', 'Poland',
            'Portugal', 'Puerto Rico', 'Qatar', 'Réunion', 'Romania',
            'Russian Federation', 'Rwanda', 'Saint BarthÉlemy',
            'Saint Helena, Ascension And Tristan Da Cunha',
            'Saint Kitts And Nevis', 'Saint Lucia',
            'Saint Martin (french Part)', 'Saint Pierre And Miquelon',
            'Saint Vincent And The Grenadines', 'Samoa', 'San Marino',
            'Sao Tome And Principe', 'Saudi Arabia', 'Senegal', 'Serbia',
            'Seychelles', 'Sierra Leone', 'Singapore',
            'Sint Maarten (dutch Part)', 'Slovakia', 'Slovenia',
            'Solomon Islands', 'Somalia', 'South Africa',
            'South Georgia And The South Sandwich Islands', 'South Sudan',
            'Space', 'Spain', 'Sri Lanka', 'Sudan', 'Suriname',
            'Svalbard And Jan Mayen', 'Swaziland', 'Sweden', 'Switzerland',
            'Syrian Arab Republic', 'Taiwan, Province Of China', 'Tajikistan',
            'Tanzania, United Republic Of', 'Thailand', 'Timor-leste', 'Togo',
            'Tokelau', 'Tonga', 'Trinidad And Tobago', 'Tunisia', 'Turkey',
            'Turkmenistan', 'Turks And Caicos Islands', 'Tuvalu', 'Uganda',
            'Ukraine', 'United Arab Emirates', 'United Kingdom',
            'United States', 'United States Minor Outlying Islands', 'Uruguay',
            'Uzbekistan', 'Vanuatu', 'Venezuela, Bolivarian Republic Of',
            'Viet Nam', 'Virgin Islands, British', 'Virgin Islands, U.S.A',
            'Wallis And Futuna', 'Western Sahara', 'Yemen', 'Zambia',
            'Zimbabwe']
