# -*- coding: utf-8 -*-

# multicloningdata.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2018 Vincent Vande Vyvre
# Licence: GPL3

# Cloning data on files in multiviewing mode

import os

from PyQt5.QtCore import Qt, QSize, QRectF
from PyQt5.QtGui import QBrush, QColor, QIcon, QPixmap, QImage
from PyQt5.QtWidgets import (QDialog, QGridLayout, QVBoxLayout, QHBoxLayout, 
                             QLabel, QSpacerItem, QGraphicsView, QPushButton, 
                             QTableWidget, QSizePolicy, QAbstractItemView,
                             QTableWidgetItem, QGraphicsScene)

from data import VERSION_STR

class  MultiCloningData:
    def __init__(self, caller, files, model, img):
        self.core = caller.core
        self.files = files
        self.model = model
        self.src_picture = img
        self.utils = caller.core.utils
        self.table_completed = False
        self.ui = Ui_Dialog(self, VERSION_STR, img, len(files))
        self.ui.show()

    def build_table(self):
        if not self.table_completed:
            self.table_items = []
            for f in self.files:
                self.table_items.append(FileItem(self, f))

            for i in self.table_items:
                self.ui.table_tbv.add_item(i)

            self.table_completed = True
        

class FileItem:
    def __init__(self, driver, path):
        self.driver = driver
        self.path = path
        self.name = os.path.basename(path)
        thpath = self.driver.utils.get_thumb_name(path)
        if not os.path.isfile(thpath):
            thpath = self.path
        img = QImage(thpath)
        self.img = img.scaledToHeight(100, Qt.SmoothTransformation)

class TableWidget(QTableWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = parent
        self.setAlternatingRowColors(True)
        self.setShowGrid(False)
        self.setIconSize(QSize(78, 46))
        self.setColumnCount(2)
        self.setRowCount(0)
        self.setColumnWidth(0, 80)
        self.hhdr = self.horizontalHeader()
        self.hhdr.setStretchLastSection(True)
        self.hhdr.setDefaultAlignment(Qt.AlignLeft)
        self.setSelectionBehavior(QAbstractItemView.SelectRows)

        item = QTableWidgetItem()
        item.setIcon(QIcon('medias/pola_icon_2.png'))
        self.setHorizontalHeaderItem(0, item)
        item = QTableWidgetItem()
        self.setHorizontalHeaderItem(1, item)
        self.item_selected = None

    def add_item(self, item):
        r = self.rowCount()
        self.insertRow(r)
        self.setRowHeight(r, 48)
        pix = QPixmap.fromImage(item.img)
        icon = QIcon()
        icon.addPixmap(pix, QIcon.Normal, QIcon.Off)
        obj = QTableWidgetItem()
        obj.setBackground(QBrush(QColor(0, 0, 0, 255)))
        obj.setTextAlignment(Qt.AlignHCenter)
        obj.setIcon(icon)
        self.setItem(r, 0, obj)
        self.obj = QTableWidgetItem()
        self.obj.setText(item.name)
        self.setItem(r, 1, self.obj)

        
class Ui_Dialog(QDialog):
    def __init__(self, driver, vers, picture, number):
        super().__init__()
        self.driver = driver
        self.setWindowModality(Qt.WindowModal)
        self.resize(350, 150)
        self.setWindowTitle("Metadata cloning - Oqapy-{}".format(vers))
        self.setSizeGripEnabled(True)
        self.setModal(True)
        self.gridLayout = QGridLayout(self)
        self.horizontalLayout_4 = QHBoxLayout()
        spc = QSpacerItem(10, 20, QSizePolicy.Minimum, QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spc)
        self.verticalLayout_2 = QVBoxLayout()
        self.horizontalLayout = QHBoxLayout()
        self.verticalLayout = QVBoxLayout()
        self.title_lbl = QLabel(self)
        self.verticalLayout.addWidget(self.title_lbl)
        spc1 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Minimum)
        self.verticalLayout.addItem(spc1)
        self.horizontalLayout.addLayout(self.verticalLayout)
        spc2 = QSpacerItem(15, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spc2)
        self.thumbnail_grv = QGraphicsView(self)
        self.thumbnail_grv.setFixedSize(QSize(100, 100))
        self.thumbnail_grv.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.thumbnail_grv.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        brush = QBrush(QColor(0, 0, 0))
        brush.setStyle(Qt.SolidPattern)
        self.thumbnail_grv.setBackgroundBrush(brush)
        self.horizontalLayout.addWidget(self.thumbnail_grv)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QHBoxLayout()
        self.selected_lbl = QLabel(self)
        self.horizontalLayout_2.addWidget(self.selected_lbl)
        self.show_btn = QPushButton(self)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/source_36x32.png"), QIcon.Normal, 
                                QIcon.Off)
        self.show_btn.setIcon(icon)
        self.horizontalLayout_2.addWidget(self.show_btn)
        spc3 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spc3)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.table_tbv = TableWidget(self)
        self.verticalLayout_2.addWidget(self.table_tbv)
        self.table_tbv.hide()
        self.horizontalLayout_3 = QHBoxLayout()
        spc4 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spc4)
        self.apply_btn = QPushButton(self)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/apply.png"), QIcon.Normal, QIcon.Off)
        self.apply_btn.setIcon(icon1)
        self.horizontalLayout_3.addWidget(self.apply_btn)
        self.cancel_btn = QPushButton(self)
        icon2 = QIcon()
        icon2.addPixmap(QPixmap("medias/close.png"), QIcon.Normal, QIcon.Off)
        self.cancel_btn.setIcon(icon2)
        self.horizontalLayout_3.addWidget(self.cancel_btn)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_4.addLayout(self.verticalLayout_2)
        self.gridLayout.addLayout(self.horizontalLayout_4, 0, 0, 1, 1)
        fname = picture.base
        self.title_lbl.setText(_("Copy metadata from image {}".format(fname)))
        self.selected_lbl.setText(_("{} files selected".format(number)))
        self.show_btn.setText(_("Show"))
        self.apply_btn.setText(_("Apply"))
        self.cancel_btn.setText(_("Cancel"))
        self.show_btn.clicked.connect(self.toggle_table_view)
        self.cancel_btn.clicked.connect(self.reject)
        self.apply_btn.clicked.connect(self.accept)
        self.scene = QGraphicsScene()
        self.thumbnail_grv.setScene(self.scene)
        self.set_thumbnail(picture)

    def set_thumbnail(self, picture):
        th = self.driver.utils.get_thumb_name(picture.uri)
        img = QImage(th)
        img = img.scaled(100, 100, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.thumbnail = QPixmap.fromImage(img)
        self.scene.setSceneRect(QRectF(self.thumbnail.rect()))
        self.scene.addPixmap(self.thumbnail)

    def toggle_table_view(self):
        if self.table_tbv.isVisible():
            self.table_tbv.hide()
            self.resize(350, 150)
            self.show_btn.setText(_("Show"))

        else:
            self.resize(350, 350)
            self.table_tbv.show()
            self.driver.build_table()
            self.show_btn.setText(_("Hide"))



