# -*- coding: utf-8 -*-

# shotsettings.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2015-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Exposure settings widget

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QFont
from PyQt5.QtWidgets import (QScrollArea, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QLabel, QSizePolicy, QSpacerItem, QFrame)

import stylesheets as style


class ShotSettings(QScrollArea):
    def __init__(self, editor, parent=None):
        super().__init__(parent)
        self.gridLayout_1 = QGridLayout(self)
        self.verticalLayout_1 = QVBoxLayout()
        self.horizontalLayout = QHBoxLayout()
        self.label = QLabel(self)
        font = QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setItalic(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.horizontalLayout.addWidget(self.label)
        self.verticalLayout = QVBoxLayout()
        spacer = QSpacerItem(20, 12, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout.addItem(spacer)
        self.line = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line.sizePolicy().hasHeightForWidth())
        self.line.setSizePolicy(sizePolicy)
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)
        self.verticalLayout.addWidget(self.line)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.verticalLayout_1.addLayout(self.horizontalLayout)
        self.horizontalLayout_1 = QHBoxLayout()
        spacer1 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_1.addItem(spacer1)
        alignment = Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter
        self.gridLayout = QGridLayout()
        self.iso_lbl = QLabel(self)
        self.iso_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.iso_lbl, 0, 0, 1, 1)
        self.iso_value_lbl = QLabel(self)
        self.iso_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.iso_value_lbl, 0, 1, 1, 1)
        self.shutter_lbl = QLabel(self)
        self.shutter_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.shutter_lbl, 1, 0, 1, 1)
        self.shutter_value_lbl = QLabel(self)
        self.shutter_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.shutter_value_lbl, 1, 1, 1, 1)
        self.fnumber_lbl = QLabel(self)
        self.fnumber_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.fnumber_lbl, 2, 0, 1, 1)
        self.fnumber_value_lbl = QLabel(self)
        self.fnumber_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.fnumber_value_lbl, 2, 1, 1, 1)
        self.focal_lbl = QLabel(self)
        self.focal_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.focal_lbl, 3, 0, 1, 1)
        self.focal_value_lbl = QLabel(self)
        self.focal_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.focal_value_lbl, 3, 1, 1, 1)
        self.program_lbl = QLabel(self)
        self.program_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.program_lbl, 4, 0, 1, 1)
        self.program_value_lbl = QLabel(self)
        self.program_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.program_value_lbl, 4, 1, 1, 1)
        self.measure_lbl = QLabel(self)
        self.measure_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.measure_lbl, 5, 0, 1, 1)
        self.measure_value_lbl = QLabel(self)
        self.measure_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.measure_value_lbl, 5, 1, 1, 1)
        self.exposure_lbl = QLabel(self)
        self.exposure_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.exposure_lbl, 6, 0, 1, 1)
        self.exposure_value_lbl = QLabel(self)
        self.exposure_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.exposure_value_lbl, 6, 1, 1, 1)
        self.correction_lbl = QLabel(self)
        self.correction_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.correction_lbl, 7, 0, 1, 1)
        self.correction_value_lbl = QLabel(self)
        self.correction_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.correction_value_lbl, 7, 1, 1, 1)
        self.whitebalance_lbl = QLabel(self)
        self.whitebalance_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.whitebalance_lbl, 8, 0, 1, 1)
        self.whitebalance_value_lbl = QLabel(self)
        self.whitebalance_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.whitebalance_value_lbl, 8, 1, 1, 1)
        self.lightsource_lbl = QLabel(self)
        self.lightsource_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.lightsource_lbl, 9, 0, 1, 1)
        self.lightsource_value_lbl = QLabel(self)
        self.lightsource_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.lightsource_value_lbl, 9, 1, 1, 1)
        self.contrast_lbl = QLabel(self)
        self.contrast_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.contrast_lbl, 10, 0, 1, 1)
        self.contrast_value_lbl = QLabel(self)
        self.contrast_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.contrast_value_lbl, 10, 1, 1, 1)
        self.saturation_lbl = QLabel(self)
        self.saturation_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.saturation_lbl, 11, 0, 1, 1)
        self.saturation_value_lbl = QLabel(self)
        self.saturation_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.saturation_value_lbl, 11, 1, 1, 1)
        self.sharpness_lbl = QLabel(self)
        self.sharpness_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.sharpness_lbl, 12, 0, 1, 1)
        self.sharpness_value_lbl = QLabel(self)
        self.sharpness_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.sharpness_value_lbl, 12, 1, 1, 1)
        self.flash_lbl = QLabel(self)
        self.flash_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.flash_lbl, 13, 0, 1, 1)
        self.flash_value_lbl = QLabel(self)
        self.flash_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.flash_value_lbl, 13, 1, 1, 1)
        self.subjectdis_lbl = QLabel(self)
        self.subjectdis_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.subjectdis_lbl, 14, 0, 1, 1)
        self.subjectdis_value_lbl = QLabel(self)
        self.subjectdis_value_lbl.setText("  - -")
        self.gridLayout.addWidget(self.subjectdis_value_lbl, 14, 1, 1, 1)
        self.horizontalLayout_1.addLayout(self.gridLayout)
        spacer6 = QSpacerItem(40, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_1.addItem(spacer6)
        self.verticalLayout_1.addLayout(self.horizontalLayout_1)
        spacer4 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout_1.addItem(spacer4)
        self.gridLayout_1.addLayout(self.verticalLayout_1, 0, 0, 1, 1)

        self.retranslateUi()
        self.set_style()

    def retranslateUi(self):
        self.label.setText(_("Exposure settings"))
        self.iso_lbl.setText(_("ISO:"))
        self.shutter_lbl.setText(_("Shutter speed:"))
        self.fnumber_lbl.setText("f-Number:")
        self.focal_lbl.setText(_("Focal length:"))
        self.program_lbl.setText(_("Program mode:"))
        self.measure_lbl.setText(_("Measuring mode:"))
        self.exposure_lbl.setText(_("Exposure mode:"))
        self.correction_lbl.setText(_("Exposure correction:"))
        self.whitebalance_lbl.setText(_("White balance:"))
        self.lightsource_lbl.setText(_("Light source:"))
        self.contrast_lbl.setText(_("Contrast:"))
        self.saturation_lbl.setText(_("Saturation:"))
        self.sharpness_lbl.setText(_("Sharpness:"))
        self.flash_lbl.setText(_("Flash:"))
        self.subjectdis_lbl.setText(_("Subject distance:"))

    def set_style(self):
        self.label.setStyleSheet(style.SUBTITLE)
        self.line.setStyleSheet(style.UNDERLINE)




