# -*- coding: utf-8 -*-

# xmppage.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2015-2016 Vincent Vande Vyvre
# Licence: GPL3
#
# XMP page widget

import re

from PyQt5.QtCore import Qt, QSize, QRect
from PyQt5.QtGui import QFont, QIcon, QPixmap, QTextCursor
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QLabel, QTabWidget, QScrollArea, QSizePolicy,
                             QFrame, QSpacerItem, QCheckBox, QPlainTextEdit,
                             QLineEdit, QComboBox, QPushButton, QTextEdit,
                             QTreeWidget, QCompleter)

class XMPPage(QWidget):
    def __init__(self, ui, parent=None):
        super().__init__(parent)
        self.ui = ui
        self.gridLayout_5 = QGridLayout(self)
        self.verticalLayout_25 = QVBoxLayout()
        self.xmp_tab = QTabWidget(self)
        self.author_tab = QWidget()
        self.verticalLayout_4 = QVBoxLayout(self.author_tab)
        self.verticalLayout_4.setContentsMargins(0, 2, 2, 2)
        self.horizontalLayout_3 = QHBoxLayout()
        spc = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spc)
        self.gridLayout = QGridLayout()
        alignment = Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter
        self.author_lbl = QLabel(self.author_tab)
        self.author_lbl.setAlignment(alignment)
        self.gridLayout.addWidget(self.author_lbl, 0, 0, 1, 1)
        self.horizontalLayout_2 = QHBoxLayout()
        self.author_led = QLineEdit(self.author_tab)
        self.horizontalLayout_2.addWidget(self.author_led)
        self.author_value_lbl = QLabel(self.author_tab)
        self.horizontalLayout_2.addWidget(self.author_value_lbl)
        self.gridLayout.addLayout(self.horizontalLayout_2, 0, 1, 1, 1)
        #self.rights_status_lbl = QLabel(self.author_tab)
        #self.rights_status_lbl.setAlignment(alignment)
        #self.gridLayout.addWidget(self.rights_status_lbl, 1, 0, 1, 1)
        #self.horizontalLayout = QHBoxLayout()
        #self.rights_cmb = QComboBox(self.author_tab)
        #self.rights_cmb.setMinimumSize(QSize(150, 0))
        #self.rights_cmb.addItems(['Uncopyrighted', 'Copyrighted'])
        #self.horizontalLayout.addWidget(self.rights_cmb)
        #spc1 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Preferred)
        #self.horizontalLayout.addItem(spc1)
        #self.gridLayout.addLayout(self.horizontalLayout, 1, 1, 1, 1)
        self.verticalLayout = QVBoxLayout()
        self.rights_lbl = QLabel(self.author_tab)
        self.rights_lbl.setAlignment(alignment)
        self.verticalLayout.addWidget(self.rights_lbl)
        spc2 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout.addItem(spc2)
        self.gridLayout.addLayout(self.verticalLayout, 2, 0, 1, 1)
        self.rights_ted = TextEdit(self.author_tab)
        self.rights_ted.setMaximumSize(QSize(16777215, 80))
        self.gridLayout.addWidget(self.rights_ted, 2, 1, 1, 1)

        self.verticalLayout_30 = QVBoxLayout()
        self.usage_lbl = QLabel(self.author_tab)
        self.usage_lbl.setAlignment(alignment)
        self.verticalLayout_30.addWidget(self.usage_lbl)
        spc20 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout_30.addItem(spc20)
        self.gridLayout.addLayout(self.verticalLayout_30, 3, 0, 1, 1)
        self.usage_ted = QPlainTextEdit(self.author_tab)
        self.usage_ted.setMaximumSize(QSize(16777215, 80))
        self.gridLayout.addWidget(self.usage_ted, 3, 1, 1, 1)

        self.verticalLayout_2 = QVBoxLayout()
        self.contact_lbl = QLabel(self.author_tab)
        self.contact_lbl.setAlignment(alignment)
        self.verticalLayout_2.addWidget(self.contact_lbl)
        self.contact_copy_btn = QPushButton(self.author_tab)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/gtk-edit.svg"), QIcon.Normal, QIcon.Off)
        self.contact_copy_btn.setIcon(icon)
        self.verticalLayout_2.addWidget(self.contact_copy_btn)
        spc3 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spc3)
        self.gridLayout.addLayout(self.verticalLayout_2, 4, 0, 1, 1)
        self.author_contact_trw = QTreeWidget(self.author_tab)
        self.author_contact_trw.setIndentation(2)
        self.author_contact_trw.setUniformRowHeights(True)
        self.author_contact_trw.setColumnCount(2)
        self.author_contact_trw.setHeaderHidden(True)
        self.author_contact_trw.setColumnWidth(0, 100)
        self.author_contact_trw.setMinimumSize(250, 220)
        self.author_contact_trw.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.gridLayout.addWidget(self.author_contact_trw, 4, 1, 1, 1)
        spc4 = QSpacerItem(20, 20, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.horizontalLayout_3.addLayout(self.gridLayout)
        self.verticalLayout_4.addLayout(self.horizontalLayout_3)

        self.verticalLayout_4.addItem(spc4)
        self.xmp_tab.addTab(self.author_tab, "")
        self.content_tab = QWidget()
        self.verticalLayout_9 = QVBoxLayout(self.content_tab)
        self.verticalLayout_9.setContentsMargins(0, 2, 2, 2)
        self.scrollArea = QScrollArea(self.content_tab)
        self.scrollArea.setWidgetResizable(True)
        self.scrollWidget = QWidget()
        self.verticalLayout_8 = QVBoxLayout(self.scrollWidget)
        self.verticalLayout_8.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_7 = QVBoxLayout()
        self.horizontalLayout_4 = QHBoxLayout()
        spc5 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spc5)
        self.gridLayout_2 = QGridLayout()
        self.title_lbl = QLabel(self.scrollWidget)
        self.title_lbl.setAlignment(alignment)
        self.gridLayout_2.addWidget(self.title_lbl, 0, 0, 1, 1)
        self.title_led = QLineEdit(self.scrollWidget)
        self.gridLayout_2.addWidget(self.title_led, 0, 1, 1, 1)
        self.verticalLayout_5 = QVBoxLayout()
        self.keywords_lbl_2 = QLabel(self.scrollWidget)
        self.keywords_lbl_2.setAlignment(alignment)
        self.verticalLayout_5.addWidget(self.keywords_lbl_2)
        spc6 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout_5.addItem(spc6)
        self.gridLayout_2.addLayout(self.verticalLayout_5, 1, 0, 1, 1)
        self.subject_ted = QPlainTextEdit(self.scrollWidget)
        self.gridLayout_2.addWidget(self.subject_ted, 1, 1, 1, 1)
        self.verticalLayout_6 = QVBoxLayout()
        self.keywords_lbl = QLabel(self.scrollWidget)
        self.keywords_lbl.setAlignment(alignment)
        self.verticalLayout_6.addWidget(self.keywords_lbl)
        spc7 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout_6.addItem(spc7)
        self.gridLayout_2.addLayout(self.verticalLayout_6, 2, 0, 1, 1)
        self.keywords_ted = QPlainTextEdit(self.scrollWidget)
        self.gridLayout_2.addWidget(self.keywords_ted, 2, 1, 1, 1)
        """
        self.place_lbl = QLabel(self.scrollWidget)
        self.place_lbl.setAlignment(alignment)
        self.gridLayout_2.addWidget(self.place_lbl, 3, 0, 1, 1)
        self.place_led = QLineEdit(self.scrollWidget)
        self.gridLayout_2.addWidget(self.place_led, 3, 1, 1, 1)
        self.city_lbl = QLabel(self.scrollWidget)
        self.city_lbl.setAlignment(alignment)
        self.gridLayout_2.addWidget(self.city_lbl, 4, 0, 1, 1)
        self.city_led = QLineEdit(self.scrollWidget)
        self.gridLayout_2.addWidget(self.city_led, 4, 1, 1, 1)
        self.state_lbl = QLabel(self.scrollWidget)
        self.state_lbl.setAlignment(alignment)
        self.gridLayout_2.addWidget(self.state_lbl, 5, 0, 1, 1)
        self.state_led = QLineEdit(self.scrollWidget)
        self.gridLayout_2.addWidget(self.state_led, 5, 1, 1, 1)
        self.country_lbl = QLabel(self.scrollWidget)
        self.country_lbl.setAlignment(alignment)
        self.gridLayout_2.addWidget(self.country_lbl, 6, 0, 1, 1)
        self.country_led = QLineEdit(self.scrollWidget)
        self.gridLayout_2.addWidget(self.country_led, 6, 1, 1, 1)
        self.country_code_lbl = QLabel(self.scrollWidget)
        self.country_code_lbl.setAlignment(alignment)
        self.gridLayout_2.addWidget(self.country_code_lbl, 7, 0, 1, 1)
        self.country_code_value_lbl = QLabel(self.scrollWidget)
        self.gridLayout_2.addWidget(self.country_code_value_lbl, 7, 1, 1, 1)
        """
        self.horizontalLayout_4.addLayout(self.gridLayout_2)
        self.verticalLayout_7.addLayout(self.horizontalLayout_4)
        self.verticalLayout_8.addLayout(self.verticalLayout_7)
        self.scrollArea.setWidget(self.scrollWidget)
        self.verticalLayout_9.addWidget(self.scrollArea)
        self.xmp_tab.addTab(self.content_tab, "")
        self.people_tab = QWidget()
        self.verticalLayout_18 = QVBoxLayout(self.people_tab)
        self.verticalLayout_18.setContentsMargins(2, 0, 0, 0)
        self.scrollArea_2 = QScrollArea(self.people_tab)
        self.scrollArea_2.setWidgetResizable(True)
        self.scrollWidget_2 = QWidget()
        self.verticalLayout_17 = QVBoxLayout(self.scrollWidget_2)
        self.verticalLayout_17.setContentsMargins(2, 0, 0, 0)
        self.verticalLayout_16 = QVBoxLayout()
        self.horizontalLayout_5 = QHBoxLayout()
        self.models_lbl = QLabel(self.scrollWidget_2)
        font = QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setItalic(True)
        font.setWeight(75)
        self.models_lbl.setFont(font)
        self.horizontalLayout_5.addWidget(self.models_lbl)
        self.verticalLayout_10 = QVBoxLayout()
        spc9 = QSpacerItem(20, 12, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_10.addItem(spc9)
        self.line = QFrame(self.scrollWidget_2)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line.sizePolicy().hasHeightForWidth())
        self.line.setSizePolicy(sizePolicy)
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_10.addWidget(self.line)
        self.horizontalLayout_5.addLayout(self.verticalLayout_10)
        self.verticalLayout_16.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_7 = QHBoxLayout()
        spc10 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spc10)
        self.gridLayout_3 = QGridLayout()
        self.verticalLayout_12 = QVBoxLayout()
        self.names_lbl = QLabel(self.scrollWidget_2)
        self.names_lbl.setAlignment(alignment)
        self.verticalLayout_12.addWidget(self.names_lbl)
        spc11 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout_12.addItem(spc11)
        self.gridLayout_3.addLayout(self.verticalLayout_12, 0, 0, 1, 1)
        self.names_ted = QPlainTextEdit(self.scrollWidget_2)
        self.names_ted.setMaximumSize(QSize(16777215, 150))
        self.gridLayout_3.addWidget(self.names_ted, 0, 1, 1, 1)
        self.ages_lbl = QLabel(self.scrollWidget_2)
        self.ages_lbl.setAlignment(alignment)
        self.gridLayout_3.addWidget(self.ages_lbl, 1, 0, 1, 1)
        self.ages_led = QLineEdit(self.scrollWidget_2)
        self.gridLayout_3.addWidget(self.ages_led, 1, 1, 1, 1)
        self.minor_ages_lbl = QLabel(self.scrollWidget_2)
        self.minor_ages_lbl.setAlignment(alignment)
        self.gridLayout_3.addWidget(self.minor_ages_lbl, 2, 0, 1, 1)
        self.hlayout = QHBoxLayout()
        self.minor_ages_cmb = QComboBox(self.scrollWidget_2)
        self.hlayout.addWidget(self.minor_ages_cmb)
        spc22 = QSpacerItem(20, 80, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.hlayout.addItem(spc22)
        self.gridLayout_3.addLayout(self.hlayout, 2, 1, 1, 1)
        self.verticalLayout_13 = QVBoxLayout()
        self.add_infos_lbl = QLabel(self.scrollWidget_2)
        self.add_infos_lbl.setAlignment(alignment)
        self.verticalLayout_13.addWidget(self.add_infos_lbl)
        spc12 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout_13.addItem(spc12)
        self.gridLayout_3.addLayout(self.verticalLayout_13, 3, 0, 1, 1)
        self.add_infos_ted = QPlainTextEdit(self.scrollWidget_2)
        self.add_infos_ted.setMaximumSize(QSize(16777215, 100))
        self.gridLayout_3.addWidget(self.add_infos_ted, 3, 1, 1, 1)
        self.horizontalLayout_7.addLayout(self.gridLayout_3)
        self.verticalLayout_16.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_6 = QHBoxLayout()
        self.artwork_lbl = QLabel(self.scrollWidget_2)
        self.artwork_lbl.setFont(font)
        self.horizontalLayout_6.addWidget(self.artwork_lbl)
        self.verticalLayout_11 = QVBoxLayout()
        spc13 = QSpacerItem(20, 12, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_11.addItem(spc13)
        self.line_2 = QFrame(self.scrollWidget_2)
        self.line_2.setSizePolicy(sizePolicy)
        self.line_2.setFrameShape(QFrame.HLine)
        self.line_2.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_11.addWidget(self.line_2)
        self.horizontalLayout_6.addLayout(self.verticalLayout_11)
        self.verticalLayout_16.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_8 = QHBoxLayout()
        spc14 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spc14)
        self.gridLayout_4 = QGridLayout()
        self.verticalLayout_14 = QVBoxLayout()
        self.artwork_desc_lbl = QLabel(self.scrollWidget_2)
        self.artwork_desc_lbl.setAlignment(alignment)
        self.verticalLayout_14.addWidget(self.artwork_desc_lbl)
        spc15 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout_14.addItem(spc15)
        self.gridLayout_4.addLayout(self.verticalLayout_14, 0, 0, 1, 1)
        self.artwork_desc_ted = QPlainTextEdit(self.scrollWidget_2)
        self.artwork_desc_ted.setMaximumSize(QSize(16777215, 100))
        self.gridLayout_4.addWidget(self.artwork_desc_ted, 0, 1, 1, 1)
        self.verticalLayout_15 = QVBoxLayout()
        self.artwork_copyright_lbl = QLabel(self.scrollWidget_2)
        self.verticalLayout_15.addWidget(self.artwork_copyright_lbl)
        spc16 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout_15.addItem(spc16)
        self.gridLayout_4.addLayout(self.verticalLayout_15, 1, 0, 1, 1)
        self.artwork_copyright_ted = QPlainTextEdit(self.scrollWidget_2)
        self.artwork_copyright_ted.setMaximumSize(QSize(16777215, 100))
        self.gridLayout_4.addWidget(self.artwork_copyright_ted, 1, 1, 1, 1)
        self.horizontalLayout_8.addLayout(self.gridLayout_4)
        self.verticalLayout_16.addLayout(self.horizontalLayout_8)
        spc17 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout_16.addItem(spc17)
        self.verticalLayout_17.addLayout(self.verticalLayout_16)
        self.scrollArea_2.setWidget(self.scrollWidget_2)
        self.verticalLayout_18.addWidget(self.scrollArea_2)
        self.xmp_tab.addTab(self.people_tab, "")
        self.other_tab = QWidget()
        self.verticalLayout_24 = QVBoxLayout(self.other_tab)
        self.verticalLayout_24.setContentsMargins(2, 0, 0, 0)
        self.scrollArea_3 = QScrollArea(self.other_tab)
        self.scrollArea_3.setWidgetResizable(True)
        self.scrollWidget_3 = QWidget()
        self.verticalLayout_23 = QVBoxLayout(self.scrollWidget_3)
        self.verticalLayout_23.setContentsMargins(2, 0, 0, 0)
        self.verticalLayout_22 = QVBoxLayout()
        self.horizontalLayout_9 = QHBoxLayout()
        self.getty_lbl = QLabel(self.scrollWidget_3)
        self.getty_lbl.setFont(font)
        self.getty_lbl.setText("Getty Images")
        self.horizontalLayout_9.addWidget(self.getty_lbl)
        self.verticalLayout_19 = QVBoxLayout()
        spc18 = QSpacerItem(40, 12, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_19.addItem(spc18)
        self.line_3 = QFrame(self.scrollWidget_3)
        self.line_3.setSizePolicy(sizePolicy)
        self.line_3.setFrameShape(QFrame.HLine)
        self.line_3.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_19.addWidget(self.line_3)
        self.horizontalLayout_9.addLayout(self.verticalLayout_19)
        self.verticalLayout_22.addLayout(self.horizontalLayout_9)
        self.horizontalLayout_12 = QHBoxLayout()
        spc19 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(spc19)
        self.getty_tbl = QTreeWidget(self.scrollWidget_3)
        #self.getty_tbl.setMaximumSize(QSize(16777215, 150))
        self.getty_tbl.setColumnCount(2)
        self.getty_tbl.setIndentation(2)
        self.getty_tbl.setUniformRowHeights(True)
        self.getty_tbl.setHeaderHidden(True)
        self.getty_tbl.setColumnWidth(0, 150)
        self.horizontalLayout_12.addWidget(self.getty_tbl)
        self.verticalLayout_22.addLayout(self.horizontalLayout_12)
        self.horizontalLayout_10 = QHBoxLayout()
        self.oqapy_lbl = QLabel(self.scrollWidget_3)
        self.oqapy_lbl.setFont(font)
        self.oqapy_lbl.setText("Oqapy")
        self.horizontalLayout_10.addWidget(self.oqapy_lbl)
        self.verticalLayout_20 = QVBoxLayout()
        spc20 = QSpacerItem(40, 12, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_20.addItem(spc20)
        self.line_4 = QFrame(self.scrollWidget_3)
        self.line_4.setSizePolicy(sizePolicy)
        self.line_4.setFrameShape(QFrame.HLine)
        self.line_4.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_20.addWidget(self.line_4)
        self.horizontalLayout_10.addLayout(self.verticalLayout_20)
        self.verticalLayout_22.addLayout(self.horizontalLayout_10)
        self.horizontalLayout_13 = QHBoxLayout()
        spc21 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_13.addItem(spc21)
        self.oqapy_tbl = QTreeWidget(self.scrollWidget_3)
        self.oqapy_tbl.setColumnCount(2)
        self.oqapy_tbl.setIndentation(2)
        self.oqapy_tbl.setUniformRowHeights(True)
        self.oqapy_tbl.setHeaderHidden(True)
        self.oqapy_tbl.setColumnWidth(0, 150)
        self.horizontalLayout_13.addWidget(self.oqapy_tbl)
        self.verticalLayout_22.addLayout(self.horizontalLayout_13)
        spc24 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Preferred)
        self.verticalLayout_22.addItem(spc24)
        self.verticalLayout_23.addLayout(self.verticalLayout_22)

        self.scrollArea_3.setWidget(self.scrollWidget_3)
        self.verticalLayout_24.addWidget(self.scrollArea_3)
        self.xmp_tab.addTab(self.other_tab, "")
        self.verticalLayout_25.addWidget(self.xmp_tab)
        self.horizontalLayout_15 = QHBoxLayout()
        self.desc_chb = QCheckBox(self)
        self.desc_chb.setChecked(True)
        self.horizontalLayout_15.addWidget(self.desc_chb)
        spc25 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_15.addItem(spc25)
        self.verticalLayout_25.addLayout(self.horizontalLayout_15)
        self.description_ted = QPlainTextEdit(self)
        self.description_ted.setMaximumSize(QSize(16777215, 80))
        self.description_ted.setReadOnly(True)
        self.verticalLayout_25.addWidget(self.description_ted)
        self.gridLayout_5.addLayout(self.verticalLayout_25, 0, 0, 1, 1)

        self.retranslateUi()
        self.minor_ages_cmb.addItems(['Unknow', '25 or Over', '24', '23', '22',
                                      '21', '20', '19', '18', '17', '16', '15',
                                      '14 or Under'])
        self.author_value_lbl.hide()
        self.xmp_tab.setCurrentIndex(0)

    def connect_signals(self):
        self.author_led.focusInEvent = self.set_author_desc
        self.author_led.editingFinished.connect(self.on_author_edited)
        #self.rights_cmb.focusInEvent = self.set_marked_desc
        self.rights_ted.focusInEvent = self.set_rights_desc
        self.rights_ted.focusOutEvent = self.on_rights_edited
        #self.rights_ted.textChanged.connect(self.on_rights_edited)
        self.usage_ted.focusInEvent = self.set_usage_desc
        self.usage_ted.focusOutEvent = self.on_usage_edited
        #self.usage_ted.textChanged.connect(self.on_usage_edited)
        self.contact_copy_btn.clicked.connect(self.on_contact_copy_clicked)
        self.author_contact_trw.focusInEvent = self.on_contact_activated
        self.title_led.focusInEvent = self.set_title_desc
        self.title_led.editingFinished.connect(self.on_title_edited)
        self.subject_ted.focusInEvent = self.set_subject_desc
        self.subject_ted.focusOutEvent = self.on_subject_edited
        #self.subject_ted.textChanged.connect(self.on_subject_edited)
        self.keywords_ted.focusInEvent = self.set_keywords_desc
        self.keywords_ted.focusOutEvent = self.on_keywords_edited
        #self.keywords_ted.textChanged.connect(self.on_keywords_edited)
        self.names_ted.focusInEvent = self.set_models_desc
        self.names_ted.focusOutEvent = self.on_names_edited
        #self.names_ted.textChanged.connect(self.on_names_edited)
        self.ages_led.focusInEvent = self.set_ages_desc
        self.ages_led.editingFinished.connect(self.on_ages_edited)
        self.minor_ages_cmb.focusInEvent = self.set_minor_ages_desc
        self.minor_ages_cmb.currentIndexChanged.connect(self.on_minor_ages_changed)
        self.add_infos_ted.focusInEvent = self.set_models_infos_desc
        self.add_infos_ted.focusOutEvent = self.on_models_infos_edited
        #self.add_infos_ted.textChanged.connect(self.on_models_infos_edited)
        self.artwork_desc_ted.focusInEvent = self.set_artwork_desc
        self.artwork_desc_ted.focusOutEvent = self.on_artwork_edited
        #self.artwork_desc_ted.textChanged.connect(self.on_artwork_edited)
        self.artwork_copyright_ted.focusInEvent = self.set_artwork_rights_desc
        self.artwork_copyright_ted.focusOutEvent = self.on_artwork_rights_edited
        #self.artwork_copyright_ted.textChanged.connect(self.on_artwork_rights_edited)
        self.desc_chb.stateChanged.connect(self.display_description)

    def on_author_edited(self):
        txt = str(self.author_led.text())
        self.ui.main.update_data('xmp', 'author', txt)
        self.ui.author_name = txt

    def on_rights_edited(self, event):
        txt = str(self.rights_ted.toPlainText()).rstrip()
        self.ui.main.update_data('xmp', 'rights', txt)
        QTextEdit.focusOutEvent(self.rights_ted, event)

    def on_usage_edited(self, event):
        txt = str(self.usage_ted.toPlainText()).rstrip()
        self.ui.main.update_data('xmp', 'terms', txt)
        QPlainTextEdit.focusOutEvent(self.usage_ted, event)

    def on_title_edited(self):
        txt = str(self.title_led.text())
        self.ui.main.update_data('xmp', 'title', txt)

    def on_subject_edited(self, event):
        txt = str(self.subject_ted.toPlainText()).rstrip()
        self.ui.main.update_data('xmp', 'subject', txt)
        QPlainTextEdit.focusOutEvent(self.subject_ted, event)

    def on_keywords_edited(self, event):
        txt = str(self.keywords_ted.toPlainText()).rstrip()
        self.ui.main.update_data('xmp', 'kwords', txt)
        QPlainTextEdit.focusOutEvent(self.keywords_ted, event)

    def on_names_edited(self, event):
        txt = str(self.names_ted.toPlainText()).rstrip()
        self.ui.main.update_data('xmp', 'people', txt)
        QPlainTextEdit.focusOutEvent(self.names_ted, event)

    def on_ages_edited(self):
        l = str(self.ages_led.text())
        numbs = re.findall('[0-9]*', l)
        ages = []
        for n in numbs:
            try:
                a = int(n)
                if a > 24:
                    ages.append(a)
            except:
                pass
        self.ui.main.update_data('xmp', 'ages', ages)

    def on_minor_ages_changed(self, idx):
        self.ui.main.update_data('xmp', 'minors', idx)

    def on_models_infos_edited(self, event):
        txt = str(self.add_infos_ted.toPlainText()).rstrip()
        self.ui.main.update_data('xmp', 'others', txt)
        QPlainTextEdit.focusOutEvent(self.add_infos_ted, event)

    def on_artwork_edited(self, event):
        txt = str(self.artwork_desc_ted.toPlainText()).rstrip()
        self.ui.main.update_data('xmp', 'artworks', txt)
        QPlainTextEdit.focusOutEvent(self.artwork_desc_ted, event)

    def on_artwork_rights_edited(self, event):
        txt = str(self.artwork_copyright_ted.toPlainText()).rstrip()
        self.ui.main.update_data('xmp', 'artrights', txt)
        QPlainTextEdit.focusOutEvent(self.artwork_copyright_ted, event)

    def set_author_desc(self, event):
        desc = self.ui.main.get_tag_description('xmp', 'author')
        self.show_description(desc)
        QLineEdit.focusInEvent(self.author_led, event)

    def set_marked_desc(self, event):
        desc = self.ui.main.get_tag_description('xmp', 'marked')
        self.show_description(desc)
        QComboBox.focusInEvent(self.rights_cmb, event)

    def set_rights_desc(self, event):
        desc = self.ui.main.get_tag_description('xmp', 'rights')
        self.show_description(desc)
        TextEdit.focusInEvent(self.rights_ted, event)

    def set_usage_desc(self, event):
        desc = self.ui.main.get_tag_description('xmp', 'terms')
        self.show_description(desc)
        QPlainTextEdit.focusInEvent(self.usage_ted, event)

    def on_contact_copy_clicked(self, event):
        self.ui.copy_contact_infos()

    def on_contact_activated(self, event):
        pass

    def set_title_desc(self, event):
        desc = self.ui.main.get_tag_description('xmp', 'title')
        self.show_description(desc)
        QLineEdit.focusInEvent(self.title_led, event)

    def set_subject_desc(self, event):
        desc = self.ui.main.get_tag_description('xmp', 'subject')
        self.show_description(desc)
        QPlainTextEdit.focusInEvent(self.subject_ted, event)

    def set_keywords_desc(self, event):
        desc = self.ui.main.get_tag_description('xmp', 'kwords')
        self.show_description(desc)
        QPlainTextEdit.focusInEvent(self.keywords_ted, event)

    def set_models_desc(self, event):
        desc = self.ui.main.get_tag_description('xmp', 'people')
        self.show_description(desc)
        QPlainTextEdit.focusInEvent(self.names_ted, event)

    def set_ages_desc(self, event):
        desc = self.ui.main.get_tag_description('xmp', 'ages')
        self.show_description(desc)
        QLineEdit.focusInEvent(self.ages_led, event)

    def set_minor_ages_desc(self, event):
        desc = self.ui.main.get_tag_description('xmp', 'minors')
        self.show_description(desc)
        QComboBox.focusInEvent(self.minor_ages_cmb, event)

    def set_models_infos_desc(self, event):
        desc = self.ui.main.get_tag_description('xmp', 'others')
        self.show_description(desc)
        QPlainTextEdit.focusInEvent(self.add_infos_ted, event)

    def set_artwork_desc(self, event):
        desc = self.ui.main.get_tag_description('xmp', 'artworks')
        self.show_description(desc)
        QPlainTextEdit.focusInEvent(self.artwork_desc_ted, event)

    def set_artwork_rights_desc(self, event):
        desc = self.ui.main.get_tag_description('xmp', 'artrights')
        self.show_description(desc)
        QPlainTextEdit.focusInEvent(self.artwork_copyright_ted, event)

    def show_description(self, desc):
        self.description_ted.clear()
        self.description_ted.appendPlainText(desc)

    def display_description(self, state):
        self.description_ted.setVisible(state)

    def retranslateUi(self):
        self.author_lbl.setText(_("Author:"))
        self.author_value_lbl.setText(_("Author's name"))
        self.rights_lbl.setText(_("Copyrights:"))
        self.usage_lbl.setText("Usage terms:")
        #self.rights_status_lbl.setText(_("Copyrights status:"))
        self.contact_lbl.setText(_("Contact infos:"))
        self.contact_copy_btn.setToolTip(_("Copy into the clipboard"))
        self.contact_copy_btn.setText(_("Copy"))
        self.xmp_tab.setTabText(0, _("XMP - Author"))
        self.title_lbl.setText(_("Title:"))
        self.keywords_lbl_2.setText(_("Subject:"))
        self.keywords_lbl.setText(_("Keywords:"))
        #self.place_lbl.setText(_("Place:"))
        #self.city_lbl.setText(_("City:"))
        #self.state_lbl.setText(_("State-province:"))
        #self.country_lbl.setText(_("Country:"))
        #self.country_code_lbl.setText(_("Country code:"))
        self.xmp_tab.setTabText(1, _("XMP - Content"))
        self.models_lbl.setText(_("Persons in image"))
        self.names_lbl.setText(_("Names:"))
        self.ages_lbl.setText(_("Ages:"))
        self.minor_ages_lbl.setText(_("Minor models:"))
        self.add_infos_lbl.setText(_("Other informations:"))
        self.artwork_lbl.setText(_("Artwork"))
        self.artwork_desc_lbl.setText(_("Description:"))
        self.artwork_copyright_lbl.setText(_("Copyright notice:"))
        self.xmp_tab.setTabText(2, _("XMP - People && Artwork"))
        self.xmp_tab.setTabText(3, _("XMP - Others"))
        self.desc_chb.setText(_("Show description"))


class TextEdit(QTextEdit):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.completer = None
        self.moveCursor(QTextCursor.End)

    def setCompleter(self, words):
        if self.completer:
            self.disconnect(self.completer, 0, self, 0)

        completer = QCompleter(words)
        completer.setWidget(self)
        completer.setCompletionMode(QCompleter.PopupCompletion)
        completer.setCaseSensitivity(Qt.CaseInsensitive)
        self.completer = completer
        self.completer.activated.connect(self.insertCompletion)

    def insertCompletion(self, comp):
        tc = self.textCursor()
        extra = (comp.length() - self.completer.completionPrefix().length())
        tc.movePosition(QTextCursor.Left)
        tc.movePosition(QTextCursor.EndOfWord)
        tc.insertText(comp.right(extra))
        self.setTextCursor(tc)

    def textUnderCursor(self):
        tc = self.textCursor()
        tc.select(QTextCursor.WordUnderCursor)
        return tc.selectedText()

    def focusInEvent(self, event):
        if self.completer:
            self.completer.setWidget(self);
        QTextEdit.focusInEvent(self, event)

    def keyPressEvent(self, event):
        if not self.completer:
            return

        if self.completer and self.completer.popup().isVisible():
            if event.key() in (Qt.Key_Enter, Qt.Key_Return, Qt.Key_Escape,
                                Qt.Key_Tab, Qt.Key_Backtab):
                event.ignore()
                return

        text = str(event.text())
        emod = event.modifiers()
        cmod, smod, nmod = Qt.ControlModifier, Qt.ShiftModifier, Qt.NoModifier
        isShortcut = emod == cmod and event.key() == Qt.Key_E
        if not isShortcut:
            QTextEdit.keyPressEvent(self, event)

        ctrlOrShift = emod in (cmod, smod)
        if ctrlOrShift and not text:
            return

        eow = "~!@#$%^&*()_+{}|:\"<>?,./;'[]\\-="
        hasModifier = emod != nmod and not ctrlOrShift
        completionPrefix = self.textUnderCursor()

        if not isShortcut and (hasModifier or not text or
                                len(completionPrefix) < 2 or
                                text[-1] in eow):
            self.completer.popup().hide()
            return

        if (completionPrefix != self.completer.completionPrefix()):
            self.completer.setCompletionPrefix(completionPrefix)
            popup = self.completer.popup()
            popup.setCurrentIndex(
                self.completer.completionModel().index(0,0))

        cr = self.cursorRect()
        cr.setWidth(self.completer.popup().sizeHintForColumn(0) + 
                self.completer.popup().verticalScrollBar().sizeHint().width())
        self.completer.complete(cr)

