# -*- coding: utf-8 -*-

# layoutpage.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2013-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Page layout tool.

from PyQt5 import QtCore, QtGui, QtWidgets

import stylesheets as styles

class LayoutPage(QtWidgets.QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.gridLayout_2 = QtWidgets.QGridLayout(self)
        self.gridLayout_2.setContentsMargins(2, 0, 2, 0)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setSpacing(0)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setVerticalSpacing(2)
        self.label = QtWidgets.QLabel(self)
        self.label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                QtCore.Qt.AlignVCenter)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 2)
        self.paper_size_lbl = QtWidgets.QLabel(self)
        self.paper_size_lbl.setObjectName("paper_size_lbl")
        self.gridLayout.addWidget(self.paper_size_lbl, 0, 2, 1, 2)
        self.label_1 = QtWidgets.QLabel(self)
        self.label_1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                QtCore.Qt.AlignVCenter)
        self.label_1.setObjectName("label_1")
        self.gridLayout.addWidget(self.label_1, 1, 0, 1, 2)
        self.preset_size_cmb = QtWidgets.QComboBox(self)
        self.preset_size_cmb.setObjectName("preset_size_cmb")
        self.gridLayout.addWidget(self.preset_size_cmb, 1, 2, 1, 1)
        self.label_2 = QtWidgets.QLabel(self)
        self.label_2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                    QtCore.Qt.AlignVCenter)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 2, 0, 1, 2)
        self.width_spb = QtWidgets.QSpinBox(self)
        self.width_spb.setMinimum(10)
        self.width_spb.setMaximum(10000)
        self.width_spb.setObjectName("width_spb")
        self.gridLayout.addWidget(self.width_spb, 2, 2, 1, 1)
        self.label_3 = QtWidgets.QLabel(self)
        self.label_3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                    QtCore.Qt.AlignVCenter)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 3, 0, 1, 2)
        self.height_spb = QtWidgets.QSpinBox(self)
        self.height_spb.setMinimum(10)
        self.height_spb.setMaximum(10000)
        self.height_spb.setObjectName("height_spb")
        self.gridLayout.addWidget(self.height_spb, 3, 2, 1, 1)
        self.proportion_chk = QtWidgets.QCheckBox(self)
        self.proportion_chk.setChecked(True)
        self.proportion_chk.setObjectName("proportion_chk")
        self.gridLayout.addWidget(self.proportion_chk, 4, 0, 1, 1)
        self.apply_resize_btn = QtWidgets.QPushButton(self)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("medias/dialog-apply.svg"), 
                                    QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.apply_resize_btn.setIcon(icon)
        self.apply_resize_btn.setIconSize(QtCore.QSize(20, 20))
        self.apply_resize_btn.setEnabled(False)
        self.apply_resize_btn.setObjectName("apply_resize_btn")
        self.gridLayout.addWidget(self.apply_resize_btn, 4, 2, 1, 1)
        self.line = QtWidgets.QFrame(self)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout.addWidget(self.line, 5, 0, 1, 3)
        self.label_4 = QtWidgets.QLabel(self)
        self.label_4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                    QtCore.Qt.AlignVCenter)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 6, 0, 1, 1)
        self.preset_pos_cmb = QtWidgets.QComboBox(self)
        self.preset_pos_cmb.setObjectName("preset_pos_cmb")
        self.gridLayout.addWidget(self.preset_pos_cmb, 6, 1, 1, 2)
        self.label_5 = QtWidgets.QLabel(self)
        self.label_5.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                    QtCore.Qt.AlignVCenter)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 7, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 7, 2, 1, 1)
        self.verticalLayout_2.addLayout(self.gridLayout)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem1 = QtWidgets.QSpacerItem(28, 20, QtWidgets.QSizePolicy.Expanding, 
                                    QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem1)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setSpacing(2)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.top_spb = QtWidgets.QDoubleSpinBox(self)
        self.top_spb.setMinimum(-1000)
        self.top_spb.setObjectName("top_spb")
        self.horizontalLayout.addWidget(self.top_spb)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem3)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.left_spb = QtWidgets.QDoubleSpinBox(self)
        self.left_spb.setMinimum(-1000)
        self.left_spb.setObjectName("left_spb")
        self.horizontalLayout_2.addWidget(self.left_spb)
        self.right_spb = QtWidgets.QDoubleSpinBox(self)
        self.right_spb.setMinimum(-1000)
        self.right_spb.setObjectName("right_spb")
        self.horizontalLayout_2.addWidget(self.right_spb)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem4)
        self.bottom_spb = QtWidgets.QDoubleSpinBox(self)
        self.bottom_spb.setMinimum(-1000)
        self.bottom_spb.setObjectName("bottom_spb")
        self.horizontalLayout_3.addWidget(self.bottom_spb)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem5)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_4.addLayout(self.verticalLayout)
        self.verticalLayout_2.addLayout(self.horizontalLayout_4)
        self.gridLayout_2.addLayout(self.verticalLayout_2, 0, 0, 1, 1)

        self.retranslateUi()
        #self.set_theme()
        QtCore.QMetaObject.connectSlotsByName(self)

        self.units = ["mm", "Point", "in", "Pica", "Didot", "Cicero", 
                        "Device pixel"]
        # Populate comboBoxs
        ico_path = ["medias/center.png", "medias/full_page.png", 
                    "medias/full_width.png", "medias/full_height.png"]
        texts = ["Normal size", "Full page", "Full width", "Full height"]

        for i, p in enumerate(ico_path):
            icon = QtGui.QIcon()
            icon.addPixmap(QtGui.QPixmap(p), QtGui.QIcon.Normal, 
                            QtGui.QIcon.Off)
            self.preset_size_cmb.addItem(icon, texts[i])

        ico_path = ["medias/top_left.png", "medias/top_right.png", 
                    "medias/center.png", "medias/bottom_left.png", 
                    "medias/bottom_right.png"]
        texts = ["Top left", "Top right", "Center", "Bottom left", 
                    "Bottom right"]
        self.preset_pos_cmb.addItem("Undefined")

        for i, p in enumerate(ico_path):
            icon = QtGui.QIcon()
            icon.addPixmap(QtGui.QPixmap(p), QtGui.QIcon.Normal, 
                            QtGui.QIcon.Off)
            self.preset_pos_cmb.addItem(icon, texts[i])

    def set_param(self, unit, margins, size):
        s, d = 1.0, 0
        sfx = " %s" % self.units[unit]

        if unit == 2:
            s, d = 0.01, 2

        self.paper_size_lbl.setText("{0} x {1}{2}".format(str(size[0]), 
                                    str(size[1]), unit))
        self.block_margins_signal(True)
        self.left_spb.setValue(margins[0])
        self.left_spb.setMaximum(size[0])
        self.left_spb.setSingleStep(s)
        self.left_spb.setSuffix(sfx)
        self.left_spb.setDecimals(d)
        self.top_spb.setValue(margins[1])
        self.top_spb.setMaximum(size[1])
        self.top_spb.setSingleStep(s)
        self.top_spb.setSuffix(sfx)
        self.top_spb.setDecimals(d)
        self.right_spb.setValue(margins[2])
        self.right_spb.setMaximum(size[0])
        self.right_spb.setSingleStep(s)
        self.right_spb.setSuffix(sfx)
        self.right_spb.setDecimals(d)
        self.bottom_spb.setValue(margins[3])
        self.bottom_spb.setMaximum(size[1])
        self.bottom_spb.setSingleStep(s)
        self.bottom_spb.setSuffix(sfx)
        self.bottom_spb.setDecimals(d)
        self.block_margins_signal(False)

    def set_adjustment(self, val):
        self.preset_size_cmb.blockSignals(True)
        self.preset_size_cmb.setCurrentIndex(0)
        self.preset_size_cmb.blockSignals(False)

    def set_margin_values(self, *args):
        self.block_margins_signal(True)
        self.left_spb.setValue(args[0])
        self.top_spb.setValue(args[1])
        self.right_spb.setValue(args[2])
        self.bottom_spb.setValue(args[3])
        self.block_margins_signal(False)

    def set_size_values(self, size):
        self.block_sizers_signal(True)
        self.width_spb.setValue(size[0])
        self.height_spb.setValue(size[1])
        self.block_sizers_signal(False)
        self.apply_resize_btn.setEnabled(False)

    def block_margins_signal(self, b):
        """Enable - disable signal from margin widgets.

        Args:
        b -- boolean
        """
        self.left_spb.blockSignals(b)
        self.top_spb.blockSignals(b)
        self.right_spb.blockSignals(b)
        self.bottom_spb.blockSignals(b)

    def block_sizers_signal(self, b):
        self.width_spb.blockSignals(b)
        self.height_spb.blockSignals(b)

    def retranslateUi(self):
        self.label.setText(_("Paper size: "))
        self.label_1.setText(_("Adjustment: "))
        self.label_2.setText(_("width: "))
        self.width_spb.setSuffix(_(" mm."))
        self.label_3.setText(_("Height: "))
        self.height_spb.setSuffix(_(" mm."))
        self.proportion_chk.setText(_("Proportionnal"))
        self.apply_resize_btn.setToolTip(_("Apply the new size"))
        self.apply_resize_btn.setText(_("Apply"))
        self.label_4.setText(_("Position: "))
        self.label_5.setText(_("Margins:"))
        self.top_spb.setSuffix(_(" mm."))
        self.top_spb.setToolTip(_("Top margin"))
        self.left_spb.setSuffix(_(" mm."))
        self.left_spb.setToolTip(_("Left margin"))
        self.right_spb.setSuffix(_(" mm."))
        self.right_spb.setToolTip(_("Right margin"))
        self.bottom_spb.setSuffix(_(" mm."))
        self.bottom_spb.setToolTip(_("Bottom margin"))

    def set_theme(self):
        self.preset_size_cmb.setStyleSheet(styles.COMBO)
        self.width_spb.setStyleSheet(styles.SPINBOX)
        self.height_spb.setStyleSheet(styles.SPINBOX)
        self.proportion_chk.setStyleSheet(styles.CHECKBOX)
        self.apply_resize_btn.setStyleSheet(styles.PUSHBUTTON)
        self.preset_pos_cmb.setStyleSheet(styles.COMBO)
        self.top_spb.setStyleSheet(styles.DSPINBOX)
        self.left_spb.setStyleSheet(styles.DSPINBOX)
        self.right_spb.setStyleSheet(styles.DSPINBOX)
        self.bottom_spb.setStyleSheet(styles.DSPINBOX)
