# -*- coding: utf-8 -*-

# previewpage.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2013-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Images preview tool.

import os

from PyQt5 import QtCore, QtGui, QtWidgets


import stylesheets as styles


class PreviewPage(QtWidgets.QWidget):
    def __init__(self, parent):
        super().__init__(parent)
        self.setMaximumSize(QtCore.QSize(2000, 240))
        self.gridLayout = QtWidgets.QGridLayout(self)
        self.gridLayout.setContentsMargins(2, 0, 2, 0)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setSpacing(0)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.filename_lbl = QtWidgets.QLabel(self)
        font = QtGui.QFont()
        font.setPointSize(11)
        self.filename_lbl.setFont(font)
        self.filename_lbl.setText("File Name")
        self.filename_lbl.setAlignment(QtCore.Qt.AlignCenter)
        self.filename_lbl.setObjectName("filename_lbl")
        self.horizontalLayout.addWidget(self.filename_lbl)
        self.fileinfo_btn = QtWidgets.QToolButton(self)
        self.fileinfo_btn.setMaximumSize(QtCore.QSize(20, 20))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("medias/infos.png"), QtGui.QIcon.Normal, 
                                        QtGui.QIcon.Off)
        self.fileinfo_btn.setIcon(icon)
        self.fileinfo_btn.setIconSize(QtCore.QSize(15, 15))
        self.fileinfo_btn.setObjectName("fileinfo_btn")
        # XXX not yet implemented
        self.fileinfo_btn.hide()
        self.horizontalLayout.addWidget(self.fileinfo_btn)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.preview = QtWidgets.QGraphicsView(self)
        self.preview.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.preview.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.preview.setObjectName("preview")
        self.verticalLayout.addWidget(self.preview)
        self.scene = QtWidgets.QGraphicsScene()
        self.preview.setScene(self.scene)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSpacing(0)
        self.left_btn = QtWidgets.QToolButton(self)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap("medias/left_arrow_20x20.png"), 
                                        QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.left_btn.setIcon(icon1)
        self.left_btn.setIconSize(QtCore.QSize(15, 15))
        self.left_btn.setShortcut("L")
        self.left_btn.setObjectName("left_btn")
        self.horizontalLayout_2.addWidget(self.left_btn)
        self.right_btn = QtWidgets.QToolButton(self)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap("medias/right_arrow_20x20.png"), 
                                        QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.right_btn.setIcon(icon2)
        self.right_btn.setIconSize(QtCore.QSize(15, 15))
        self.right_btn.setShortcut("R")
        self.right_btn.setObjectName("right_btn")
        self.horizontalLayout_2.addWidget(self.right_btn)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.previous_btn = QtWidgets.QToolButton(self)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap("medias/collapse_white_right.png"), 
                                        QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.previous_btn.setIcon(icon3)
        self.previous_btn.setIconSize(QtCore.QSize(15, 15))
        self.previous_btn.setObjectName("previous_btn")
        self.horizontalLayout_2.addWidget(self.previous_btn)
        self.next_btn = QtWidgets.QToolButton(self)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap("medias/collapse_white_left.png"), 
                                        QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.next_btn.setIcon(icon4)
        self.next_btn.setIconSize(QtCore.QSize(15, 15))
        self.next_btn.setObjectName("next_btn")
        self.horizontalLayout_2.addWidget(self.next_btn)
        spacerItem2 = QtWidgets.QSpacerItem(18, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem2)
        spacerItem3 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Fixed, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.select_btn = QtWidgets.QPushButton(self)
        self.select_btn.setText("OK")
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap("medias/dialog-apply.svg"), 
                                        QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.select_btn.setIcon(icon5)
        self.select_btn.setIconSize(QtCore.QSize(15, 15))
        self.select_btn.setEnabled(False)
        self.select_btn.setObjectName("select_btn")
        self.horizontalLayout_2.addWidget(self.select_btn)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.gridLayout.addLayout(self.verticalLayout, 0, 0, 1, 1)

        self.retranslateUi()
        #self.set_theme()
        QtCore.QMetaObject.connectSlotsByName(self)

        self.previous_btn.clicked.connect(self.previous_image)
        self.next_btn.clicked.connect(self.next_image)
        self.left_btn.clicked.connect(self.rotate_left)
        self.right_btn.clicked.connect(self.rotate_right)

    def get_viewer_size(self):
        """Returns size of graphics view."""
        return (self.preview.width(), self.preview.height())

    def configure(self, imgs, idx):
        self.images = imgs
        self.current = idx - 1
        self.count = len(self.images)
        self.next_image()

    def previous_image(self):
        self.current = (self.current - 1) % self.count
        self.set_image(self.images[self.current])
        self.select_btn.setEnabled(True)

    def next_image(self):
        self.current = (self.current + 1) % self.count
        self.set_image(self.images[self.current])
        self.select_btn.setEnabled(True)

    def set_image(self, path):
        self.qimage = QtGui.QImage(path)
        self.set_pixmap()
        self.set_file_name(path)

    def set_pixmap(self):
        w, h = self.get_viewer_size()
        pix = QtGui.QPixmap(self.qimage.scaled(w, h, 
                            QtCore.Qt.KeepAspectRatio, 
                            QtCore.Qt.SmoothTransformation))

        self.change_preview(pix)

    def change_preview(self, pixmap):
        self.scene.clear()
        self.scene.setSceneRect(QtCore.QRectF(pixmap.rect()))
        self.scene.addPixmap(pixmap)

    def set_file_name(self, path):
        name = os.path.split(path)[1]
        self.filename_lbl.setText(name)

    def rotate_left(self):
        self.rotate(-90)

    def rotate_right(self):
        self.rotate(90)

    def rotate(self, angle):
        matrix = QtGui.QTransform()
        self.qimage = self.qimage.transformed(matrix.rotate(angle), 
                             QtCore.Qt.SmoothTransformation)

        self.set_pixmap()

    def retranslateUi(self):
        self.left_btn.setToolTip(_("Rotate to left."))
        self.right_btn.setToolTip(_("Rotate to right."))
        self.previous_btn.setToolTip(_("Previous image."))
        self.previous_btn.setShortcut("P")
        self.next_btn.setToolTip(_("Next image."))
        self.next_btn.setShortcut("N")
        self.select_btn.setToolTip(_("Select the current image."))
        self.select_btn.setShortcut("S")

    def set_theme(self):
        self.filename_lbl.setStyleSheet(styles.TITLE)
        self.preview.setStyleSheet(styles.VIEWER)
        self.fileinfo_btn.setStyleSheet(styles.TBUTTON)
        self.left_btn.setStyleSheet(styles.TBUTTON)
        self.right_btn.setStyleSheet(styles.TBUTTON)
        self.previous_btn.setStyleSheet(styles.TBUTTON)
        self.next_btn.setStyleSheet(styles.TBUTTON)
        self.select_btn.setStyleSheet(styles.PUSHBUTTON)
