# -*- coding: utf-8 -*-

# printerpage.py
# this file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2013-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Printer settings tool.

from PyQt5 import QtCore, QtGui, QtWidgets

import stylesheets as styles

class PrinterPage(QtWidgets.QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.gridLayout_2 = QtWidgets.QGridLayout(self)
        self.gridLayout_2.setContentsMargins(2, 0, 2, 0)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setVerticalSpacing(2)
        self.label = QtWidgets.QLabel(self)
        self.label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                QtCore.Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSpacing(0)
        self.output_cmb = QtWidgets.QComboBox(self)
        self.output_cmb.setMinimumSize(QtCore.QSize(150, 0))
        self.output_cmb.setObjectName("output_cmb")
        self.horizontalLayout.addWidget(self.output_cmb)
        self.printer_sett_btn = QtWidgets.QToolButton(self)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("medias/settings_red.png"), 
                                QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.printer_sett_btn.setIcon(icon)
        self.printer_sett_btn.setIconSize(QtCore.QSize(20, 20))
        self.printer_sett_btn.setObjectName("printer_sett_btn")
        self.horizontalLayout.addWidget(self.printer_sett_btn)
        self.gridLayout.addLayout(self.horizontalLayout, 0, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(self)
        self.label_2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                QtCore.Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.orientation_cmb = QtWidgets.QComboBox(self)
        self.orientation_cmb.setObjectName("orientation_cmb")
        self.gridLayout.addWidget(self.orientation_cmb, 1, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self)
        self.label_3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                QtCore.Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label_3, 2, 0, 1, 1)
        self.color_cmb = QtWidgets.QComboBox(self)
        self.color_cmb.setObjectName("color_cmb")
        self.gridLayout.addWidget(self.color_cmb, 2, 1, 1, 1)
        self.label_4 = QtWidgets.QLabel(self)
        self.label_4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                QtCore.Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label_4, 3, 0, 1, 1)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSpacing(0)
        self.resol_spb = QtWidgets.QSpinBox(self)
        self.resol_spb.setMinimum(72)
        self.resol_spb.setMaximum(5000)
        self.resol_spb.setProperty("value", 150)
        self.resol_spb.setObjectName("resol_spb")
        self.horizontalLayout_2.addWidget(self.resol_spb)
        self.apply_resol_btn = QtWidgets.QToolButton(self)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap("medias/dialog-apply.svg"), 
                                QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.apply_resol_btn.setIcon(icon1)
        self.apply_resol_btn.setIconSize(QtCore.QSize(20, 20))
        self.apply_resol_btn.setEnabled(False)
        self.apply_resol_btn.setObjectName("apply_resol_btn")
        self.horizontalLayout_2.addWidget(self.apply_resol_btn)
        self.gridLayout.addLayout(self.horizontalLayout_2, 3, 1, 1, 1)
        self.gridLayout_2.addLayout(self.gridLayout, 0, 0, 1, 1)

        self.orientations = [_("Portrait"), _("Landscape")]
        self.orientation_cmb.addItems(self.orientations)
        self.orientation_cmb.setCurrentIndex(0)
        self.colors = [_("Colors"), _("Grayscale")]
        self.color_cmb.addItems(self.colors)
        self.outputs = ["To file pdf", "To file postscript"]
        self.output_cmb.addItems(self.outputs)

        self.retranslateUi()
        #self.set_theme()
        self.resol_spb.valueChanged.connect(self.on_resolution_edited)
        QtCore.QMetaObject.connectSlotsByName(self)

    def on_resolution_edited(self, val):
        self.apply_resol_btn.setEnabled(True)

    def retranslateUi(self):
        self.label.setText(_("Printer: "))
        self.printer_sett_btn.setToolTip(_("Printer settings"))
        self.label_2.setText(_("Orientation: "))
        self.label_3.setText(_("Color: "))
        self.label_4.setText(_("Resolution: "))
        self.resol_spb.setSuffix(_(" dpi"))
        self.apply_resol_btn.setToolTip(_("Apply"))

    def set_theme(self):
        self.output_cmb.setStyleSheet(styles.COMBO)
        self.printer_sett_btn.setStyleSheet(styles.TBUTTON)
        self.orientation_cmb.setStyleSheet(styles.COMBO)
        self.color_cmb.setStyleSheet(styles.COMBO)
        self.resol_spb.setStyleSheet(styles.SPINBOX)
        self.apply_resol_btn.setStyleSheet(styles.TBUTTON)
