# recentscleaner.py   
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2010-2018 Vincent Vande Vyvre
# Licence: GPL3
#
# Dialog box for purge the recent folders list.

import os

from PyQt5.QtGui import QIcon, QPixmap
from PyQt5.QtWidgets import (QDialog, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QAbstractItemView, QToolButton, QPushButton,
                             QLabel, QSpacerItem, QSizePolicy,
                             QListWidget, QListWidgetItem)


class RecentsCleaner(QDialog):
    def __init__(self, core):
        """Setup the dialog box.

        """
        super().__init__()
        self.core = core
        self.recents = core.recents.path_strs
        self.resize(563, 655)
        self.gridLayout = QGridLayout(self)
        self.gridLayout.setContentsMargins(2, 2, 2, 2)
        self.verticalLayout = QVBoxLayout()
        self.path_viewer = QListWidget(self)
        self.path_viewer.setTabKeyNavigation(True)
        self.path_viewer.setAlternatingRowColors(True)
        self.path_viewer.setSelectionMode(QAbstractItemView.MultiSelection)
        self.verticalLayout.addWidget(self.path_viewer)
        self.horizontalLayout = QHBoxLayout()
        self.supp_btn = QToolButton(self)
        ic = QIcon()
        ic.addPixmap(QPixmap("medias/clean_24x27.png"), QIcon.Normal, QIcon.Off)
        self.supp_btn.setIcon(ic)
        self.horizontalLayout.addWidget(self.supp_btn)
        self.label = QLabel()
        self.horizontalLayout.addWidget(self.label)
        spacerItem = QSpacerItem(338, 20, QSizePolicy.Expanding, 
                                            QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.close_btn = QPushButton(self)
        self.horizontalLayout.addWidget(self.close_btn)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.gridLayout.addLayout(self.verticalLayout, 0, 0, 1, 1)

        self.retranslateUi()
        self.close_btn.clicked.connect(self.accept)
        self.supp_btn.clicked.connect(self.suppress)
        self.path_viewer.itemSelectionChanged.connect(self.selection_changed)
        self.show()
        self.read_list()

    def retranslateUi(self):
        self.setWindowTitle(_("Clean recent folders list"))
        self.supp_btn.setToolTip(_("To withdraw the selected folders"))
        self.close_btn.setText(_("Close"))

    def read_list(self):
        """Extract the recent folders list.

        """
        self.selected = []        

        if not self.recents:
            self.label.setText("No recent folders")

        # Show datas
        for r in self.recents:
            data = QListWidgetItem(self.path_viewer)
            data.setText(r)

            # Validate data
            if not os.path.isdir(r):
                data.setSelected(True)

        if self.selected:
            self.label.setText(_("The selected folders are invalid!"))    

    def suppress(self):
        """Remove one or more recent folder(s).

        """
        sel = self.selected[:]

        for s in sel:
            self.path_viewer.takeItem(s)
            fld = self.recents.pop(s)
            self.core.recents.remove_path(fld)

        self.label.setText("")

    def selection_changed(self):
        """Get folder selected

        """
        self.selected = []
        select = self.path_viewer.selectedItems()

        for s in select:
            self.selected.append(self.path_viewer.row(s))

        self.selected.sort()
        self.selected.reverse()
