# -*- coding: utf-8 -*-

# aerialviewpage.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2011-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Config page for aerial view
# !! This page is no longer used !!

from PyQt5.QtCore import QSize, Qt
from PyQt5.QtGui import QIcon, QPixmap, QFont
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QLabel, QSpacerItem, QFrame, QSizePolicy, QSpinBox,
                             QDialog)

import stylesheets as style

from utils import LineEdit, EditButton, UndoButton, ShortcutEditor

class AerialViewPage(QWidget):
    def __init__(self, parent):
        super().__init__(parent)
        self.parent_ = parent
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/plus.png"), QIcon.Normal, QIcon.Off)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/gtk-refresh.png"), QIcon.Normal, QIcon.Off)
        self.gridLayout_2 = QGridLayout(self)
        self.gridLayout_2.setContentsMargins(20, -1, -1, -1)
        self.verticalLayout_4 = QVBoxLayout()
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setSpacing(1)
        self.horizontalLayout_3 = QHBoxLayout()
        self.title_lbl = QLabel(self)
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.title_lbl.setFont(font)
        self.horizontalLayout_3.addWidget(self.title_lbl)
        spacerItem = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.line = QFrame(self)
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)
        self.verticalLayout.addWidget(self.line)
        self.verticalLayout_4.addLayout(self.verticalLayout)
        spacerItem1 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_4.addItem(spacerItem1)
        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setSpacing(10)
        self.size_lbl = QLabel(self)
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(11)
        font.setBold(True)
        font.setItalic(True)
        self.size_lbl.setFont(font)
        self.horizontalLayout_4.addWidget(self.size_lbl)
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setSpacing(0)
        spacerItem2 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_2.addItem(spacerItem2)
        self.line_3 = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line_3.sizePolicy().hasHeightForWidth())
        self.line_3.setSizePolicy(sizePolicy)
        self.line_3.setFrameShape(QFrame.HLine)
        self.line_3.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_2.addWidget(self.line_3)
        self.horizontalLayout_4.addLayout(self.verticalLayout_2)
        self.verticalLayout_4.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5 = QHBoxLayout()
        spacerItem3 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem3)
        self.label_2 = QLabel(self)
        self.label_2.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.horizontalLayout_5.addWidget(self.label_2)
        self.height_spb = QSpinBox(self)
        self.height_spb.setMinimumSize(QSize(100, 20))
        self.height_spb.setMaximumSize(QSize(16777215, 20))
        self.height_spb.setSuffix(" px")
        self.height_spb.setMinimum(100)
        self.height_spb.setMaximum(400)
        self.height_spb.setProperty("value", 200)
        self.height_spb.setStyleSheet(style.SPINBOX)
        self.horizontalLayout_5.addWidget(self.height_spb)
        spacerItem4 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem4)
        self.verticalLayout_4.addLayout(self.horizontalLayout_5)
        spacerItem5 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_4.addItem(spacerItem5)
        self.horizontalLayout_6 = QHBoxLayout()
        self.horizontalLayout_6.setSpacing(10)
        self.shortcuts_lbl = QLabel(self)
        self.shortcuts_lbl.setFont(font)
        self.horizontalLayout_6.addWidget(self.shortcuts_lbl)
        self.verticalLayout_3 = QVBoxLayout()
        spacerItem6 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_3.addItem(spacerItem6)
        self.line_2 = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line_2.sizePolicy().hasHeightForWidth())
        self.line_2.setSizePolicy(sizePolicy)
        self.line_2.setFrameShape(QFrame.HLine)
        self.line_2.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_3.addWidget(self.line_2)
        self.horizontalLayout_6.addLayout(self.verticalLayout_3)
        self.verticalLayout_4.addLayout(self.horizontalLayout_6)
        self.gridLayout = QGridLayout()
        spacerItem7 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem7, 0, 0, 1, 1)
        self.label_3 = QLabel(self)
        self.label_3.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label_3, 0, 1, 1, 1)
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setSpacing(0)
        self.show_led = LineEdit(self, None, "show")
        self.horizontalLayout.addWidget(self.show_led)
        self.show_change_btn = EditButton(self, None, icon, "show")
        self.show_change_btn.setObjectName("show_change_btn")
        self.horizontalLayout.addWidget(self.show_change_btn)
        self.show_reverse_btn = UndoButton(self, None, icon1, "show")
        self.show_reverse_btn.setObjectName("show_reverse_btn")
        self.horizontalLayout.addWidget(self.show_reverse_btn)
        self.gridLayout.addLayout(self.horizontalLayout, 0, 2, 1, 1)
        spacerItem8 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem8, 0, 3, 1, 1)
        spacerItem9 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem9, 1, 0, 1, 1)
        self.label_4 = QLabel(self)
        self.label_4.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label_4, 1, 1, 1, 1)
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setSpacing(0)
        self.permanent_led = LineEdit(self, None, "permanent")
        self.horizontalLayout_2.addWidget(self.permanent_led)
        self.permanent_change_btn = EditButton(self, None, icon, "permanent")
        self.permanent_change_btn.setObjectName("permanent_change_btn")
        self.horizontalLayout_2.addWidget(self.permanent_change_btn)
        self.permanent_reverse_btn = UndoButton(self, None, icon1, "permanent")
        self.permanent_reverse_btn.setObjectName("permanent_reverse_btn")
        self.horizontalLayout_2.addWidget(self.permanent_reverse_btn)
        self.gridLayout.addLayout(self.horizontalLayout_2, 1, 2, 1, 1)
        spacerItem10 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem10, 1, 3, 1, 1)
        self.verticalLayout_4.addLayout(self.gridLayout)
        spacerItem11 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem11)
        self.gridLayout_2.addLayout(self.verticalLayout_4, 0, 0, 1, 1)

        self.retranslateUi()
        self.set_style()
        self.set_connections()

    def set_config_values(self, conf, shortcuts):
        """Set the configuration values in widgets.

        Keyword arguments:
        conf -- dictionnary of user's prefs

        Keys:
        "aerial_view": {"show_hide": "A",
                        "permanent": "D"}
        """
        self.conf = conf
        self.shortcuts = shortcuts["aerialview"]
        self.height_spb.setValue(int(conf["aerialview"]["size"]))
        self.show_led.setText(self.shortcuts["show_hide"])
        self.permanent_led.setText(self.shortcuts["permanent"])
        self.show_reverse_btn.setEnabled(False)
        self.permanent_reverse_btn.setEnabled(False)

    def set_connections(self):
        self.height_spb.valueChanged.connect(self.parent_.value_changed)


    def on_value_changed(self, sender):
        if sender == "show":
            self.show_reverse_btn.setEnabled(True)

        else:
            self.permanent_reverse_btn.setEnabled(True)

        self.parent_.value_changed()

    def undo_change(self, sender):
        if sender == "show":
            self.show_led.setText(self.shortcuts["show_hide"])
            self.show_reverse_btn.setEnabled(False)

        else:
            self.permanent_led.setText(self.shortcuts["permanent"])
            self.permanent_reverse_btn.setEnabled(False)

    def edit_shortcut(self, sender):
        if sender == "show":
            w = self.show_led

        else:
            w = self.permanent_led

        sht = unicode(w.text()).split("+")
        if len(sht) == 1:
            self.mod = " "
            self.key = sht[0]

        else:
            self.mod = sht[0]
            self.key = sht[1]

        Dial = QDialog()
        sh = ShortcutEditor(Dial, self, self.mod, self.key)
        reply = Dial.exec_()

        if reply:
            sc = ""
            if self.mod != " ":
                sc = sc.join([self.mod, "+"])

            sc = "".join([sc, self.key])
            w.setText(sc)

    def apply_change(self):
        self.conf["aerialview"]["size"] = int(self.height_spb.value())
        self.shortcuts["show_hide"] = self.show_led.text()
        self.shortcuts["permanent"] = self.permanent_led.text()

    def retranslateUi(self):
        self.title_lbl.setText(_("Aerial View - Configuration"))
        self.size_lbl.setText(_("Size"))
        self.label_2.setText(_("Height:"))
        self.shortcuts_lbl.setText(_("Shortcuts"))
        self.label_3.setText(_("Show - hide:"))
        self.label_4.setText(_("Permanent:"))

    def set_style(self):
        self.title_lbl.setStyleSheet(style.SUBTITLE)
        self.line.setStyleSheet(style.UNDERLINE)
        self.size_lbl.setStyleSheet(style.SUBTITLE)
        self.line_2.setStyleSheet(style.UNDERLINE)
        self.line_3.setStyleSheet(style.UNDERLINE)

