# -*- coding: utf-8 -*-

# croppage.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2011-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Config page for crop tool


from PyQt5.QtCore import Qt, QSize
from PyQt5.QtGui import QIcon, QPixmap, QFont
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QLabel, QSpacerItem, QSizePolicy, QFrame,
                             QComboBox, QRadioButton, QSpinBox, QCheckBox,
                             QDialog, QButtonGroup)

import stylesheets as style

from settings.utils import LineEdit, EditButton, UndoButton, ShortcutEditor

class CropPage(QWidget):
    def __init__(self, parent):
        super().__init__(parent)
        self.parent_ = parent
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/plus.png"), QIcon.Normal, QIcon.Off)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/gtk-refresh.png"), 
                                QIcon.Normal, QIcon.Off)
        self.gridLayout_2 = QGridLayout(self)
        self.gridLayout_2.setContentsMargins(20, -1, -1, -1)
        self.verticalLayout_4 = QVBoxLayout()
        self.verticalLayout_5 = QVBoxLayout()
        self.verticalLayout_5.setSpacing(1)
        self.horizontalLayout_16 = QHBoxLayout()
        self.title_lbl = QLabel(self)
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.title_lbl.setFont(font)
        self.horizontalLayout_16.addWidget(self.title_lbl)
        sp = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_16.addItem(sp)
        self.verticalLayout_5.addLayout(self.horizontalLayout_16)
        self.line = QFrame(self)
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_5.addWidget(self.line)
        self.verticalLayout_4.addLayout(self.verticalLayout_5)
        sp0 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_4.addItem(sp0)
        self.verticalLayout_3 = QVBoxLayout()
        self.verticalLayout_2 = QVBoxLayout()
        self.horizontalLayout_17 = QHBoxLayout()
        self.horizontalLayout_17.setSpacing(10)
        self.ratio_lbl = QLabel(self)
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(11)
        font.setBold(True)
        font.setItalic(True)
        self.ratio_lbl.setFont(font)
        self.horizontalLayout_17.addWidget(self.ratio_lbl)
        self.verticalLayout_6 = QVBoxLayout()
        self.verticalLayout_6.setSpacing(0)
        sp1 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_6.addItem(sp1)
        self.line_3 = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line_3.sizePolicy().hasHeightForWidth())
        self.line_3.setSizePolicy(sizePolicy)
        self.line_3.setFrameShape(QFrame.HLine)
        self.line_3.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_6.addWidget(self.line_3)
        self.horizontalLayout_17.addLayout(self.verticalLayout_6)
        self.verticalLayout_2.addLayout(self.horizontalLayout_17)
        self.horizontalLayout_24 = QHBoxLayout()
        sp2 = QSpacerItem(28, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_24.addItem(sp2)
        self.verticalLayout = QVBoxLayout()
        self.horizontalLayout_2 = QHBoxLayout()
        self.same_as_rdo = QRadioButton(self)
        self.same_as_rdo.setChecked(True)
        self.buttonGroup = QButtonGroup(self)
        self.buttonGroup.addButton(self.same_as_rdo)
        self.horizontalLayout_2.addWidget(self.same_as_rdo)
        sp3 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(sp3)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QHBoxLayout()
        self.defined_rdo = QRadioButton(self)
        self.buttonGroup.addButton(self.defined_rdo)
        self.horizontalLayout_3.addWidget(self.defined_rdo)
        self.defined_cmb = QComboBox(self)
        self.defined_cmb.setEnabled(False)
        self.defined_cmb.setMinimumSize(QSize(120, 0))
        self.horizontalLayout_3.addWidget(self.defined_cmb)
        self.reverse_chb = QCheckBox(self)
        self.reverse_chb.setEnabled(False)
        self.horizontalLayout_3.addWidget(self.reverse_chb)
        sp4 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(sp4)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_23 = QHBoxLayout()
        self.custom_rdo = QRadioButton(self)
        self.buttonGroup.addButton(self.custom_rdo)
        self.horizontalLayout_23.addWidget(self.custom_rdo)
        sp5 = QSpacerItem(18, 20, QSizePolicy.Minimum, QSizePolicy.Minimum)
        self.horizontalLayout_23.addItem(sp5)
        self.horizontalLayout_4 = QHBoxLayout()
        align = Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter
        self.label_2 = QLabel(self)
        self.label_2.setAlignment(align)
        self.horizontalLayout_4.addWidget(self.label_2)
        self.width_spb = QSpinBox(self)
        self.width_spb.setEnabled(False)
        self.width_spb.setMinimumSize(QSize(80, 0))
        self.horizontalLayout_4.addWidget(self.width_spb)
        self.label_3 = QLabel(self)
        self.label_3.setAlignment(align)
        self.horizontalLayout_4.addWidget(self.label_3)
        self.height_spb = QSpinBox(self)
        self.height_spb.setEnabled(False)
        self.height_spb.setMinimumSize(QSize(80, 0))
        self.horizontalLayout_4.addWidget(self.height_spb)
        self.append_chb = QCheckBox(self)
        self.append_chb.setEnabled(False)
        self.horizontalLayout_4.addWidget(self.append_chb)
        sp6 = QSpacerItem(13, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(sp6)
        self.horizontalLayout_23.addLayout(self.horizontalLayout_4)
        self.verticalLayout.addLayout(self.horizontalLayout_23)
        self.horizontalLayout = QHBoxLayout()
        self.label_4 = QLabel(self)
        self.horizontalLayout.addWidget(self.label_4)
        sp7 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(sp7)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_7 = QHBoxLayout()
        sp8 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(sp8)
        self.constrain_width_rdo = QRadioButton(self)
        self.constrain_width_rdo.setChecked(True)
        self.buttonGroup_2 = QButtonGroup(self)
        self.buttonGroup_2.addButton(self.constrain_width_rdo)
        self.horizontalLayout_7.addWidget(self.constrain_width_rdo)
        sp9 = QSpacerItem(40, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(sp9)
        self.constrain_height_rdo = QRadioButton(self)
        self.buttonGroup_2.addButton(self.constrain_height_rdo)
        self.horizontalLayout_7.addWidget(self.constrain_height_rdo)
        sp10 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(sp10)
        self.verticalLayout.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_24.addLayout(self.verticalLayout)
        self.verticalLayout_2.addLayout(self.horizontalLayout_24)
        sp11 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_2.addItem(sp11)
        self.horizontalLayout_18 = QHBoxLayout()
        self.horizontalLayout_18.setSpacing(10)
        self.grids_lbl = QLabel(self)
        self.grids_lbl.setFont(font)
        self.horizontalLayout_18.addWidget(self.grids_lbl)
        self.verticalLayout_7 = QVBoxLayout()
        self.verticalLayout_7.setSpacing(0)
        sp12 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_7.addItem(sp12)
        self.line_4 = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line_4.sizePolicy().hasHeightForWidth())
        self.line_4.setSizePolicy(sizePolicy)
        self.line_4.setFrameShape(QFrame.HLine)
        self.line_4.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_7.addWidget(self.line_4)
        self.horizontalLayout_18.addLayout(self.verticalLayout_7)
        self.verticalLayout_2.addLayout(self.horizontalLayout_18)
        self.horizontalLayout_8 = QHBoxLayout()
        sp13 = QSpacerItem(10, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(sp13)
        self.grid_chb = QCheckBox(self)
        self.horizontalLayout_8.addWidget(self.grid_chb)
        sp14 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(sp14)
        self.verticalLayout_2.addLayout(self.horizontalLayout_8)
        sp14 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_2.addItem(sp14)
        sp15 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_2.addItem(sp15)
        self.horizontalLayout_20 = QHBoxLayout()
        self.horizontalLayout_20.setSpacing(10)
        self.shortcuts_lbl = QLabel(self)
        self.shortcuts_lbl.setFont(font)
        self.horizontalLayout_20.addWidget(self.shortcuts_lbl)
        self.verticalLayout_9 = QVBoxLayout()
        self.verticalLayout_9.setSpacing(0)
        sp16 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_9.addItem(sp16)
        self.line_6 = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line_6.sizePolicy().hasHeightForWidth())
        self.line_6.setSizePolicy(sizePolicy)
        self.line_6.setFrameShape(QFrame.HLine)
        self.line_6.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_9.addWidget(self.line_6)
        self.horizontalLayout_20.addLayout(self.verticalLayout_9)
        self.verticalLayout_2.addLayout(self.horizontalLayout_20)
        self.horizontalLayout_25 = QHBoxLayout()
        sp17 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_25.addItem(sp17)
        self.gridLayout = QGridLayout()
        self.label_19 = QLabel(self)
        self.label_19.setAlignment(align)
        self.gridLayout.addWidget(self.label_19, 0, 0, 1, 1)
        self.horizontalLayout_21 = QHBoxLayout()
        self.horizontalLayout_21.setSpacing(0)
        self.setup_led = LineEdit(self, None,"setup")
        self.horizontalLayout_21.addWidget(self.setup_led)
        self.setup_change_btn = EditButton(self, None, icon, "setup")
        self.horizontalLayout_21.addWidget(self.setup_change_btn)
        self.setup_reverse_btn = UndoButton(self, None, icon1, "setup")
        self.horizontalLayout_21.addWidget(self.setup_reverse_btn)
        self.gridLayout.addLayout(self.horizontalLayout_21, 0, 1, 1, 1)
        self.label_23 = QLabel(self)
        self.label_23.setAlignment(align)
        self.gridLayout.addWidget(self.label_23, 5, 0, 1, 1)
        self.horizontalLayout_15 = QHBoxLayout()
        self.horizontalLayout_15.setSpacing(0)
        self.extract_led = LineEdit(self, None,"extract")
        self.horizontalLayout_15.addWidget(self.extract_led)
        self.extract_change_btn = EditButton(self, None, icon, "extract")
        self.horizontalLayout_15.addWidget(self.extract_change_btn)
        self.extract_reverse_btn = UndoButton(self, None, icon1, "extract")
        self.horizontalLayout_15.addWidget(self.extract_reverse_btn)
        self.gridLayout.addLayout(self.horizontalLayout_15, 5, 1, 1, 1)
        self.horizontalLayout_25.addLayout(self.gridLayout)
        sp18 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_25.addItem(sp18)
        self.verticalLayout_2.addLayout(self.horizontalLayout_25)
        self.horizontalLayout_22 = QHBoxLayout()
        sp19 = QSpacerItem(10, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_22.addItem(sp19)
        self.label = QLabel(self)
        self.horizontalLayout_22.addWidget(self.label)
        self.verticalLayout_2.addLayout(self.horizontalLayout_22)
        self.verticalLayout_3.addLayout(self.verticalLayout_2)
        sp20 = QSpacerItem(10, 60, QSizePolicy.Fixed, QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(sp20)        
        self.verticalLayout_4.addLayout(self.verticalLayout_3)
        self.gridLayout_2.addLayout(self.verticalLayout_4, 0, 0, 1, 1)

        self.leds = [self.setup_led, self.extract_led]

        self.undos = [self.setup_reverse_btn, self.extract_reverse_btn]

        self.senders = ["setup", "extract"]

        # Not yet implemented
        self.append_chb.hide()

        self.retranslateUi()
        self.set_style()
        self.set_connections()

    def set_connections(self):
        self.same_as_rdo.toggled.connect(self.set_ratio_sameas)
        self.defined_rdo.toggled.connect(self.set_ratio_defined)
        self.custom_rdo.toggled.connect(self.set_ratio_custom)
        self.width_spb.valueChanged.connect(self.on_custom_value_changed)
        self.height_spb.valueChanged.connect(self.on_custom_value_changed)
        self.append_chb.stateChanged.connect(self.on_append_changed)

    def set_config_values(self, conf, shortcuts):
        """Set the configuration values in widgets.

        Args:
        conf -- dictionnary of user's prefs

        Keys:
        "crop": {"constrain": False,
                 "current_ratio": None,
                 "reverse": False,
                 "force_width": True,
                 "grid": False,
                 "mode": ("same_as",),
                 "ratios": ["1:1", "4:3", "3:2", "16:9", "1.85:1", "2.39:1"]}
        """
        self.conf = conf
        cfg = conf["crop"]
        self.shortcuts = shortcuts["crop_tool"]
        self.defined_cmb.addItems(cfg["ratios"])
        val = cfg["mode"]
        if val[0] == "same_as":
            self.same_as_rdo.setChecked(True)

        elif val[0] == "defined":
            self.defined_rdo.setChecked(True)

        else:
            self.custom_rdo.setChecked(True)

        self.reverse_chb.setChecked(cfg["reverse"])
        self.constrain_height_rdo.setChecked(not cfg["force_width"])
        self.setup_led.setText(self.shortcuts["activate"])
        self.extract_led.setText(self.shortcuts["apply"])
        for u in self.undos:
            u.setEnabled(False)

    def apply_change(self):
        """Apply settings to config dictionnary.

        """
        self.conf["crop"]["reverse"] = self.reverse_chb.isChecked()
        self.conf["crop"]["force_width"] = self.constrain_width_rdo.isChecked()
        if self.same_as_rdo.isChecked():
            value = ("same_as",)

        elif self.defined_rdo.isChecked():
            value = ("defined", str(self.defined_cmb.currentText()))

        else:
            value = ("custom", self.width_spb.value(), self.height_spb.value())

        self.conf["crop"]["mode"] = value
        self.shortcuts["activate"] = self.setup_led.text()
        self.shortcuts["apply"] = self.extract_led.text()

    def on_value_changed(self, sender):
        """Enable undo button when a shortcut has changed.

        Args:
        sender -- shortcut lineEdit
        """
        self.undos[self.senders.index(sender)].setEnabled(True)
        self.parent_.value_changed()

    def edit_shortcut(self, sender):
        """Call shortcut editor dialog box when an edit button is pressed.

        Args:
        sender -- shortcut edit button
        """
        led = self.leds[self.senders.index(sender)]
        sht = unicode(led.text()).split("+")
        if len(sht) == 1:
            self.mod = " "
            self.key = sht[0]

        else:
            self.mod = sht[0]
            self.key = sht[1]

        Dial = QDialog()
        sh = ShortcutEditor(Dial, self, self.mod, self.key)
        reply = Dial.exec_()

        if reply:
            sc = ""
            if self.mod != " ":
                sc = "".join([self.mod, "+"])

            sc = "".join([sc, self.key])
            led.setText(sc)

    def undo_change(self, sender):
        """Undo the last change in shortcut group.

        Args:
        sender -- shortcut undo button
        """
        idx = self.senders.index(sender)
        if not idx:
            conf = self.shortcuts["activate"]

        else:
            conf = self.shortcuts["apply"]

        self.leds[idx].setText(conf)
        self.undos[idx].setEnabled(False)

    def set_ratio_sameas(self, b):
        if b:
            self.enable_defined(False)
            self.enable_custom(False)

    def set_ratio_defined(self, b):
        if b:
            self.enable_defined(True)
            self.enable_custom(False)

    def set_ratio_custom(self, b):
        if b:
            self.enable_defined(False)
            self.enable_custom(True)

    def enable_defined(self, b):
        self.defined_cmb.setEnabled(b)
        self.reverse_chb.setEnabled(b)

    def enable_custom(self, b):
        self.width_spb.setEnabled(b)
        self.height_spb.setEnabled(b)
        self.append_chb.setEnabled(b)

    def on_custom_value_changed(self, value):
        self.append_chb.setChecked(False)
        if value == 0.0:
            self.append_chb.setEnabled(False)

        else:
            self.append_chb.setEnabled(self.compare_ratio())

    def on_append_changed(self, b):
        if b is not False:
            r = self.get_ratio_string()
            self.defined_cmb.addItem(r)
            self.conf["crop"]["ratios"].append(r)

    def compare_ratio(self):
        r = self.get_ratio_string()
        if self.defined_cmb.findText(r) != -1:
            return False

        return True

    def get_ratio_string(self):
        w, h = self.width_spb.value(), self.height_spb.value()
        if w % 1 == 0:
            w = int(w)

        if h % 1 == 0:
            h = int(h)

        return ":".join([str(w), str(h)])

    def set_defined_number(self):
        s = str(self.defined_cmb.currentText())
        w, h = s.split(":")
        return eval(w), eval(h)

    def retranslateUi(self):
        self.title_lbl.setText(_("Crop Tool - Configuration"))
        self.ratio_lbl.setText(_("Ratio constraint"))
        self.same_as_rdo.setText(_("Same as image"))
        self.defined_rdo.setText(_("Defined"))
        self.reverse_chb.setToolTip(_("Reverse width / height"))
        self.reverse_chb.setText(_("Reverse"))
        self.custom_rdo.setText(_("Custom"))
        self.label_2.setText(_("width:"))
        self.label_3.setText(_("height:"))
        self.append_chb.setToolTip(_("Append this ratio into predefined list"))
        self.append_chb.setText(_("Append"))
        self.label_4.setText(_(
                "When resizing with corner grips the new size is constrain to:"))
        self.constrain_width_rdo.setText(_("width"))
        self.constrain_height_rdo.setText(_("height"))
        self.grids_lbl.setText(_("Grids"))
        self.grid_chb.setText(_("Always show grids"))
        self.label_19.setText(_("Start crop tool:"))
        self.label_23.setText(_("Extract the new image:"))
        self.extract_led.setText(_("Enter"))
        self.shortcuts_lbl.setText(_("Shortcuts"))
        self.label_23.setText(_("Extract the new image:"))
        self.label.setWordWrap(True)
        self.label.setText(_(
                "The selection area can be moved with the <b>Arrow</b> keys, "\
                "<b>Ctrl</b> and <b>Alt</b> keys may be combined with these"\
                " keys for enlarge and reduce the size of selection. "\
                "<b>Maj</b> key is used as accelerator."))

    def set_style(self):
        self.title_lbl.setStyleSheet(style.SUBTITLE)
        self.line.setStyleSheet(style.UNDERLINE)
        self.ratio_lbl.setStyleSheet(style.SUBTITLE)
        self.line_3.setStyleSheet(style.UNDERLINE)
        self.same_as_rdo.setStyleSheet(style.RADIO)
        self.defined_rdo.setStyleSheet(style.RADIO)
        self.defined_cmb.setStyleSheet(style.COMBO)
        self.reverse_chb.setStyleSheet(style.CHECKBOX)
        self.custom_rdo.setStyleSheet(style.RADIO)
        self.width_spb.setStyleSheet(style.SPINBOX)
        self.height_spb.setStyleSheet(style.SPINBOX)
        self.append_chb.setStyleSheet(style.CHECKBOX)
        self.constrain_width_rdo.setStyleSheet(style.RADIO)
        self.constrain_height_rdo.setStyleSheet(style.RADIO)
        self.grids_lbl.setStyleSheet(style.SUBTITLE)
        self.line_4.setStyleSheet(style.UNDERLINE)
        self.grid_chb.setStyleSheet(style.CHECKBOX)
        self.shortcuts_lbl.setStyleSheet(style.SUBTITLE)
        self.line_6.setStyleSheet(style.UNDERLINE)
