# -*- coding: utf-8 -*-

# g9npage.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2011-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Config page for geolocation


import os
import logging
lgg = logging.getLogger(__name__)

from math import modf

from PyQt5.QtCore import Qt, QSize, pyqtSignal, QObject, QUrl
from PyQt5.QtGui import QIcon, QPixmap, QFont
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QLabel, QSpacerItem, QSizePolicy, QFrame,
                             QComboBox, QSpinBox, QDoubleSpinBox, QPushButton)
from PyQt5 import QtWebKitWidgets, QtWebKit

from g9n.mapviewer import MapViewer
import stylesheets as style

class G9nPage(QWidget):
    def __init__(self, parent):
        super().__init__(parent)
        self.parent_ = parent
        self.gridLayout_2 = QGridLayout(self)
        self.gridLayout_2.setContentsMargins(20, -1, -1, -1)
        self.verticalLayout_4 = QVBoxLayout()
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setSpacing(1)
        self.horizontalLayout = QHBoxLayout()
        self.title_lbl = QLabel(self)
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.title_lbl.setFont(font)
        self.horizontalLayout.addWidget(self.title_lbl)
        sp = QSpacerItem(178, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(sp)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.line = QFrame(self)
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)
        self.verticalLayout.addWidget(self.line)
        self.verticalLayout_4.addLayout(self.verticalLayout)
        sp1 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_4.addItem(sp1)
        self.horizontalLayout_6 = QHBoxLayout()
        self.label_2 = QLabel(self)
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(11)
        font.setBold(True)
        font.setItalic(True)
        self.label_2.setFont(font)
        self.horizontalLayout_6.addWidget(self.label_2)
        self.verticalLayout_2 = QVBoxLayout()
        sp2 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_2.addItem(sp2)
        self.line_2 = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line_2.sizePolicy().hasHeightForWidth())
        self.line_2.setSizePolicy(sizePolicy)
        self.line_2.setFrameShape(QFrame.HLine)
        self.line_2.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_2.addWidget(self.line_2)
        self.horizontalLayout_6.addLayout(self.verticalLayout_2)
        self.verticalLayout_4.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_4 = QHBoxLayout()
        sp3 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(sp3)
        self.start_lbl = QLabel(self)
        self.start_lbl.setAlignment(Qt.AlignRight|Qt.AlignTrailing|
                                        Qt.AlignVCenter)
        self.horizontalLayout_4.addWidget(self.start_lbl)
        self.site_cmb = QComboBox(self)
        self.site_cmb.setMinimumSize(QSize(150, 0))
        self.horizontalLayout_4.addWidget(self.site_cmb)
        sp4 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(sp4)
        self.verticalLayout_4.addLayout(self.horizontalLayout_4)
        sp5 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_4.addItem(sp5)
        self.horizontalLayout_7 = QHBoxLayout()
        self.label_5 = QLabel(self)
        self.label_5.setFont(font)
        self.horizontalLayout_7.addWidget(self.label_5)
        self.verticalLayout_3 = QVBoxLayout()
        sp6 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_3.addItem(sp6)
        self.line_3 = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line_3.sizePolicy().hasHeightForWidth())
        self.line_3.setSizePolicy(sizePolicy)
        self.line_3.setFrameShape(QFrame.HLine)
        self.line_3.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_3.addWidget(self.line_3)
        self.horizontalLayout_7.addLayout(self.verticalLayout_3)
        self.verticalLayout_4.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_2 = QHBoxLayout()
        sp7 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(sp7)
        self.gridLayout = QGridLayout()
        self.lat_lbl = QLabel(self)
        self.lat_lbl.setAlignment(Qt.AlignRight|Qt.AlignTrailing|
                                        Qt.AlignVCenter)
        self.gridLayout.addWidget(self.lat_lbl, 0, 0, 1, 1)
        self.la_deg_spb = QSpinBox(self)
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, 
                                        QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.la_deg_spb.sizePolicy()
                                        .hasHeightForWidth())
        self.la_deg_spb.setSizePolicy(sizePolicy)
        self.la_deg_spb.setMinimumSize(QSize(80, 0))
        self.la_deg_spb.setMinimum(-90)
        self.la_deg_spb.setMaximum(90)
        self.la_deg_spb.setProperty("value", 50)
        self.gridLayout.addWidget(self.la_deg_spb, 0, 1, 1, 1)
        self.la_min_spb = QSpinBox(self)
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, 
                                        QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.la_min_spb.sizePolicy()
                                        .hasHeightForWidth())
        self.la_min_spb.setSizePolicy(sizePolicy)
        self.la_min_spb.setMinimumSize(QSize(80, 0))
        self.la_min_spb.setMaximum(59)
        self.gridLayout.addWidget(self.la_min_spb, 0, 2, 1, 1)
        self.la_sec_spb = QSpinBox(self)
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, 
                                        QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.la_sec_spb.sizePolicy()
                                        .hasHeightForWidth())
        self.la_sec_spb.setSizePolicy(sizePolicy)
        self.la_sec_spb.setMinimumSize(QSize(80, 0))
        self.la_sec_spb.setMaximum(59)
        self.gridLayout.addWidget(self.la_sec_spb, 0, 3, 1, 1)
        self.label_3 = QLabel(self)
        self.label_3.setText(" = ")
        self.gridLayout.addWidget(self.label_3, 0, 4, 1, 1)
        self.la_dec_spb = QDoubleSpinBox(self)
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, 
                                        QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.la_dec_spb.sizePolicy()
                                        .hasHeightForWidth())
        self.la_dec_spb.setSizePolicy(sizePolicy)
        self.la_dec_spb.setMinimumSize(QSize(80, 0))
        self.la_dec_spb.setMinimum(-90.0)
        self.la_dec_spb.setMaximum(900000.0)
        self.la_dec_spb.setProperty("value", 50.0)
        self.gridLayout.addWidget(self.la_dec_spb, 0, 5, 1, 1)
        self.lon_lbl = QLabel(self)
        self.lon_lbl.setAlignment(Qt.AlignRight|Qt.AlignTrailing|
                                        Qt.AlignVCenter)
        self.gridLayout.addWidget(self.lon_lbl, 1, 0, 1, 1)
        self.lo_deg_spb = QSpinBox(self)
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, 
                                        QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lo_deg_spb.sizePolicy()
                                        .hasHeightForWidth())
        self.lo_deg_spb.setSizePolicy(sizePolicy)
        self.lo_deg_spb.setMinimumSize(QSize(80, 0))
        self.lo_deg_spb.setMinimum(-90)
        self.lo_deg_spb.setMaximum(90)
        self.gridLayout.addWidget(self.lo_deg_spb, 1, 1, 1, 1)
        self.lo_min_spb = QSpinBox(self)
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, 
                                        QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lo_min_spb.sizePolicy()
                                        .hasHeightForWidth())
        self.lo_min_spb.setSizePolicy(sizePolicy)
        self.lo_min_spb.setMinimumSize(QSize(80, 0))
        self.lo_min_spb.setMaximum(59)
        self.gridLayout.addWidget(self.lo_min_spb, 1, 2, 1, 1)
        self.lo_sec_spb = QSpinBox(self)
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, 
                                        QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lo_sec_spb.sizePolicy()
                                        .hasHeightForWidth())
        self.lo_sec_spb.setSizePolicy(sizePolicy)
        self.lo_sec_spb.setMinimumSize(QSize(80, 0))
        self.lo_sec_spb.setMaximum(59)
        self.gridLayout.addWidget(self.lo_sec_spb, 1, 3, 1, 1)
        self.label_4 = QLabel(self)
        self.label_4.setText(" = ")
        self.gridLayout.addWidget(self.label_4, 1, 4, 1, 1)
        self.lo_dec_spb = QDoubleSpinBox(self)
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, 
                                        QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lo_dec_spb.sizePolicy()
                                        .hasHeightForWidth())
        self.lo_dec_spb.setSizePolicy(sizePolicy)
        self.lo_dec_spb.setMinimumSize(QSize(80, 0))
        self.lo_dec_spb.setMinimum(-90.0)
        self.lo_dec_spb.setMaximum(900000.0)
        self.lo_dec_spb.setProperty("value", 0.0)
        self.gridLayout.addWidget(self.lo_dec_spb, 1, 5, 1, 1)
        self.zoom_lbl = QLabel(self)
        self.zoom_lbl.setAlignment(Qt.AlignRight|Qt.AlignTrailing|
                                        Qt.AlignVCenter)
        self.gridLayout.addWidget(self.zoom_lbl, 2, 0, 1, 1)
        self.zoom_spb = QSpinBox(self)
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, 
                                        QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.zoom_spb.sizePolicy()
                                        .hasHeightForWidth())
        self.zoom_spb.setSizePolicy(sizePolicy)
        self.zoom_spb.setMinimumSize(QSize(80, 0))
        self.zoom_spb.setMaximum(20)
        self.zoom_spb.setProperty("value", 10)
        self.gridLayout.addWidget(self.zoom_spb, 2, 1, 1, 1)
        sp8 = QSpacerItem(80, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.gridLayout.addItem(sp8, 2, 2, 1, 1)
        sp9 = QSpacerItem(80, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.gridLayout.addItem(sp9, 2, 3, 1, 1)
        sp10 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.gridLayout.addItem(sp10, 2, 4, 1, 1)
        sp11 = QSpacerItem(80, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.gridLayout.addItem(sp11, 2, 5, 1, 1)
        self.horizontalLayout_2.addLayout(self.gridLayout)
        sp12 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(sp12)
        self.verticalLayout_4.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QHBoxLayout()
        sp13 = QSpacerItem(10, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(sp13)
        self.locate_btn = QPushButton(self)
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/magnifier_36x30.png"), 
                                        QIcon.Normal, QIcon.Off)
        self.locate_btn.setIcon(icon)
        self.horizontalLayout_3.addWidget(self.locate_btn)
        sp14 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(sp14)
        self.verticalLayout_4.addLayout(self.horizontalLayout_3)
        self.webView = MapViewer(self)
        self.webView.setUrl(QUrl("about:blank"))
        self.verticalLayout_4.addWidget(self.webView)
        self.horizontalLayout_5 = QHBoxLayout()
        sp15 = QSpacerItem(158, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(sp15)
        self.set_btn = QPushButton(self)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/dialog-apply.svg"), 
                                        QIcon.Normal, QIcon.Off)
        self.set_btn.setIcon(icon1)
        self.horizontalLayout_5.addWidget(self.set_btn)
        sp16 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(sp16)
        self.verticalLayout_4.addLayout(self.horizontalLayout_5)
        self.gridLayout_2.addLayout(self.verticalLayout_4, 0, 0, 1, 1)

        self.site_cmb.addItems(['Google Map', 'Open Street Map'])
        self.map_values = False
        self.retranslateUi()
        self.set_style()

        self.site_cmb.currentIndexChanged.connect(self.on_site_changed)
        self.la_dec_spb.valueChanged.connect(self.on_latitude_dec_changed)
        self.la_deg_spb.valueChanged.connect(self.on_latitude_changed)
        self.la_min_spb.valueChanged.connect(self.on_latitude_changed)
        self.la_sec_spb.valueChanged.connect(self.on_latitude_changed)
        self.lo_dec_spb.valueChanged.connect(self.on_longitude_dec_changed)
        self.lo_deg_spb.valueChanged.connect(self.on_longitude_changed)
        self.lo_min_spb.valueChanged.connect(self.on_longitude_changed)
        self.lo_sec_spb.valueChanged.connect(self.on_longitude_changed)
        self.locate_btn.clicked.connect(self.locate)
        self.webView.coordinateChanged.connect(self.on_map_changed)
        self.set_btn.clicked.connect(self.update_coordinate)
        self.temp = None

    def set_config_values(self, conf, shortcuts):
        """Set the configuration values in widgets.

        Args:
        conf -- Dictionnary of user's prefs

        Keys:
        "g9n": {"expandsize": (700, 600),
                "default": (50.0, 4.0, 6),
                "userpoints": {}}
        """
        self.cfg = conf
        try:
            self.latitude = conf["g9n"]["default"][0]
        except:
            self.latitude = 53.0

        try:
            self.longitude = conf["g9n"]["default"][1]
        except:
            self.longitude = 0.0

        self.zoom = 12

        # FIXME No user pref ?
        self.site_cmb.setCurrentIndex(1)
        self.webView.server = 1
        self.la_dec_spb.setValue(self.latitude)
        self.lo_dec_spb.setValue(self.longitude)
        self.zoom_spb.setValue(self.zoom)
        self.map_values = [self.latitude, self.longitude, self.zoom]
        self.webView._setup((self.latitude, self.longitude, self.zoom), self.temp)

    def on_site_changed(self, idx):
        pass

    def on_latitude_dec_changed(self, lat):
        vals = self.decimal_to_dms(lat)
        self.set_latitude_values(vals)
        self.locate_btn.setEnabled(True)

    def on_longitude_dec_changed(self, long):
        vals = self.decimal_to_dms(long)
        self.set_longitude_values(vals)
        self.locate_btn.setEnabled(True)

    def on_latitude_changed(self, val):
        d = self.la_deg_spb.value()
        m = self.la_min_spb.value()
        s = self.la_sec_spb.value()
        dec = self.dms_to_decimal(d, m, s)
        self.set_decimal_latitude(dec)
        self.locate_btn.setEnabled(True)

    def on_longitude_changed(self, val):
        d = self.lo_deg_spb.value()
        m = self.lo_min_spb.value()
        s = self.lo_sec_spb.value()
        dec = self.dms_to_decimal(d, m, s)
        self.set_decimal_longitude(dec)
        self.locate_btn.setEnabled(True)

    def on_map_changed(self, val):
        try:
            lat, lon, zoom = [float(v) for v in val]
        except ValueError:
            # Coordinates are not set
            return

        self.map_values = (lat, lon, zoom)
        self.set_btn.setEnabled(True)

    def set_decimal_latitude(self, val):
        self.la_dec_spb.blockSignals(True)
        self.la_dec_spb.setValue(val)
        self.la_dec_spb.blockSignals(False)

    def set_decimal_longitude(self, val):
        self.lo_dec_spb.blockSignals(True)
        self.lo_dec_spb.setValue(val)
        self.lo_dec_spb.blockSignals(False)

    def set_latitude_values(self, vals):
        self._block_signals(True)
        self.la_deg_spb.setValue(vals[0])
        self.la_min_spb.setValue(vals[1])
        self.la_sec_spb.setValue(vals[2])
        self._block_signals(False)

    def set_longitude_values(self, vals):
        self._block_signals(True)
        self.lo_deg_spb.setValue(vals[0])
        self.lo_min_spb.setValue(vals[1])
        self.lo_sec_spb.setValue(vals[2])
        self._block_signals(False)

    def locate(self):
        la = self.la_dec_spb.value()
        lo = self.lo_dec_spb.value()
        z = self.zoom_spb.value()
        self.locate_btn.setEnabled(False)
        self.webView.go_to(la, lo, z)

    def update_coordinate(self):
        self.la_dec_spb.setValue(self.map_values[0])
        self.lo_dec_spb.setValue(self.map_values[1])
        self.zoom_spb.setValue(self.map_values[2])
        self.locate_btn.setEnabled(False)
        self.set_btn.setEnabled(False)

    def apply_change(self):
        """Apply settings to config dictionnary.

        """
        # FIXME Add to prefs
        #if self.site_cmb.currentIndex() == 0:
        la = self.la_dec_spb.value()
        lo = self.lo_dec_spb.value()
        z = self.zoom_spb.value()
        self.cfg["g9n"]["default"] = (la, lo, z)

    def decimal_to_dms(self, d):
        """Convert decimal degrees to degrees, minutes, seconds.

        Args:
        d -- float

        Returns:
        list(+-degrees, abs(min.), abs(sec.))
        """
        i, d = modf(d)
        j, m = modf(i*60)
        s = j*60
        return [int(d), int(abs(m)), abs(s)]

    def dms_to_decimal(self, d, m, s):
        """Convert degrees, minutes, seconds to decimal degrees.

        Args:
        d, m, s -- degrees, minutes, seconds

        Returns:
        float
        """
        dec = (abs(d) + round((m / 60.0), 4) + round((s / 3600.0), 4))
        return (-1 if d < 0 else 1) * dec

    def _block_signals(self, b):
        self.la_deg_spb.blockSignals(b)
        self.la_min_spb.blockSignals(b)
        self.la_sec_spb.blockSignals(b)
        self.lo_deg_spb.blockSignals(b)
        self.lo_min_spb.blockSignals(b)
        self.lo_sec_spb.blockSignals(b)

    def retranslateUi(self):
        self.title_lbl.setText(_("Geolocation - Configuration"))
        self.label_2.setText(_("Server"))
        self.start_lbl.setText(_("Startup with:"))
        self.label_5.setText(_("Default coordinates"))
        self.lat_lbl.setText(_("Latitude:"))
        self.la_deg_spb.setSuffix(_(" deg."))
        self.la_min_spb.setSuffix(_(" min."))
        self.la_sec_spb.setSuffix(_(" sec."))
        self.lon_lbl.setText(_("Longitude:"))
        self.lo_deg_spb.setSuffix(_(" deg."))
        self.lo_min_spb.setSuffix(_(" min."))
        self.lo_sec_spb.setSuffix(_(" sec."))
        self.zoom_lbl.setText(_("Zoom:"))
        self.locate_btn.setText(_("Locate"))
        self.set_btn.setText(_("Set coordinates"))

    def set_style(self):
        self.title_lbl.setStyleSheet(style.SUBTITLE)
        self.line.setStyleSheet(style.UNDERLINE)
        self.label_2.setStyleSheet(style.SUBTITLE)
        self.line_2.setStyleSheet(style.UNDERLINE)
        self.site_cmb.setStyleSheet(style.COMBO)
        self.label_5.setStyleSheet(style.SUBTITLE)
        self.line_3.setStyleSheet(style.UNDERLINE)
        self.la_deg_spb.setStyleSheet(style.SPINBOX)
        self.la_min_spb.setStyleSheet(style.SPINBOX)
        self.la_sec_spb.setStyleSheet(style.SPINBOX)
        self.la_dec_spb.setStyleSheet(style.DSPINBOX)
        self.lo_deg_spb.setStyleSheet(style.SPINBOX)
        self.lo_min_spb.setStyleSheet(style.SPINBOX)
        self.lo_sec_spb.setStyleSheet(style.SPINBOX)
        self.lo_dec_spb.setStyleSheet(style.DSPINBOX)
        self.zoom_spb.setStyleSheet(style.SPINBOX)
        self.locate_btn.setStyleSheet(style.PUSHBUTTONDIAL)
        self.set_btn.setStyleSheet(style.PUSHBUTTONDIAL)


