# -*- coding: utf-8 -*-

# gridpage.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2012-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Config page for grid

from PyQt5.QtCore import Qt, QSize
from PyQt5.QtGui import QIcon, QPixmap, QFont, QColor
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QLabel, QSpacerItem, QSizePolicy, QFrame,
                             QComboBox, QRadioButton, QSpinBox, QCheckBox,
                             QDialog)

import stylesheets as style

from VWidgets_PyQt5.vcolorcombobox import VColorComboBox
from settings.utils import LineEdit, EditButton, UndoButton, ShortcutEditor

class GridPage(QWidget):
    def __init__(self, parent):
        super().__init__(parent)
        self.parent_ = parent
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/plus.png"), QIcon.Normal, QIcon.Off)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/gtk-refresh.png"), 
                                        QIcon.Normal, QIcon.Off)
        self.gridLayout_2 = QGridLayout(self)
        self.gridLayout_2.setContentsMargins(20, -1, -1, -1)
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout_4 = QVBoxLayout()
        self.verticalLayout_4.setSpacing(1)
        self.horizontalLayout_2 = QHBoxLayout()
        self.title_lbl = QLabel(self)
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.title_lbl.setFont(font)
        self.horizontalLayout_2.addWidget(self.title_lbl)
        sp = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(sp)
        self.verticalLayout_4.addLayout(self.horizontalLayout_2)
        self.line = QFrame(self)
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_4.addWidget(self.line)
        self.verticalLayout.addLayout(self.verticalLayout_4)
        sp1 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout.addItem(sp1)
        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setSpacing(10)
        self.style_lbl = QLabel(self)
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(11)
        font.setBold(True)
        font.setItalic(True)
        self.style_lbl.setFont(font)
        self.horizontalLayout_3.addWidget(self.style_lbl)
        self.verticalLayout_5 = QVBoxLayout()
        sp2 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_5.addItem(sp2)
        self.line_4 = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line_4.sizePolicy().hasHeightForWidth())
        self.line_4.setSizePolicy(sizePolicy)
        self.line_4.setFrameShape(QFrame.HLine)
        self.line_4.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_5.addWidget(self.line_4)
        self.horizontalLayout_3.addLayout(self.verticalLayout_5)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.gridLayout = QGridLayout()
        sp3 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout.addItem(sp3, 0, 0, 1, 1)
        self.line_lbl = QLabel(self)
        self.line_lbl.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.gridLayout.addWidget(self.line_lbl, 0, 1, 1, 1)
        self.style_cmb = QComboBox(self)
        self.gridLayout.addWidget(self.style_cmb, 0, 2, 1, 1)
        sp4 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.gridLayout.addItem(sp4, 0, 3, 1, 1)
        sp5 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout.addItem(sp5, 1, 0, 1, 1)
        self.color_lbl = QLabel(self)
        self.color_lbl.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.gridLayout.addWidget(self.color_lbl, 1, 1, 1, 1)
        self.color_cmb = VColorComboBox(self)
        self.gridLayout.addWidget(self.color_cmb, 1, 2, 1, 1)
        sp6 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.gridLayout.addItem(sp6, 1, 3, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        sp7 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout.addItem(sp7)
        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setSpacing(10)
        self.settings_lbl = QLabel(self)
        self.settings_lbl.setFont(font)
        self.horizontalLayout_4.addWidget(self.settings_lbl)
        self.verticalLayout_6 = QVBoxLayout()
        sp8 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_6.addItem(sp8)
        self.line_5 = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line_5.sizePolicy().hasHeightForWidth())
        self.line_5.setSizePolicy(sizePolicy)
        self.line_5.setFrameShape(QFrame.HLine)
        self.line_5.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_6.addWidget(self.line_5)
        self.horizontalLayout_4.addLayout(self.verticalLayout_6)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5 = QHBoxLayout()
        sp9 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(sp9)
        self.gridLayout_3 = QGridLayout()
        self.lines_rdo = QRadioButton(self)
        self.gridLayout_3.addWidget(self.lines_rdo, 0, 0, 1, 1)
        self.h_lines_spb = QSpinBox(self)
        self.gridLayout_3.addWidget(self.h_lines_spb, 0, 1, 1, 1)
        self.v_lines_spb = QSpinBox(self)
        self.gridLayout_3.addWidget(self.v_lines_spb, 0, 2, 1, 1)
        self.gap_rdo = QRadioButton(self)
        self.gridLayout_3.addWidget(self.gap_rdo, 1, 0, 1, 1)
        self.h_gap_spb = QSpinBox(self)
        self.gridLayout_3.addWidget(self.h_gap_spb, 1, 1, 1, 1)
        self.v_gap_spb = QSpinBox(self)
        self.gridLayout_3.addWidget(self.v_gap_spb, 1, 2, 1, 1)
        self.ratio_chb = QCheckBox(self)
        self.gridLayout_3.addWidget(self.ratio_chb, 2, 0, 1, 1)
        self.ratio_cmb = QComboBox(self)
        self.ratio_cmb.setEnabled(False)
        self.ratio_cmb.setMinimumSize(QSize(120, 0))
        self.gridLayout_3.addWidget(self.ratio_cmb, 2, 1, 1, 1)
        self.reverse_chb = QCheckBox(self)
        self.reverse_chb.setEnabled(False)
        self.gridLayout_3.addWidget(self.reverse_chb, 2, 2, 1, 1)
        sp10 = QSpacerItem(40, 20, QSizePolicy.MinimumExpanding, 
                                        QSizePolicy.Minimum)
        self.gridLayout_3.addItem(sp10, 3, 0, 1, 1)
        self.custom_h_spb = QSpinBox(self)
        self.custom_h_spb.setEnabled(False)
        self.custom_h_spb.setMinimumSize(QSize(80, 0))
        self.gridLayout_3.addWidget(self.custom_h_spb, 3, 1, 1, 1)
        self.custom_v_spb = QSpinBox(self)
        self.custom_v_spb.setEnabled(False)
        self.custom_v_spb.setMinimumSize(QSize(80, 0))
        self.gridLayout_3.addWidget(self.custom_v_spb, 3, 2, 1, 1)
        sp11 = QSpacerItem(40, 20, QSizePolicy.MinimumExpanding, 
                                        QSizePolicy.Minimum)
        self.gridLayout_3.addItem(sp11, 4, 0, 1, 1)
        self.append_chb = QCheckBox(self)
        self.append_chb.setEnabled(False)
        self.gridLayout_3.addWidget(self.append_chb, 4, 1, 1, 1)
        sp12 = QSpacerItem(120, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.gridLayout_3.addItem(sp12, 4, 2, 1, 1)
        self.horizontalLayout_5.addLayout(self.gridLayout_3)
        sp13 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(sp13)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        sp14 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout.addItem(sp14)
        self.horizontalLayout_6 = QHBoxLayout()
        self.horizontalLayout_6.setSpacing(10)
        self.grids_lbl = QLabel(self)
        self.grids_lbl.setFont(font)
        self.horizontalLayout_6.addWidget(self.grids_lbl)
        self.verticalLayout_7 = QVBoxLayout()
        sp15 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_7.addItem(sp15)
        self.line_6 = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line_6.sizePolicy().hasHeightForWidth())
        self.line_6.setSizePolicy(sizePolicy)
        self.line_6.setFrameShape(QFrame.HLine)
        self.line_6.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_7.addWidget(self.line_6)
        self.horizontalLayout_6.addLayout(self.verticalLayout_7)
        self.verticalLayout.addLayout(self.horizontalLayout_6)
        self.horizontalLayout = QHBoxLayout()
        sp16 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(sp16)
        self.label_16 = QLabel(self)
        self.label_16.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.horizontalLayout.addWidget(self.label_16)
        self.horizontalLayout_11 = QHBoxLayout()
        self.horizontalLayout_11.setSpacing(0)
        self.set_grid_led = LineEdit(self, None, "grid")
        self.horizontalLayout_11.addWidget(self.set_grid_led)
        self.grid_change_btn = EditButton(self, None, icon, "grid")
        self.horizontalLayout_11.addWidget(self.grid_change_btn)
        self.grid_reverse_btn = UndoButton(self, None, icon1, "grid")
        self.horizontalLayout_11.addWidget(self.grid_reverse_btn)
        self.horizontalLayout.addLayout(self.horizontalLayout_11)
        sp17 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(sp17)
        self.verticalLayout.addLayout(self.horizontalLayout)
        sp18 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout.addItem(sp18)
        self.gridLayout_2.addLayout(self.verticalLayout, 0, 0, 1, 1)

        st = _("Solid")
        self.style_cmb.addItem(st)
        st = _("Dash")
        self.style_cmb.addItem(st)

        self.leds = [self.set_grid_led]
        self.undos = [self.grid_reverse_btn]
        self.senders = ["grid"]

        self.retranslateUi()
        self.set_style()
        self.set_connections()

    def set_config_values(self, conf, shortcuts):
        """Set the configuration values in widgets.

        Args:
        conf -- dictionnary of user's prefs

        Keys:
        2150 -- config dict
                {'line_style': 'solid', 'color': (192, 192, 192), 
                'thickness': 1, 'type_ratio': 'sameAs', 'ratio': 0, 
                'mode': 'line', 'numH': 3, 'numV': 5, 'offsetX': 0, 
                'offsetY': 0}
        5006 -- predefined ratios
        2532 -- show-hide shortcut
        "grid": {"line_style": "solid", 
                 "color": "#c0c0c0", 
                 "thickness": 1,
                 "type_ratio": "sameAs", 
                 "ratio": 0, 
                 "mode": "lines",
                 "numH": 3, 
                 "numV": 5, 
                 "offsetX": 0, 
                 "offsetY": 0}
        """
        self.conf = conf
        self.shortcuts = shortcuts['grid']
        self.ratios = conf["crop"]["ratios"]

        if self.conf["grid"]['line_style'] == 'dash':
            self.style_cmb.setCurrentIndex(1)

        self.set_color()

        if self.conf["grid"]['mode'] == 'lines':
            self.lines_rdo.setChecked(True)

        else:
            self.gap_rdo.setChecked(True)
            self.set_lines_widgets(False)

        self.update_values()
        st = _("Same as Image")
        self.ratio_cmb.addItem(st)
        self.ratio_cmb.addItems(self.ratios)
        st = _("Custom")
        self.ratio_cmb.addItem(st)

        ratio = self.conf["grid"]['type_ratio']
        if ratio:
            self.ratio_chb.setChecked(True)
            self.set_ratio_widgets(True)

            if self.conf["grid"]['type_ratio'] == 'sameAs':
                self.ratio_cmb.setCurrentIndex(0)

            else:
                ratio = self.get_ratio_string(ratio)
                if ratio in self.ratios:
                    self.ratio_cmb.setCurrentIndex(self.ratios.index(ratio))
                
                else:
                    self.ratio_cmb.setCurrentIndex(self.ratio_cmb.count() - 1)
                    h, v = ratio.split(':')
                    self.custom_h_spb.setValue(int(h))
                    self.custom_v_spb.setValue(int(v))
                    self.custom_h_spb.setEnabled(True)
                    self.custom_v_spb.setEnabled(True)

        self.set_grid_led.setText(self.shortcuts["show_hide"])

    def on_value_changed(self, sender):
        """Enable undo button when a shortcut has changed.

        Args:
        sender -- shortcut lineEdit
        """
        self.undos[self.senders.index(sender)].setEnabled(True)
        self.parent_.value_changed()

    def update_values(self):
        self.block_signals(True)
        self.h_lines_spb.setValue(self.conf["grid"]['numH'])
        self.v_lines_spb.setValue(self.conf["grid"]['numV'])
        self.h_gap_spb.setValue(self.conf["grid"]['offsetX'])
        self.v_gap_spb.setValue(self.conf["grid"]['offsetY'])
        self.block_signals(False)

    def set_color(self):
        self.color_cmb.set_current_color(QColor(self.conf["grid"]['color']))
      
    def block_signals(self, b):
        self.h_lines_spb.blockSignals(b)
        self.v_lines_spb.blockSignals(b)
        self.h_gap_spb.blockSignals(b)
        self.v_gap_spb.blockSignals(b)

    def set_ratio_widgets(self, state):
        self.ratio_cmb.setEnabled(state)
        self.ratio_cmb.setCurrentIndex(0)

    def set_custom_settings(self, b):
        self.custom_h_spb.setValue(1)
        self.custom_v_spb.setValue(1)
        self.custom_h_spb.setEnabled(b)
        self.custom_v_spb.setEnabled(b)

    def set_defined_ratio(self, idx):
        ratio = self.ratios[idx-1]
        self.conf["grid"]['type_ratio'] = 'predefined'
        self.conf["grid"]['ratio'] = ratio

    def set_lines_widgets(self, b):
        """Set enabled - disabled lines and gap spinBox.

        Args -- boolean, if true, set lines spinbox enabled
        """
        self.h_lines_spb.setEnabled(b)
        self.v_lines_spb.setEnabled(b)
        self.h_gap_spb.setEnabled(not b)
        self.v_gap_spb.setEnabled(not b)

    def get_ratio_string(self, r):
        """Return the ratio as string.

        Args:
        r -- Tuple or Fraction() instance

        Returns:
        str("numerator:denominator")
        """
        if isinstance(r, tuple):
            return ":".join([str(r[0]), str(r[1])])

        return ":".join([str(r.numerator), str(r.denominator)])

    def set_connections(self):
        self.style_cmb.currentIndexChanged.connect(self.on_style_changed)
        self.color_cmb.currentColorChanged.connect(self.on_color_changed)
        self.lines_rdo.toggled.connect(self.on_lines_checked)
        self.h_lines_spb.valueChanged.connect(self.on_h_lines_value_changed)
        self.v_lines_spb.valueChanged.connect(self.on_v_lines_value_changed)
        self.h_gap_spb.valueChanged.connect(self.on_h_gap_value_changed)
        self.v_gap_spb.valueChanged.connect(self.on_v_gap_value_changed)
        self.ratio_chb.stateChanged.connect(self.on_ratio_checked)
        self.ratio_cmb.currentIndexChanged.connect(self.on_ratio_defined)
        #self.reverse_chb.stateChanged.connect(self.on_reverse_checked)
        self.custom_h_spb.valueChanged.connect(self.on_custom_values_changed)
        self.custom_v_spb.valueChanged.connect(self.on_custom_values_changed)

    def on_style_changed(self, idx):
        self.conf["grid"]['line_style'] = 'dash' if idx else 'solid'

    def on_color_changed(self, name, color):
        self.conf["grid"]['color'] = color.name()

    def on_lines_checked(self, b):
        self.conf["grid"]['mode'] = 'lines' if b else 'gap'
        self.set_lines_widgets(b)

    def on_h_lines_value_changed(self, val):
        self.conf["grid"]['numH'] = val

    def on_v_lines_value_changed(self, val):
        self.conf["grid"]['numV'] = val

    def on_h_gap_value_changed(self, val):
        self.conf["grid"]['offsetX'] = val

    def on_v_gap_value_changed(self, val):
        self.conf["grid"]['offsetY'] = val

    def on_ratio_checked(self, b):
        self.conf["grid"]['ratio'] = 'sameAs' if b else 0
        self.set_ratio_widgets(b)

    def on_ratio_defined(self, idx):
        if len(self.ratios) + 1 == idx:
            self.set_custom_settings(True)
            return

        if not idx:
            self.conf["grid"]['ratio'] = 'sameAs'

        else:
            self.set_defined_ratio(idx)
        self.set_custom_settings(False)

    def on_custom_values_changed(self, val):
        h = self.custom_h_spb.value()
        v = self.custom_v_spb.value()
        self.conf["grid"]['ratio'] = (h, v)

    def edit_shortcut(self, sender):
        if sender == "grid":
            w = self.set_grid_led

        sht = unicode(w.text()).split("+")
        if len(sht) == 1:
            self.mod = " "
            self.key = sht[0]

        else:
            self.mod = sht[0]
            self.key = sht[1]

        Dial = QDialog()
        sh = ShortcutEditor(Dial, self, self.mod, self.key)
        reply = Dial.exec_()

        if reply:
            sc = ""
            if self.mod != " ":
                sc = sc.join([self.mod, "+"])

            sc = "".join([sc, self.key])
            w.setText(sc)

    def undo_change(self, sender):
        if sender == "grid":
            self.set_grid_led.setText(self.shortcuts["show_hide"])
            self.grid_reverse_btn.setEnabled(False)

    def apply_change(self):
        """Apply settings to config dictionnary.

        """
        self.shortcuts["show_hide"] = self.set_grid_led.text()

    def retranslateUi(self):
        self.title_lbl.setText(_("Grid Tool - Configuration"))
        self.style_lbl.setText(_("Style"))
        self.line_lbl.setText(_("Lines:"))
        self.color_lbl.setText(_("Color:"))
        self.settings_lbl.setText(_("Settings"))
        self.lines_rdo.setText(_("Lines:"))
        self.h_lines_spb.setToolTip(_("Horizontal lines"))
        self.v_lines_spb.setToolTip(_("Vertical lines"))
        self.gap_rdo.setText(_("Gap:"))
        self.h_gap_spb.setToolTip(_("Horizontal gap"))
        self.v_gap_spb.setToolTip(_("Vertical gap"))
        self.ratio_chb.setText(_("Ratio:"))
        self.reverse_chb.setToolTip(_("Reverse width / height"))
        self.reverse_chb.setText(_("Reverse"))
        self.custom_h_spb.setToolTip(_("Horizontal value"))
        self.custom_v_spb.setToolTip(_("Vertical value"))
        self.append_chb.setToolTip(_("Append this ratio into predefined list"))
        self.append_chb.setText(_("Append"))
        self.grids_lbl.setText(_("Grids"))
        self.label_16.setText(_("Show - Hide the grid:"))


    def set_style(self):
        self.title_lbl.setStyleSheet(style.SUBTITLE)
        self.line.setStyleSheet(style.UNDERLINE)
        self.style_lbl.setStyleSheet(style.SUBTITLE)
        self.line_4.setStyleSheet(style.UNDERLINE)
        self.style_cmb.setStyleSheet(style.COMBO)
        self.color_cmb.setStyleSheet(style.COMBO_DIAL)
        self.settings_lbl.setStyleSheet(style.SUBTITLE)
        self.line_5.setStyleSheet(style.UNDERLINE)
        self.lines_rdo.setStyleSheet(style.RADIO)
        self.h_lines_spb.setStyleSheet(style.SPINBOX)
        self.v_lines_spb.setStyleSheet(style.SPINBOX)
        self.gap_rdo.setStyleSheet(style.RADIO)
        self.h_gap_spb.setStyleSheet(style.SPINBOX)
        self.v_gap_spb.setStyleSheet(style.SPINBOX)
        self.ratio_chb.setStyleSheet(style.CHECKBOX)
        self.ratio_cmb.setStyleSheet(style.COMBO)
        self.reverse_chb.setStyleSheet(style.CHECKBOX)
        self.custom_h_spb.setStyleSheet(style.SPINBOX)
        self.custom_v_spb.setStyleSheet(style.SPINBOX)
        self.append_chb.setStyleSheet(style.CHECKBOX)
        self.grids_lbl.setStyleSheet(style.SUBTITLE)
        self.line_6.setStyleSheet(style.UNDERLINE)
