# -*- coding: utf-8 -*-

# magnifierpage.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2011-2017 Vincent Vande Vyvre
# Licence: GPL3


from PyQt5.QtCore import Qt, QSize
from PyQt5.QtGui import QIcon, QPixmap, QFont
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QLabel, QSpacerItem, QSizePolicy, QFrame,
                             QRadioButton, QSpinBox, QDialog, QButtonGroup)

import stylesheets as style

from settings.utils import LineEdit, EditButton, UndoButton, ShortcutEditor

class MagnifierPage(QWidget):
    def __init__(self, parent):
        super().__init__(parent)
        self.parent_ = parent
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/plus.png"), QIcon.Normal, QIcon.Off)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/gtk-refresh.png"), 
                        QIcon.Normal, QIcon.Off)
        self.gridLayout_3 = QGridLayout(self)
        self.gridLayout_3.setContentsMargins(20, -1, -1, -1)
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout_3 = QVBoxLayout()
        self.verticalLayout_3.setSpacing(1)
        self.horizontalLayout_8 = QHBoxLayout()
        self.loupe_lbl = QLabel(self)
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.loupe_lbl.setFont(font)
        self.horizontalLayout_8.addWidget(self.loupe_lbl)
        sp = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(sp)
        self.verticalLayout_3.addLayout(self.horizontalLayout_8)
        self.line = QFrame(self)
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_3.addWidget(self.line)
        self.verticalLayout.addLayout(self.verticalLayout_3)
        sp1 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout.addItem(sp1)
        self.horizontalLayout_9 = QHBoxLayout()
        self.horizontalLayout_9.setSpacing(10)
        self.style_lbl = QLabel(self)
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(11)
        font.setBold(True)
        font.setItalic(True)
        self.style_lbl.setFont(font)
        self.horizontalLayout_9.addWidget(self.style_lbl)
        self.verticalLayout_4 = QVBoxLayout()
        sp2 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_4.addItem(sp2)
        self.line_3 = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line_3.sizePolicy().hasHeightForWidth())
        self.line_3.setSizePolicy(sizePolicy)
        self.line_3.setFrameShape(QFrame.HLine)
        self.line_3.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_4.addWidget(self.line_3)
        self.horizontalLayout_9.addLayout(self.verticalLayout_4)
        self.verticalLayout.addLayout(self.horizontalLayout_9)
        self.horizontalLayout = QHBoxLayout()
        sp3 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(sp3)
        self.gridLayout_2 = QGridLayout()
        align = Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter
        self.label_4 = QLabel(self)
        self.label_4.setAlignment(align)
        self.gridLayout_2.addWidget(self.label_4, 0, 0, 1, 1)
        self.square_rdo = QRadioButton(self)
        self.buttonGroup = QButtonGroup(self)
        self.buttonGroup.addButton(self.square_rdo)
        self.gridLayout_2.addWidget(self.square_rdo, 0, 1, 1, 1)
        sp4 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout_2.addItem(sp4, 1, 0, 1, 1)
        self.circle_rdo = QRadioButton(self)
        self.circle_rdo.setChecked(True)
        self.buttonGroup.addButton(self.circle_rdo)
        self.gridLayout_2.addWidget(self.circle_rdo, 1, 1, 1, 1)
        self.label = QLabel(self)
        self.label.setAlignment(align)
        self.gridLayout_2.addWidget(self.label, 2, 0, 1, 1)
        self.size_spb = QSpinBox(self)
        self.size_spb.setMinimumSize(QSize(75, 20))
        self.size_spb.setMaximumSize(QSize(16777215, 20))
        self.size_spb.setMinimum(80)
        self.size_spb.setMaximum(500)
        self.size_spb.setProperty("value", 160)
        self.gridLayout_2.addWidget(self.size_spb, 2, 1, 1, 1)
        self.horizontalLayout.addLayout(self.gridLayout_2)
        sp5 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(sp5)
        self.verticalLayout.addLayout(self.horizontalLayout)
        sp6 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout.addItem(sp6)
        self.horizontalLayout_10 = QHBoxLayout()
        self.horizontalLayout_10.setSpacing(10)
        self.magnification_lbl = QLabel(self)
        self.magnification_lbl.setFont(font)
        self.horizontalLayout_10.addWidget(self.magnification_lbl)
        self.verticalLayout_5 = QVBoxLayout()
        sp7 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_5.addItem(sp7)
        self.line_4 = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line_4.sizePolicy().hasHeightForWidth())
        self.line_4.setSizePolicy(sizePolicy)
        self.line_4.setFrameShape(QFrame.HLine)
        self.line_4.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_5.addWidget(self.line_4)
        self.horizontalLayout_10.addLayout(self.verticalLayout_5)
        self.verticalLayout.addLayout(self.horizontalLayout_10)
        self.horizontalLayout_2 = QHBoxLayout()
        sp8 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(sp8)
        self.label_5 = QLabel(self)
        self.label_5.setAlignment(align)
        self.horizontalLayout_2.addWidget(self.label_5)
        self.factor_spb = QSpinBox(self)
        self.factor_spb.setMinimumSize(QSize(75, 20))
        self.factor_spb.setMaximumSize(QSize(16777215, 20))
        self.factor_spb.setMinimum(50)
        self.factor_spb.setMaximum(500)
        self.factor_spb.setProperty("value", 100)
        self.horizontalLayout_2.addWidget(self.factor_spb)
        sp9 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(sp9)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        sp10 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout.addItem(sp10)
        self.horizontalLayout_11 = QHBoxLayout()
        self.horizontalLayout_11.setSpacing(10)
        self.shortcuts_lbl = QLabel(self)
        self.shortcuts_lbl.setFont(font)
        self.horizontalLayout_11.addWidget(self.shortcuts_lbl)
        self.verticalLayout_6 = QVBoxLayout()
        sp11 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_6.addItem(sp11)
        self.line_5 = QFrame(self)
        self.line_5.setSizePolicy(sizePolicy)
        self.line_5.setFrameShape(QFrame.HLine)
        self.line_5.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_6.addWidget(self.line_5)
        self.horizontalLayout_11.addLayout(self.verticalLayout_6)
        self.verticalLayout.addLayout(self.horizontalLayout_11)
        self.horizontalLayout_3 = QHBoxLayout()
        sp12 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(sp12)
        self.gridLayout = QGridLayout()
        self.label_20 = QLabel(self)
        self.label_20.setAlignment(align)
        self.gridLayout.addWidget(self.label_20, 0, 0, 1, 1)
        self.horizontalLayout_13 = QHBoxLayout()
        self.horizontalLayout_13.setSpacing(0)
        self.show_led = LineEdit(self, None, "show")
        self.horizontalLayout_13.addWidget(self.show_led)
        self.show_change_btn = EditButton(self, None, icon, "show")
        self.horizontalLayout_13.addWidget(self.show_change_btn)
        self.show_reverse_btn = UndoButton(self, None, icon1, "show")
        self.horizontalLayout_13.addWidget(self.show_reverse_btn)
        self.gridLayout.addLayout(self.horizontalLayout_13, 0, 1, 1, 1)
        self.label_19 = QLabel(self)
        self.label_19.setAlignment(align)
        self.gridLayout.addWidget(self.label_19, 1, 0, 1, 1)
        self.horizontalLayout_12 = QHBoxLayout()
        self.horizontalLayout_12.setSpacing(0)
        self.focus_led = LineEdit(self, None, "focus")
        self.horizontalLayout_12.addWidget(self.focus_led)
        self.focus_change_btn = EditButton(self, None, icon, "focus")
        self.horizontalLayout_12.addWidget(self.focus_change_btn)
        self.focus_reverse_btn = UndoButton(self, None, icon1, "focus")
        self.horizontalLayout_12.addWidget(self.focus_reverse_btn)
        self.gridLayout.addLayout(self.horizontalLayout_12, 1, 1, 1, 1)
        self.horizontalLayout_3.addLayout(self.gridLayout)
        sp13 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(sp13)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        sp14 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout.addItem(sp14)
        self.gridLayout_3.addLayout(self.verticalLayout, 0, 0, 1, 1)
        self.leds = [self.show_led, self.focus_led]
        self.undos = [self.show_reverse_btn, self.focus_reverse_btn]
        self.senders = ["show", "focus"]
        self.retranslateUi()
        self.set_style()

    def set_config_values(self, conf, shortcuts):
        """Set the configuration values in widgets.

        Args:
        conf -- dictionnary of user's prefs
        "magnifier": {"factor": 1.0,
                      "size": 170,
                      "shape": "circle"}
        """
        self.conf = conf
        self.shortcuts = shortcuts["magnifier"]
        self.square_rdo.setChecked(conf["magnifier"]["shape"] == "square")
        self.size_spb.setValue(conf["magnifier"]["size"])
        self.factor_spb.setValue(int(conf["magnifier"]["factor"] * 100))
        self.show_led.setText(self.shortcuts["show_hide"])
        self.focus_led.setText(self.shortcuts["set_focus"])

        for u in self.undos:
            u.setEnabled(False)

    def apply_change(self):
        """Apply settings to config dictionnary.

        """
        self.conf["magnifier"]["factor"] = self.factor_spb.value() / 100.0
        self.conf["magnifier"]["size"] = self.size_spb.value()
        shape = ("circle", "square")[self.square_rdo.isChecked()]
        self.conf["magnifier"]["shape"] = shape
        self.shortcuts["show_hide"] = self.show_led.text()
        self.shortcuts["set_focus"] = self.focus_led.text()

    def on_value_changed(self, sender):
        """Enable undo button when a shortcut has changed.

        Args:
        sender -- shortcut lineEdit
        """
        self.undos[self.senders.index(sender)].setEnabled(True)
        self.parent_.value_changed()

    def edit_shortcut(self, sender):
        """Call shortcut editor dialog box when an edit button is pressed.

        Args:
        sender -- shortcut edit button
        """
        led = self.leds[self.senders.index(sender)]
        sht = led.text().split("+")

        if len(sht) == 1:
            self.mod = " "
            self.key = sht[0]

        else:
            self.mod = sht[0]
            self.key = sht[1]

        Dial = QDialog()
        sh = ShortcutEditor(Dial, self, self.mod, self.key)
        reply = Dial.exec_()

        if reply:
            sc = ""
            if self.mod != " ":
                sc = sc.join([self.mod, "+"])

            sc = "".join([sc, self.key])
            led.setText(sc)

    def undo_change(self, sender):
        """Undo the last change in shortcut group.

        Args:
        sender -- shortcut undo button
        """
        idx = self.senders.index(sender)
        if not idx:
            cfg = self.shortcuts["show_hide"]

        else:
            cfg = self.shortcuts["set_focus"]

        self.leds[idx].setText(cfg)
        self.undos[idx].setEnabled(False)

    def retranslateUi(self):
        self.loupe_lbl.setText(_("Magnifying Glass - Configuration"))
        self.style_lbl.setText(_("Style"))
        self.label_4.setText(_("Shape:"))
        self.square_rdo.setText(_("Square"))
        self.circle_rdo.setText(_("Circle"))
        self.label.setText(_("Size:"))
        self.size_spb.setSuffix(_(" pxl"))
        self.magnification_lbl.setText(_("Magnification"))
        self.label_5.setText(_("Magnification factor:"))
        self.factor_spb.setSuffix(_(" %"))
        self.shortcuts_lbl.setText(_("Shortcuts"))
        self.label_20.setText(_("Show - Hide:"))
        self.label_19.setText(_("Set the focus on the magnifier:"))

    def set_style(self):
        self.loupe_lbl.setStyleSheet(style.SUBTITLE)
        self.line.setStyleSheet(style.UNDERLINE)
        self.square_rdo.setStyleSheet(style.RADIO)
        self.circle_rdo.setStyleSheet(style.RADIO)
        self.size_spb.setStyleSheet(style.SPINBOX)
        self.magnification_lbl.setStyleSheet(style.SUBTITLE)
        self.line_4.setStyleSheet(style.UNDERLINE)
        self.factor_spb.setStyleSheet(style.SPINBOX)
        self.shortcuts_lbl.setStyleSheet(style.SUBTITLE)
        self.line_5.setStyleSheet(style.UNDERLINE)

