# -*- coding: utf-8 -*-

# sortingpage.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2011-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Config page for sorting mode


from PyQt5.QtCore import Qt, QSize
from PyQt5.QtGui import QIcon, QPixmap, QFont
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QLabel, QSpacerItem, QSizePolicy, QFrame,
                             QSpinBox, QCheckBox, QDialog)

import stylesheets as style

from settings.utils import LineEdit, EditButton, UndoButton, ShortcutEditor

class SortingPage(QWidget):
    def __init__(self, parent):
        super().__init__(parent)
        self.parent_ = parent
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/plus.png"), QIcon.Normal, QIcon.Off)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/gtk-refresh.png"), 
                                QIcon.Normal, QIcon.Off)

        self.gridLayout_4 = QGridLayout(self)
        self.gridLayout_4.setContentsMargins(20, -1, -1, -1)
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setSpacing(1)
        self.horizontalLayout_15 = QHBoxLayout()
        self.title_lbl = QLabel(self)
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.title_lbl.setFont(font)
        self.horizontalLayout_15.addWidget(self.title_lbl)
        sp = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_15.addItem(sp)
        self.verticalLayout_2.addLayout(self.horizontalLayout_15)
        self.line = QFrame(self)
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_2.addWidget(self.line)
        self.verticalLayout.addLayout(self.verticalLayout_2)
        sp1 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout.addItem(sp1)
        self.horizontalLayout_16 = QHBoxLayout()
        self.horizontalLayout_16.setSpacing(10)
        self.target_lbl = QLabel(self)
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(11)
        font.setBold(True)
        font.setItalic(True)
        self.target_lbl.setFont(font)
        self.target_lbl.setFrameShape(QFrame.NoFrame)
        self.horizontalLayout_16.addWidget(self.target_lbl)
        self.verticalLayout_3 = QVBoxLayout()
        sp2 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_3.addItem(sp2)
        self.line_2 = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line_2.sizePolicy().hasHeightForWidth())
        self.line_2.setSizePolicy(sizePolicy)
        self.line_2.setFrameShape(QFrame.HLine)
        self.line_2.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_3.addWidget(self.line_2)
        self.horizontalLayout_16.addLayout(self.verticalLayout_3)
        self.verticalLayout.addLayout(self.horizontalLayout_16)
        self.horizontalLayout_5 = QHBoxLayout()
        sp3 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(sp3)
        align = Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter
        self.label = QLabel(self)
        self.label.setAlignment(align)
        self.horizontalLayout_5.addWidget(self.label)
        self.targets_numb_spb = QSpinBox(self)
        self.targets_numb_spb.setMinimumSize(QSize(75, 20))
        self.targets_numb_spb.setMaximumSize(QSize(16777215, 20))
        self.targets_numb_spb.setMinimum(6)
        self.targets_numb_spb.setMaximum(24)
        self.horizontalLayout_5.addWidget(self.targets_numb_spb)
        sp4 = QSpacerItem(20, 10, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(sp4)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        sp5 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout.addItem(sp5)
        self.horizontalLayout_17 = QHBoxLayout()
        self.horizontalLayout_17.setSpacing(10)
        self.shortcuts_lbl = QLabel(self)
        self.shortcuts_lbl.setFont(font)
        self.shortcuts_lbl.setFrameShape(QFrame.NoFrame)
        self.horizontalLayout_17.addWidget(self.shortcuts_lbl)
        self.verticalLayout_4 = QVBoxLayout()
        sp6 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_4.addItem(sp6)
        self.line_3 = QFrame(self)
        self.line_3.setSizePolicy(sizePolicy)
        self.line_3.setFrameShape(QFrame.HLine)
        self.line_3.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_4.addWidget(self.line_3)
        self.horizontalLayout_17.addLayout(self.verticalLayout_4)
        self.verticalLayout.addLayout(self.horizontalLayout_17)
        self.gridLayout_3 = QGridLayout()
        sp7 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout_3.addItem(sp7, 0, 0, 1, 1)
        self.label_13 = QLabel(self)
        self.label_13.setAlignment(align)
        self.gridLayout_3.addWidget(self.label_13, 0, 1, 1, 1)
        self.horizontalLayout_6 = QHBoxLayout()
        self.horizontalLayout_6.setSpacing(0)
        self.next_right_tab_led = LineEdit(self, None, "next_right_tab")
        self.horizontalLayout_6.addWidget(self.next_right_tab_led)
        self.next_right_tab_change_btn = EditButton(self, None, icon, "next_right_tab")
        self.horizontalLayout_6.addWidget(self.next_right_tab_change_btn)
        self.next_right_tab_reverse_btn = UndoButton(self, None, icon1, 
                                                    "next_right_tab")
        self.horizontalLayout_6.addWidget(self.next_right_tab_reverse_btn)
        self.gridLayout_3.addLayout(self.horizontalLayout_6, 0, 2, 1, 1)
        sp8 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.gridLayout_3.addItem(sp8, 0, 3, 1, 1)
        sp9 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout_3.addItem(sp9, 1, 0, 1, 1)
        self.label_14 = QLabel(self)
        self.label_14.setAlignment(align)
        self.gridLayout_3.addWidget(self.label_14, 1, 1, 1, 1)
        self.horizontalLayout_11 = QHBoxLayout()
        self.horizontalLayout_11.setSpacing(0)
        self.next_left_tab_led = LineEdit(self, None, "next_left_tab")
        self.horizontalLayout_11.addWidget(self.next_left_tab_led)
        self.next_left_tab_change_btn = EditButton(self, None, icon, "next_left_tab")
        self.horizontalLayout_11.addWidget(self.next_left_tab_change_btn)
        self.next_left_tab_reverse_btn = UndoButton(self, None, icon1, "next_left_tab")
        self.horizontalLayout_11.addWidget(self.next_left_tab_reverse_btn)
        self.gridLayout_3.addLayout(self.horizontalLayout_11, 1, 2, 1, 1)
        sp10 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.gridLayout_3.addItem(sp10, 1, 3, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout_3)
        self.horizontalLayout_2 = QHBoxLayout()
        sp11 = QSpacerItem(10, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(sp11)
        self.label_15 = QLabel(self)
        self.horizontalLayout_2.addWidget(self.label_15)
        sp12 = QSpacerItem(228, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(sp12)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_18 = QHBoxLayout()
        sp13 = QSpacerItem(18, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_18.addItem(sp13)
        self.gridLayout = QGridLayout()
        self.label_16 = QLabel(self)
        self.label_16.setAlignment(align)
        self.gridLayout.addWidget(self.label_16, 0, 0, 1, 1)
        self.horizontalLayout_8 = QHBoxLayout()
        self.horizontalLayout_8.setSpacing(0)
        self.next_right_button_led = LineEdit(self, None, "next_right_button")
        self.horizontalLayout_8.addWidget(self.next_right_button_led)
        self.next_right_button_change_btn = EditButton(self, None,
                                           icon, "next_right_button")
        self.horizontalLayout_8.addWidget(self.next_right_button_change_btn)
        self.next_right_button_reverse_btn = UndoButton(self, None,
                                           icon1, "next_right_button")
        self.horizontalLayout_8.addWidget(self.next_right_button_reverse_btn)
        self.gridLayout.addLayout(self.horizontalLayout_8, 0, 1, 1, 1)
        self.label_17 = QLabel(self)
        self.label_17.setAlignment(align)
        self.gridLayout.addWidget(self.label_17, 1, 0, 1, 1)
        self.horizontalLayout_7 = QHBoxLayout()
        self.horizontalLayout_7.setSpacing(0)
        self.next_left_button_led = LineEdit(self, None, "next_left_button")
        self.horizontalLayout_7.addWidget(self.next_left_button_led)
        self.next_left_button_change_btn = EditButton(self, None, icon, 
                                            "next_left_button")
        self.horizontalLayout_7.addWidget(self.next_left_button_change_btn)
        self.next_left_button_reverse_btn = UndoButton(self, None, icon1, 
                                            "next_left_button")
        self.horizontalLayout_7.addWidget(self.next_left_button_reverse_btn)
        self.gridLayout.addLayout(self.horizontalLayout_7, 1, 1, 1, 1)
        self.label_18 = QLabel(self)
        self.label_18.setAlignment(align)
        self.gridLayout.addWidget(self.label_18, 2, 0, 1, 1)
        self.horizontalLayout_9 = QHBoxLayout()
        self.horizontalLayout_9.setSpacing(0)
        self.first_letter_led = LineEdit(self, None, "first_letter")
        self.horizontalLayout_9.addWidget(self.first_letter_led)
        self.first_letter_change_btn = EditButton(self, None, icon, "first_letter")
        self.horizontalLayout_9.addWidget(self.first_letter_change_btn)
        self.first_letter_reverse_btn = UndoButton(self, None, icon1, "first_letter")
        self.horizontalLayout_9.addWidget(self.first_letter_reverse_btn)
        self.gridLayout.addLayout(self.horizontalLayout_9, 2, 1, 1, 1)
        self.label_22 = QLabel(self)
        self.label_22.setAlignment(align)
        self.gridLayout.addWidget(self.label_22, 3, 0, 1, 1)
        self.horizontalLayout_10 = QHBoxLayout()
        self.horizontalLayout_10.setSpacing(0)
        self.unselect_led = LineEdit(self, None, "unselect")
        self.horizontalLayout_10.addWidget(self.unselect_led)
        self.unselect_change_btn = EditButton(self, None, icon, "unselect")
        self.horizontalLayout_10.addWidget(self.unselect_change_btn)
        self.unselect_reverse_btn = UndoButton(self, None, icon1, "unselect")
        self.horizontalLayout_10.addWidget(self.unselect_reverse_btn)
        self.gridLayout.addLayout(self.horizontalLayout_10, 3, 1, 1, 1)
        self.horizontalLayout_18.addLayout(self.gridLayout)
        sp14 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_18.addItem(sp14)
        self.verticalLayout.addLayout(self.horizontalLayout_18)
        self.horizontalLayout_3 = QHBoxLayout()
        sp15 = QSpacerItem(13, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(sp15)
        self.label_20 = QLabel(self)
        self.horizontalLayout_3.addWidget(self.label_20)
        sp16 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(sp16)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_14 = QHBoxLayout()
        sp17 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_14.addItem(sp17)
        self.gridLayout_2 = QGridLayout()
        self.label_21 = QLabel(self)
        self.label_21.setAlignment(align)
        self.gridLayout_2.addWidget(self.label_21, 0, 0, 1, 1)
        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setSpacing(0)
        self.copy_led = LineEdit(self, None, "copy")
        self.horizontalLayout_4.addWidget(self.copy_led)
        self.copy_change_btn = EditButton(self, None, icon, "copy")
        self.horizontalLayout_4.addWidget(self.copy_change_btn)
        self.copy_reverse_btn = UndoButton(self, None, icon1, "copy")
        self.horizontalLayout_4.addWidget(self.copy_reverse_btn)
        self.gridLayout_2.addLayout(self.horizontalLayout_4, 0, 1, 1, 1)
        self.label_23 = QLabel(self)
        self.label_23.setAlignment(align)
        self.gridLayout_2.addWidget(self.label_23, 1, 0, 1, 1)
        self.horizontalLayout_12 = QHBoxLayout()
        self.horizontalLayout_12.setSpacing(0)
        self.move_led = LineEdit(self, None, "move")
        self.horizontalLayout_12.addWidget(self.move_led)
        self.move_change_btn = EditButton(self, None, icon, "move")
        self.horizontalLayout_12.addWidget(self.move_change_btn)
        self.move_reverse_btn = UndoButton(self, None, icon1, "move")
        self.horizontalLayout_12.addWidget(self.move_reverse_btn)
        self.gridLayout_2.addLayout(self.horizontalLayout_12, 1, 1, 1, 1)
        self.label_24 = QLabel(self)
        self.label_24.setAlignment(align)
        self.gridLayout_2.addWidget(self.label_24, 2, 0, 1, 1)
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setSpacing(0)
        self.close_folder_led = LineEdit(self, None, "close_folder")
        self.horizontalLayout.addWidget(self.close_folder_led)
        self.close_folder_change_btn = EditButton(self, None, icon, "close_folder")
        self.horizontalLayout.addWidget(self.close_folder_change_btn)
        self.close_folder_reverse_btn = UndoButton(self, None, icon1, "close_folder")
        self.horizontalLayout.addWidget(self.close_folder_reverse_btn)
        self.gridLayout_2.addLayout(self.horizontalLayout, 2, 1, 1, 1)
        self.label_25 = QLabel(self)
        self.label_25.setAlignment(align)
        self.gridLayout_2.addWidget(self.label_25, 3, 0, 1, 1)
        self.horizontalLayout_13 = QHBoxLayout()
        self.horizontalLayout_13.setSpacing(0)
        self.open_folder_led = LineEdit(self, None, "open_folder")
        self.horizontalLayout_13.addWidget(self.open_folder_led)
        self.open_folder_change_btn = EditButton(self, None, icon, "open_folder")
        self.horizontalLayout_13.addWidget(self.open_folder_change_btn)
        self.open_folder_reverse_btn = UndoButton(self, None, icon1, "open_folder")
        self.horizontalLayout_13.addWidget(self.open_folder_reverse_btn)
        self.gridLayout_2.addLayout(self.horizontalLayout_13, 3, 1, 1, 1)
        self.horizontalLayout_14.addLayout(self.gridLayout_2)
        sp18 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_14.addItem(sp18)
        self.verticalLayout.addLayout(self.horizontalLayout_14)
        sp19 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout.addItem(sp19)
        self.gridLayout_4.addLayout(self.verticalLayout, 0, 0, 1, 1)

        self.leds = [self.next_right_tab_led, self.next_left_tab_led, 
                    self.next_right_button_led, self.next_left_button_led,
                    self.first_letter_led, self.unselect_led,
                    self.copy_led, self.move_led, self.close_folder_led, 
                    self.open_folder_led]

        self.undos = [self.next_right_tab_reverse_btn, 
                        self.next_left_tab_reverse_btn,
                        self.next_right_button_reverse_btn, 
                        self.next_left_button_reverse_btn,
                        self.first_letter_reverse_btn, 
                        self.unselect_reverse_btn,
                        self.copy_reverse_btn, 
                        self.move_reverse_btn, 
                        self.close_folder_reverse_btn, 
                        self.open_folder_reverse_btn]

        self.senders = ["next_right_tab", "next_left_tab", "next_right_button", 
                        "next_left_button", "first_letter", "unselect",
                        "copy", "move", "close_folder", "open_folder"]

        self.retranslateUi()
        self.set_style()

    def set_config_values(self, conf, shortcuts):
        """Set the configuration values in widgets.

        Args:
        conf -- dictionnary of user's prefs
        shortcuts -- dictionnary of shortcuts

        Config keys:
        1050 -- number of target's folders
        2510 -- shift to next right tab
        2511 -- shift to next left tab
        2512 -- next right button
        2513 -- next left button
        2514 -- first letter
        2515 -- unselect
        2516 -- copy image
        2517 -- move image
        2518 -- close folder
        2519 -- open folder
        "sorting": {"number": 6,
                    "paths": []}

        shortcuts keys:
        "sorting": {"next_right_tab": "Ctrl+Right",
                    "next_left_tab": "Ctrl+Left",
                    "next_right_button": "Shift+Right",
                    "next_left_button": "Shift+Left",
                    "unselect": "Shift+Down",
                    "copy": "C",
                    "move": "V",
                    "close_folder": "W",
                    "open_folder": "N"}

        """
        self.conf = conf
        self.shortcuts = shortcuts["sorting"]
        self.targets_numb_spb.setValue(conf["sorting"]["number"])
        self.next_right_tab_led.setText(self.shortcuts["next_right_tab"])
        self.next_left_tab_led.setText(self.shortcuts["next_left_tab"])
        self.next_right_button_led.setText(self.shortcuts["next_right_button"])
        self.next_left_button_led.setText(self.shortcuts["next_left_button"])
        #txt = self.shortcuts[] + "+(first letter)"
        #self.first_letter_led.setText(txt)
        self.unselect_led.setText(self.shortcuts["unselect"])
        self.copy_led.setText(self.shortcuts["copy"])
        self.move_led.setText(self.shortcuts["move"])
        self.close_folder_led.setText(self.shortcuts["close_folder"])
        self.open_folder_led.setText(self.shortcuts["open_folder"])
        for u in self.undos:
            u.setEnabled(False)

    def apply_change(self):
        """Apply settings to config dictionnary.

        """
        self.conf["sorting"]["number"] = self.targets_numb_spb.value()
        # FIXME Shortcuts
        self.shortcuts["next_right_tab"] = self.next_right_tab_led.text()
        self.shortcuts["next_left_tab"] = self.next_left_tab_led.text()
        self.shortcuts["next_right_button"] = self.next_right_button_led.text()
        self.shortcuts["next_left_button"] = self.next_left_button_led.text()
        #s = self.first_letter_led.text().replace("+(first letter)", "")
        #self.shortcuts[] = s
        self.shortcuts["unselect"] = self.unselect_led.text()
        self.shortcuts["copy"] = self.copy_led.text()
        self.shortcuts["move"] = self.move_led.text()
        self.shortcuts["close_folder"] = self.close_folder_led.text()
        self.shortcuts["open_folder"] = self.open_folder_led.text()

    def on_value_changed(self, sender):
        """Enable undo button when a shortcut has changed.

        Args:
        sender -- shortcut lineEdit
        """
        self.undos[self.senders.index(sender)].setEnabled(True)
        self.parent_.value_changed()

    def edit_shortcut(self, sender):
        """Call shortcut editor dialog box when an edit button is pressed.

        Args:
        sender -- shortcut edit button
        """
        led = self.leds[self.senders.index(sender)]
        sht = unicode(led.text()).split("+")

        if len(sht) == 1:
            self.mod = " "
            self.key = sht[0]

        else:
            self.mod = sht[0]
            self.key = sht[1]

        Dial = QDialog()
        sh = ShortcutEditor(Dial, self, self.mod, self.key)
        reply = Dial.exec_()

        if reply:
            sc = ""
            if sender == "first_letter":
                if self.mod == " ":
                    # This shorcut need a modifier
                    return

                led.setText(self.mod + "+(first letter)")
                return

            if self.mod != " ":
                sc = sc.join([self.mod, "+"])

            sc = "".join([sc, self.key])
            led.setText(sc)

    def undo_change(self, sender):
        """Undo the last change in shortcut group.

        Args:
        sender -- shortcut undo button
        """
        idx = self.senders.index(sender)
        self.leds[idx].setText(self.shortcuts[sender])
        self.undos[idx].setEnabled(False)

    def retranslateUi(self):
        self.title_lbl.setText(_("Sorting Mode - Configuration"))
        self.target_lbl.setText(_("Targets"))
        self.label.setText(_("Number of targets:"))
        self.shortcuts_lbl.setText(_("Shortcuts"))
        self.label_13.setText(_("Shift to next tab to the right:"))
        self.label_14.setText(_("Shift to next tab to the left:"))
        self.label_15.setText(_("Selection of one target button."))
        self.label_16.setText(_("Left to right:"))
        self.label_17.setText(_("Right to left:"))
        self.label_18.setText(_("With first letter of folder:"))
        self.label_22.setText(_("Unselect:"))
        self.label_20.setText(_("When a target button is selected,"
                                " the following shortcuts are available\n"
                                "for this target :"))
        self.label_21.setText(_("Copy the current image in this folder:"))
        self.label_23.setText(_("Move the current image in this folder:"))
        self.label_24.setText(_("Close this folder:"))
        self.label_25.setText(_("Open a new folder:"))

    def set_style(self):
        self.title_lbl.setStyleSheet(style.SUBTITLE)
        self.line.setStyleSheet(style.UNDERLINE)
        self.target_lbl.setStyleSheet(style.SUBTITLE)
        self.line_2.setStyleSheet(style.UNDERLINE)
        self.targets_numb_spb.setStyleSheet(style.SPINBOX)
        self.shortcuts_lbl.setStyleSheet(style.SUBTITLE)
        self.line_3.setStyleSheet(style.UNDERLINE)
