# -*- coding: utf-8 -*-

# startpage.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2011-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Config page for startup


import os

from PyQt5.QtCore import Qt, QSize
from PyQt5.QtGui import QFont
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QLabel, QSpacerItem, QSizePolicy, QFrame, QSpinBox,
                             QRadioButton, QButtonGroup, QCheckBox)

import stylesheets as style

from VWidgets_PyQt5.vlineedit import VLineEdit

class StartPage(QWidget):
    def __init__(self, parent):
        super().__init__(parent)
        self.parent_ = parent
        self.core = parent.core
        self.gridLayout_4 = QGridLayout(self)
        self.gridLayout_4.setContentsMargins(20, -1, -1, -1)
        self.verticalLayout_7 = QVBoxLayout()
        self.verticalLayout_6 = QVBoxLayout()
        self.verticalLayout_6.setSpacing(1)
        self.horizontalLayout_7 = QHBoxLayout()
        self.title_lbl = QLabel(self)
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.title_lbl.setFont(font)
        self.horizontalLayout_7.addWidget(self.title_lbl)
        sp = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(sp)
        self.verticalLayout_6.addLayout(self.horizontalLayout_7)
        self.line = QFrame(self)
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_6.addWidget(self.line)
        self.verticalLayout_7.addLayout(self.verticalLayout_6)
        sp1 = QSpacerItem(17, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_7.addItem(sp1)
        align = Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(11)
        font.setBold(True)
        font.setItalic(True)
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setSpacing(10)
        self.toolpanel_lbl = QLabel(self)
        self.toolpanel_lbl.setFont(font)
        self.toolpanel_lbl.setAlignment(align)
        self.horizontalLayout_2.addWidget(self.toolpanel_lbl)
        self.verticalLayout_2 = QVBoxLayout()
        sp7 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_2.addItem(sp7)
        self.line_3 = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line_3.sizePolicy().hasHeightForWidth())
        self.line_3.setSizePolicy(sizePolicy)
        self.line_3.setFrameShape(QFrame.HLine)
        self.line_3.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_2.addWidget(self.line_3)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)
        self.verticalLayout_7.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_8 = QHBoxLayout()
        sp8 = QSpacerItem(10, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(sp8)
        self.toolpanel_chb = QCheckBox(self)
        self.horizontalLayout_8.addWidget(self.toolpanel_chb)
        sp9 = QSpacerItem(188, 17, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(sp9)
        self.verticalLayout_7.addLayout(self.horizontalLayout_8)
        self.horizontalLayout_9 = QHBoxLayout()
        sp10 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(sp10)
        self.gridLayout_2 = QGridLayout()
        self.gridLayout_2.setVerticalSpacing(2)
        self.side_lbl = QLabel(self)
        self.side_lbl.setAlignment(align)
        self.gridLayout_2.addWidget(self.side_lbl, 0, 0, 1, 1)
        self.panel_on_right_rdo = QRadioButton(self)
        self.buttonGroup_2 = QButtonGroup(self)
        self.buttonGroup_2.setObjectName("buttonGroup_2")
        self.buttonGroup_2.addButton(self.panel_on_right_rdo)
        self.gridLayout_2.addWidget(self.panel_on_right_rdo, 0, 1, 1, 1)
        sp11 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout_2.addItem(sp11, 1, 0, 1, 1)
        self.panel_on_left_rdo = QRadioButton(self)
        self.buttonGroup_2.addButton(self.panel_on_left_rdo)
        self.gridLayout_2.addWidget(self.panel_on_left_rdo, 1, 1, 1, 1)
        self.horizontalLayout_9.addLayout(self.gridLayout_2)
        sp12 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(sp12)
        self.verticalLayout_7.addLayout(self.horizontalLayout_9)
        self.horizontalLayout_10 = QHBoxLayout()
        sp13 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_10.addItem(sp13)
        self.advice_lbl = QLabel(self)
        self.advice_lbl.setEnabled(False)
        self.horizontalLayout_10.addWidget(self.advice_lbl)
        sp14 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_10.addItem(sp14)
        self.verticalLayout_7.addLayout(self.horizontalLayout_10)
        sp15 = QSpacerItem(17, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_7.addItem(sp15)
        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setSpacing(10)
        self.folder_lbl = QLabel(self)
        self.folder_lbl.setFont(font)
        self.folder_lbl.setAlignment(align)
        self.horizontalLayout_3.addWidget(self.folder_lbl)
        self.verticalLayout_3 = QVBoxLayout()
        sp16 = QSpacerItem(254, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_3.addItem(sp16)
        self.line_4 = QFrame(self)
        self.line_4.setSizePolicy(sizePolicy)
        self.line_4.setFrameShape(QFrame.HLine)
        self.line_4.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_3.addWidget(self.line_4)
        self.horizontalLayout_3.addLayout(self.verticalLayout_3)
        self.verticalLayout_7.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_11 = QHBoxLayout()
        sp17 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_11.addItem(sp17)
        self.gridLayout_3 = QGridLayout()
        self.open_lbl = QLabel(self)
        self.open_lbl.setAlignment(align)
        self.gridLayout_3.addWidget(self.open_lbl, 0, 0, 1, 1)
        self.last_open_rdo = QRadioButton(self)
        self.buttonGroup_3 = QButtonGroup(self)
        self.buttonGroup_3.setObjectName("buttonGroup_3")
        self.buttonGroup_3.addButton(self.last_open_rdo)
        self.gridLayout_3.addWidget(self.last_open_rdo, 0, 1, 1, 2)
        sp18 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout_3.addItem(sp18, 0, 3, 1, 2)
        sp19 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout_3.addItem(sp19, 1, 0, 1, 1)
        self.choosed_folder_rdo = QRadioButton(self)
        self.buttonGroup_3.addButton(self.choosed_folder_rdo)
        self.gridLayout_3.addWidget(self.choosed_folder_rdo, 1, 1, 1, 1)
        self.choose_folder_led = VLineEdit(self)
        self.choose_folder_led.setSizePolicy(sizePolicy)
        self.choose_folder_led.setMinimumSize(QSize(220, 0))
        self.choose_folder_led.setResetButton(True)
        self.choose_folder_led.setHistoricButton(True)
        self.choose_folder_led.setCommandButton(True)
        self.choose_folder_led.set_command_icon("medias/magnifier_36x30.png")
        self.choose_folder_led._set_command_enabled(True)
        self.choose_folder_led.setEnabled(False)
        self.gridLayout_3.addWidget(self.choose_folder_led, 1, 2, 1, 3)
        sp20 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout_3.addItem(sp20, 2, 0, 1, 1)
        self.dont_open_rdo = QRadioButton(self)
        self.buttonGroup_3.addButton(self.dont_open_rdo)
        self.gridLayout_3.addWidget(self.dont_open_rdo, 2, 1, 1, 3)
        sp21 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout_3.addItem(sp21, 2, 4, 1, 1)
        self.horizontalLayout_11.addLayout(self.gridLayout_3)
        sp22 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_11.addItem(sp22)
        self.verticalLayout_7.addLayout(self.horizontalLayout_11)
        sp23 = QSpacerItem(17, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_7.addItem(sp23)
        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setSpacing(10)
        self.recent_lbl = QLabel(self)
        self.recent_lbl.setFont(font)
        self.recent_lbl.setAlignment(align)
        self.horizontalLayout_4.addWidget(self.recent_lbl)
        self.verticalLayout_4 = QVBoxLayout()
        sp24 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_4.addItem(sp24)
        self.line_5 = QFrame(self)
        self.line_5.setSizePolicy(sizePolicy)
        self.line_5.setFrameShape(QFrame.HLine)
        self.line_5.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_4.addWidget(self.line_5)
        self.horizontalLayout_4.addLayout(self.verticalLayout_4)
        self.verticalLayout_7.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_12 = QHBoxLayout()
        sp25 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(sp25)
        self.num_recent_lbl = QLabel(self)
        self.num_recent_lbl.setAlignment(align)
        self.horizontalLayout_12.addWidget(self.num_recent_lbl)
        self.recents_spb = QSpinBox(self)
        self.recents_spb.setMinimum(1)
        self.horizontalLayout_12.addWidget(self.recents_spb)
        sp26 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(sp26)
        self.verticalLayout_7.addLayout(self.horizontalLayout_12)
        sp27 = QSpacerItem(17, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_7.addItem(sp27)
        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setSpacing(10)
        self.trash_lbl = QLabel(self)
        self.trash_lbl.setFont(font)
        self.trash_lbl.setAlignment(align)
        self.horizontalLayout_5.addWidget(self.trash_lbl)
        self.verticalLayout_5 = QVBoxLayout()
        sp28 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_5.addItem(sp28)
        self.line_6 = QFrame(self)
        self.line_6.setSizePolicy(sizePolicy)
        self.line_6.setFrameShape(QFrame.HLine)
        self.line_6.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_5.addWidget(self.line_6)
        self.horizontalLayout_5.addLayout(self.verticalLayout_5)
        self.verticalLayout_7.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_14 = QHBoxLayout()
        sp31 = QSpacerItem(10, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_14.addItem(sp31)
        self.empty_trash_chb = QCheckBox(self)
        self.horizontalLayout_14.addWidget(self.empty_trash_chb)
        sp32 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_14.addItem(sp32)
        self.verticalLayout_7.addLayout(self.horizontalLayout_14)
        sp33 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout_7.addItem(sp33)
        self.gridLayout_4.addLayout(self.verticalLayout_7, 0, 0, 1, 1)

        self.panel_on_left_rdo.setChecked(True)

        self.retranslateUi()
        self.set_style()
        self.set_connections()

    def set_config_values(self, conf, shortcuts):
        """Set the configuration values in widgets.

        Args:
        conf -- dictionnary of user's prefs
        shortcuts -- there's no shortcuts in this page

        Keys:
        "startup": {"size": (1000, 700),
                    "mode": 0,
                    "trailer": 1,
                    "panel": 2,
                    "open": "",
                    "always_open": "",
                    "empty_trash": False,
                    "2screens": True,
                    "map_server": 0,
                    "numb_recents": 20}
        """
        self.conf = conf
        self.toolpanel_chb.setChecked(True)
        if conf["startup"]["panel"] == 2:
            self.panel_on_right_rdo.setChecked(True)

        if conf["startup"]["always_open"]:
            self.choose_folder_led.setText(conf["startup"]["always_open"])

        elif conf["startup"]["open"]:
            self.last_open_rdo.setChecked(True)

        else:
            self.dont_open_rdo.setChecked(True)

        self.recents_spb.setValue(conf["startup"]["numb_recents"])
        self.choose_folder_led.add_historic_entries(conf["folders"]["recents"], 
                                                    True)
        self.empty_trash_chb.setChecked(conf["startup"]["empty_trash"])

    def set_connections(self):
        self.toolpanel_chb.stateChanged.connect(self.set_toolpanel_widgets)
        self.buttonGroup_2.buttonClicked.connect(self.set_advice_lbl)
        self.choose_folder_led.editingFinished.connect(self.check_for_folder)
        self.choose_folder_led.commandTriggered.connect(self.choose_folder)
        self.choosed_folder_rdo.toggled.connect(self.activate_widgets)

    def apply_change(self):
        """Apply settings to config dictionnary.

        """
        self.conf["startup"]["panel"] = self.get_dockpanel_side()
        self.conf["startup"]["open"] = ''
        self.conf["startup"]["always_open"] = ''

        if self.last_open_rdo.isChecked():
            try:
                fld = self.core.source_fld.where
            except AttributeError:
                # No source folder defined
                fld = self.core.recents.get_first

            if fld is None:
                fld = ""

            self.conf["startup"]["open"] = fld

        elif self.choosed_folder_rdo.isChecked(): 
            self.conf["startup"]["always_open"] = self.choose_folder_led.text()

        self.conf["startup"]["numb_recents"] = self.recents_spb.value()
        self.conf["startup"]["empty_trash"] = self.empty_trash_chb.isChecked()

    def choose_folder(self):
        """Browse to folder.

        """
        fld = self.parent_.get_dir()
        if not os.path.isdir(fld):
            return

        self.choose_folder_led.setText(fld)

    def set_toolpanel_widgets(self, state):
        self.panel_on_right_rdo.setEnabled(state)
        self.panel_on_left_rdo.setEnabled(state)

    def set_advice_lbl(self, btn):
        self.advice_lbl.setEnabled(True)
        
    def activate_widgets(self, b):
        self.choose_folder_led.setEnabled(b)

    def check_for_folder(self):
        path = self.choose_folder_led.text()
        if not path:
            return

        if not os.path.isdir(path):
            self.choose_folder_led.setStyleSheet('QLineEdit {color: #FF0000;}')

        else:
            self.choose_folder_led.setStyleSheet('QLineEdit {color: #000000;}')
            return True

    def get_dockpanel_side(self):
        if self.panel_on_right_rdo.isChecked():
            return 2

        return 1

    def retranslateUi(self):
        self.title_lbl.setText(_("Startup - Configuration"))
        self.toolpanel_lbl.setText(_("Properties panel "))
        self.toolpanel_chb.setText(_("Show the properties panel"))
        self.side_lbl.setText("Side:")
        self.panel_on_right_rdo.setText(_("On the right side"))
        self.panel_on_left_rdo.setText(_("On the left side"))
        self.advice_lbl.setText(_("<i> This change will be applied at the"
                            " next startup of Oqapy.</i>"))
        self.folder_lbl.setText(_("Current folder "))
        self.open_lbl.setText(_("Open folder:"))
        self.last_open_rdo.setText(_("Last open folder"))
        self.choosed_folder_rdo.setText(_("Always open: "))
        self.choose_folder_led.setInformativeText(_("Folder path"))
        self.choose_folder_led.setToolTip(_("Enter path of folder."))
        self.choose_folder_led.command_button.setToolTip("Browse")
        self.dont_open_rdo.setText(_("Don\'t open any folder"))
        self.recent_lbl.setText(_("Recent folders "))
        self.num_recent_lbl.setText(_("Number of recent folders:"))
        self.trash_lbl.setText(_("Trash can "))
        self.empty_trash_chb.setText(_("To empty the trash can at startup"))

    def set_style(self):
        self.title_lbl.setStyleSheet(style.SUBTITLE)
        self.line.setStyleSheet(style.UNDERLINE)
        self.toolpanel_lbl.setStyleSheet(style.SUBTITLE)
        self.line_3.setStyleSheet(style.UNDERLINE)
        self.toolpanel_chb.setStyleSheet(style.CHECKBOX)
        self.panel_on_right_rdo.setStyleSheet(style.RADIO)
        self.panel_on_left_rdo.setStyleSheet(style.RADIO)
        self.folder_lbl.setStyleSheet(style.SUBTITLE)
        self.line_4.setStyleSheet(style.UNDERLINE)
        self.last_open_rdo.setStyleSheet(style.RADIO)
        self.choosed_folder_rdo.setStyleSheet(style.RADIO)
        self.choose_folder_led.setStyleSheet(style.LINEEDIT)
        self.dont_open_rdo.setStyleSheet(style.RADIO)
        self.recent_lbl.setStyleSheet(style.SUBTITLE)
        self.line_5.setStyleSheet(style.UNDERLINE)
        self.recents_spb.setStyleSheet(style.SPINBOX)
        self.trash_lbl.setStyleSheet(style.SUBTITLE)
        self.line_6.setStyleSheet(style.UNDERLINE)
        self.empty_trash_chb.setStyleSheet(style.CHECKBOX)

