# -*- coding: utf-8 -*-

# trailerpage.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2011-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Config page for trailer

import time

from fractions import Fraction

from PyQt5.QtCore import Qt, QSize
from PyQt5.QtGui import QIcon, QPixmap, QFont
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QLabel, QSpacerItem, QSizePolicy, QFrame,
                             QSpinBox, QCheckBox, QDialog, QGraphicsView,
                             QGraphicsScene, QGraphicsPixmapItem)

import stylesheets as style

from settings.utils import LineEdit, EditButton, UndoButton, ShortcutEditor
from traileritemdrawer import TrailerItemDrawer
from tools.path import Path

class TrailerPage(QWidget):
    def __init__(self, parent):
        super().__init__(parent)
        self.parent_ = parent
        self.temp = None
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/plus.png"), QIcon.Normal, QIcon.Off)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/gtk-refresh.png"), 
                                        QIcon.Normal, QIcon.Off)
        self.gridLayout_2 = QGridLayout(self)
        self.gridLayout_2.setContentsMargins(20, -1, -1, -1)
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_4 = QVBoxLayout()
        self.verticalLayout_4.setSpacing(1)
        self.horizontalLayout_5 = QHBoxLayout()
        self.title_lbl = QLabel(self)
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.title_lbl.setFont(font)
        self.horizontalLayout_5.addWidget(self.title_lbl)
        sp = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(sp)
        self.verticalLayout_4.addLayout(self.horizontalLayout_5)
        self.line = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line.sizePolicy().hasHeightForWidth())
        self.line.setSizePolicy(sizePolicy)
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_4.addWidget(self.line)
        self.verticalLayout_2.addLayout(self.verticalLayout_4)
        sp1 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_2.addItem(sp1)
        self.horizontalLayout_11 = QHBoxLayout()
        self.horizontalLayout_11.setSpacing(10)
        self.style_lbl = QLabel(self)
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(11)
        font.setBold(True)
        font.setItalic(True)
        font.setWeight(75)
        self.style_lbl.setFont(font)
        self.horizontalLayout_11.addWidget(self.style_lbl)
        self.verticalLayout_5 = QVBoxLayout()
        sp2 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_5.addItem(sp2)
        self.line_2 = QFrame(self)
        self.line_2.setSizePolicy(sizePolicy)
        self.line_2.setFrameShape(QFrame.HLine)
        self.line_2.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_5.addWidget(self.line_2)
        self.horizontalLayout_11.addLayout(self.verticalLayout_5)
        self.verticalLayout_2.addLayout(self.horizontalLayout_11)
        self.horizontalLayout_2 = QHBoxLayout()
        sp3 = QSpacerItem(10, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(sp3)
        self.verticalLayout = QVBoxLayout()
        self.gridLayout = QGridLayout()
        align = Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter
        self.label_4 = QLabel(self)
        self.label_4.setAlignment(align)
        self.gridLayout.addWidget(self.label_4, 0, 0, 1, 1)
        self.name_chb = QCheckBox(self)
        self.gridLayout.addWidget(self.name_chb, 0, 1, 1, 1)
        sp4 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.gridLayout.addItem(sp4, 1, 0, 1, 1)
        self.infos_chb = QCheckBox(self)
        self.gridLayout.addWidget(self.infos_chb, 1, 1, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        sp6 = QSpacerItem(20, 18, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout.addItem(sp6)
        self.horizontalLayout_2.addLayout(self.verticalLayout)
        sp7 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(sp7)
        self.verticalLayout_8 = QVBoxLayout()
        self.preview = Viewer(self)
        self.preview.setMaximumSize(QSize(16777215, 280))
        self.verticalLayout_8.addWidget(self.preview)
        sp18 = QSpacerItem(20, 20, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout_8.addItem(sp18)
        self.horizontalLayout_2.addLayout(self.verticalLayout_8)
        sp8 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(sp8)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        sp9 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_2.addItem(sp9)
        self.horizontalLayout_12 = QHBoxLayout()
        self.horizontalLayout_12.setSpacing(10)
        self.size_lbl = QLabel(self)
        self.size_lbl.setFont(font)
        self.size_lbl.setStyleSheet("QLabel {color: #555555;}")
        self.horizontalLayout_12.addWidget(self.size_lbl)
        self.verticalLayout_6 = QVBoxLayout()
        sp10 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_6.addItem(sp10)
        self.line_3 = QFrame(self)
        self.line_3.setSizePolicy(sizePolicy)
        self.line_3.setFrameShape(QFrame.HLine)
        self.line_3.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_6.addWidget(self.line_3)
        self.horizontalLayout_12.addLayout(self.verticalLayout_6)
        self.verticalLayout_2.addLayout(self.horizontalLayout_12)
        self.horizontalLayout = QHBoxLayout()
        sp11 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(sp11)
        self.label = QLabel(self)
        self.label.setAlignment(align)
        self.horizontalLayout.addWidget(self.label)
        self.thumb_height_spb = QSpinBox(self)
        self.thumb_height_spb.setMinimumSize(QSize(80, 20))
        self.thumb_height_spb.setMaximumSize(QSize(16777215, 20))
        self.thumb_height_spb.setSuffix(" px")
        self.thumb_height_spb.setMinimum(60)
        self.thumb_height_spb.setMaximum(250)
        self.thumb_height_spb.setSingleStep(5)
        self.thumb_height_spb.setProperty("value", 100)
        self.horizontalLayout.addWidget(self.thumb_height_spb)
        sp12 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(sp12)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        sp13 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_2.addItem(sp13)
        self.horizontalLayout_13 = QHBoxLayout()
        self.horizontalLayout_13.setSpacing(10)
        self.shortcuts_lbl = QLabel(self)
        self.shortcuts_lbl.setFont(font)
        self.shortcuts_lbl.setStyleSheet("QLabel {color: #555555;}")
        self.horizontalLayout_13.addWidget(self.shortcuts_lbl)
        self.verticalLayout_7 = QVBoxLayout()
        sp14 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_7.addItem(sp14)
        self.line_4 = QFrame(self)
        self.line_4.setSizePolicy(sizePolicy)
        self.line_4.setFrameShape(QFrame.HLine)
        self.line_4.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_7.addWidget(self.line_4)
        self.horizontalLayout_13.addLayout(self.verticalLayout_7)
        self.verticalLayout_2.addLayout(self.horizontalLayout_13)
        self.horizontalLayout_3 = QHBoxLayout()
        sp15 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(sp15)
        self.gridLayout_3 = QGridLayout()
        self.label_16 = QLabel(self)
        self.label_16.setAlignment(align)
        self.gridLayout_3.addWidget(self.label_16, 0, 0, 1, 1)
        self.horizontalLayout_8 = QHBoxLayout()
        self.horizontalLayout_8.setSpacing(0)
        self.show_led = LineEdit(self, None, "show")
        self.horizontalLayout_8.addWidget(self.show_led)
        self.show_change_btn = EditButton(self, None, icon, "show")
        self.horizontalLayout_8.addWidget(self.show_change_btn)
        self.show_reverse_btn = UndoButton(self, None, icon1, "show")
        self.horizontalLayout_8.addWidget(self.show_reverse_btn)
        self.gridLayout_3.addLayout(self.horizontalLayout_8, 0, 1, 1, 1)
        self.label_17 = QLabel(self)
        self.label_17.setAlignment(align)
        self.gridLayout_3.addWidget(self.label_17, 1, 0, 1, 1)
        self.horizontalLayout_7 = QHBoxLayout()
        self.horizontalLayout_7.setSpacing(0)
        self.forward_led = LineEdit(self, None, "forward")
        self.horizontalLayout_7.addWidget(self.forward_led)
        self.forward_change_btn = EditButton(self, None, icon, "forward")
        self.horizontalLayout_7.addWidget(self.forward_change_btn)
        self.forward_reverse_btn = UndoButton(self, None, icon1, "forward")
        self.horizontalLayout_7.addWidget(self.forward_reverse_btn)
        self.gridLayout_3.addLayout(self.horizontalLayout_7, 1, 1, 1, 1)
        self.label_18 = QLabel(self)
        self.label_18.setAlignment(align)
        self.gridLayout_3.addWidget(self.label_18, 2, 0, 1, 1)
        self.horizontalLayout_9 = QHBoxLayout()
        self.horizontalLayout_9.setSpacing(0)
        self.reverse_led = LineEdit(self, None, "reverse")
        self.horizontalLayout_9.addWidget(self.reverse_led)
        self.reverse_change_btn = EditButton(self, None, icon, "reverse")
        self.horizontalLayout_9.addWidget(self.reverse_change_btn)
        self.reverse_reverse_btn = UndoButton(self, None, icon1, "reverse")
        self.horizontalLayout_9.addWidget(self.reverse_reverse_btn)
        self.gridLayout_3.addLayout(self.horizontalLayout_9, 2, 1, 1, 1)
        self.label_22 = QLabel(self)
        self.label_22.setAlignment(align)
        self.gridLayout_3.addWidget(self.label_22, 3, 0, 1, 1)
        self.horizontalLayout_6 = QHBoxLayout()
        self.horizontalLayout_6.setSpacing(0)
        self.fast_forward_led = LineEdit(self, None, "fast_forward")
        self.horizontalLayout_6.addWidget(self.fast_forward_led)
        self.fast_forward_change_btn = EditButton(self, None, icon, "fast_forward")
        self.horizontalLayout_6.addWidget(self.fast_forward_change_btn)
        self.fast_forward_reverse_btn = UndoButton(self, None, icon1, "fast_forward")
        self.horizontalLayout_6.addWidget(self.fast_forward_reverse_btn)
        self.gridLayout_3.addLayout(self.horizontalLayout_6, 3, 1, 1, 1)
        self.label_23 = QLabel(self)
        self.label_23.setAlignment(align)
        self.gridLayout_3.addWidget(self.label_23, 4, 0, 1, 1)
        self.horizontalLayout_10 = QHBoxLayout()
        self.horizontalLayout_10.setSpacing(0)
        self.fast_reverse_led = LineEdit(self, None, "fast_reverse")
        self.horizontalLayout_10.addWidget(self.fast_reverse_led)
        self.fast_reverse_change_btn = EditButton(self, None, icon, "fast_reverse")
        self.horizontalLayout_10.addWidget(self.fast_reverse_change_btn)
        self.fast_reverse_reverse_btn = UndoButton(self, None, icon1, "fast_reverse")
        self.horizontalLayout_10.addWidget(self.fast_reverse_reverse_btn)

        self.gridLayout_3.addLayout(self.horizontalLayout_10, 4, 1, 1, 1)
        self.horizontalLayout_3.addLayout(self.gridLayout_3)
        sp16 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(sp16)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        sp17 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(sp17)
        self.gridLayout_2.addLayout(self.verticalLayout_2, 0, 0, 1, 1)

        self.leds = [self.show_led, self.forward_led, self.reverse_led, 
                    self.fast_forward_led, self.fast_reverse_led]
        self.undos = [self.show_reverse_btn, self.forward_reverse_btn, 
                    self.reverse_reverse_btn, self.fast_forward_reverse_btn,
                    self.fast_reverse_reverse_btn]
        self.senders = ["show", "forward", "reverse", "fast_forward", 
                        "fast_reverse"]

        #self.up_down_rdo.setChecked(True)

        self.retranslateUi()
        self.set_style()

    def set_config_values(self, conf, shortcuts):
        """Set the configuration values in widgets.

        Args:
        conf -- dictionnary of user's prefs
        shortcuts -- dictionnary of shortcuts

        Config keys:
        1022 -- Height of thumbnails
        1027 -- Text position "outline" or "inline"
        1028 -- Style 1 to 6
        2560 -- Show - hide shortcut
        2561 -- Forward shortcut
        2562 -- Reverse shortcut
        2563 -- Fast forward shortcut
        2564 -- Fast reverse shortcut
        "trailer": {"size": 80,
                    "style": 3}

        Shortcuts keys:
        "trailer": {"show": "T",
                    "forward": "J",
                    "reverse": "H",
                    "fast_forward": "G",
                    "fast_reverse": "F"}
        """
        self.conf = conf
        self.shortcuts = shortcuts["trailer"]
        self.thumb_height_spb.setValue(conf["trailer"]["size"])
        if conf["trailer"]["style"] in (1, 3):
            self.name_chb.setChecked(True)

        if conf["trailer"]["style"] in (2, 3):
            self.infos_chb.setChecked(True)

        self.show_led.setText(self.shortcuts["show"])
        self.forward_led.setText(self.shortcuts["forward"])
        self.reverse_led.setText(self.shortcuts["reverse"])
        self.fast_forward_led.setText(self.shortcuts["fast_forward"])
        self.fast_reverse_led.setText(self.shortcuts["fast_reverse"])

        for u in self.undos:
            u.setEnabled(False)

        self.set_datas()
        self.thumb_height_spb.valueChanged.connect(self.on_size_changed)
        self.name_chb.stateChanged.connect(self.on_style_changed)
        self.infos_chb.stateChanged.connect(self.on_style_changed)

    def apply_change(self):
        """Apply settings to config dictionnary.

        """
        self.conf["trailer"]["size"] = self.thumb_height_spb.value()

        style = 0
        if self.name_chb.isChecked():
            style += 1

        if self.infos_chb.isChecked():
            style += 2

        self.conf["trailer"]["style"] = style
        self.shortcuts["show"] = self.show_led.text()
        self.shortcuts["forward"] = self.forward_led.text()
        self.shortcuts["reverse"] = self.reverse_led.text()
        self.shortcuts["fast_forward"] = self.fast_forward_led.text()
        self.shortcuts["fast_reverse"] = self.fast_reverse_led.text()

    def on_size_changed(self, val):
        """Size of thumbnails has changed.

        Args:
        val -- spinBox's value
        """        
        self.set_preview(size=val)

    def on_style_changed(self, b):
        self.set_preview()

    def on_value_changed(self, sender):
        self.undos[self.senders.index(sender)].setEnabled(True)
        self.parent_.value_changed()

    def edit_shortcut(self, sender):
        led = self.leds[self.senders.index(sender)]
        sht = led.text().split("+")
        if len(sht) == 1:
            self.mod = " "
            self.key = sht[0]
        else:
            self.mod = sht[0]
            self.key = sht[1]

        Dial = QDialog()
        sh = ShortcutEditor(Dial, self, self.mod, self.key)
        reply = Dial.exec_()

        if reply:
            sc = ""
            if self.mod != " ":
                sc = sc.join([self.mod, "+"])

            sc = "".join([sc, self.key])
            led.setText(sc)

    def undo_change(self, sender):
        idx = self.senders.index(sender)
        self.leds[idx].setText(self.shortcuts[sender])
        self.undos[idx].setEnabled(False)

    def set_datas(self):
        self.set_preview(self.conf["trailer"]["size"])

    def set_preview(self, size=None):
        if size is None:
            size = self.thumb_height_spb.value()

        style = 0
        if self.name_chb.isChecked():
            style += 1

        if self.infos_chb.isChecked():
            style += 2

        self.thumbs = []
        tb = TrailerItemDrawer(self.temp)
        tb.configure({'size': size, 'style': style})
        tb.itemCreated.connect(self.add_thumbnail)
        th = Path("medias/thumb_O_01.jpg")
        basename = int(str(time.time()).replace('.', '')[4:])
        data = {'exp': Fraction('1/45'), 'fn': Fraction('59/10'), 'iso': 100,
                'orient': 1}
        tb.draw(str(basename), th, data)
        th = Path("medias/thumb_O_02.jpg")
        tb.draw(str(basename+1), th, data)
        self.update_viewer()       

    def add_thumbnail(self, path, data, orig):
        self.thumbs.append(path)

    def update_viewer(self):
        pix = QPixmap(self.thumbs[0])
        pix_1 = QPixmap(self.thumbs[1])
        h = pix.height()
        w = pix.width()
        w_1 = pix_1.width()
        self.preview.set_size(w + w_1 + 10, h + 4)
        item = QGraphicsPixmapItem(None)
        self.preview.scene.addItem(item)
        item.setPixmap(pix)
        item.setPos(2.0, 1.0)
        item_1 = QGraphicsPixmapItem(None)
        self.preview.scene.addItem(item_1)
        item_1.setPixmap(pix_1)
        item_1.setPos(w + 6.0, 1.0)

    def activate_text_pos(self, b):
        self.insert_rdo.setEnabled(b)
        self.up_down_rdo.setEnabled(b)

    def retranslateUi(self):
        self.title_lbl.setText(_("Trailer - Configuration"))
        self.style_lbl.setText(_("Style"))
        self.label_4.setText(_("Text:"))
        self.name_chb.setText(_("File name"))
        self.infos_chb.setText(_("Exposure\'s infos"))
        self.size_lbl.setText(_("Size"))
        self.label.setText(_("Height of the thumbnails:"))
        self.shortcuts_lbl.setText(_("Shortcuts"))
        self.label_16.setText(_("Show - Hide:"))
        self.label_17.setText(_("Forward:"))
        self.label_18.setText(_("Reverse:"))
        self.label_22.setText(_("Fast forward:"))
        self.label_23.setText(_("Fast reverse:"))

    def set_style(self):
        self.title_lbl.setStyleSheet(style.SUBTITLE)
        self.line.setStyleSheet(style.UNDERLINE)
        self.style_lbl.setStyleSheet(style.SUBTITLE)
        self.line_2.setStyleSheet(style.UNDERLINE)
        self.name_chb.setStyleSheet(style.CHECKBOX)
        self.infos_chb.setStyleSheet(style.CHECKBOX)
        self.size_lbl.setStyleSheet(style.SUBTITLE)
        self.line_3.setStyleSheet(style.UNDERLINE)
        self.thumb_height_spb.setStyleSheet(style.SPINBOX)
        self.shortcuts_lbl.setStyleSheet(style.SUBTITLE)
        self.line_4.setStyleSheet(style.UNDERLINE)


class Viewer(QGraphicsView):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setMaximumSize(QSize(16777215, 200))
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setStyleSheet("QWidget { background-color: #242424}")
        self.scene = QGraphicsScene()
        self.setScene(self.scene)

    def set_size(self, w, h):
        self.scene.clear()
        self.setMinimumSize(QSize(w, h))
        self.setMaximumSize(QSize(w, h))
        self.scene.setSceneRect(0.0, 0.0, w, h)
