# -*- coding: utf-8 -*-

# utils.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2011-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Some tools for config pages

from PyQt5 import QtCore, QtGui, QtWidgets


import stylesheets as style

combo_style = """
        QComboBox {border: 2px solid #8A8E96;
            border-radius: 6px;
            color: #FFFFFF;}
        QComboBox::hover {border: 2px solid #3975D7;}
        QComboBox::drop-down {background: qlineargradient(
            x1: 0, y1: 0, x2: 0, y2: 1,
            stop: 0 #FFFFFF, stop: 0.4 #BBBBFF, stop: 1.0 #0000FF);
            border-top: 1px solid #0000FF;
            border-right: 1px solid #0000FF;
            border-top-right-radius: 4px;
            border-bottom-right-radius: 4px;}
        QComboBox::down-arrow {image: url(medias/combo_spin.png);}
        QComboBox QAbstractItemView {background-color: #404040;
            border: 2px solid #000000;
            border-radius: 6px;
            qproperty-iconSize: 50px 24px;
            padding: 2px, 2px, 2px, 2px;}"""

class EditButton(QtWidgets.QToolButton):
    def __init__(self, page, parent=None, icon=None, who=None):
        super().__init__(parent)
        self.setMinimumSize(QtCore.QSize(0, 20))
        self.setMaximumSize(QtCore.QSize(16777215, 20))
        self.setIcon(icon)
        self.setIconSize(QtCore.QSize(20, 20))
        self.setToolTip(_("Edit shortcut"))
        self.page = page
        self.who = who
        self.setStyleSheet(style.TOOLBUTTONDIAL)

    def mousePressEvent(self, event):
        QtWidgets.QToolButton.mousePressEvent(self, event)
        self.page.edit_shortcut(self.who)
        

class UndoButton(QtWidgets.QToolButton):
    def __init__(self, page, parent=None, icon=None, who=None):
        super().__init__(parent)
        self.setMinimumSize(QtCore.QSize(0, 20))
        self.setMaximumSize(QtCore.QSize(16777215, 20))
        self.setIcon(icon)
        self.setIconSize(QtCore.QSize(20, 20))
        self.setToolTip(_("Undo"))
        self.page = page
        self.who = who
        self.setStyleSheet(style.TOOLBUTTONDIAL)

    def mousePressEvent(self, event):
        QtWidgets.QToolButton.mousePressEvent(self, event)
        self.page.undo_change(self.who)


class LineEdit(QtWidgets.QLineEdit):
    def __init__(self, page, parent=None, who=None):
        super().__init__(parent)
        self.setMinimumSize(QtCore.QSize(0, 20))
        self.setMaximumSize(QtCore.QSize(16777215, 20))
        self.setReadOnly(True)
        # FIXME replace this lambda !
        self.textChanged.connect(lambda idx=who: page.on_value_changed(who))
        self.setStyleSheet(style.LINEEDIT)


class ShortcutEditor:
    def __init__(self, Editor, page, mod, key):
        Editor.setObjectName("Editor")
        Editor.resize(403, 168)
        self.page = page
        self.gridLayout_2 = QtWidgets.QGridLayout(Editor)
        self.gridLayout_2.setContentsMargins(5, 5, 5, 5)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.mod_cmb = QtWidgets.QComboBox(Editor)
        self.mod_cmb.setObjectName("mod_cmb")
        self.gridLayout.addWidget(self.mod_cmb, 0, 0, 1, 1)
        self.label = QtWidgets.QLabel(Editor)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setWeight(75)
        font.setBold(True)
        self.label.setFont(font)
        self.label.setText("+")
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 1, 1, 1)
        self.key_cmb = QtWidgets.QComboBox(Editor)
        self.key_cmb.setObjectName("key_cmb")
        self.gridLayout.addWidget(self.key_cmb, 0, 2, 1, 1)
        self.label_2 = QtWidgets.QLabel(Editor)
        self.label_2.setAlignment(QtCore.Qt.AlignCenter)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(15, 20, QtWidgets.QSizePolicy.Fixed, 
                                    QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem1, 1, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(Editor)
        self.label_3.setAlignment(QtCore.Qt.AlignCenter)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 1, 2, 1, 1)
        self.horizontalLayout.addLayout(self.gridLayout)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.label_4 = QtWidgets.QLabel(Editor)
        self.label_4.setObjectName("label_4")
        self.verticalLayout.addWidget(self.label_4)
        self.buttonBox = QtWidgets.QDialogButtonBox(Editor)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.
                                    QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)
        self.gridLayout_2.addLayout(self.verticalLayout, 0, 0, 1, 1)

        self.mods = [" ", "Shift", "Ctrl", "Alt", "Meta"]
        self.mod_cmb.addItems(self.mods)

        self.keys = []
        for i in range(65, 91):
            self.keys.append(chr(i))

        self.keys.extend(["+", "-", "Esc", "Space", "Right", "Left", "Home", 
                          "End", "Page Up", "Page Down", "Delete", "Enter", 
                          "Up", "Down", "F1", "F2", "F3", "F4", "F5", "F6", 
                          "F7", "F8", "F9", "F10", "F11", "F12"])

        self.key_cmb.addItems(self.keys)
        self.mod_cmb.setCurrentIndex(self.mod_cmb.findText(mod))
        self.key_cmb.setCurrentIndex(self.key_cmb.findText(key))
        self.mod_cmb.currentIndexChanged.connect(self.on_modifier_changed)
        self.key_cmb.currentIndexChanged.connect(self.on_key_changed)
        self.key_cmb.activated.connect(self.on_key_changed)
        self.buttonBox.accepted.connect(Editor.accept)
        self.buttonBox.rejected.connect(Editor.reject)
        self.retranslateUi(Editor)

    def on_modifier_changed(self, idx):
        mod = self.mods[idx]
        self.page.mod = self.mods[idx]

    def on_key_changed(self, idx):
        key = self.keys[idx]
        self.page.key = self.keys[idx]

    def retranslateUi(self, Editor):
        Editor.setWindowTitle(_("Shortcut Editor - Oqapy"))
        self.label_2.setText(_("Modifier"))
        self.label_3.setText(_("Key"))
        self.label_4.setWordWrap(True)
        self.label_4.setText("<u>Note: </u>Modifier <b>Meta</b>, also know as key <b>Super</b>, "
                            "corresponds to the <b>Control</b> key on Mac OS X.  "
                            "On Windows keyboard, this corresponds to the <b>Windows</b> key.")

