# -*- coding: utf-8 -*-

# viewingpage.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2011-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Config page for viewing mode

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QIcon, QPixmap, QFont, QColor
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QLabel, QDialog, QSizePolicy, QSpacerItem,
                             QCheckBox, QFrame)

import stylesheets as style

from settings.utils import LineEdit, EditButton, UndoButton, ShortcutEditor
from VWidgets_PyQt5.vcolorcombobox import VColorComboBox

from data import VIEWER_PLT 

class ViewingPage(QWidget):
    def __init__(self, parent):
        super().__init__(parent)
        self.parent_ = parent
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/plus.png"), QIcon.Normal, QIcon.Off)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/gtk-refresh.png"), QIcon.Normal, QIcon.Off)
        self.gridLayout_2 = QGridLayout(self)
        self.gridLayout_2.setContentsMargins(20, -1, -1, -1)
        self.verticalLayout_4 = QVBoxLayout()
        self.verticalLayout_3 = QVBoxLayout()
        self.verticalLayout_3.setSpacing(1)
        self.horizontalLayout_3 = QHBoxLayout()
        self.title_lbl = QLabel(self)
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.title_lbl.setFont(font)
        self.horizontalLayout_3.addWidget(self.title_lbl)
        sp = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(sp)
        self.verticalLayout_3.addLayout(self.horizontalLayout_3)
        self.line = QFrame(self)
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_3.addWidget(self.line)
        self.verticalLayout_4.addLayout(self.verticalLayout_3)
        sp1 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_4.addItem(sp1)
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setSpacing(10)
        self.viewer_lbl = QLabel(self)
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(11)
        font.setBold(True)
        font.setItalic(True)
        self.viewer_lbl.setFont(font)
        self.horizontalLayout.addWidget(self.viewer_lbl)
        self.verticalLayout = QVBoxLayout()
        sp2 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout.addItem(sp2)
        self.line_2 = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line_2.sizePolicy().hasHeightForWidth())
        self.line_2.setSizePolicy(sizePolicy)
        self.line_2.setFrameShape(QFrame.HLine)
        self.line_2.setFrameShadow(QFrame.Sunken)
        self.verticalLayout.addWidget(self.line_2)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.verticalLayout_4.addLayout(self.horizontalLayout)
        self.horizontalLayout_4 = QHBoxLayout()
        sp3 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(sp3)
        self.back_col_lbl = QLabel(self)
        self.back_col_lbl.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.horizontalLayout_4.addWidget(self.back_col_lbl)
        self.background_color_cmb = VColorComboBox(self)
        self.horizontalLayout_4.addWidget(self.background_color_cmb)
        sp4 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(sp4)
        self.verticalLayout_4.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5 = QHBoxLayout()
        sp5 = QSpacerItem(18, 10, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(sp5)
        self.show_scrBar_chb = QCheckBox(self)
        self.horizontalLayout_5.addWidget(self.show_scrBar_chb)
        sp6 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(sp6)
        self.verticalLayout_4.addLayout(self.horizontalLayout_5)
        sp7 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_4.addItem(sp7)
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setSpacing(10)
        self.shortcut_lbl = QLabel(self)
        self.shortcut_lbl.setFont(font)
        self.horizontalLayout_2.addWidget(self.shortcut_lbl)
        self.verticalLayout_2 = QVBoxLayout()
        sp8 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_2.addItem(sp8)
        self.line_3 = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line_3.sizePolicy().hasHeightForWidth())
        self.line_3.setSizePolicy(sizePolicy)
        self.line_3.setFrameShape(QFrame.HLine)
        self.line_3.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_2.addWidget(self.line_3)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)
        self.verticalLayout_4.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_11 = QHBoxLayout()
        sp9 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_11.addItem(sp9)
        self.gridLayout = QGridLayout()
        self.label = QLabel(self)
        self.label.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.horizontalLayout_6 = QHBoxLayout()
        self.horizontalLayout_6.setSpacing(0)
        self.next_image_led = LineEdit(self, None, "next_image")
        self.horizontalLayout_6.addWidget(self.next_image_led)
        self.next_image_change_btn = EditButton(self, None, icon, "next_image")
        self.horizontalLayout_6.addWidget(self.next_image_change_btn)
        self.next_image_reverse_btn = UndoButton(self, None, icon1, "next_image")
        self.horizontalLayout_6.addWidget(self.next_image_reverse_btn)
        self.gridLayout.addLayout(self.horizontalLayout_6, 0, 1, 1, 1)
        self.label_2 = QLabel(self)
        self.label_2.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.horizontalLayout_7 = QHBoxLayout()
        self.horizontalLayout_7.setSpacing(0)
        self.prev_image_led = LineEdit(self, None, "prev_image")
        self.horizontalLayout_7.addWidget(self.prev_image_led)
        self.prev_image_change_btn = EditButton(self, None, icon, "prev_image")
        self.horizontalLayout_7.addWidget(self.prev_image_change_btn)
        self.prev_image_reverse_btn = UndoButton(self, None, icon1, "prev_image")
        self.horizontalLayout_7.addWidget(self.prev_image_reverse_btn)
        self.gridLayout.addLayout(self.horizontalLayout_7, 1, 1, 1, 1)
        self.label_3 = QLabel(self)
        self.label_3.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label_3, 2, 0, 1, 1)
        self.horizontalLayout_8 = QHBoxLayout()
        self.horizontalLayout_8.setSpacing(0)
        self.rotation_r_led = LineEdit(self, None, "rotate_r")
        self.horizontalLayout_8.addWidget(self.rotation_r_led)
        self.rotation_r_change_btn = EditButton(self, None, icon, "rotate_r")
        self.horizontalLayout_8.addWidget(self.rotation_r_change_btn)
        self.rotation_r_reverse_btn = UndoButton(self, None, icon1, "rotate_r")
        self.horizontalLayout_8.addWidget(self.rotation_r_reverse_btn)
        self.gridLayout.addLayout(self.horizontalLayout_8, 2, 1, 1, 1)
        self.label_4 = QLabel(self)
        self.label_4.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label_4, 3, 0, 1, 1)
        self.horizontalLayout_9 = QHBoxLayout()
        self.horizontalLayout_9.setSpacing(0)
        self.rotation_l_led = LineEdit(self, None, "rotate_l")
        self.horizontalLayout_9.addWidget(self.rotation_l_led)
        self.rotation_l_change_btn = EditButton(self, None, icon, "rotate_l")
        self.horizontalLayout_9.addWidget(self.rotation_l_change_btn)
        self.rotation_l_reverse_btn = UndoButton(self, None, icon1, "rotate_l")
        self.horizontalLayout_9.addWidget(self.rotation_l_reverse_btn)
        self.gridLayout.addLayout(self.horizontalLayout_9, 3, 1, 1, 1)
        self.label_5 = QLabel(self)
        self.label_5.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label_5, 4, 0, 1, 1)
        self.horizontalLayout_10 = QHBoxLayout()
        self.horizontalLayout_10.setSpacing(0)
        self.del_img_led = LineEdit(self, None, "del_image")
        self.horizontalLayout_10.addWidget(self.del_img_led)
        self.del_img_change_btn = EditButton(self, None, icon, "del_image")
        self.horizontalLayout_10.addWidget(self.del_img_change_btn)
        self.del_img_reverse_btn = UndoButton(self, None, icon1, "del_image")
        self.horizontalLayout_10.addWidget(self.del_img_reverse_btn)
        self.gridLayout.addLayout(self.horizontalLayout_10, 4, 1, 1, 1)
        self.horizontalLayout_11.addLayout(self.gridLayout)
        sp10 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_11.addItem(sp10)
        self.verticalLayout_4.addLayout(self.horizontalLayout_11)
        sp11 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(sp11)
        self.gridLayout_2.addLayout(self.verticalLayout_4, 0, 0, 1, 1)

        self.leds = [self.next_image_led, self.prev_image_led, 
                     self.rotation_r_led, self.rotation_l_led, 
                     self.del_img_led]

        self.undos = [self.next_image_reverse_btn, self.prev_image_reverse_btn,
                      self.rotation_r_reverse_btn, self.rotation_l_reverse_btn,
                      self.del_img_reverse_btn]

        self.senders = ["next_image", "prev_image", "rotate_r", "rotate_l",
                        "del_image"] 

        self.retranslateUi()
        self.set_style()

    def set_config_values(self, conf, shortcuts):
        """Set the configuration values in widgets.

        Args:
        conf -- Dictionnary of user's prefs
        shortcuts -- Dictionnary of shortcuts

        Config keys:
        1010 -- Background color
        1026 -- Show scrollBar
        2500 -- Next image
        2501 -- previous image
        2502 -- Rotation to right
        2503 -- Rotation to Left
        2508 -- Delete current image
        "style": {"bgrd": "#343434",
                  "scrlbars": True,
                  "stretch": False,
                  "thumb_size": 150,
                  "sel_color": 2,
                  "tab_icon_size": 150}

        Shortcuts keys:
        "viewing": {"next_image": "Space",
                    "prev_image": "Ctrl+Space",
                    "rotate_r": "R",
                    "rotate_l": "L",
                    "del_image": "Del"}
        """
        self.conf = conf
        self.shortcuts = shortcuts["viewing"]
        self.show_scrBar_chb.setChecked(conf["style"]["scrlbars"])
        self.next_image_led.setText(self.shortcuts["next_image"])
        self.prev_image_led.setText(self.shortcuts["prev_image"])
        self.rotation_r_led.setText(self.shortcuts["rotate_r"])
        self.rotation_l_led.setText(self.shortcuts["rotate_l"])
        self.del_img_led.setText(self.shortcuts["del_image"])

        for u in self.undos:
            u.setEnabled(False)

        self.set_colors()

    def apply_change(self):
        """Apply settings to config dictionnary.

        """
        color = self.background_color_cmb.get_current_color()[1].name()
        self.conf["style"]["bgrd"] = color
        self.conf["style"]["scrlbars"] = self.show_scrBar_chb.isChecked()
        self.shortcuts["next_image"] = self.next_image_led.text()
        self.shortcuts["prev_image"] = self.prev_image_led.text()
        self.shortcuts["rotate_r"] = self.rotation_r_led.text()
        self.shortcuts["rotate_l"] = self.rotation_l_led.text()
        self.shortcuts["del_image"] = self.del_img_led.text()

    def on_value_changed(self, sender):
        """Enable undo button when a shortcut has changed.

        Args:
        sender -- shortcut lineEdit
        """
        self.undos[self.senders.index(sender)].setEnabled(True)
        self.parent_.value_changed()

    def edit_shortcut(self, sender):
        """Call shortcut editor dialog box when an edit button is pressed.

        Args:
        sender -- shortcut edit button
        """
        led = self.leds[self.senders.index(sender)]
        sht = led.text().split("+")

        if len(sht) == 1:
            self.mod = " "
            self.key = sht[0]

        else:
            self.mod = sht[0]
            self.key = sht[1]

        Dial = QDialog()
        shed = ShortcutEditor(Dial, self, self.mod, self.key)
        reply = Dial.exec_()

        if reply:
            sc = ""
            if self.mod != " ":
                sc = sc.join([self.mod, "+"])

            sc += self.key
            led.setText(sc)

    def undo_change(self, sender):
        """Undo the last change in shortcut group.

        Args:
        sender -- shortcut undo button
        """
        idx = self.senders.index(sender)
        self.leds[idx].setText(self.shortcuts[sender])
        self.undos[idx].setEnabled(False)

    def set_colors(self):
        self.background_color_cmb.set_palette(VIEWER_PLT)
        self.background_color_cmb.addMoreChoice(False)
        val = self.conf["style"]["bgrd"]
        c = QColor(val if val != -1 else '#080C14')
        self.background_color_cmb.set_current_color(c)

    def retranslateUi(self):
        self.title_lbl.setText("Viewing Mode - Configuration")
        self.viewer_lbl.setText("Viewer")
        self.back_col_lbl.setText("Background color:")
        self.show_scrBar_chb.setText("Show scrollbars")
        self.shortcut_lbl.setText("Shortcuts")
        self.label.setText("Next image:")
        self.label_2.setText(_("Previous image: "))
        self.label_3.setText(_("Rotation to the right:"))
        self.label_4.setText(_("Rotation to the left:"))
        self.label_5.setText(_("Delete the current image:"))

    def set_style(self):
        self.title_lbl.setStyleSheet(style.SUBTITLE)
        self.line.setStyleSheet(style.UNDERLINE)
        self.viewer_lbl.setStyleSheet(style.SUBTITLE)
        self.line_2.setStyleSheet(style.UNDERLINE)
        self.background_color_cmb.setStyleSheet(style.COMBO_DIAL)
        self.show_scrBar_chb.setStyleSheet(style.CHECKBOX)
        self.shortcut_lbl.setStyleSheet(style.SUBTITLE)
        self.line_3.setStyleSheet(style.UNDERLINE)

