# -*- coding: utf-8 -*-

# zoompage.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2011-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Config page for zoom

from PyQt5.QtCore import Qt, QSize
from PyQt5.QtGui import QFont, QIcon, QPixmap
from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QHBoxLayout,
                             QSizePolicy, QDialog, QSpinBox, QSpacerItem,
                             QLabel, QFrame)

import stylesheets as style

from settings.utils import LineEdit, EditButton, UndoButton, ShortcutEditor

class ZoomPage(QWidget):
    def __init__(self, parent):
        super().__init__(parent)
        self.parent_ = parent
        icon = QIcon()
        icon.addPixmap(QPixmap("medias/plus.png"),QIcon.Normal, QIcon.Off)
        icon1 = QIcon()
        icon1.addPixmap(QPixmap("medias/gtk-refresh.png"), QIcon.Normal, QIcon.Off)
        self.gridLayout_2 = QGridLayout(self)
        self.gridLayout_2.setContentsMargins(20, -1, -1, -1)
        self.verticalLayout_4 = QVBoxLayout()
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setSpacing(1)
        self.horizontalLayout_6 = QHBoxLayout()
        self.zoom_lbl = QLabel(self)
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.zoom_lbl.setFont(font)
        self.horizontalLayout_6.addWidget(self.zoom_lbl)
        sp = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(sp)
        self.verticalLayout.addLayout(self.horizontalLayout_6)
        self.line = QFrame(self)
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)
        self.verticalLayout.addWidget(self.line)
        self.verticalLayout_4.addLayout(self.verticalLayout)
        sp1 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_4.addItem(sp1)
        self.horizontalLayout_7 = QHBoxLayout()
        self.horizontalLayout_7.setSpacing(10)
        self.mouse_lbl = QLabel(self)
        font = QFont()
        font.setFamily("Ubuntu")
        font.setPointSize(11)
        font.setBold(True)
        font.setItalic(True)
        self.mouse_lbl.setFont(font)
        self.horizontalLayout_7.addWidget(self.mouse_lbl)
        self.verticalLayout_2 = QVBoxLayout()
        sp2 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_2.addItem(sp2)
        self.line_2 = QFrame(self)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line_2.sizePolicy().hasHeightForWidth())
        self.line_2.setSizePolicy(sizePolicy)
        self.line_2.setFrameShape(QFrame.HLine)
        self.line_2.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_2.addWidget(self.line_2)
        self.horizontalLayout_7.addLayout(self.verticalLayout_2)
        self.verticalLayout_4.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_2 = QHBoxLayout()
        sp3 = QSpacerItem(40, 20, QSizePolicy.Expanding,QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(sp3)
        self.label = QLabel(self)
        self.horizontalLayout_2.addWidget(self.label)
        self.mouse_inc_spb = QSpinBox(self)
        self.mouse_inc_spb.setMinimumSize(QSize(75, 0))
        self.mouse_inc_spb.setSuffix(" %")
        self.mouse_inc_spb.setMinimum(1)
        self.mouse_inc_spb.setProperty("value", 6)
        self.horizontalLayout_2.addWidget(self.mouse_inc_spb)
        sp4 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(sp4)
        self.verticalLayout_4.addLayout(self.horizontalLayout_2)
        sp5 = QSpacerItem(20, 13, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_4.addItem(sp5)
        self.horizontalLayout_8 = QHBoxLayout()
        self.horizontalLayout_8.setSpacing(10)
        self.shortcuts_lbl = QLabel(self)
        self.shortcuts_lbl.setFont(font)
        self.horizontalLayout_8.addWidget(self.shortcuts_lbl)
        self.verticalLayout_3 = QVBoxLayout()
        sp6 = QSpacerItem(20, 10, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_3.addItem(sp6)
        self.line_3 = QFrame(self)
        self.line_3.setSizePolicy(sizePolicy)
        self.line_3.setFrameShape(QFrame.HLine)
        self.line_3.setFrameShadow(QFrame.Sunken)
        self.verticalLayout_3.addWidget(self.line_3)
        self.horizontalLayout_8.addLayout(self.verticalLayout_3)
        self.verticalLayout_4.addLayout(self.horizontalLayout_8)
        self.horizontalLayout_9 = QHBoxLayout()
        sp7 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(sp7)
        self.gridLayout = QGridLayout()
        self.label_2 = QLabel(self)
        self.label_2.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label_2, 0, 0, 1, 1)
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setSpacing(0)
        self.zoom_in_led = LineEdit(self, None, "plus")
        self.horizontalLayout.addWidget(self.zoom_in_led)
        self.zoom_in_change_btn = EditButton(self, None, icon, "plus")
        self.horizontalLayout.addWidget(self.zoom_in_change_btn)
        self.zoom_in_reverse_btn = UndoButton(self, None, icon1, "plus")
        self.horizontalLayout.addWidget(self.zoom_in_reverse_btn)
        self.gridLayout.addLayout(self.horizontalLayout, 0, 1, 1, 1)
        align = Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter
        self.label_3 = QLabel(self)
        self.label_3.setAlignment(align)
        self.gridLayout.addWidget(self.label_3, 1, 0, 1, 1)
        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setSpacing(0)
        self.zoom_out_led = LineEdit(self, None, "minus")
        self.horizontalLayout_3.addWidget(self.zoom_out_led)
        self.zoom_out_change_btn = EditButton(self, None, icon, "minus")
        self.horizontalLayout_3.addWidget(self.zoom_out_change_btn)
        self.zoom_out_reverse_btn = UndoButton(self, None, icon1, "minus")
        self.horizontalLayout_3.addWidget(self.zoom_out_reverse_btn)
        self.gridLayout.addLayout(self.horizontalLayout_3, 1, 1, 1, 1)
        self.label_4 = QLabel(self)
        self.label_4.setAlignment(align)
        self.gridLayout.addWidget(self.label_4, 2, 0, 1, 1)
        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setSpacing(0)
        self.fit_in_led = LineEdit(self, None, "fit_in_view")
        self.horizontalLayout_4.addWidget(self.fit_in_led)
        self.fit_in_change_btn = EditButton(self, None, icon, "fit_in_view")
        self.horizontalLayout_4.addWidget(self.fit_in_change_btn)
        self.fit_in_reverse_btn = UndoButton(self, None, icon1, "fit_in_view")
        self.horizontalLayout_4.addWidget(self.fit_in_reverse_btn)
        self.gridLayout.addLayout(self.horizontalLayout_4, 2, 1, 1, 1)
        self.label_5 = QLabel(self)
        self.label_5.setAlignment(align)
        self.gridLayout.addWidget(self.label_5, 3, 0, 1, 1)
        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setSpacing(0)
        self.full_size_led = LineEdit(self, None, "full_size")
        self.horizontalLayout_5.addWidget(self.full_size_led)
        self.full_size_change_btn = EditButton(self, None, icon, "full_size")
        self.horizontalLayout_5.addWidget(self.full_size_change_btn)
        self.full_size_reverse_btn = UndoButton(self, None, icon1, "full_size")
        self.horizontalLayout_5.addWidget(self.full_size_reverse_btn)
        self.gridLayout.addLayout(self.horizontalLayout_5, 3, 1, 1, 1)
        self.horizontalLayout_9.addLayout(self.gridLayout)
        sp8 = QSpacerItem(20, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(sp8)
        self.verticalLayout_4.addLayout(self.horizontalLayout_9)
        sp9 = QSpacerItem(20, 38, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(sp9)
        self.gridLayout_2.addLayout(self.verticalLayout_4, 0, 0, 1, 1)

        self.leds = [self.zoom_in_led, self.zoom_out_led,
                        self.fit_in_led, self.full_size_led]
        self.undos = [self.zoom_in_reverse_btn, self.zoom_out_reverse_btn,
                        self.fit_in_reverse_btn, self.full_size_reverse_btn]
        self.senders = ["plus", "minus", "fit_in_view", "full_size"]

        self.retranslateUi()
        self.set_style()

    def set_config_values(self, conf, shortcuts):
        """Set the configuration values in widgets.

        Args:
        conf -- dictionnary of user's prefs
        shortcuts -- dictionnary of shortcuts

        Config keys:
        1012 -- zoom increment
        2550 -- zoom in
        2551 -- zoom out
        2552 -- fit in view
        2553 -- full size
        "zoom": {"step": 0.06}

        Shortcuts keys:
        "zoom": {"plus": "+",
                 "minus": "-",
                 "fit_in_view": "Home",
                 "full_size": "End"}
        """
        self.conf = conf
        self.shortcuts = shortcuts["zoom"]
        self.mouse_inc_spb.setValue(conf["zoom"]["step"] * 100)
        self.zoom_in_led.setText(self.shortcuts["plus"])
        self.zoom_out_led.setText(self.shortcuts["minus"])
        self.fit_in_led.setText(self.shortcuts["fit_in_view"])
        self.full_size_led.setText(self.shortcuts["full_size"])

        for u in self.undos:
            u.setEnabled(False)

    def apply_change(self):
        """Apply settings to config dictionnary.

        """
        self.conf["zoom"]["step"] = self.mouse_inc_spb.value() / 100.0
        self.shortcuts["plus"] = self.zoom_in_led.text()
        self.shortcuts["minus"] = self.zoom_out_led.text()
        self.shortcuts["fit_in_view"] = self.fit_in_led.text()
        self.shortcuts["full_size"] = self.full_size_led.text()

    def on_value_changed(self, sender):
        """Enable undo button when a shortcut has changed.

        Args:
        sender -- shortcut lineEdit
        """
        self.undos[self.senders.index(sender)].setEnabled(True)
        self.parent_.value_changed()

    def edit_shortcut(self, sender):
        """Call shortcut editor dialog box when an edit button is pressed.

        Args:
        sender -- shortcut edit button
        """
        led = self.leds[self.senders.index(sender)]
        if led.text() == "+":
            self.mod = " "
            self.key = "+"

        else:
            sht = led.text().split("+")
            if len(sht) == 1:
                self.mod = " "
                self.key = sht[0]

            else:
                self.mod = sht[0]
                self.key = sht[1]

        Dial = QDialog()
        sh = ShortcutEditor(Dial, self, self.mod, self.key)
        reply = Dial.exec_()

        if reply:
            sc = ""
            if self.mod != " ":
                sc = sc.join([self.mod, "+"])

            sc = "".join([sc, self.key])
            led.setText(sc)

    def undo_change(self, sender):
        """Undo the last change in shortcut group.

        Args:
        sender -- shortcut undo button
        """
        idx = self.senders.index(sender)
        self.leds[idx].setText(self.shortcuts[sender])
        self.undos[idx].setEnabled(False)

    def retranslateUi(self):
        self.zoom_lbl.setText(_("Zoom - Configuration"))
        self.mouse_lbl.setText(_("Mouse"))
        self.label.setText(_("Mouse wheel increment:"))
        self.shortcuts_lbl.setText(_("Shortcuts"))
        self.label_2.setText(_("Zoom in:"))
        self.label_3.setText(_("Zoom out:"))
        self.label_4.setText(_("Fit in view:"))
        self.label_5.setText(_("Full size:"))

    def set_style(self):
        self.zoom_lbl.setStyleSheet(style.SUBTITLE)
        self.line.setStyleSheet(style.UNDERLINE)
        self.mouse_lbl.setStyleSheet(style.SUBTITLE)
        self.line_2.setStyleSheet(style.UNDERLINE)
        self.mouse_inc_spb.setStyleSheet(style.SPINBOX)
        self.shortcuts_lbl.setStyleSheet(style.SUBTITLE)
        self.line_3.setStyleSheet(style.UNDERLINE)

