# -*- coding: utf-8 -*-

# ui_imagefinder.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Copyright: 2012-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# Gui for the image finder dialog

import os
import math
import logging
lgg = logging.getLogger(__name__)

from PyQt5 import QtCore, QtGui, QtWidgets


import stylesheets as theme

from VWidgets_PyQt5.vlineedit import VLineEdit

LOC = QtCore.QLocale.system().name()
LANG  = QtCore.QLocale(LOC)


class UiImageFinder(QtWidgets.QDialog):
    def __init__(self, ifinder, parent=None):
        super(UiImageFinder, self).__init__(parent)
        self.ifinder = ifinder
        self.setWindowModality(QtCore.Qt.WindowModal)
        self.resize(550, 500)
        self.setSizeGripEnabled(True)
        self.setModal(True)
        pal = theme.get_palette()
        self.setPalette(pal)
        self.gridLayout_6 = QtWidgets.QGridLayout(self)
        self.gridLayout_6.setContentsMargins(2, 2, 2, 2)
        self.verticalLayout_13 = QtWidgets.QVBoxLayout()
        self.tab_wdg = QtWidgets.QTabWidget(self)
        self.tab_wdg.setPalette(pal)
        self.tab = QtWidgets.QWidget()
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.tab)
        self.verticalLayout_10.setContentsMargins(4, 0, 4, 0)
        self.verticalLayout_9 = QtWidgets.QVBoxLayout()
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setSpacing(0)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.title_1 = TitleWidget(self.tab)
        self.verticalLayout_9.addWidget(self.title_1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 8, QtWidgets.QSizePolicy.Minimum, 
                                        QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout_9.addItem(spacerItem1)
        self.verticalLayout_8 = QtWidgets.QVBoxLayout()
        self.verticalLayout_8.setSpacing(0)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSpacing(0)
        self.folder_led = VLineEdit(self.tab)
        self.folder_led.setMinimumSize(QtCore.QSize(300, 0))
        self.folder_led.setMaximumSize(QtCore.QSize(16777215, 22))
        self.folder_led.setResetButton(True)
        self.folder_led.setHistoricButton(True)
        self.horizontalLayout_2.addWidget(self.folder_led)
        self.browse_btn = QtWidgets.QToolButton(self.tab)
        self.browse_btn.setMaximumSize(QtCore.QSize(22, 22))
        self.horizontalLayout_2.addWidget(self.browse_btn)
        self.verticalLayout_8.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.subfolder_chb = QtWidgets.QCheckBox(self.tab)
        self.horizontalLayout_11.addWidget(self.subfolder_chb)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_11.addItem(spacerItem2)
        self.verticalLayout_8.addLayout(self.horizontalLayout_11)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.ignore_thumbs_chb = QtWidgets.QCheckBox(self.tab)
        self.ignore_thumbs_chb.setChecked(True)
        self.horizontalLayout_12.addWidget(self.ignore_thumbs_chb)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(spacerItem3)
        self.verticalLayout_8.addLayout(self.horizontalLayout_12)
        self.verticalLayout_9.addLayout(self.verticalLayout_8)
        spacerItem4 = QtWidgets.QSpacerItem(20, 16, QtWidgets.QSizePolicy.Minimum, 
                                        QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout_9.addItem(spacerItem4)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setSpacing(0)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.title_2 = TitleWidget(self.tab)
        self.verticalLayout_9.addWidget(self.title_2)
        spacerItem6 = QtWidgets.QSpacerItem(20, 8, QtWidgets.QSizePolicy.Minimum, 
                                        QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout_9.addItem(spacerItem6)
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setVerticalSpacing(0)
        self.label_3 = QtWidgets.QLabel(self.tab)
        self.gridLayout.addWidget(self.label_3, 0, 0, 1, 1)
        self.fname = EditWidget(self, 1, self.tab)
        self.gridLayout.addWidget(self.fname, 0, 1, 1, 1)
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem7, 1, 0, 1, 1)
        self.case_1_chb = QtWidgets.QCheckBox(self.tab)
        self.gridLayout.addWidget(self.case_1_chb, 1, 1, 1, 1)
        self.horizontalLayout_13.addLayout(self.gridLayout)
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_13.addItem(spacerItem8)
        self.verticalLayout_9.addLayout(self.horizontalLayout_13)
        # Dates
        self.horizontalLayout_18 = QtWidgets.QHBoxLayout()
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setSpacing(2)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.label_4 = QtWidgets.QLabel(self.tab)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(11)
        self.label_4.setFont(font)
        self.horizontalLayout_4.addWidget(self.label_4)
        spacerItem16 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem16)
        self.verticalLayout_5.addLayout(self.horizontalLayout_4)
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setVerticalSpacing(2)
        spacerItem17 = QtWidgets.QSpacerItem(30, 20, QtWidgets.QSizePolicy.Fixed, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem17, 0, 0, 1, 1)
        self.create_chb = QtWidgets.QCheckBox(self.tab)
        self.gridLayout_2.addWidget(self.create_chb, 0, 1, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.tab)
        self.label_5.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                        QtCore.Qt.AlignVCenter)
        self.gridLayout_2.addWidget(self.label_5, 0, 2, 1, 1)
        self.date_create_from_ded = QtWidgets.QDateEdit(self.tab)
        self.date_create_from_ded.setDisplayFormat("d/MM/yyyy")
        self.date_create_from_ded.setCalendarPopup(True)
        self.calendar_1 = QtWidgets.QCalendarWidget()
        self.date_create_from_ded.setCalendarWidget(self.calendar_1)
        self.gridLayout_2.addWidget(self.date_create_from_ded, 0, 3, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.tab)
        self.label_7.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                        QtCore.Qt.AlignVCenter)
        self.gridLayout_2.addWidget(self.label_7, 0, 4, 1, 1)
        self.date_create_to_ded = QtWidgets.QDateEdit(self.tab)
        self.date_create_to_ded.setDisplayFormat("d/MM/yyyy")
        self.date_create_to_ded.setCalendarPopup(True)
        self.calendar_2 = QtWidgets.QCalendarWidget()
        self.date_create_to_ded.setCalendarWidget(self.calendar_2)
        self.gridLayout_2.addWidget(self.date_create_to_ded, 0, 5, 1, 1)
        spacerItem18 = QtWidgets.QSpacerItem(30, 20, QtWidgets.QSizePolicy.Fixed, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem18, 1, 0, 1, 1)
        self.modif_chb = QtWidgets.QCheckBox(self.tab)
        self.gridLayout_2.addWidget(self.modif_chb, 1, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.tab)
        self.label_6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                        QtCore.Qt.AlignVCenter)
        self.gridLayout_2.addWidget(self.label_6, 1, 2, 1, 1)
        self.date_modif_from_ded = QtWidgets.QDateEdit(self.tab)
        self.date_modif_from_ded.setDisplayFormat("d/MM/yyyy")
        self.date_modif_from_ded.setCalendarPopup(True)
        self.calendar_3 = QtWidgets.QCalendarWidget()
        self.date_modif_from_ded.setCalendarWidget(self.calendar_3)
        self.gridLayout_2.addWidget(self.date_modif_from_ded, 1, 3, 1, 1)
        self.label_8 = QtWidgets.QLabel(self.tab)
        self.label_8.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                        QtCore.Qt.AlignVCenter)
        self.gridLayout_2.addWidget(self.label_8, 1, 4, 1, 1)
        self.date_modif_to_ded = QtWidgets.QDateEdit(self.tab)
        self.date_modif_to_ded.setDisplayFormat("d/MM/yyyy")
        self.date_modif_to_ded.setCalendarPopup(True)
        self.calendar_4 = QtWidgets.QCalendarWidget()
        self.date_modif_to_ded.setCalendarWidget(self.calendar_4)
        self.gridLayout_2.addWidget(self.date_modif_to_ded, 1, 5, 1, 1)
        spacerItem19 = QtWidgets.QSpacerItem(30, 20, QtWidgets.QSizePolicy.Fixed, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem19, 2, 0, 1, 1)
        spacerItem20 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem20, 2, 1, 1, 1)
        spacerItem21 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem21, 2, 2, 1, 1)
        self.unique_day_chb = QtWidgets.QCheckBox(self.tab)
        self.gridLayout_2.addWidget(self.unique_day_chb, 2, 3, 1, 1)
        spacerItem22 = QtWidgets.QSpacerItem(13, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem22, 2, 4, 1, 1)
        spacerItem23 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem23, 2, 5, 1, 1)
        self.verticalLayout_5.addLayout(self.gridLayout_2)
        self.horizontalLayout_18.addLayout(self.verticalLayout_5)
        spacerItem24 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_18.addItem(spacerItem24)
        self.verticalLayout_9.addLayout(self.horizontalLayout_18)
        # Sizes
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setSpacing(2)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.label_9 = QtWidgets.QLabel(self.tab)
        self.label_9.setFont(font)
        self.horizontalLayout_5.addWidget(self.label_9)
        spacerItem9 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem9)
        self.verticalLayout_4.addLayout(self.horizontalLayout_5)
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setVerticalSpacing(2)
        spacerItem10 = QtWidgets.QSpacerItem(30, 20, QtWidgets.QSizePolicy.Fixed, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem10, 0, 0, 1, 1)
        self.fsize_chb = QtWidgets.QCheckBox(self.tab)
        self.gridLayout_3.addWidget(self.fsize_chb, 0, 1, 1, 1)
        self.label_10 = QtWidgets.QLabel(self.tab)
        self.label_10.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                        QtCore.Qt.AlignVCenter)
        self.gridLayout_3.addWidget(self.label_10, 0, 2, 1, 1)
        self.min_file_spb = QtWidgets.QSpinBox(self.tab)
        self.min_file_spb.setSuffix(" ko")
        self.min_file_spb.setMaximum(999999)
        self.gridLayout_3.addWidget(self.min_file_spb, 0, 3, 1, 1)
        self.label_11 = QtWidgets.QLabel(self.tab)
        self.label_11.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                        QtCore.Qt.AlignVCenter)
        self.gridLayout_3.addWidget(self.label_11, 0, 4, 1, 1)
        self.max_file_spb = QtWidgets.QSpinBox(self.tab)
        self.max_file_spb.setSuffix(" ko")
        self.max_file_spb.setMaximum(999999)
        self.max_file_spb.setProperty("value", 1000)
        self.gridLayout_3.addWidget(self.max_file_spb, 0, 5, 1, 1)
        spacerItem11 = QtWidgets.QSpacerItem(30, 20, QtWidgets.QSizePolicy.Fixed, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem11, 1, 0, 1, 1)
        self.psize_chb = QtWidgets.QCheckBox(self.tab)
        self.gridLayout_3.addWidget(self.psize_chb, 1, 1, 1, 1)
        self.label_13 = QtWidgets.QLabel(self.tab)
        self.label_13.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                        QtCore.Qt.AlignVCenter)
        self.gridLayout_3.addWidget(self.label_13, 1, 2, 1, 1)
        self.min_pict_spb = QtWidgets.QSpinBox(self.tab)
        self.min_pict_spb.setSuffix(" px")
        self.min_pict_spb.setMaximum(999999)
        self.gridLayout_3.addWidget(self.min_pict_spb, 1, 3, 1, 1)
        self.label_12 = QtWidgets.QLabel(self.tab)
        self.label_12.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                        QtCore.Qt.AlignVCenter)
        self.gridLayout_3.addWidget(self.label_12, 1, 4, 1, 1)
        self.max_pict_spb = QtWidgets.QSpinBox(self.tab)
        self.max_pict_spb.setSuffix(" px")
        self.max_pict_spb.setMaximum(99999999)
        self.max_pict_spb.setProperty("value", 1000)
        self.gridLayout_3.addWidget(self.max_pict_spb, 1, 5, 1, 1)
        self.verticalLayout_4.addLayout(self.gridLayout_3)
        self.horizontalLayout_14.addLayout(self.verticalLayout_4)
        spacerItem12 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_14.addItem(spacerItem12)
        self.verticalLayout_9.addLayout(self.horizontalLayout_14)
        spacerItem13 = QtWidgets.QSpacerItem(20, 18, QtWidgets.QSizePolicy.Minimum, 
                                        QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_9.addItem(spacerItem13)
        self.verticalLayout_10.addLayout(self.verticalLayout_9)
        self.tab_wdg.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.verticalLayout_14 = QtWidgets.QVBoxLayout(self.tab_2)
        self.verticalLayout_14.setContentsMargins(4, -1, 4, -1)
        self.verticalLayout_7 = QtWidgets.QVBoxLayout()
        self.horizontalLayout_17 = QtWidgets.QHBoxLayout()
        self.gridLayout_4 = QtWidgets.QGridLayout()
        self.gridLayout_4.setVerticalSpacing(2)
        self.uid_chb = QtWidgets.QCheckBox(self.tab_2)
        self.gridLayout_4.addWidget(self.uid_chb, 0, 0, 1, 1)
        self.identifier_led = QtWidgets.QLineEdit(self.tab_2)
        self.identifier_led.setMinimumSize(QtCore.QSize(280, 0))
        self.gridLayout_4.addWidget(self.identifier_led, 0, 1, 1, 1)

        spacerItem25 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem25, 1, 0, 1, 1)
        self.all_occur = QtWidgets.QCheckBox(self.tab_2)
        self.gridLayout_4.addWidget(self.all_occur, 1, 1, 1, 1)

        spacerItem14 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem14, 2, 0, 1, 1)
        self.warning_lbl = QtWidgets.QLabel(self.tab_2)
        self.warning_lbl.setStyleSheet("QLabel {color: red;}")
        self.warning_lbl.hide()
        self.gridLayout_4.addWidget(self.warning_lbl, 2, 1, 1, 1)
        self.horizontalLayout_17.addLayout(self.gridLayout_4)
        spacerItem15 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_17.addItem(spacerItem15)
        self.verticalLayout_7.addLayout(self.horizontalLayout_17)

        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setSpacing(0)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.title_3 = TitleWidget(self.tab_2)
        self.verticalLayout_7.addWidget(self.title_3)
        spacerItem26 = QtWidgets.QSpacerItem(20, 13, QtWidgets.QSizePolicy.Minimum, 
                                        QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout_7.addItem(spacerItem26)
        self.horizontalLayout_19 = QtWidgets.QHBoxLayout()
        self.gridLayout_5 = QtWidgets.QGridLayout()
        self.label_15 = QtWidgets.QLabel(self.tab_2)
        self.gridLayout_5.addWidget(self.label_15, 0, 0, 1, 1)
        # Data
        self.title = EditWidget(self, 2, self.tab_2)
        self.gridLayout_5.addWidget(self.title, 0, 1, 1, 1)

        self.label_16 = QtWidgets.QLabel(self.tab_2)
        self.gridLayout_5.addWidget(self.label_16, 1, 0, 1, 1)

        self.author = EditWidget(self, 3, self.tab_2)
        self.gridLayout_5.addWidget(self.author, 1, 1, 1, 1)

        self.label_17 = QtWidgets.QLabel(self.tab_2)
        self.gridLayout_5.addWidget(self.label_17, 2, 0, 1, 1)

        self.tag = EditWidget(self, 4, self.tab_2)
        self.gridLayout_5.addWidget(self.tag, 2, 1, 1, 1)

        self.label_20 = QtWidgets.QLabel(self.tab_2)
        self.gridLayout_5.addWidget(self.label_20, 3, 0, 1, 1)

        self.place = EditWidget(self, 5, self.tab_2)
        self.gridLayout_5.addWidget(self.place, 3, 1, 1, 1)

        self.horizontalLayout_19.addLayout(self.gridLayout_5)
        spacerItem27 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Preferred, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_19.addItem(spacerItem27)
        self.verticalLayout_7.addLayout(self.horizontalLayout_19)
        spacerItem32 = QtWidgets.QSpacerItem(20, 13, QtWidgets.QSizePolicy.Minimum, 
                                        QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout_7.addItem(spacerItem32)
        # GPS
        self.title_4 = TitleWidget(self.tab_2)
        self.verticalLayout_7.addWidget(self.title_4)
        spacerItem31 = QtWidgets.QSpacerItem(20, 13, QtWidgets.QSizePolicy.Minimum, 
                                        QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout_7.addItem(spacerItem31)
        self.gridLayout_7 = QtWidgets.QGridLayout()
        self.label_23 = QtWidgets.QLabel(self.tab_2)
        self.gridLayout_7.addWidget(self.label_23, 0, 0, 1, 1)
        self.lo_deg = QtWidgets.QSpinBox(self.tab_2)
        self.lo_deg.setMaximum(179)
        self.lo_deg.setWrapping(True)
        self.gridLayout_7.addWidget(self.lo_deg, 0, 1, 1, 1)
        self.lo_min = QtWidgets.QSpinBox(self.tab_2)
        self.lo_min.setMaximum(59)
        self.lo_min.setWrapping(True)
        self.gridLayout_7.addWidget(self.lo_min, 0, 2, 1, 1)
        self.lo_sec = QtWidgets.QSpinBox(self.tab_2)
        self.lo_sec.setMaximum(60)
        self.lo_sec.setWrapping(True)
        self.gridLayout_7.addWidget(self.lo_sec, 0, 3, 1, 1)
        self.lo_ref = QtWidgets.QComboBox(self.tab_2)
        self.lo_ref.setMaximumSize(QtCore.QSize(90, 16777215))
        self.gridLayout_7.addWidget(self.lo_ref, 0, 4, 1, 1)
        self.lo_dec = QtWidgets.QDoubleSpinBox(self.tab_2)
        self.lo_dec.setDecimals(8)
        self.lo_dec.setMaximum(180.0)
        self.lo_dec.setMinimum(-180.0)
        self.lo_dec.setWrapping(True)
        self.gridLayout_7.addWidget(self.lo_dec, 0, 5, 1, 1)
        self.label_24 = QtWidgets.QLabel(self.tab_2)
        self.gridLayout_7.addWidget(self.label_24, 1, 0, 1, 1)
        self.la_deg = QtWidgets.QSpinBox(self.tab_2)
        self.la_deg.setMaximum(179)
        self.la_deg.setWrapping(True)
        self.gridLayout_7.addWidget(self.la_deg, 1, 1, 1, 1)
        self.la_min = QtWidgets.QSpinBox(self.tab_2)
        self.la_min.setMaximum(59)
        self.la_min.setWrapping(True)
        self.gridLayout_7.addWidget(self.la_min, 1, 2, 1, 1)
        self.la_sec = QtWidgets.QSpinBox(self.tab_2)
        self.la_sec.setMaximum(60)
        self.la_sec.setWrapping(True)
        self.gridLayout_7.addWidget(self.la_sec, 1, 3, 1, 1)
        self.la_ref = QtWidgets.QComboBox(self.tab_2)
        self.la_ref.setMaximumSize(QtCore.QSize(90, 16777215))
        self.gridLayout_7.addWidget(self.la_ref, 1, 4, 1, 1)
        self.la_dec = QtWidgets.QDoubleSpinBox(self.tab_2)
        self.la_dec.setDecimals(8)
        self.la_dec.setMaximum(90.0)
        self.la_dec.setMinimum(-90.0)
        self.la_dec.setWrapping(True)
        self.gridLayout_7.addWidget(self.la_dec, 1, 5, 1, 1)
        self.label_25 = QtWidgets.QLabel(self.tab_2)
        self.gridLayout_7.addWidget(self.label_25, 2, 0, 1, 1)
        self.accurate = QtWidgets.QDoubleSpinBox(self.tab_2)
        self.accurate.setMinimum(0.1)
        self.accurate.setMaximum(300.0)
        self.accurate.setSingleStep(0.1)
        self.gridLayout_7.addWidget(self.accurate, 2, 1, 1, 1)
        self.use_coord_chb = QtWidgets.QCheckBox(self.tab_2)
        self.gridLayout_7.addWidget(self.use_coord_chb, 2, 2, 1, 2)
        self.verticalLayout_7.addLayout(self.gridLayout_7)
        #----------------------------------------------------
        spacerItem28 = QtWidgets.QSpacerItem(20, 17, QtWidgets.QSizePolicy.Minimum, 
                                        QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_7.addItem(spacerItem28)
        self.verticalLayout_14.addLayout(self.verticalLayout_7)
        self.tab_wdg.addTab(self.tab_2, "")
        self.tab_3 = QtWidgets.QWidget()
        self.verticalLayout_12 = QtWidgets.QVBoxLayout(self.tab_3)
        self.verticalLayout_12.setContentsMargins(4, -1, 4, -1)
        self.verticalLayout_11 = QtWidgets.QVBoxLayout()
        self.table = TableWidget(self, self.tab_3)
        self.verticalLayout_11.addWidget(self.table)
        self.prog = ProgressWidget(self.tab_3)
        self.verticalLayout_11.addWidget(self.prog)
        self.horizontalLayout_16 = QtWidgets.QHBoxLayout()
        self.remove_btn = QtWidgets.QPushButton(self.tab_3)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap("medias/clean_24x27.png"), 
                                        QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.remove_btn.setIcon(icon5)
        self.horizontalLayout_16.addWidget(self.remove_btn)
        self.label_19 = QtWidgets.QLabel(self.tab_3)
        self.horizontalLayout_16.addWidget(self.label_19)
        spacerItem29 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_16.addItem(spacerItem29)
        self.found_lbl = QtWidgets.QLabel(self.tab_3)
        self.horizontalLayout_16.addWidget(self.found_lbl)
        self.verticalLayout_11.addLayout(self.horizontalLayout_16)
        self.verticalLayout_12.addLayout(self.verticalLayout_11)
        self.tab_wdg.addTab(self.tab_3, "")
        self.verticalLayout_13.addWidget(self.tab_wdg)
        self.horizontalLayout_15 = QtWidgets.QHBoxLayout()
        self.all_crit_chb = QtWidgets.QCheckBox(self)
        self.horizontalLayout_15.addWidget(self.all_crit_chb)
        spacerItem30 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_15.addItem(spacerItem30)
        self.search_btn = QtWidgets.QPushButton(self)
        self.search_btn.setMaximumSize(QtCore.QSize(16777215, 30))
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap("medias/self-apply.svg"), 
                                        QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.search_btn.setIcon(icon1)
        self.horizontalLayout_15.addWidget(self.search_btn)
        self.reset_btn = QtWidgets.QPushButton(self)
        self.reset_btn.setMaximumSize(QtCore.QSize(16777215, 30))
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap("medias/gtk-refresh.png"), 
                                        QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.reset_btn.setIcon(icon2)
        self.horizontalLayout_15.addWidget(self.reset_btn)
        self.open_btn = QtWidgets.QPushButton(self)
        self.open_btn.setMinimumSize(QtCore.QSize(0, 30))
        self.open_btn.setMaximumSize(QtCore.QSize(16777215, 30))
        self.horizontalLayout_15.addWidget(self.open_btn)
        self.close_btn = QtWidgets.QPushButton(self)
        self.close_btn.setMaximumSize(QtCore.QSize(16777215, 30))
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap("medias/close.png"), 
                                        QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.close_btn.setIcon(icon3)
        self.horizontalLayout_15.addWidget(self.close_btn)
        spacerItem33 = QtWidgets.QSpacerItem(30, 20, QtWidgets.QSizePolicy.Fixed, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_15.addItem(spacerItem33)
        self.verticalLayout_13.addLayout(self.horizontalLayout_15)
        self.gridLayout_6.addLayout(self.verticalLayout_13, 0, 0, 1, 1)

        self.set_style()
        self.retranslateUi()
        self.tab_wdg.setCurrentIndex(0)
        self.identifier_led.setEnabled(False)
        self.all_occur.setEnabled(False)
        self.all_crit_chb.setChecked(True)
        self.remove_btn.setEnabled(False)
        self.open_btn.setEnabled(False)
        self.folder_led.setFocus(True)
        QtCore.QMetaObject.connectSlotsByName(self)
        self.show()
        self.on_subfolder_selected(False)
        self.set_creation_date_edit(False)
        self.set_modif_date_edit(False)
        self.set_file_size_edit(False)
        self.set_picture_size_edit(False)
        self.set_coord_edit(False)

        self.browse_btn.clicked.connect(self.call_file_dialog)
        self.subfolder_chb.stateChanged.connect(self.on_subfolder_selected)
        self.create_chb.stateChanged.connect(self.on_creation_date_selected)
        self.date_create_from_ded.dateChanged.connect(self.on_creation_date_changed)
        self.modif_chb.stateChanged.connect(self.on_modif_date_selected)
        self.date_modif_from_ded.dateChanged.connect(self.on_modif_date_changed)
        self.unique_day_chb.stateChanged.connect(self.on_unique_day_selected)
        self.fsize_chb.stateChanged.connect(self.on_fsize_selected)
        self.min_file_spb.valueChanged.connect(self.on_fsize_changed)
        self.psize_chb.stateChanged.connect(self.on_psize_selected)
        self.min_pict_spb.valueChanged.connect(self.on_psize_changed)
        self.uid_chb.stateChanged.connect(self.on_uid_checked)
        self.identifier_led.textChanged.connect(self.on_identifier_changed)
        self.lo_deg.valueChanged.connect(self.on_long_dms_changed)
        self.lo_min.valueChanged.connect(self.on_long_dms_changed)
        self.lo_sec.valueChanged.connect(self.on_long_dms_changed)
        self.lo_ref.currentIndexChanged.connect(self.on_long_ref_changed)
        self.lo_dec.valueChanged.connect(self.on_long_dec_changed)
        self.la_deg.valueChanged.connect(self.on_lat_dms_changed)
        self.la_min.valueChanged.connect(self.on_lat_dms_changed)
        self.la_sec.valueChanged.connect(self.on_lat_dms_changed)
        self.la_ref.currentIndexChanged.connect(self.on_lat_ref_changed)
        self.la_dec.valueChanged.connect(self.on_lat_dec_changed)
        self.accurate.valueChanged.connect(self.on_accurate_changed)
        self.use_coord_chb.stateChanged.connect(self.on_coord_selected)
        self.prog.btn.clicked.connect(self.cancel_process)
        self.remove_btn.clicked.connect(self.table.remove_files)
        self.search_btn.clicked.connect(self.run_searching)
        self.reset_btn.clicked.connect(self.reset_all)
        self.open_btn.clicked.connect(self.ifinder.send_selected_files)
        self.close_btn.clicked.connect(self.reject)

    def call_file_dialog(self):
        title = _("Get folder name")
        folder = os.path.expanduser("~")
        f_name = QtWidgets.QFileDialog.getExistingDirectory(self, 
                    title, folder,
                    QtWidgets.QFileDialog.DontResolveSymlinks
                    | QtWidgets.QFileDialog.ShowDirsOnly)

        if f_name:
            self.folder_led.setText(f_name)
            self.folder_led._update_historic()

    def _on_info_requested(self, idx):
        """Called when an info button is clicked.

        Args:
        idx -- index of button
        """
        geo = self.geometry()
        self.help = Help(idx, geo)

    def set_creation_date_edit(self, b):
        """Set enabled the creation date widgets enabled.

        Args:
        b -- boolean
        """
        self.date_create_from_ded.setEnabled(b)
        self.date_create_to_ded.setEnabled(b)

    def set_modif_date_edit(self, b):
        """Set enabled the modification date widgets enabled.

        Args:
        b -- boolean
        """
        self.date_modif_from_ded.setEnabled(b)
        self.date_modif_to_ded.setEnabled(b)

    def on_subfolder_selected(self, state):
        """Called when the checkbox 'Include all subfolders' date is (un)checked.

        Args:
        b -- boolean
        """
        self.ignore_thumbs_chb.setEnabled(state)

    def on_creation_date_selected(self, state):
        """Called when the checkbox 'Creation date' is (un)checked.

        Args:
        b -- boolean
        """
        self.set_creation_date_edit(state)
        if state:
            self.modif_chb.setChecked(False)
            self.on_unique_day_selected(self.unique_day_chb.isChecked())

    def on_creation_date_changed(self, date):
        """Called when the first creation dateEdit is edited.

        Set the second dateEdit one day later.

        Args:
        date -- QDate
        """
        self.date_create_to_ded.setMinimumDate(date.addDays(1))

    def on_modif_date_selected(self, state):
        """Called when the checkbox 'Modification date' is (un)checked.

        Args:
        b -- boolean
        """
        self.set_modif_date_edit(state)
        if state:
            self.create_chb.setChecked(False)
            self.on_unique_day_selected(self.unique_day_chb.isChecked())

    def on_modif_date_changed(self, date):
        """Called when the first creation dateEdit is edited.

        Set the second dateEdit one day later.

        Args:
        date -- QDate
        """
        self.date_modif_to_ded.setMinimumDate(date.addDays(1))

    def on_unique_day_selected(self, state):
        if self.create_chb.isChecked():
            self.date_create_to_ded.setEnabled(not state)

        elif self.modif_chb.isChecked():
            self.date_modif_to_ded.setEnabled(not state)

    def set_file_size_edit(self, b):
        self.min_file_spb.setEnabled(b)
        self.max_file_spb.setEnabled(b)

    def set_picture_size_edit(self, b):
        self.min_pict_spb.setEnabled(b)
        self.max_pict_spb.setEnabled(b)

    def on_fsize_selected(self, state):
        self.set_file_size_edit(state)

    def on_fsize_changed(self, size):
        self.max_file_spb.setMinimum(size)

    def on_psize_selected(self, state):
        self.set_picture_size_edit(state)

    def on_psize_changed(self, size):
        self.max_pict_spb.setMinimum(size)

    def on_uid_checked(self, state):
        if state:
            self.create_chb.setChecked(False)
            self.modif_chb.setChecked(False)
            self.fsize_chb.setChecked(False)
            self.psize_chb.setChecked(False)
            self.use_coord_chb.setChecked(False)
            self.all_crit_chb.setChecked(False)

        self.identifier_led.setEnabled(state)
        self.all_occur.setEnabled(state)
        state = not state
        self.create_chb.setEnabled(state)
        self.modif_chb.setEnabled(state)
        self.fsize_chb.setEnabled(state)
        self.psize_chb.setEnabled(state)
        self.use_coord_chb.setEnabled(state)
        self.fname.led.setEnabled(state)
        self.title.led.setEnabled(state)
        self.author.led.setEnabled(state)
        self.tag.led.setEnabled(state)
        self.place.led.setEnabled(state)
        self.all_crit_chb.setEnabled(state)

    def on_identifier_changed(self, txt):
        self.warning_lbl.hide()
        if txt:
            try:
                txt = str(txt)
                i = int("0x" + txt, 16)
            except (UnicodeEncodeError, ValueError):
                self.warning_lbl.show()

    def set_coord_edit(self, b):
        self.lo_deg.setEnabled(b)
        self.lo_min.setEnabled(b)
        self.lo_sec.setEnabled(b)
        self.lo_ref.setEnabled(b)
        self.lo_dec.setEnabled(b)
        self.la_deg.setEnabled(b)
        self.la_min.setEnabled(b)
        self.la_sec.setEnabled(b)
        self.la_ref.setEnabled(b)
        self.la_dec.setEnabled(b)
        self.accurate.setEnabled(b)

    def on_long_dms_changed(self, val):
        vals = (self.lo_deg.value(), self.lo_min.value(), self.lo_sec.value())
        dec = self.dms_to_decimal(vals)

        if self.lo_ref.currentIndex == 1:
            dec *= -1

        self.lo_dec.blockSignals(True)
        self.lo_dec.setValue(dec)
        self.lo_dec.blockSignals(False)

    def on_long_ref_changed(self, idx):
        lon = abs(self.lo_dec.value())

        if idx:
            lon *= -1

        self.lo_dec.blockSignals(True)
        self.lo_dec.setValue(lon)
        self.lo_dec.blockSignals(False)

    def on_long_dec_changed(self, val):
        d, m, s = self.decimal_to_dms(abs(val))
        self.set_long_signals(True)
        self.lo_deg.setValue(d)
        self.lo_min.setValue(m)
        self.lo_sec.setValue(s)
        self.lo_ref.setCurrentIndex(not val > 0)
        self.set_long_signals(False)

    def on_lat_dms_changed(self, val):
        vals = (self.la_deg.value(), self.la_min.value(), self.la_sec.value())
        dec = self.dms_to_decimal(vals)

        if self.la_ref.currentIndex == 1:
            dec *= -1

        self.la_dec.blockSignals(True)
        self.la_dec.setValue(dec)
        self.la_dec.blockSignals(False)

    def on_lat_ref_changed(self, idx):
        lat = abs(self.la_dec.value())

        if idx:
            lat *= -1

        self.la_dec.blockSignals(True)
        self.la_dec.setValue(lat)
        self.la_dec.blockSignals(False)

    def on_lat_dec_changed(self, val):
        d, m, s = self.decimal_to_dms(abs(val))
        self.set_lat_signals(True)
        self.la_deg.setValue(d)
        self.la_min.setValue(m)
        self.la_sec.setValue(s)
        self.la_ref.setCurrentIndex(not val > 0)
        self.set_lat_signals(False)

    def on_accurate_changed(self, val):
        pass

    def on_coord_selected(self, state):
        self.set_coord_edit(state)

    def run_searching(self):
        root = self.folder_led.text()

        if not os.path.isdir(root):
            self.show_dir_warning(root)
            return

        self.ifinder.set_searching(root)        

    def reset_all(self):
        self.fname.led.setText("")
        self.create_chb.setChecked(False)
        self.modif_chb.setChecked(False)
        self.unique_day_chb.setChecked(False)
        self.fsize_chb.setChecked(False)
        self.psize_chb.setChecked(False)
        self.identifier_led.setText("")
        self.title.led.setText("")
        self.author.led.setText("")
        self.tag.led.setText("")
        self.place.led.setText("")
        self.use_coord_chb.setChecked(False)
        self.table.clearContents()
        self.table.setRowCount(0)
        self.open_btn.setEnabled(False)

    def cancel_process(self):
        self.ifinder.cancel = True

    def get_lat_dms(self):
        return (self.la_deg.value(), self.la_min.value(), self.la_sec.value())

    def get_geo_coord(self):
        return (self.lo_dec.value(), self.la_dec.value(), self.accurate.value())

    def set_long_signals(self, b):
        self.lo_deg.blockSignals(b)
        self.lo_min.blockSignals(b)
        self.lo_sec.blockSignals(b)
        self.lo_ref.blockSignals(b)

    def set_lat_signals(self, b):
        self.la_deg.blockSignals(b)
        self.la_min.blockSignals(b)
        self.la_sec.blockSignals(b)
        self.la_ref.blockSignals(b)

    def decimal_to_dms(self, d):
        """Convert decimal degrees into degrees, minutes, seconds.

        Args:
        d -- float(degrees)
        """
        i, d = math.modf(d)
        j, m = math.modf(i*60)
        s = j*60
        return [int(d), int(abs(m)), abs(s)]

    def dms_to_decimal(self, dms):
        """Convert degrees-minutes-seconds value to decimal.

        Args:
        dms -- (degrees, minutes, seconds)
        """
        return (dms[0] + (dms[1] / 60.0) + (dms[2] / 3600.0))

    def show_dir_warning(self, path):
        mssg = QtWidgets.QMessageBox()
        mssg.setIcon(QtWidgets.QMessageBox.Warning)
        mssg.setText(_("Folder < {0} >".format(path)))
        mssg.setInformativeText(_("doesn't exists !"))
        close = QtWidgets.QPushButton(_("Close"))
        mssg.addButton(close, QtWidgets.QMessageBox.ActionRole)
        mssg.setDefaultButton(close)
        reply = mssg.exec_()

    def set_files_number_label(self, numb):
        num = str(LANG.toString(numb))
        self.found_lbl.setText(_("Found %s files" % num))

    def retranslateUi(self):
        self.setWindowTitle(_("Images finder - Oqapy"))
        self.title_1.label.setText(_("Folder"))
        self.folder_led.setProperty("informativeText_", _("Folder"))
        self.browse_btn.setToolTip(_("Browse"))
        self.browse_btn.setText("...")
        self.subfolder_chb.setText(_("Include all subfolders"))
        self.ignore_thumbs_chb.setText(_("Ignore subfolders 'thumbnails'"))
        self.title_2.label.setText(_("Search Criteria"))
        self.label_3.setText(_("File's names:  "))
        self.fname.led.setProperty("informativeText_", _("Enter the text"))
        self.case_1_chb.setText(_("Case sensitive"))
        self.label_4.setText(_("Dates"))
        self.label_9.setText(_("Size"))
        self.fsize_chb.setText(_("Files"))
        self.label_10.setText(_("min.:"))
        self.label_11.setText(_("max.:"))
        self.psize_chb.setText(_("Images"))
        self.label_13.setText(_("min.:"))
        self.label_12.setText(_("max.:"))
        self.tab_wdg.setTabText(0, _("File"))
        self.create_chb.setText(_("Creation"))
        self.label_5.setText(_("from: "))
        self.label_7.setText(_("to:"))
        self.modif_chb.setText(_("Modification"))
        self.label_6.setText(_("from: "))
        self.label_8.setText(_("to:"))
        self.unique_day_chb.setText(_("Unique day"))
        self.title_3.label.setText(_("Data"))
        self.uid_chb.setText(_("Unique identifier: "))
        self.all_occur.setText(_("All occurrences"))
        self.all_occur.setToolTip(_("Check the box to search all "
                                    "copies of the image"))
        self.warning_lbl.setText(_("Hexadecimal not valid: 0~9 & A~F"))
        self.label_15.setText(_("Title: "))
        self.title.led.setProperty("informativeText_", 
                                        _("Image title"))
        self.label_16.setText(_("Author: "))
        self.author.led.setProperty("informativeText_", 
                                        _("Author name"))
        self.label_17.setText(_("Keywords: "))
        self.tag.led.setProperty("informativeText_", 
                                        _("Image comments"))
        self.label_20.setText(_("Scene: "))
        self.place.led.setProperty("informativeText_", 
                                        _("Image localisation"))
        self.title_4.label.setText("Geographical coordinates")
        self.label_23.setText(_("Longitude:"))
        self.lo_deg.setSuffix(_(" deg."))
        self.lo_min.setSuffix(_(" min."))
        self.lo_sec.setSuffix(_(" sec."))
        self.lo_ref.addItem(_("East"))
        self.lo_ref.addItem(_("West"))
        self.label_24.setText(_("Latitude:"))
        self.la_deg.setSuffix(_(" deg."))
        self.la_min.setSuffix(_(" min."))
        self.la_sec.setSuffix(_(" sec."))
        self.la_ref.addItem(_("North"))
        self.la_ref.addItem(_("South"))
        self.label_25.setText(_("Accuracy:"))
        self.accurate.setToolTip(_("Accuracy in kilometers"))
        self.accurate.setSuffix(_(" km."))
        self.use_coord_chb.setText(_("Use coordinates"))
        self.use_coord_chb.setToolTip(_("Check the box to use the geographical criterion"))
        self.tab_wdg.setTabText(1, _("Metadata"))
        self.remove_btn.setToolTip(_("Remove selected files from the list"))
        self.remove_btn.setText(_("Remove"))
        self.label_19.setText(_("0 files selected"))
        self.found_lbl.setText(_("Found 0 files"))
        self.tab_wdg.setTabText(2, _("Result"))
        self.table.horizontalHeaderItem(1).setText(_("Name"))
        self.table.horizontalHeaderItem(2).setText(_("Image size"))
        self.table.horizontalHeaderItem(3).setText(_("Folder"))
        self.all_crit_chb.setText(_("All criteria"))
        self.all_crit_chb.setToolTip(_("Use all criteria for each file."))
        self.search_btn.setToolTip(_("Launch the research"))
        self.search_btn.setText(_("Research"))
        self.reset_btn.setToolTip(_("Reset all criteria"))
        self.reset_btn.setText(_("Reset"))
        self.open_btn.setToolTip(_("Open the file list in Oqapy"))
        self.open_btn.setText(_("Open"))
        self.close_btn.setToolTip(_("Cancel and close"))
        self.close_btn.setText(_("Cancel"))

    def set_style(self):
        self.tab_wdg.setStyleSheet(theme.TABDIAL)
        self.tab.setStyleSheet(theme.TABPAGEDIAL)
        self.tab_2.setStyleSheet(theme.TABPAGEDIAL)
        self.tab_3.setStyleSheet(theme.TABPAGEDIAL)
        self.title_1.label.setStyleSheet(theme.SUBTITLE)
        self.title_1.sep.setStyleSheet(theme.UNDERLINE)
        self.title_2.label.setStyleSheet(theme.SUBTITLE)
        self.title_2.sep.setStyleSheet(theme.UNDERLINE)
        self.title_3.label.setStyleSheet(theme.SUBTITLE)
        self.title_3.sep.setStyleSheet(theme.UNDERLINE)
        self.title_4.label.setStyleSheet(theme.SUBTITLE)
        self.title_4.sep.setStyleSheet(theme.UNDERLINE)
        self.folder_led.setStyleSheet(theme.LINEEDIT)
        self.folder_led.historic_button.setStyleSheet(theme.MENU)
        self.browse_btn.setStyleSheet(theme.TOOLBUTTONDIAL)
        self.subfolder_chb.setStyleSheet(theme.CHECKBOX)
        self.ignore_thumbs_chb.setStyleSheet(theme.CHECKBOX)
        self.label_3.setStyleSheet(theme.LABEL)
        self.fname.led.setStyleSheet(theme.LINEEDIT)
        self.case_1_chb.setStyleSheet(theme.CHECKBOX)
        self.label_4.setStyleSheet(theme.LABEL)
        self.create_chb.setStyleSheet(theme.CHECKBOX)
        self.label_5.setStyleSheet(theme.LABEL)
        self.date_create_from_ded.setStyleSheet(theme.DATEEDIT)
        self.calendar_1.setStyleSheet(theme.CALENDAR)
        self.label_7.setStyleSheet(theme.LABEL)
        self.date_create_to_ded.setStyleSheet(theme.DATEEDIT)
        self.calendar_2.setStyleSheet(theme.CALENDAR)
        self.modif_chb.setStyleSheet(theme.CHECKBOX)
        self.label_6.setStyleSheet(theme.LABEL)
        self.date_modif_from_ded.setStyleSheet(theme.DATEEDIT)
        self.calendar_3.setStyleSheet(theme.CALENDAR)
        self.label_8.setStyleSheet(theme.LABEL)
        self.date_modif_to_ded.setStyleSheet(theme.DATEEDIT)
        self.calendar_4.setStyleSheet(theme.CALENDAR)
        self.unique_day_chb.setStyleSheet(theme.CHECKBOX)
        self.label_9.setStyleSheet(theme.LABEL)
        self.fsize_chb.setStyleSheet(theme.CHECKBOX)
        self.label_10.setStyleSheet(theme.LABEL)
        self.min_file_spb.setStyleSheet(theme.SPINBOX)
        self.label_11.setStyleSheet(theme.LABEL)
        self.max_file_spb.setStyleSheet(theme.SPINBOX)
        self.psize_chb.setStyleSheet(theme.CHECKBOX)
        self.label_12.setStyleSheet(theme.LABEL)
        self.min_pict_spb.setStyleSheet(theme.SPINBOX)
        self.label_13.setStyleSheet(theme.LABEL)
        self.max_pict_spb.setStyleSheet(theme.SPINBOX)
        self.uid_chb.setStyleSheet(theme.CHECKBOX)
        self.identifier_led.setStyleSheet(theme.LINEEDIT)
        self.all_occur.setStyleSheet(theme.CHECKBOX)
        self.title_3.label.setStyleSheet(theme.SUBTITLE)
        self.label_15.setStyleSheet(theme.LABEL)
        self.title.led.setStyleSheet(theme.LINEEDIT)
        self.label_16.setStyleSheet(theme.LABEL)
        self.author.led.setStyleSheet(theme.LINEEDIT)
        self.label_17.setStyleSheet(theme.LABEL)
        self.tag.led.setStyleSheet(theme.LINEEDIT)
        self.label_20.setStyleSheet(theme.LABEL)
        self.place.led.setStyleSheet(theme.LINEEDIT)
        self.title_4.label.setStyleSheet(theme.SUBTITLE)
        self.label_23.setStyleSheet(theme.LABEL)
        self.lo_deg.setStyleSheet(theme.SPINBOX)
        self.lo_min.setStyleSheet(theme.SPINBOX)
        self.lo_sec.setStyleSheet(theme.SPINBOX)
        self.lo_ref.setStyleSheet(theme.COMBO)
        self.lo_dec.setStyleSheet(theme.DSPINBOX)
        self.label_24.setStyleSheet(theme.LABEL)
        self.la_deg.setStyleSheet(theme.SPINBOX)
        self.la_min.setStyleSheet(theme.SPINBOX)
        self.la_sec.setStyleSheet(theme.SPINBOX)
        self.la_ref.setStyleSheet(theme.COMBO)
        self.la_dec.setStyleSheet(theme.DSPINBOX)
        self.label_25.setStyleSheet(theme.LABEL)
        self.accurate.setStyleSheet(theme.DSPINBOX)
        self.use_coord_chb.setStyleSheet(theme.CHECKBOX)
        self.table.hhdr.setStyleSheet(theme.HEADERVIEW)
        self.table.setStyleSheet(theme.TABLEVIEW)
        self.table.verticalScrollBar().setStyleSheet(
                                theme.VSCROLLBARDIAL)
        self.prog.bar.setStyleSheet(theme.PROGRESSBAR)
        self.remove_btn.setStyleSheet(theme.PUSHBUTTONDIAL)
        self.all_crit_chb.setStyleSheet(theme.CHECKBOX)
        self.search_btn.setStyleSheet(theme.PUSHBUTTONDIAL)
        self.reset_btn.setStyleSheet(theme.PUSHBUTTONDIAL)
        self.open_btn.setStyleSheet(theme.PUSHBUTTONDIAL)
        self.close_btn.setStyleSheet(theme.PUSHBUTTONDIAL)


class TitleWidget(QtWidgets.QWidget):
    def __init__(self, parent=None):
        super(TitleWidget, self).__init__(parent)
        self.vl = QtWidgets.QVBoxLayout(self)
        self.vl.setSpacing(0)
        self.hl = QtWidgets.QHBoxLayout()
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(12)
        font.setWeight(75)
        font.setBold(True)
        self.label = QtWidgets.QLabel(self)
        self.label.setFont(font)
        self.hl.addWidget(self.label)
        spacer = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.hl.addItem(spacer)
        self.vl.addLayout(self.hl)
        self.sep = QtWidgets.QFrame(self)
        self.sep.setFrameShadow(QtWidgets.QFrame.Plain)
        self.sep.setMidLineWidth(1)
        self.sep.setFrameShape(QtWidgets.QFrame.HLine)
        self.sep.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.vl.addWidget(self.sep)
 
class EditWidget(QtWidgets.QWidget):
    def __init__(self, ui, idx, parent=None):
        super(EditWidget, self).__init__(parent)
        self.ui = ui
        self.idx = idx
        self.vl = QtWidgets.QVBoxLayout(self)
        self.vl.setSpacing(0)
        self.vl.setContentsMargins(0, 0, 0, 0)
        self.hl = QtWidgets.QHBoxLayout()
        self.hl.setSpacing(0)
        self.led = VLineEdit(self)
        self.led.setMinimumSize(QtCore.QSize(250, 0))
        self.led.setMaximumSize(QtCore.QSize(16777215, 22))
        self.led.setResetButton(True)
        self.hl.addWidget(self.led)
        self.btn = QtWidgets.QToolButton(self)
        self.btn.setMaximumSize(QtCore.QSize(22, 22))
        self.btn.setAutoRaise(True)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("medias/infos.png"), QtGui.QIcon.Normal, 
                                        QtGui.QIcon.Off)
        self.btn.setIcon(icon)
        self.btn.setIconSize(QtCore.QSize(22, 22))
        self.hl.addWidget(self.btn)
        self.vl.addLayout(self.hl)

        self.led.resetTriggered.connect(self.on_reset_clicked)
        self.btn.clicked.connect(self.on_info_clicked)

    def on_reset_clicked(self):
        self.led.setText("")

    def on_info_clicked(self):
        self.ui._on_info_requested(self.idx)

class ProgressWidget(QtWidgets.QWidget):
    def __init__(self, parent=None):
        super(ProgressWidget, self).__init__(parent)
        self.layout = QtWidgets.QHBoxLayout(self)
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.bar = QtWidgets.QProgressBar(self)
        self.bar.setMaximumSize(QtCore.QSize(16777215, 12))
        self.bar.setMinimum(0)
        self.bar.setMaximum(100)
        self.bar.setTextVisible(False)
        self.layout.addWidget(self.bar)
        self.btn = QtWidgets.QToolButton(self)
        self.btn.setMaximumSize(QtCore.QSize(22, 22))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("medias/close.png"), QtGui.QIcon.Normal, 
                                        QtGui.QIcon.Off)
        self.btn.setIcon(icon)
        self.btn.setIconSize(QtCore.QSize(22, 22))
        self.btn.setAutoRaise(True)
        self.layout.addWidget(self.btn)
        self.label = QtWidgets.QLabel(self)
        self.layout.addWidget(self.label)

    def show_progress(self, b):
        lgg.info("Show progress: {0}".format(b))
        if b:
            self.bar.setMaximum(0)

        else:
            self.bar.setMaximum(100)
            self.bar.setValue(100)

        QtCore.QCoreApplication.processEvents()

    def set_text(self, path, numb=None):
        if path == "":
            self.label.setText("")
            return

        name = os.path.split(path)[1]
        num = str(LANG.toString(numb))
        tx = "".join([name, ": ", num, " files"])
        self.label.setText(tx)

class TableWidget(QtWidgets.QTableWidget):
    def __init__(self, ui, parent=None):
        super(TableWidget, self).__init__(parent)
        self.ui = ui
        self.setAlternatingRowColors(True)
        self.setShowGrid(False)
        self.setIconSize(QtCore.QSize(78, 46))
        self.setColumnCount(4)
        self.setRowCount(0)
        self.setColumnWidth(0, 80)
        self.setColumnWidth(1, 150)
        self.setColumnWidth(2, 130)
        self.hhdr = self.horizontalHeader()
        self.hhdr.setStretchLastSection(True)
        self.hhdr.setDefaultAlignment(QtCore.Qt.AlignLeft)
        self.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.cellClicked.connect(self.on_cell_activated)
        self.itemSelectionChanged.connect(self.set_selection)

        item = QtWidgets.QTableWidgetItem()
        item.setIcon(QtGui.QIcon('medias/pola_icon_2.png'))
        self.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.setHorizontalHeaderItem(3, item)

        self.corner_btn = self.findChild(QtWidgets.QAbstractButton)

        self.hhdr.sectionResized.connect(self.column_resized)
        self.painter = QtGui.QPainter()
        self.item_selected = None

    def add_item(self, item):
        r = self.rowCount()
        self.insertRow(r)
        self.setRowHeight(r, 48)  
        pix = self.draw_icon(item.img)
        icon = QtGui.QIcon()
        icon.addPixmap(pix, QtGui.QIcon.Normal, QtGui.QIcon.Off)
        obj = QtWidgets.QTableWidgetItem()
        obj.setBackground(QtGui.QBrush(QtGui.QColor(0, 0, 0, 255)))
        obj.setTextAlignment(QtCore.Qt.AlignHCenter)
        obj.setIcon(icon)
        self.setItem(r, 0, obj)
        self.obj = TableWidgetItem(self)
        self.obj.set_text(item.name)
        self.setItem(r, 1, self.obj)
        obj = QtWidgets.QTableWidgetItem()
        obj.setText(self.get_formatted_size(item.psize))
        self.setItem(r, 2, obj)
        obj = QtWidgets.QTableWidgetItem()
        obj.setText(item.fld)
        self.setItem(r, 3, obj)

    def get_formatted_number(self, numb):
        return LANG.toString(numb)

    def get_formatted_size(self, size):
        width = self.get_formatted_number(size[0])
        height = self.get_formatted_number(size[1])

        return "".join([width, " x ", height, " px"])

    def get_icon_background(self):
        color = QtGui.QColor(0, 0, 0, 0)
        pix = QtGui.QPixmap(QtCore.QSize(80, 48))
        pix.fill(color)

        return pix

    def draw_icon(self, img):
        pict = img.scaledToHeight(48, QtCore.Qt.SmoothTransformation)
        w, h = float(pict.width()), float(pict.height())
        offset = (80 - w) / 2
        ic = self.get_icon_background()
        self.painter.begin(ic)
        self.painter.drawImage(QtCore.QRectF(offset, 0.0, w, 48.0), 
                                pict, QtCore.QRectF(0.0, 0.0, w, h))
        self.painter.end()

        return ic

    def column_resized(self, row, old, new):
        """Called when a column is resized.

        The width of the the column 0 is fixed to keep the alignment of the
        icons.
        """ 
        if not row:
            self.horizontalHeader().blockSignals(True)
            self.resizeColumnToContents(0)
            self.horizontalHeader().blockSignals(False)

    def selectAll(self):
        """By-pass the method selectAll() called with the corner button.

        """
        pass

    def show_properties(self):
        """Call the extended properties box.

        """
        geo = self.ui.geometry()
        item = self.ui.ifinder.items[self.item_selected]
        self.menu = ItemPropertiesBox(item, geo, self)

    def on_cell_activated(self, r, c):
        self.item_selected = r

    def mousePressEvent(self, event):
        QtWidgets.QTableWidget.mousePressEvent(self, event)
        if event.button() == 2:
            timer = QtCore.QTimer()
            timer.singleShot(300, self.show_properties)
            timer.start()

    def set_selection(self):
        self.ui.remove_btn.setEnabled(len(self.selectedItems()))
        self.ui.label_19.setText(_("{0} files selected"
                                .format(str(len(self.selectedItems())/4))))

    def remove_files(self):
        sel = set([])
        for item in self.selectedItems():
            sel.add(item.row())

        sel = list(sel)
        sel.sort()
        sel.reverse()
        for s in sel:
            self.removeRow(s)

        self.ui.ifinder.remove_from_selection(sel)

class TableWidgetItem(QtWidgets.QTableWidgetItem):
    def __init__(self, table, parent=None):
        super(TableWidgetItem, self).__init__()
        self.table = table

    def set_text(self, text):
        self.setText(text)

class ItemPropertiesBox(QtWidgets.QDialog):
    def __init__(self, item, pos, parent=None):
        super(ItemPropertiesBox, self).__init__(parent)
        self.resize(460, 240)
        self.setWindowFlags(QtCore.Qt.Tool | QtCore.Qt.FramelessWindowHint)
        self.setStyleSheet("""QWidget {color: white;
                                        background: qlineargradient(
                                        x1: 0, y1: 0, x2: 1, y2: 1,
                                        stop: 0 #333333, stop: 1 #000000);
                                        border: 2px solid #9C9C9C;
                                        border-radius: 8px;}""")
        LABEL = """QLabel {background: transparent;
                                    border: transparent;
                                    color: #FFFFFF;}"""
        self.gridLayout_2 = QtWidgets.QGridLayout(self)
        self.gridLayout_2.setContentsMargins(4, 4, 4, 4)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.view = QtWidgets.QGraphicsView(self)
        self.view.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.view.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.view.setStyleSheet("""QWidget {background-color: #080C14;
                                        border: 2px solid #F0F0FF;
                                        border-radius: 6px;}""")
        self.horizontalLayout_6.addWidget(self.view)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setSpacing(0)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.name_lbl = QtWidgets.QLabel(self)
        font = QtGui.QFont()
        font.setFamily("Trebuchet MS")
        font.setPointSize(11)
        font.setWeight(75)
        font.setBold(True)
        self.name_lbl.setFont(font)
        self.name_lbl.setStyleSheet(LABEL)
        self.horizontalLayout.addWidget(self.name_lbl)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.gridLayout = QtWidgets.QGridLayout()
        self.label_2 = QtWidgets.QLabel(self)
        self.label_2.setStyleSheet(LABEL)
        self.label_2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                    QtCore.Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label_2, 0, 0, 1, 1)
        font1 = QtGui.QFont()
        font1.setFamily("Trebuchet MS")
        font1.setPointSize(11)
        font1.setItalic(True)
        self.title_lbl = QtWidgets.QLabel(self)
        self.title_lbl.setFont(font1)
        self.title_lbl.setText("  --")
        self.title_lbl.setStyleSheet(LABEL)
        self.gridLayout.addWidget(self.title_lbl, 0, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self)
        self.label_3.setStyleSheet(LABEL)
        self.label_3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                    QtCore.Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label_3, 1, 0, 1, 1)
        self.psize_lbl = QtWidgets.QLabel(self)
        self.psize_lbl.setFont(font1)
        self.psize_lbl.setText("  --")
        self.psize_lbl.setStyleSheet(LABEL)
        self.gridLayout.addWidget(self.psize_lbl, 1, 1, 1, 1)
        self.label_4 = QtWidgets.QLabel(self)
        self.label_4.setStyleSheet(LABEL)
        self.label_4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                    QtCore.Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label_4, 2, 0, 1, 1)
        self.date_lbl = QtWidgets.QLabel(self)
        self.date_lbl.setFont(font1)
        self.date_lbl.setText("  --")
        self.date_lbl.setStyleSheet(LABEL)
        self.gridLayout.addWidget(self.date_lbl, 2, 1, 1, 1)
        self.label_5 = QtWidgets.QLabel(self)
        self.label_5.setStyleSheet(LABEL)
        self.label_5.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                    QtCore.Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label_5, 3, 0, 1, 1)
        self.iso_lbl = QtWidgets.QLabel(self)
        self.iso_lbl.setFont(font1)
        self.iso_lbl.setText("  --")
        self.iso_lbl.setStyleSheet(LABEL)
        self.gridLayout.addWidget(self.iso_lbl, 3, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(self)
        self.label_6.setStyleSheet(LABEL)
        self.label_6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                    QtCore.Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label_6, 4, 0, 1, 1)
        self.speed_lbl = QtWidgets.QLabel(self)
        self.speed_lbl.setFont(font1)
        self.speed_lbl.setText("  --")
        self.speed_lbl.setStyleSheet(LABEL)
        self.gridLayout.addWidget(self.speed_lbl, 4, 1, 1, 1)
        self.label_7 = QtWidgets.QLabel(self)
        self.label_7.setStyleSheet(LABEL)
        self.label_7.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                    QtCore.Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label_7, 5, 0, 1, 1)
        self.fnumb_lbl = QtWidgets.QLabel(self)
        self.fnumb_lbl.setFont(font1)
        self.fnumb_lbl.setText("  --")
        self.fnumb_lbl.setStyleSheet(LABEL)
        self.gridLayout.addWidget(self.fnumb_lbl, 5, 1, 1, 1)
        self.label_8 = QtWidgets.QLabel(self)
        self.label_8.setStyleSheet(LABEL)
        self.label_8.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                    QtCore.Qt.AlignVCenter)
        self.gridLayout.addWidget(self.label_8, 6, 0, 1, 1)
        self.author_lbl = QtWidgets.QLabel(self)
        self.author_lbl.setFont(font1)
        self.author_lbl.setText("  --")
        self.author_lbl.setStyleSheet(LABEL)
        self.gridLayout.addWidget(self.author_lbl, 6, 1, 1, 1)
        self.horizontalLayout_2.addLayout(self.gridLayout)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_6.addLayout(self.verticalLayout)
        self.verticalLayout_2.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.label_9 = QtWidgets.QLabel(self)
        self.label_9.setStyleSheet(LABEL)
        self.label_9.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                    QtCore.Qt.AlignVCenter)
        self.horizontalLayout_3.addWidget(self.label_9)
        self.scene_lbl = QtWidgets.QLabel(self)
        self.scene_lbl.setFont(font1)
        self.scene_lbl.setText("  --")
        self.scene_lbl.setStyleSheet(LABEL)
        self.horizontalLayout_3.addWidget(self.scene_lbl)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem2)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.label_10 = QtWidgets.QLabel(self)
        self.label_10.setStyleSheet(LABEL)
        self.label_10.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|
                                    QtCore.Qt.AlignVCenter)
        self.horizontalLayout_4.addWidget(self.label_10)
        self.tags_lbl = QtWidgets.QLabel(self)
        self.tags_lbl.setFont(font1)
        self.tags_lbl.setText("  --")
        self.tags_lbl.setStyleSheet(LABEL)
        self.horizontalLayout_4.addWidget(self.tags_lbl)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem3)
        self.verticalLayout_2.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                    QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem4)
        self.close_btn = QtWidgets.QPushButton(self)
        self.close_btn.setStyleSheet(theme.PUSHBUTTONDIAL)
        self.close_btn.setMaximumSize(QtCore.QSize(16777215, 28))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("medias/close.png"), QtGui.QIcon.Normal, 
                                    QtGui.QIcon.Off)
        self.close_btn.setIcon(icon)
        self.horizontalLayout_5.addWidget(self.close_btn)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.gridLayout_2.addLayout(self.verticalLayout_2, 0, 0, 1, 1)

        self.retranslateUi()
        self.set_properties(item)
        QtCore.QMetaObject.connectSlotsByName(self)

        self.close_btn.clicked.connect(self.accept)
        h = (pos.width() - 460) / 2 + pos.x()
        v = (pos.height() - 240) / 2 + pos.y()
        self.move(h, v)
        self.show()

    def set_properties(self, item):
        self.display_image(item.img)
        self.name_lbl.setText(item.name)
        self.psize_lbl.setText(self.get_formatted_size(item.psize))
        if item.data:
            self.set_data(item.data)

    def display_image(self, img):
        scene = QtWidgets.QGraphicsScene()
        scene.setSceneRect(0.0, 0.0, float(img.width()), float(img.height()))
        scene.addPixmap(QtGui.QPixmap(img))
        self.view.setMaximumSize(img.size())
        self.view.setScene(scene)

    def set_data(self, data):
        if data[0][0]:
            self.title_lbl.setText(data[0][0])
        if data[0][1]:
            self.date_lbl.setText(data[0][1])
        if data[0][2]:
            self.author_lbl.setText(data[0][2])
        if data[0][4]:
            self.tags_lbl.setText(data[0][4])
        ch = [i for i in data[1] if i is not None]
        if ch:
            ch = " ".join(ch)
            self.scene_lbl.setText(ch)
        if data[3]['iso'] is not None:
            self.iso_lbl.setText(str(data[3]['iso']))
        if data[3]['exp'] is not None:
            s = data[3]['exp']
            self.speed_lbl.setText("/".join(["1", str(int
                                    (s.denominator / s.numerator))]))
        if data[3]['fn'] is not None:
            f = data[3]['fn']
            self.fnumb_lbl.setText(str(round(f.numerator / f.denominator, 2)))

    def get_formatted_number(self, numb):
        return LANG.toString(numb)

    def get_formatted_size(self, size):
        width = self.get_formatted_number(size[0])
        height = self.get_formatted_number(size[1])

        return "".join([width, " x ", height, " px"])

    def retranslateUi(self):
        self.label_2.setText(_("Title:  "))
        self.label_3.setText(_("Image size:  "))
        self.label_4.setText(_("Date:  "))
        self.label_5.setText(_("ISO:  "))
        self.label_6.setText(_("Exposure:  "))
        self.label_7.setText(_("Fnumber:  "))
        self.label_8.setText(_("Author:  "))
        self.label_9.setText(_("Scene:  "))
        self.label_10.setText(_("Keywords:  "))
        self.close_btn.setText(_("Close"))


class Help(QtWidgets.QWidget):
    def __init__(self, idx, pos, parent=None):
        super(Help, self).__init__(parent)
        self.pos = pos
        self.setWindowFlags(QtCore.Qt.Tool | QtCore.Qt.FramelessWindowHint)
        self.gridLayout = QtWidgets.QGridLayout(self)
        self.gridLayout.setContentsMargins(4, 4, 4, 4)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.title_lbl = QtWidgets.QLabel(self)
        font = QtGui.QFont()
        font.setFamily("Trebuchet MS")
        font.setPointSize(11)
        font.setWeight(75)
        font.setBold(True)
        self.title_lbl.setFont(font)
        self.horizontalLayout.addWidget(self.title_lbl)
        self.desc_lbl = QtWidgets.QLabel(self)
        self.horizontalLayout.addWidget(self.desc_lbl)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.editor = QtWidgets.QPlainTextEdit(self)
        self.editor.setTextInteractionFlags(QtCore.Qt.NoTextInteraction)
        self.editor.setStyleSheet(theme.TEXTEDIT)
        self.verticalLayout.addWidget(self.editor)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, 
                                        QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.close_btn = QtWidgets.QPushButton(self)
        self.close_btn.setMaximumSize(QtCore.QSize(150, 28))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("medias/close.png"), QtGui.QIcon.Normal, 
                                        QtGui.QIcon.Off)
        self.close_btn.setIcon(icon)
        self.close_btn.setText(_("Close"))
        self.close_btn.setStyleSheet(theme.PUSHBUTTONDIAL)
        self.horizontalLayout_2.addWidget(self.close_btn)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.gridLayout.addLayout(self.verticalLayout, 0, 0, 1, 1)

        QtCore.QMetaObject.connectSlotsByName(self)
        self.close_btn.clicked.connect(self.close)

        self.text_2 = _("\nThe case and the order of the terms are ignored.")

        self.set_title(idx)


    def move_to(self, w, h):
        x = (self.pos.width() - w) / 2 + self.pos.x()
        y = (self.pos.height() - h) / 2 + self.pos.y()
        self.resize(w, h)
        self.move(x, y)
        self.show()

    def set_title(self, idx):
        if idx == 1:
            self.title_lbl.setText(_("File's names:  "))
            self.desc_lbl.setText(_("Search the text into the name of the files."))
            self.show_name_info()

        elif idx == 2:
            self.title_lbl.setText(_("Title:  "))
            self.desc_lbl.setText(_("Search the text into the the title "\
                                "of the picture."))
            self.show_title_info()

        elif idx == 3:
            self.title_lbl.setText(_("Author:  "))
            self.desc_lbl.setText(_("Search the text into the name of the "\
                                "author of the picture."))
            self.show_author_info()

        elif idx == 4:
            self.title_lbl.setText(_("Keywords:  "))
            self.desc_lbl.setText(_("Search the text into the keywords, also "\
                                "called tags."))
            self.show_tags_info()

        else:
            self.title_lbl.setText(_("Scene:  "))
            self.desc_lbl.setText(_("Search the text into the scene description."))
            self.show_scene_info()

    def show_name_info(self):
        txt = _("""<p>Enter one or more terms wich must be in the file name.</p></p>
<p>Separate the facultative terms by a <i>space</i> and mandatory terms by a <i>plus</i> sign.</p><p> </p>
<p>Separate the exclusive terms by a <i>minus</i> sign.</p>""")
        self.editor.appendHtml(txt)

        txt = _("\nThe order of the terms is ignored.\n\nExamples:")
        self.editor.appendPlainText(txt)

        txt = _("""
<p><i>[ <b>italy 2010</b> ]</i> matches the files <i>"Italy.jpg"</i>, <i>"Italy_2012.jpg</i>" and <i>"London_2010.jpg"</i></p>
<p><i>[ <b>italy + 2008</b> ]</i> matches the files <i>"Italy_2008.jpg"</i>, <i>"July_2008_Italy.jpg"</i> and refuse <i>"Italy.jpg"</i>, <i>"Italy_2010.jpg"</i></p>
<p><i>[ <b>italy ombria + 2008 - roma</b> ]</i> matches <i>"Italy_2008.jpg"</i>, <i>"ombria_july_2008.jpg"</i> and refuse <i>"Italy.jpg"</i>, <i>"2008_Italy_Roma.jpg"</i></p><p></p>""")
        self.editor.appendHtml(txt)

        txt = _("<p>Check the box <b>Case sensitive</b> if necessary.</p>")
        self.editor.appendHtml(txt)
        self.move_to(750, 340)

    def show_title_info(self):
        txt = _("""<p>Enter one or more terms wich must be in the title field.</p></p>
<p>Separate the facultative terms by a <i>space</i> and mandatory terms by a <i>plus</i> sign.</p><p> </p>
<p>Separate the exclusive terms by a <i>minus</i> sign.</p>""")
        self.editor.appendHtml(txt)

        self.editor.appendPlainText(self.text_2)
        self.move_to(620, 180)

    def show_author_info(self):
        txt = _("""<p>Enter one or more terms wich must be in the author field.</p></p>
<p>Separate the facultative terms by a <i>space</i> and mandatory terms by a <i>plus</i> sign.</p><p> </p>
<p>Separate the exclusive terms by a <i>minus</i> sign.</p>""")
        self.editor.appendHtml(txt)

        self.editor.appendPlainText(self.text_2)
        self.move_to(620, 180)

    def show_tags_info(self):
        txt = _("""<p>Enter one or more terms wich must be in the keywords field.</p></p>
<p>Separate the facultative terms by a <i>space</i> and mandatory terms by a <i>plus</i> sign.</p>
<p>Separate the exclusive terms by a <i>minus</i> sign.</p>""")
        self.editor.appendHtml(txt)

        self.editor.appendPlainText(self.text_2)
        self.move_to(620, 180)

    def show_scene_info(self):
        txt = _("""<p>The scene description content all the fields present in the IPTC-Scene category.</p><p> </p>""")
        self.editor.appendHtml(txt)

        txt = _("""<p>Enter one or more terms wich must be in the scene fields.</p></p>
<p>Separate the facultative terms by a <i>space</i> and mandatory terms by a <i>plus</i> sign.</p><p> </p>
<p>Separate the exclusive terms by a <i>minus</i> sign.</p>""")
        self.editor.appendHtml(txt)

        self.editor.appendPlainText(self.text_2)
        self.move_to(620, 220)
