# -*- coding: utf-8 -*-

# splashscreen.py
# This file is part of Oqapy
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2012-2017 Vincent Vande Vyvre
# Licence: GPL3 
#
# Splash screen

from PyQt5.QtCore import Qt, QSize, QRect, QCoreApplication
from PyQt5.QtGui import QFont, QPainter, QPixmap
from PyQt5.QtWidgets import (QWidget, QSizePolicy, QGridLayout, QVBoxLayout,
                             QHBoxLayout, QSpacerItem, QLabel, QApplication)

SSW = """QWidget {
            background-color: transparent;
            border-radius: 5px;}"""
SSL = """QLabel {
            background-color: transparent;
            color: #F4F4F4;}"""

class SplashScreen(QWidget):
    def __init__(self, version, width, height, parent=None):
        super().__init__(parent, Qt.SplashScreen)
        self.setGeometry(width/2-150, height/2-125, 300, 250)
        #self.setStyleSheet(SSW)
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        self.setSizePolicy(sizePolicy)
        self.setWindowTitle("Form")
        self.gridLayout = QGridLayout(self)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setSpacing(0)
        self.verticalLayout_2 = QVBoxLayout()
        spacer = QSpacerItem(20, 150, QSizePolicy.Minimum, QSizePolicy.Fixed)
        self.verticalLayout_2.addItem(spacer)
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setSpacing(0)
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setSpacing(0)
        self.title_lbl = QLabel(self)
        font = QFont()
        font.setFamily("Sawasdee")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.title_lbl.setFont(font)
        self.title_lbl.setText("oqapy %s" % version)
        self.title_lbl.setMaximumSize(QSize(16777215, 36))
        self.title_lbl.setAlignment(Qt.AlignRight|Qt.AlignTrailing)
        self.title_lbl.setStyleSheet(SSL)
        self.verticalLayout.addWidget(self.title_lbl)
        self.licence_lbl = QLabel(self)
        font = QFont()
        font.setFamily("Sawasdee")
        font.setPointSize(10)
        self.licence_lbl.setFont(font)
        self.licence_lbl.setText("Licence GPL3")
        self.licence_lbl.setMaximumSize(QSize(16777215, 18))
        self.licence_lbl.setAlignment(Qt.AlignRight|Qt.AlignTrailing)
        self.licence_lbl.setStyleSheet(SSL)
        self.verticalLayout.addWidget(self.licence_lbl)
        self.infos_lbl = QLabel(self)
        self.infos_lbl.setFont(font)
        self.infos_lbl.setText('Initialisation ...')
        self.infos_lbl.setMaximumSize(QSize(16777215, 18))
        self.infos_lbl.setAlignment(Qt.AlignCenter)
        self.infos_lbl.setStyleSheet(SSL)
        self.verticalLayout.addWidget(self.infos_lbl)
        self.horizontalLayout.addLayout(self.verticalLayout)
        spacer1 = QSpacerItem(15, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacer1)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.gridLayout.addLayout(self.verticalLayout_2, 0, 0, 1, 1)
        self.show()
        QCoreApplication.processEvents()

    def show_message(self, message):
        self.infos_lbl.setText(message)
        QCoreApplication.processEvents()

    def paintEvent(self, event):
        print('Paint event')
        painter = QPainter()
        painter.begin(self)
        pix = QPixmap('medias/splashscreen.png')
        rect = QRect(0, 0, 300, 250)
        painter.drawPixmap(rect, pix, rect)
        painter.end()
        QCoreApplication.processEvents()
        QWidget.paintEvent(self, event)


if __name__ == "__main__":
    import sys
    app = QApplication(sys.argv)
    ui = SplashScreen('3.0.0', 1600, 900)
    ui.show()
    sys.exit(app.exec_())

