# stylesheets.py
# This file is part of Oqapy serie 3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2012-2017 Vincent Vande Vyvre
# Licence: GPL3
#
# style sheets definitions

from PyQt5.QtGui import QPalette, QBrush, QColor
from PyQt5.QtWidgets import  QApplication
from PyQt5.QtCore import Qt

# Common widgets
# Main Window
MAINWINDOW = """QMainWindow {background: #181E26;
                    color: #FFFFFF;}
                QMainWindow::separator {background: #181E26;
                    color: #FFFFFF;}
                QMainWindow::separator:hover {background: #383E46;}"""

# Base style
WIDGET = """QWidget {background: #181E26;
                color: #FFFFFF;}"""

# Tool bar
TOOLS = """QWidget {background: #181E26;
                color: #000000;}"""

# Viewers
VIEWER = """QWidget {background-color: #343434;
                border: 1px solid #8A8E96;
                border-radius: 4px;}"""

# QPlainTextEdit
TEXTEDIT = """QAbstractScrollArea {background-color: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #FFFFFF, stop: 1.0 #EFEFFF);
                border: 1px solid #8A8E96;
                border-radius: 4px;
                color: #000000;}"""

# QScrollArea
SCROLLAREA = """QScrollArea {background-color: #080C14;
                border: 2px solid #8A8E96;
                border-radius: 4px;}"""

# Menu bar
MENUBAR = """QMenuBar {background-color: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #101010, stop: 0.4 #606060,
                stop: 0.5 #050505, stop: 1.0 #000000);
                border: 1px solid #000000;
                border-bottom-color: #000000;
                border-top-left-radius: 4px;
                border-top-right-radius: 4px;
                min-width: 8ex;
                padding: 2px;
                color: #FFFFFF;}
            QMenuBar::item {spacing: 3px;
                padding: 1px 4px;
                background: transparent;
                border-radius: 4px;}
            QMenuBar::item:selected {color: #6E82C8;}
            QMenuBar::item:pressed {background: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #101010, stop: 0.4 #6E82C8,
                stop: 0.5 #050505, stop: 1.0 #1050A4);
                border: 1px solid #000000;
                border-bottom-color: #000000;
                border-top-left-radius: 4px;
                border-top-right-radius: 4px;
                min-width: 8ex;
                padding: 2px;
                color: #FFFFFF;}"""

# Menu, used for menu bar, button popup menu and context menu
MENU = """QMenu {background: transparent;
                border: 2px solid #8A8E96;
                border-radius: 8px;
                color: #181E26;}
            QMenu::item {background: #FFFFFF;
                padding: 2px 25px 2px 20px;
                border: 1px solid transparent;}
            QMenu::item:selected {background-color: #E8E8F8;
                border-color: #0F2D78;
                border-radius: 4px;
                color: #0F2D78;}
            QMenu::item:disabled {color: #A0A0A0;}
            QMenu::icon:checked {background: gray;
                border: 1px inset gray;
                position: absolute;
                top: 1px;
                right: 1px;
                bottom: 1px;
                left: 1px;}
            QMenu::separator {height: 2px;
                background: #BABDD6;}"""

# QToolButton with popup menu, see BUTTONPOPUP for the magnifier button
POPUPMENU = """QToolButton {background-color: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #787C84, stop: 0.4 #8A8E96,
                stop: 1.0 #505460);
                border: 1px solid #8A8E96;
                border-radius: 4px;
                min-width: 4ex;}
            QToolButton:hover {border: 1px #181E26;
                border-radius: 4px;}
            QToolButton:pressed {background-color: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #DADBDE, stop: 1 #F6F7FA);}
            QToolButton[popupMode="1"] {padding-right: 20px;}
            QToolButton::menu-button {background-color: transparent;
                border: 1px solid #8A8E96;
                border-radius: 2px;}
            QToolButton::menu-arrow {image: url(medias/popup_up_10.png);}"""

# Not used
DOCKWIDGET = """QDockWidget {background: #181E26;
                color: #FFFFFF;}"""

#------------------------------------------------------------------------------
# Tool bar
# Button's container
CONTAINER = """QWidget {background: #1A1C20;
                    border-top: 1px solid #000000;
                    border-left: 1px solid #000000;
                    border-right: 1px solid #A0A0A0;
                    border-bottom: 1px solid #A0A0A0;
                    border-radius: 6px;}"""
# QToolButton
TOOLBUTTON = """QToolButton {background: #1A1C20;
                    border: transparent;}
                QToolButton:hover {border: 2px solid #CC1D00;
                    border-radius: 4px;}
                QToolButton:pressed {background: qlineargradient(
                    x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #dadbde, stop: 1 #F6F7Fa);}"""

# QToolButton in statusBar
STATUSBUTTON = """QToolButton {background: #1A1C20;
                    border: 1px solid #8A8E96;
                    border-radius: 4px;
                    min-width: 6ex;}
                QToolButton:hover {border: 2px solid #CC1D00;
                    border-radius: 4px;}
                QToolButton:pressed {background: qlineargradient(
                    x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #dadbde, stop: 1 #F6F7Fa);}"""

# QToolButton with popupmenu
BUTTONPOPUP = """QToolButton {background: #1A1C20;
                    border: transparent;
                    padding-right: 10px;
                    min-width: 4ex;}
                QToolButton:hover {border: 2px solid #CC1D00;
                    border-radius: 4px;}
                QToolButton:pressed {background: qlineargradient(
                    x1: 0, y1: 0, x2: 0, y2: 1,
                    stop: 0 #dadbde, stop: 1 #F6F7Fa);}
                QToolButton[popupMode="1"] {padding-right: 20px;}
                QToolButton::menu-button {background-color: transparent;
                    border: 1px solid #181E26;
                    border-radius: 2px;}"""
#------------------------------------------------------------------------------

# QToolButton for the viewer header
HEADERBUTTON = """QToolButton {background-color: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #101010, stop: 0.4 #606060,
                stop: 0.5 #050505, stop: 1.0 #000000);
                border: 1px solid #505050;
                border-radius: 4px;
                padding: 2px;
                color: #FFFFFF;}
            QToolButton:hover {border: 2px solid #CC1D00;}
            QToolButton:pressed {background-color: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #000000, stop: 0.4 #050505,
                stop: 0.5 #606060, stop: 1.0 #101010);}"""

TOOLBUTTON2 = """QToolButton {background-color: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #787C84, stop: 0.4 #8A8E96,
                stop: 1.0 #505460);
                border: 1px solid #8A8E96;
                border-radius: 4px;
                min-width: 6ex;
                padding: 2px;
                color: #FFFFFF;}
            QToolButton:hover {border: 1px #181E26;
                border-radius: 4px;}
            QToolButton:pressed {background-color: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #dadbde, stop: 1 #F6F7Fa);}"""

# QToolButton for the properties panel
TBUTTON = """QToolButton {background-color: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #101010, stop: 0.4 #606060,
                stop: 0.5 #050505, stop: 1.0 #000000);
                border-radius: 4px;
                padding: 2px;
                color: #FFFFFF;}
            QToolButton:hover {border: 2px solid #CC1D00;}
            QToolButton:pressed {background-color: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #000000, stop: 0.4 #050505,
                stop: 0.5 #606060, stop: 1.0 #101010);}"""

# QPushButton
PUSHBUTTON = """QPushButton {background-color: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #101010, stop: 0.4 #606060,
                stop: 0.5 #050505, stop: 1.0 #000000);
                border-radius: 4px;
                padding: 2px;
                color: #FFFFFF;}
            QPushButton:hover {border: 2px solid #CC1D00;}
            QPushButton:pressed {background-color: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #000000, stop: 0.4 #050505,
                stop: 0.5 #606060, stop: 1.0 #101010);}"""

# QPushButton for dialog boxes
PUSHBUTTONDIAL = """QPushButton {background-color: qlineargradient(
                        x1: 0, y1: 0, x2: 0, y2: 1,
                        stop: 0 #F4F4F4, stop: 0.4 #E8E8E8,
                        stop: 0.5 #E8E8E8, stop: 1.0 #F4F4F4);
                        border: 2px solid #8A8E96;
                        border-radius: 4px;
                        padding: 2px;
                        color: #000000;}
                    QPushButton:hover {border: 2px solid #CC1D00;}
                    QPushButton:pressed {background-color: qlineargradient(
                        x1: 0, y1: 0, x2: 0, y2: 1,
                        stop: 0 #000000, stop: 0.4 #050505,
                        stop: 0.5 #606060, stop: 1.0 #101010);}"""

# QToolButton for dialog boxes
TOOLBUTTONDIAL = """QToolButton {background-color: qlineargradient(
                        x1: 0, y1: 0, x2: 0, y2: 1,
                        stop: 0 #F4F4F4, stop: 0.4 #E8E8E8,
                        stop: 0.5 #E8E8E8, stop: 1.0 #F4F4F4);
                        border: 2px solid #8A8E96;
                        border-radius: 4px;
                        padding: 2px;
                        color: #000000;}
                    QToolButton:hover {border: 2px solid #CC1D00;}
                    QToolButton:pressed {background-color: qlineargradient(
                        x1: 0, y1: 0, x2: 0, y2: 1,
                        stop: 0 #000000, stop: 0.4 #050505,
                        stop: 0.5 #606060, stop: 1.0 #101010);}"""

# QTabWidget, only tabs, use tabPage_style for pages
TAB = """QTabWidget::pane {border-top: 2px solid #000000;}
            QTabWidget::tab-bar {left: 5px;}
            QTabBar::tab:!selected {margin-top: 2px;}
            QTabBar::tab:selected {
                border-color: #000000;
                border-bottom-color: #0000FF;}
            QTabBar::tab {background: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #101010, stop: 0.4 #606060,
                stop: 0.5 #050505, stop: 1.0 #000000);
                border: 2px solid #000000;
                border-bottom-color: #000000;
                border-top-left-radius: 4px;
                border-top-right-radius: 4px;
                min-width: 8ex;
                padding: 2px;
                color: #FFFFFF;}"""

# Pages in QTabWidget
TABPAGE = """QWidget {background: #181E26;
                color: #FFFFFF;}"""

# QTabWidget for dialog boxes, only tabs, use TABPAGEDIAL for pages
TABDIAL = """QTabWidget::pane {border-top: 2px solid #9A9A9A;
                    border-bottom: 2px solid #9A9A9A;}
                QTabWidget::tab-bar {left: 5px;}
                QTabBar::tab:!selected {margin-top: 2px;}
                QTabBar::tab:selected {
                    border-color: #3975D7;
                    border-bottom-color: #AAC5E0;}
                QTabBar::tab {background: qlineargradient(
                    x1: 0, y1: 0, x2: 0, y2: 1,
                    stop: 0 #F4F4F4, stop: 0.4 #E8E8E8,
                    stop: 0.5 #E0E0E0, stop: 1.0 #9A9A9A);
                    border: 2px solid #9A9A9A;
                    border-top-left-radius: 4px;
                    border-top-right-radius: 4px;
                    min-width: 8ex;
                    padding: 2px;
                    color: #000000;}"""

# Pages in QTabWidget for dialog boxes
TABPAGEDIAL = """QWidget {background: transparent;}"""

# QLabel  for dialog boxes, background: same color as QPalette.Active for  
# window, border is needed for overwrite the border parent.
LABEL = """QFrame {background: #F4F4F4;
                    color: #000000;
                    border: 1px solid #F4F4F4;}"""

# QLineEdit, VLineEdit
LINEEDIT = """QLineEdit {border: 2px solid #A0A5B2;
                border-radius: 6px;
                background: white;
                color: black;
                selection-background-color: darkgray;}"""

# reset, historic and command buttons for VLineEdit
LINEEDITBUTTON = """QToolButton {border-color: transparent;
                    background: transparent;
                    color: black;}"""

# Informative text in VLineEdit
LINEEDITPLACEHOLDER = """QLabel {color: #A0A0A0;
                background-color: transparent;
                border-color: transparent;}"""

# Pixmap needed: combo_spin.png
# TODO add a inactive state
COMBO = """QComboBox {border: 2px solid #A0A5B2;
                border-radius: 6px;
                background: #FFFFFF;
                color: #000508;
                selection-color: #0000FF;
                selection-background-color: #E8E8F8;}
            QComboBox::drop-down {background: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #FFFFFF, stop: 0.4 #BBBBFF, stop: 1.0 #0000FF);
                border-top: 1px solid #0000FF;
                border-right: 1px solid #0000FF;
                border-top-right-radius: 4px;
                border-bottom-right-radius: 4px;}
            QComboBox::down-arrow {image: url(medias/combo_spin.png);}
            QComboBox QAbstractItemView {background-color: #F4F4F4;
                color: #000000;
                border: 2px solid #000000;
                border-radius: 6px;
                outline: none;}"""

# QCombobox for dialog boxes
# QComboBox background is not used, icon's items are not resized
# Pixmap needed: combo_spin.png
# TODO add a inactive state
COMBO_DIAL = """QComboBox {border: 2px solid #8A8E96;
                    border-radius: 6px;
                    selection-color: #0000FF;
                    selection-background-color: #E8E8F8;}
                QComboBox::hover {border: 2px solid #3975D7;}
                QComboBox::drop-down {background: qlineargradient(
                    x1: 0, y1: 0, x2: 0, y2: 1,
                    stop: 0 #FFFFFF, stop: 0.4 #BBBBFF, stop: 1.0 #0000FF);
                    border-top: 1px solid #0000FF;
                    border-right: 1px solid #0000FF;
                    border-top-right-radius: 4px;
                    border-bottom-right-radius: 4px;}
                QComboBox::down-arrow {image: url(medias/combo_spin.png);}
                QComboBox QAbstractItemView {background-color: #F4F4F4;
                    color: #000000;
                    border: 2px solid #000000;
                    border-radius: 6px;
                    min-width: 100px;}"""

# QComboBox with dark QAbstractItemView used i.e. for VColorComboBox
# Pixmap needed: combo_spin.png
DARKCOMBO = """QComboBox {border: 2px solid #8A8E96;
                    border-radius: 6px;}
                QComboBox::hover {border: 2px solid #3975D7;}
                QComboBox::drop-down {background: qlineargradient(
                    x1: 0, y1: 0, x2: 0, y2: 1,
                    stop: 0 #FFFFFF, stop: 0.4 #BBBBFF, stop: 1.0 #0000FF);
                    border-top: 1px solid #0000FF;
                    border-right: 1px solid #0000FF;
                    border-top-right-radius: 4px;
                    border-bottom-right-radius: 4px;}
                QComboBox::down-arrow {image: url(medias/combo_spin.png);}
                QComboBox QAbstractItemView {background-color: #404040;
                    border: 2px solid #000000;
                    border-radius: 6px;
                    qproperty-iconSize: 50px 24px;
                    padding: 2px, 2px, 2px, 2px;
                    color: #FFFFFF;}"""

# QDateEdit
# Pixmaps needed: spinup.png, spindown.png
# TODO add a inactive state
DATEEDIT = """QDateEdit {
                border: 2px solid #8A8E96;
                border-radius: 6px;
                color: #000508;}
            QDateEdit::hover {border: 2px solid #3975D7;} 
            QDateEdit::drop-down {background: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #FFFFFF, stop: 0.4 #BBBBFF, stop: 1.0 #0000FF);
                border-top: 1px solid #0000FF;
                border-right: 1px solid #0000FF;
                border-top-right-radius: 4px;
                border-bottom-right-radius: 4px;}
            QDateEdit::down-arrow {image: url(medias/combo_spin.png);}
            QDateEdit::disabled {color: #787C84;}"""

#QCalendar
# Pixmaps needed: spinup.png, spindown.png
CALENDAR = """QMenu {background-color:qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #cccccc, 
                stop: 1 #333333);}
            QToolButton {background-color: qlineargradient(x1: 0, y1: 0,
                x2: 0, y2: 1, stop: 0 #cccccc, stop: 1 #333333);
                height: 20px; width: 50px}
            QAbstractItemView {selection-background-color: 
                rgb(255, 174, 0);}
            QToolButton::menu-arrow {}
            QToolButton::menu-button {}
            QToolButton::menu-indicator{width: 10px;}
            QToolButton::menu-indicator:open{top: 10px; left: 10px;}
            QListView {background-color: white;}
            QSpinBox {background-color: white;
                border: 2px solid #8A8E96;
                border-radius: 6px;
                color: #000508;}
            QSpinBox::up-button {subcontrol-origin: border;
                subcontrol-position: top right;
                width: 16px;
                image: url(medias/spinup.png) 1;
                border-top-right-radius: 5px;
                border-top: 1px solid #0000FF;
                border-right: 1px solid #0000FF;
                border-bottom: none;}
            QSpinBox::down-button {subcontrol-origin: border;
                subcontrol-position: bottom right;
                width: 16px;
                image: url(medias/spindown.png) 1;
                border-bottom-right-radius: 5px;
                border-top: none;
                border-bottom: 1px solid #0000FF;
                border-right: 1px solid #0000FF;}"""

# QSpinBox
# Pixmaps needed: spinup.png, spindown.png
SPINBOX = """QSpinBox {background: #FFFFFF;
                    border: 2px solid #8A8E96;
                    border-radius: 6px;
                    color: #000508;}
            QSpinBox::hover {border: 2px solid #3975D7;} 
            QSpinBox::up-button {subcontrol-origin: border;
                subcontrol-position: top right;
                width: 16px;
                image: url(medias/spinup.png) 1;
                border-top-right-radius: 5px;
                border-top: 1px solid #0000FF;
                border-right: 1px solid #0000FF;
                border-bottom: none;}
            QSpinBox::down-button {subcontrol-origin: border;
                subcontrol-position: bottom right;
                width: 16px;
                image: url(medias/spindown.png) 1;
                border-bottom-right-radius: 5px;
                border-top: none;
                border-bottom: 1px solid #0000FF;
                border-right: 1px solid #0000FF;}
            QSpinBox::disabled {color: #787C84;}"""

# QDoubleSpinBox
# Pixmaps needed: spinup.png, spindown.png
DSPINBOX = """QDoubleSpinBox {background: #FFFFFF;
                    border: 2px solid #8A8E96;
                    border-radius: 6px;
                    color: #000508;}
                QDoubleSpinBox::hover {border: 2px solid #3975D7;} 
                QDoubleSpinBox::up-button {subcontrol-origin: border;
                    subcontrol-position: top right;
                    width: 16px;
                    image: url(medias/spinup.png) 1;
                    border-top-right-radius: 5px;
                    border-top: 1px solid #0000FF;
                    border-right: 1px solid #0000FF;
                    border-bottom: none;}
                QDoubleSpinBox::down-button {subcontrol-origin: border;
                    subcontrol-position: bottom right;
                    width: 16px;
                    image: url(medias/spindown.png) 1;
                    border-bottom-right-radius: 5px;
                    border-top: none;
                    border-bottom: 1px solid #0000FF;
                    border-right: 1px solid #0000FF;}
                QDoubleSpinBox::disabled {color: #787C84;}"""

# QStatusBar
STATUSBAR = """QStatusBar {background-color: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #101010, stop: 0.4 #606060,
                stop: 0.5 #050505, stop: 1.0 #000000);
                border-radius: 4px;
                min-width: 8ex;
                min-height: 3ex;
                padding: 2px;
                color: #FFFFFF;}
               QStatusBar::item {border: 1px solid #000000;
                border-radius: 4px;}"""

# QToolButton for mode buttons in statusBar
STATEBUTTON = """QToolButton {background-color: #000000;
                border-top: 1px solid #FFFFFF;
                border-left: 1px solid #EFEFEF;
                border-right: 1px solid #EFEFEF;
                border-bottom: 1px solid #A0A0A0;
                border-radius: 4px;
                color: #FFFFFF;}"""

# QCheckBox
# Pixmaps needed: box_unchecked.png, box_unchecked_hover.png,
#                 box_checked.png, box_checked_hover.png
CHECKBOX = """QCheckBox::indicator {
                width: 20px;
                height: 20px;}
            QCheckBox::indicator:unchecked {
                image: url(medias/box_unchecked.png);}
            QCheckBox::indicator:unchecked:hover {
                image: url(medias/box_unchecked_hover.png);}
            QCheckBox::indicator:checked {
                image: url(medias/box_checked.png);}
            QCheckBox::indicator:checked:hover {
                image: url(medias/box_checked_hover.png);}"""

# QRadioButton
# Pixmaps needed: box_unchecked.png, box_unchecked_hover.png,
#                 radioButton_checked.png, radioButton_checked_hover.png
RADIO = """QRadioButton::indicator {
                width: 20px;
                height: 20px;}
            QRadioButton::indicator:unchecked {
                image: url(medias/box_unchecked.png);}
            QRadioButton::indicator:unchecked:hover {
                image: url(medias/box_unchecked_hover.png);}
            QRadioButton::indicator:checked {
                image: url(medias/radioButton_checked.png);}
            QRadioButton::indicator:checked:hover {
                image: url(medias/radioButton_checked_hover.png);}"""

# Tabs of the image toolBox Metadata, Color and Geolocation
#TOOLBOX = """QToolBox::tab {background-color: qlineargradient(
                #x1: 0, y1: 0, x2: 0, y2: 1,
                #stop: 0 #101010, stop: 0.4 #606060,
                #stop: 0.5 #050505, stop: 1.0 #000000);
                #border-radius: 2px;
                #padding: 2px;
                #color: #FFFFFF;}"""

# Horizontal scrollBar
HSCROLLBAR ="""QScrollBar:horizontal {border: 2px solid #888A85;
                border-radius: 6px;
                background: solid #505460;
                height: 16px;
                margin: 0px 40px 0 0px;}
            QScrollBar::handle:horizontal {
                background: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #FFFFFF, stop: 0.7 #3975D7,
                stop: 1.0 #C0C0FF);
                border: 1px solid #0000FF;
                border-radius: 5px;
                min-width: 20px;}
            QScrollBar::add-line:horizontal {
                background: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #FFFFFF, stop: 0.7 #3975D7,
                stop: 1.0 #C0C0FF);
                border-radius: 4px;
                width: 16px;
                subcontrol-position: right;
                subcontrol-origin: margin;
                border: 1px solid #888A85;}
            QScrollBar::sub-line:horizontal {
                background: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #FFFFFF, stop: 0.7 #3975D7,
                stop: 1.0 #C0C0FF);
                border-radius: 4px;
                width: 16px;
                subcontrol-position: top right;
                subcontrol-origin: margin;
                border: 1px solid #888A85;
                position: absolute;
                right: 20px;}
            QScrollBar:left-arrow:horizontal {
                image: url(medias/scroll_spinleft.png);} 
            QScrollBar::right-arrow:horizontal {
                image: url(medias/scroll_spinright.png);}
            QScrollBar::add-page:horizontal, QScrollBar::sub-page:horizontal {
                background: none;}"""

# Vertical scrollBar
VSCROLLBAR = """QScrollBar:vertical {border: 2px solid #888A85;
                border-radius: 6px;
                background: solid #505460;
                width: 16px;
                margin: 0px 0px 40px 0;}
            QScrollBar::handle:vertical {
                background: qlineargradient(
                x1: 0, y1: 1, x2: 1, y2: 1,
                stop: 0 #FFFFFF, stop: 0.7 #3975D7,
                stop: 1.0 #C0C0FF);
                border: 1px solid #0000FF;
                border-radius: 5px;
                min-height: 20px;}
            QScrollBar::add-line:vertical {
                background: qlineargradient(
                x1: 0, y1: 1, x2: 1, y2: 1,
                stop: 0 #FFFFFF, stop: 0.7 #3975D7,
                stop: 1.0 #C0C0FF);
                border-radius: 4px;
                height: 16px;
                subcontrol-position: bottom;
                subcontrol-origin: margin;
                border: 1px solid #888A85;}
            QScrollBar::sub-line:vertical {
                background: qlineargradient(
                x1: 0, y1: 1, x2: 1, y2: 1,
                stop: 0 #FFFFFF, stop: 0.7 #3975D7,
                stop: 1.0 #C0C0FF);
                border-radius: 4px;
                height: 16px;
                subcontrol-position: right bottom;
                subcontrol-origin: margin;
                border: 1px solid #888A85;
                position: absolute;
                bottom: 20px;}
            QScrollBar:up-arrow:vertical {
                image: url(medias/scroll_spinup.png);}
            QScrollBar::down-arrow:vertical {
                image: url(medias/scroll_spindown.png);}
            QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical {
                background: none;}"""

# QVerticalScrollBar for dialog boxes
VSCROLLBARDIAL = """QScrollBar:vertical {border: 2px solid #9C9C9C;
                        border-radius: 6px;
                        background: solid #505460;
                        width: 16px;
                        margin: 0px 0px 40px 0;}
                    QScrollBar::handle:vertical {
                        background: qlineargradient(
                        x1: 0, y1: 1, x2: 1, y2: 1,
                        stop: 0 #FFFFFF, stop: 0.7 #304060,
                        stop: 1.0 #000000);
                        border: 1px solid #000000;
                        border-radius: 5px;
                        min-height: 20px;}
                    QScrollBar::add-line:vertical {
                        background: qlineargradient(
                        x1: 0, y1: 1, x2: 1, y2: 1,
                        stop: 0 #FFFFFF, stop: 0.7 #304060,
                        stop: 1.0 #000000);
                        border-radius: 4px;
                        height: 16px;
                        subcontrol-position: bottom;
                        subcontrol-origin: margin;
                        border: 1px solid #9C9C9C;}
                    QScrollBar::sub-line:vertical {
                        background: qlineargradient(
                        x1: 0, y1: 1, x2: 1, y2: 1,
                        stop: 0 #FFFFFF, stop: 0.7 #304060,
                        stop: 1.0 #000000);
                        border-radius: 4px;
                        height: 16px;
                        subcontrol-position: right bottom;
                        subcontrol-origin: margin;
                        border: 1px solid #9C9C9C;
                        position: absolute;
                        bottom: 20px;}
                    QScrollBar:up-arrow:vertical {
                        image: url(medias/scroll_spinup.png);}
                    QScrollBar::down-arrow:vertical {
                        image: url(medias/scroll_spindown.png);}
                    QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical {
                        background: none;}"""

# QProgressBar
PROGRESSBAR = """QProgressBar {border: 2px solid #A0A5B2;
                border-radius: 4px;
                text-align: center;}
            QProgressBar::chunk {background: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #FFFFFF, stop: 0.7 #3975D7,
                stop: 1.0 #C0C0FF);
                border: 1px solid #0000FF;
                border-radius: 2px;}"""

SLIDER = """QSlider::groove:horizontal {border: 2px solid #A0A5B2;
                border-radius: 2px;
                height: 4px;
                background: #E8E8E8;}
            QSlider::handle:horizontal {background: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #FFFFFF, stop: 0.7 #3975D7,
                stop: 1.0 #C0C0FF);
                width: 20px;
                border-top: 1px solid #FFFFFF;
                border-left: 1px solid #0000FF;
                border-right: 1px solid #0000FF;
                border-bottom: 2px solid #0000CC;
                border-radius: 2px;
                margin: -3px 0;}"""

# -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

# Dialog boxes
# Subtitle
# Label
SUBTITLE = "QLabel {color: #555555;}"
# Underline
UNDERLINE = "QFrame {color: #555555;}"


# -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

# Metadata widget
# Page
DATAPAGE = "QWidget {background: #181E26;}"

# Title bar
DATATITLE = """QWidget {background-color: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #101010, stop: 0.4 #606060,
                stop: 0.5 #050505, stop: 1.0 #000000);
                border: 1px solid #000000;
                border-bottom-color: #000000;
                border-top-left-radius: 4px;
                border-top-right-radius: 4px;
                padding: 2px;
                color: #FFFFFF;}"""

# Editor header
DATAHEADER = """QHeaderView {background-color: #000000;
                    border: 1px solid #000000;
                    border-bottom-color: #000000;
                    border-top-left-radius: 4px;
                    border-top-right-radius: 4px;
                    color: #000000;}
                QHeaderView::section {background-color: #101A15;
                    border: 1px solid #101A15;
                    color: #FFFFFF;}"""

# QHeaderView, used for the QTableWidget in the image finder dialog
HEADERVIEW = """QHeaderView::section {
                        background-color: qlineargradient(x1:0, y1:0, x2:0, y2:1,
                        stop:0 #F4F4F4, stop: 0.5 #E8E8E8,
                        stop: 0.6 #E0E0E0, stop:1 #9A9A9A);
                        border-top: 1px solid #9A9A9A;
                        border-bottom: 2px solid #9A9A9A;
                        border-right: 1px solid #9A9A9A;
                        color: #080C14;
                        padding-left: 30px;}"""

# QTableView
TABLEVIEW = """QTableView {alternate-background-color: #F0F0F0;
                        selection-background-color: #AAAAAA;}"""

# QTreeView
DATATREE = """QTreeView {background: #181E26;
                border: 1px solid #000000;
                color: #FFFFFF;
                show-decoration-selected: 0;}
            QTreeView::item {border-top-color: 1px solid #000000;
                border-bottom-color: 1px solid #000000;}
            QTreeView::item:selected {background: #181E26;}"""

DATATREEHEADER = """QHeaderView {background-color: #000000;
                        border: 1px solid #000000;
                        border-bottom-color: #000000;
                        border-top-left-radius: 4px;
                        border-top-right-radius: 4px;
                        color: #000000;}
                    QHeaderView::section {background-color: #101A15;
                        border: 1px solid #101A15;
                        color: #FFFFFF;}"""

# QPlainTextEdit
TEXTEDIT2 = """QPlainTextEdit {background: qlineargradient(
                            x1: 0, y1: 0, x2: 0, y2: 1,
                            stop: 0 #FAFAFF, stop: 1.0 #EAEAEE);
                            border: 2px solid #8A8E96;
                            border-radius: 6px;
                            color: #000000;}"""

# -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

# Header bar for tool box
# Note: for the main header bar we add "padding: 2px;"
HEADER = """QWidget {background-color: qlineargradient(
                x1: 0, y1: 0, x2: 0, y2: 1,
                stop: 0 #101010, stop: 0.3 #606060,
                stop: 0.4 #050505, stop: 1.0 #000000);
                border: 1px solid #000000;
                border-bottom-color: #000000;
                border-top-left-radius: 4px;
                border-top-right-radius: 4px;
                color: #FFFFFF;}"""

# Title
TITLE = """QLabel {color: #FFFFFF;
                background-color: transparent;
                border-color: transparent;}"""

# Expand-collapse button
EXPAND = """QPushButton {background-color: transparent;
                border-color: transparent;}"""

# -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

def get_palette():
    palette = QApplication.palette()
    brush = QBrush(QColor(85, 170, 127))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Active, QPalette.Button, brush)
    # QAction text in Qmenu
    brush = QBrush(QColor('#181E26'))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Active, QPalette.ButtonText, brush)
    brush = QBrush(QColor(255, 255, 255))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Active, QPalette.Base, brush)
    brush = QBrush(QColor(250, 250, 252))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Active, QPalette.Window, brush)
    brush = QBrush(QColor(138, 142, 156))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Active, QPalette.Shadow, brush)
    brush = QBrush(QColor(40, 41, 83))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Active, QPalette.Highlight, brush)
    brush = QBrush(QColor(80, 84, 96))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Active, QPalette.ToolTipBase, brush)
    brush = QBrush(QColor(154, 154, 154))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Active, QPalette.ToolTipText, brush)
    brush = QBrush(QColor(85, 170, 127))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Inactive, QPalette.Button, brush)
    brush = QBrush(QColor(128, 0, 255))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Inactive, QPalette.ButtonText, brush)
    brush = QBrush(QColor(255, 255, 255))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Inactive, QPalette.Base, brush)
    brush = QBrush(QColor(246, 250, 250))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Inactive, QPalette.Window, brush)
    brush = QBrush(QColor(138, 142, 156))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Inactive, QPalette.Shadow, brush)
    brush = QBrush(QColor(40, 41, 83))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Inactive, QPalette.Highlight, brush)
    brush = QBrush(QColor(255, 4, 6))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Inactive, QPalette.ToolTipBase, brush)
    brush = QBrush(QColor(154, 154, 154))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Inactive, QPalette.ToolTipText, brush)
    brush = QBrush(QColor(85, 170, 127))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Disabled, QPalette.Button, brush)
    brush = QBrush(QColor(128, 0, 255))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Disabled, QPalette.ButtonText, brush)
    brush = QBrush(QColor(246, 250, 250))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Disabled, QPalette.Base, brush)
    brush = QBrush(QColor(246, 250, 250))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Disabled, QPalette.Window, brush)
    brush = QBrush(QColor(138, 142, 156))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Disabled, QPalette.Shadow, brush)
    brush = QBrush(QColor(217, 217, 217))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Disabled, QPalette.Highlight, brush)
    brush = QBrush(QColor(80, 84, 96))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Disabled, QPalette.ToolTipBase, brush)
    brush = QBrush(QColor(154, 154, 154))
    brush.setStyle(Qt.SolidPattern)
    palette.setBrush(QPalette.Disabled, QPalette.ToolTipText, brush)
    return palette
