# -*- coding: utf-8 -*-

# thumbnailer.py
# This file is part of Oqapy-3
#
# Author: Vincent Vande Vyvre <vincent.vandevyvre@oqapy.eu>
# Home: http://www.oqapy.eu
# Copyright: 2016 Vincent Vande Vyvre
# Licence: GPL3
#
# Thumbnail drawer

import logging
lgg = logging.getLogger(__name__)

from metadata.metadata import Metadata

from PyQt5 import QtCore, QtGui

class Thumbnailer:
    def __init__(self, image, thumbname, orientation=1, size=200, data=None):
        self.image = image
        self.thumbname = thumbname
        self.orientation = orientation
        self.size = size
        self.data = data

    def draw_and_save(self):
        if isinstance(self.image, str):
            qimg = QtGui.QImage(self.image)
            if qimg.isNull():
                qimg = QtGui.QImage("medias/No_Preview_200x150.jpg")

        else:
            qimg = self.image

        if self.orientation != 1:
            matrix = QtGui.QTransform()
            angles = {3: 180, 6: 90, 8: -90}
            qimg = qimg.transformed(matrix.rotate(angles[self.orientation]), 
                                    QtCore.Qt.SmoothTransformation)

        if qimg.width() > self.size or qimg.height() > self.size:
            qimg = qimg.scaled(self.size, self.size, 
                               QtCore.Qt.KeepAspectRatio, 
                               QtCore.Qt.SmoothTransformation)
                
        qimg.save(self.thumbname, None, -1)

    def copy_metadata(self):
        """Copy the metadata into a thumbnail file.

        """
        data = self.data.get_data_for_thumbnail()
        # The thumbnail has been re-oriented if necessary
        data['orientation'] = 1
        md = Metadata(self.thumbname)
        if md.get_data():
            md.write_thumbnail_data(data)

        else:
            lgg.warning('Read thumbnail data error: %s' % md.last_error)

